// SettingsDialog.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"   	// main symbols
#include "aerosubc.h"
#include "aeroglss.h"
#include "SettingsDialog.h"
#include "registry.h"
#include "desktopCalendar.h"
#include "MainHelperQ.h"
#include "GdiplusWrapper.h"
#include "utility.h"
#include "AboutDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

Helper m_Helper;

extern "C"
{
	typedef BOOL (*LPSetHook)(HWND hWnd);
	typedef BOOL (*LPClearHook)(HWND hWnd);
}

/////////////////////////////////////////////////////////////////////////////
// CStaticCtrl control

CStaticCtrl::CStaticCtrl()
{
}

CStaticCtrl::~CStaticCtrl()
{
}

void CStaticCtrl::DrawItem(LPDRAWITEMSTRUCT lpDrawItemStruct)
{
	int ctrlID = GetDlgCtrlID();
	switch(ctrlID)
	{
	case IDC_PREVIEW:
		SettingsDialog->m_SettingsDialogImagesQueued.DrawPreview();
		break;
	case IDC_PREVIEW2:
		SettingsDialog->m_SettingsDialogQueueImages.DrawPreview2();
		break;
	case IDC_SAMPLE_TEXT:
		SettingsDialog->m_SettingsDialogMisc.DrawSampleText();
		break;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CSettingsDialog dialog

CSettingsDialog::CSettingsDialog() : CDialog(CSettingsDialog::IDD), uiTimer(0)
{
	clock = new CClock();
}

CSettingsDialog::~CSettingsDialog()
{
	delete clock;

	if (hAOT != NULL)
	{
		LPClearHook pClearHook = (LPSetHook)GetProcAddress(hAOT, "ClearHook");
		if (pClearHook != NULL)
		{
			(pClearHook)(this->m_hWnd);
		}
		FreeLibrary(hAOT);
		hAOT = NULL;
	}

	// Remove the system tray icon.
	Shell_NotifyIcon(NIM_DELETE, &niData);

	SettingsDialog = NULL;
}

BEGIN_MESSAGE_MAP(CSettingsDialog, CDialog)
	ON_MESSAGE(WM_NI_MSG, OnNotifyIconMsg)
	ON_WM_TIMER()
	ON_MESSAGE(WM_HIDEWINDOW, OnHideMainWindow)
	ON_BN_CLICKED(IDC_APPLY, OnBnClickedApply)
	ON_COMMAND(ID_POPUPMENU_ABOUT, OnPopupMenuAbout)
	ON_COMMAND(ID_CONFIGUREDESKTOPCALENDAR, OnShowConfigWindow)
	ON_COMMAND(ID_SETWALLPAPERTONEXTIMAGE, OnSetToNextImage)
	ON_COMMAND(ID_EXITWPQ, OnExit)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CSettingsDialog message handlers

BOOL CSettingsDialog::DestroyWindow()
{
	m_TabSetting.DestroyWindow();
	return CDialog::DestroyWindow();
}

void CSettingsDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDOK, m_BtnOk);
	DDX_Control(pDX, IDCANCEL, m_BtnCancel);
	DDX_Control(pDX, IDC_APPLY, m_btnApply);
}
IMPLEMENT_DYNAMIC(CSettingsDialog, CDialog)

BOOL CSettingsDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);

	hAOT = LoadLibrary(_T("AOTHook.dll"));

	if (hAOT != NULL)
	{
		LPSetHook pSetHook = (LPSetHook)GetProcAddress(hAOT, "SetHook");
		if (pSetHook != NULL)
		{
			(pSetHook)(this->m_hWnd);
		}
	}

	// Initialize or reset dialog controls.
	ResetMainDialog();

	// Display the tray icon.
	niData.cbSize = NOTIFYICONDATA_V1_SIZE;
	niData.hWnd = this->m_hWnd;
	niData.uID = IDR_MAINFRAME;
	niData.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	niData.uCallbackMessage = WM_NI_MSG;
	niData.hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	_tcscpy(niData.szTip, _T("Desktop Calendar"));

	Shell_NotifyIcon(NIM_ADD, &niData);

	clock->Start();

	m_TabSetting.EnableStackedTabs(FALSE);
	m_TabSetting.AddPage(&m_SettingsDialogQueueImages);
	m_TabSetting.AddPage(&m_SettingsDialogImagesQueued);
	m_TabSetting.AddPage(&m_SettingsDialogMisc);

	m_TabSetting.SetTreeViewMode(TRUE, TRUE);
	m_TabSetting.SetTreeWidth(150);
	m_TabSetting.Create(this, WS_CHILD | WS_VISIBLE | WS_TABSTOP);
	m_TabSetting.ModifyStyleEx(0, WS_EX_CONTROLPARENT);

	CRect rcSheet, rcPage, rectTree;
	GetWindowRect(&rcSheet);
	ScreenToClient(&rcSheet);

	translate();

	m_TabSetting.SetActivePage(0);
	m_TabSetting.GetActivePage()->GetClientRect(rcPage);
	if (rcPage.Width() > rcSheet.Width())
		rcSheet.right += (rcPage.Width() - rcSheet.Width());
	if (rcPage.Height() > rcSheet.Height())
		rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

	m_TabSetting.SetActivePage(1);
	m_TabSetting.GetActivePage()->GetClientRect(rcPage);
	if (rcPage.Width() > rcSheet.Width())
		rcSheet.right += (rcPage.Width() - rcSheet.Width());
	if (rcPage.Height() > rcSheet.Height())
		rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

	m_TabSetting.SetActivePage(2);
	m_TabSetting.GetActivePage()->GetClientRect(rcPage);
	if (rcPage.Width() > rcSheet.Width())
		rcSheet.right += (rcPage.Width() - rcSheet.Width());
	if (rcPage.Height() > rcSheet.Height())
		rcSheet.bottom += (rcPage.Height() - rcSheet.Height());

	m_TabSetting.RefillPageTree();

	m_TabSetting.SetActivePage(0);
	m_TabSetting.GetTabControl()->ShowWindow(SW_HIDE);

	m_TabSetting.GetClientRect(&rcSheet);
	m_TabSetting.MoveWindow(&rcSheet);

	CString cstrCmdLine;

	// Get the command line.
	cstrCmdLine = AfxGetApp()->m_lpCmdLine;

	// If we send a command line of "-noexit", then keep the window up.
	if (cstrCmdLine.CompareNoCase(_T("-noexit")) == 0)
		m_SettingsDialogMisc.iExitAfter = 0;
	else
		PostMessage(WM_HIDEWINDOW);

	// Change the wallpaper.
	if (m_SettingsDialogMisc.iUseTimer == 1 && m_SettingsDialogMisc.iMinutes > 0)
		uiTimer = (UINT)SetTimer(1, m_SettingsDialogMisc.iMinutes * 60000, NULL);
	else
		m_SettingsDialogImagesQueued.ChangeWallpaper(m_SettingsDialogImagesQueued.m_chkRandomize.GetCheck());

	// get time
	time_t ltime;
	time(&ltime);
	// convert to local time
	struct tm* pTime;
	pTime = localtime(&ltime);

	int timerValue = pTime->tm_hour * 60 * 60 * 1000;
	timerValue += pTime->tm_min * 60 * 1000;
	timerValue += pTime->tm_sec * 1000;

	SetTimer(3, 24 * 60 * 60 * 1000 - timerValue, NULL);

	UpdateData(false);

	return TRUE;
}

void CSettingsDialog::translate()
{
	TC_ITEM item;
	CString buf;

	item.mask = TCIF_TEXT;

	m_SettingsDialogQueueImages.translate();
	m_SettingsDialogImagesQueued.translate();
	m_SettingsDialogMisc.translate();

	CPropertyPage *page = m_TabSetting.GetActivePage();
 	int i = 0;

	if(page == &m_SettingsDialogQueueImages)
		i = 0;
	else if(page == &m_SettingsDialogImagesQueued)
		i = 1;
	else if(page == &m_SettingsDialogMisc)
		i = 2;

	m_TabSetting.SetActivePage(0);
	buf = m_Helper.translateWindowTextString(&m_SettingsDialogQueueImages, IDD_SETTINGS_QUEUE_IMAGES);
	item.pszText = (LPTSTR)buf.GetString();
	m_TabSetting.GetTabControl()->SetItem(0, &item);
	m_TabSetting.SetActivePage(1);
	buf = m_Helper.translateWindowTextString(&m_SettingsDialogImagesQueued, IDD_SETTINGS_IMAGES_QUEUED);
	item.pszText = (LPTSTR)buf.GetString();
	m_TabSetting.GetTabControl()->SetItem(1, &item);
	m_TabSetting.SetActivePage(2);
	buf = m_Helper.translateWindowTextString(&m_SettingsDialogMisc, IDD_SETTINGS_MISC);
	item.pszText = (LPTSTR)buf.GetString();
	m_TabSetting.GetTabControl()->SetItem(2, &item);

	m_TabSetting.RefillPageTree();

	m_TabSetting.SetActivePage(i);

	m_Helper.translateWindowText(this, IDD_SETTINGS);
	m_Helper.translateDlgItemText(this, IDOK);
	m_Helper.translateDlgItemText(this, IDCANCEL);
	m_Helper.translateDlgItemText(this, IDC_APPLY);
}

void CSettingsDialog::OnOK()
{
	UpdateData(TRUE);
	// Apply changes, then minimize.
	ApplyChanges(SettingsDialog->m_btnApply.IsWindowEnabled());

	SettingsDialog->m_btnApply.EnableWindow(FALSE);
	SendMessage(WM_HIDEWINDOW);
}

void CSettingsDialog::OnCancel()
{
	m_SettingsDialogQueueImages.ResetMainDialog();
	m_SettingsDialogImagesQueued.ResetMainDialog();
	m_SettingsDialogMisc.ResetMainDialog();
	SendMessage(WM_HIDEWINDOW);
}

LRESULT CSettingsDialog::OnNotifyIconMsg(WPARAM wParam, LONG lParam)
{
	// Show a right-click menu from the system tray icon 
	// if they right-click or double click it.
	switch (lParam)
	{
	case WM_LBUTTONDBLCLK:
		SetForegroundWindow();
		break;
	case WM_RBUTTONUP:
		POINT point;
		CMenu menu;

		GetCursorPos(&point);

		menu.LoadMenu(IDR_NOTIFYICON_MENU);
		m_Helper.translateMenu(&menu, IDR_NOTIFYICON_MENU);
		CMenu* pMenu = menu.GetSubMenu(0);
		pMenu->SetDefaultItem(ID_CONFIGUREDESKTOPCALENDAR);

		SetForegroundWindow();

		pMenu->TrackPopupMenu(TPM_RIGHTALIGN | TPM_LEFTBUTTON | TPM_RIGHTBUTTON, point.x, point.y, this);

		PostMessage(WM_NULL, 0, 0);
		menu.DestroyMenu();
		break;
	}

	return TRUE;
}

void CSettingsDialog::OnTimer(UINT nIDEvent)
{
	CDialog::OnTimer(nIDEvent);

	if (nIDEvent == 1)
	{
		// When the timer is fired, change the wallpaper to the next image.
		m_SettingsDialogImagesQueued.ChangeWallpaper(m_SettingsDialogImagesQueued.m_chkRandomize.GetCheck());
	}
	else if (nIDEvent == 3)
	{
		// When the timer is fired, change the wallpaper to the next image.
		m_SettingsDialogImagesQueued.ChangeWallpaper(m_SettingsDialogImagesQueued.m_chkRandomize.GetCheck());
		KillTimer(3);

		// get time
		time_t ltime;
		time(&ltime);
		// convert to local time
		struct tm* pTime;
		pTime = localtime(&ltime);

		int timerValue = pTime->tm_hour * 60 * 60 * 1000;
		timerValue += pTime->tm_min * 60 * 1000;
		timerValue += pTime->tm_sec * 1000;

		SetTimer(3, 24 * 60 * 60 * 1000 - timerValue, NULL);
	}
}

void CSettingsDialog::OnShowConfigWindow()
{
	CRect trayrect;

	GetTrayWndRect(&trayrect);

	DrawAnimatedRects(IDANI_CAPTION, &trayrect, &rcLastSize);

	MoveWindow(&rcLastSize);
}

LRESULT CSettingsDialog::OnHideMainWindow(WPARAM wParam, LONG lParam)
{
	RECT rcTray, rcWindow;

	GetTrayWndRect(&rcTray);
	GetWindowRect(&rcWindow);

	rcLastSize = rcWindow;

	MoveWindow(rcTray.left, rcTray.top, 0, 0);

	DrawAnimatedRects(IDANI_CAPTION, &rcWindow, &rcTray);

	return TRUE;
}

void CSettingsDialog::OnSetToNextImage()
{
	ApplyChanges();
}

void CSettingsDialog::ApplyChanges(BOOL bChangeWallpaper /*= TRUE*/)
{
	// Delete all keys under desktopCalendar in the registry.
	CRegistry::RegDelete(_T("Software"), _T(""), _T("Desktop Calendar_V1.0"));
	m_TabSetting.PressButton(PSBTN_APPLYNOW);
	m_SettingsDialogQueueImages.ApplyChanges();
	m_SettingsDialogMisc.ApplyChanges();
	m_SettingsDialogImagesQueued.ApplyChanges(bChangeWallpaper);

	// If they chose to change the wallpaper every X minutes, kill any 
	// previous timers and start a new one based on the minutes entered.
	if (m_SettingsDialogMisc.iUseTimer == 1)
	{
		if (uiTimer != 0)
			KillTimer(uiTimer);

		if (m_SettingsDialogMisc.iMinutes > 0)
			uiTimer = (UINT)SetTimer(1, m_SettingsDialogMisc.iMinutes * 60000, NULL);
	}
	else
	{
		// Or just kill it if they didn't choose to use a timer.
		if (uiTimer != 0)
			KillTimer(uiTimer);
	}

	// Ask them if they want to exit now if they chose to exit after changing wallpaper.
	if (m_SettingsDialogMisc.iExitAfter == 1)
	{
		if (AfxMessageBox(m_Helper.MyLoadString(IDS_EXITAFTERCHANGE), MB_YESNO) == IDYES)
			OnExit();
	}
}

void CSettingsDialog::OnBnClickedApply()
{
	// Apply changes without closing/minimizing.
	UpdateData(true);
	ApplyChanges();

	SettingsDialog->m_btnApply.EnableWindow(FALSE);
}

void CSettingsDialog::OnExit()
{
	CDialog::OnCancel();
}

void CSettingsDialog::OnPopupMenuAbout()
{
	AboutDialog = AboutDialog->getInstance();
}

void CSettingsDialog::ResetMainDialog()
{
	int iScreenWidth, iScreenHeight;
	CString cstrScreenRes, cstrColors;
	CRect rect;

	CDC* pDC = GetDC();  // Get the device context of this window.
	// Get the screen dimensions.
	iScreenWidth = GetDeviceCaps(pDC->m_hDC, HORZRES);
	iScreenHeight = GetDeviceCaps(pDC->m_hDC, VERTRES);

	if (pDC)
		ReleaseDC(pDC);  // Release the device context retrieved earlier.

	// Move the window to the center of the screen.
	GetWindowRect(&rect);
	MoveWindow(iScreenWidth / 2 - (rect.Width()) / 2, iScreenHeight / 2 - (rect.Height()) / 2, (rect.Width()), (rect.Height()));

	SettingsDialog->m_btnApply.EnableWindow(FALSE);
	
	UpdateData(false);
}
