// SettingsDialogImagesQueued.cpp : fichier d'implmentation
//

#include "stdafx.h"
#include "resource.h"   	// main symbols
#include "aerosubc.h"
#include "aeroglss.h"
#include "SettingsDialog.h"
#include "registry.h"
#include "MainHelperQ.h"
#include "GdiplusWrapper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Bote de dialogue CSettingsDialogImagesQueued
IMPLEMENT_DYNAMIC(CSettingsDialogImagesQueued, CPropertyPage)

CSettingsDialogImagesQueued::CSettingsDialogImagesQueued() : CPropertyPage(CSettingsDialogImagesQueued::IDD)
{
	m_buildDesktopImage = new CBuildDesktopImage(this->m_hWnd);

	// Get a pointer to the IActiveDesktop interface.
	// We use this to set the wallpaper when not using .BMP files.
	pIActiveDesktop = NULL;
	HRESULT hr = CoCreateInstance(CLSID_ActiveDesktop, NULL, CLSCTX_INPROC_SERVER, IID_IActiveDesktop, (void**)&pIActiveDesktop);

	if (FAILED(hr))
		pIActiveDesktop = NULL;
}

CSettingsDialogImagesQueued::~CSettingsDialogImagesQueued()
{
	delete m_buildDesktopImage;

	// Release the IActiveDesktop interface.
	if (pIActiveDesktop != NULL)
	{
		pIActiveDesktop->Release();
		pIActiveDesktop = NULL;
	}
}

void CSettingsDialogImagesQueued::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_QLIST, m_lbQList);
	DDX_Control(pDX, IDC_RANDOMIZE, m_chkRandomize);
	DDX_Text(pDX, IDC_NBROWS, m_nbRows);
	DDX_Text(pDX, IDC_NBCOLS, m_nbCols);
	DDX_Control(pDX, IDC_PREVIEW, m_picPreview);
	DDX_Control(pDX, IDC_WIDTH, m_stWidth);
	DDX_Control(pDX, IDC_HEIGHT, m_stHeight);
	DDX_Control(pDX, IDC_TYPE, m_stType);
	DDX_Control(pDX, IDC_POSITION, m_comboPosition);
	DDX_Control(pDX, IDC_ARROWUP, m_btnArrowUp);
	DDX_Control(pDX, IDC_ARROWDOWN, m_btnArrowDown);
	DDX_Control(pDX, IDC_SETCURRENT, m_btnSetCurrent);
	DDX_Control(pDX, IDC_SPIN1, m_spin1);
	DDX_Control(pDX, IDC_SPIN2, m_spin2);
}

BOOL CSettingsDialogImagesQueued::OnInitDialog()
{
	CPropertyPage::OnInitDialog();

	AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);

	// Set bitmaps for the arrow buttons.
	m_btnArrowUp.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWUP)));
	m_btnArrowDown.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWDOWN)));

	m_buildDesktopImage->SetHWnd(SettingsDialog->m_hWnd);
	m_buildDesktopImage->SetIcon();

	translate();

	m_spin1.SetRange(1, 20);
	m_spin2.SetRange(1, 20);

	// Initialize or reset dialog controls.
	ResetMainDialog();

	return TRUE;
}

BOOL CSettingsDialogImagesQueued::OnEraseBkgnd(CDC* pDC) 
{
	CDwmApiImpl dwmApi;
	if(dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
	{
		CRect rcClient;
		GetClientRect(&rcClient);

		VERIFY(pDC->PatBlt(0, 0, rcClient.Width(), rcClient.Height(), BLACKNESS));
		return TRUE;
	}
	return CPropertyPage::OnEraseBkgnd(pDC);
}

void CSettingsDialogImagesQueued::DrawPreview()
{
	USES_CONVERSION;

	CString cstrWidth, cstrHeight, cstrType;
	UINT uiWidth, uiHeight;
	UINT thumbWidth, thumbHeight;
	int iAdjTop, iAdjLeft;
	int iCount = 0;
	double dRatio;
	CDC* pDC;
	TCHAR szBuf[MAX_PATH];
	RECT rect;
	CWnd* pWndTemp = NULL;
	BOOL bExists = TRUE;

	if (!IsWindowVisible())
		return;

	// Initialize the Graphics class in GDI+.
	pDC = m_picPreview.GetWindowDC();
	Graphics graphics(pDC->m_hDC);

	// Fill the preview ara with a WHITE background.
	m_picPreview.GetClientRect(&rect);
	rect.left += 1;
	rect.top += 1;
	rect.right -= 1;
	rect.bottom -= 1;
	pDC->FillSolidRect(&rect, RGB(255, 255, 255));

	iCount = m_lbQList.GetCount();  // QList has focus, so get its count.

	if (iCount > 0)
	{
		// Get the path to the image from the list with focus.
		m_lbQList.GetText(m_lbQList.GetCurSel(), szBuf);

		// Make sure there is an image chosen.
		if (szBuf[0] != '/0')
		{
			// Make sure the image exists
			CFile file;
			if (file.Open(szBuf, CFile::modeRead))
				file.Close();
			else
				bExists = FALSE;

			if (bExists)
			{
				try
				{
					// Use the Image class to display a thumbnail of the image.
					Image image(T2CW(szBuf));

					// Determine the appropriate size of the thumbnail preview
					// given the image size ratio and the preview window size ratio.
					uiWidth = image.GetWidth();
					uiHeight = image.GetHeight();
					dRatio = ((double)uiWidth) / ((double)uiHeight);

					// If the width is larger than the height of the image, 
					// set the width of the thumbnail to the width of the preview area
					// and calculate the thumbnail height by using the ratios.
					// If the height is larger than the width of the image, 
					// set the height of the thumbnail to the height of the preview area
					// and calculate the thumbnail width by using the ratios.
					if (uiWidth > uiHeight)
					{
						thumbWidth = rect.right - rect.left;
						thumbHeight = (UINT)(thumbWidth / dRatio);

						if (thumbHeight == 0)
							thumbHeight = 1;
						if (thumbHeight > (UINT)(rect.bottom - rect.top))
							thumbHeight = rect.bottom - rect.top;

						// Adjust things to make the preview not paint over the control border.
						iAdjTop = 1 + ((rect.bottom - rect.top) / 2) - (thumbHeight / 2);
						iAdjLeft = 1;
					}
					else
					{
						thumbHeight = rect.bottom - rect.top;
						thumbWidth = (UINT)(uiWidth * thumbHeight / uiHeight);

						if (thumbWidth == 0)
							thumbWidth = 1;
						if (thumbWidth > (UINT)(rect.right - rect.left))
							thumbWidth = rect.right - rect.left;

						// Adjust things to make the preview not paint over the control border.
						iAdjTop = 1;
						iAdjLeft = 1 + ((rect.right - rect.left) / 2) - (thumbWidth / 2);
					}

					// Get the thumbnail and display it in the preview control.
					Image* pThumbnail = image.GetThumbnailImage(thumbWidth, thumbHeight, NULL, NULL);
					graphics.DrawImage(pThumbnail, iAdjLeft, iAdjTop, pThumbnail->GetWidth(), pThumbnail->GetHeight());
					delete pThumbnail;

					// Display the image width and height.
					CString buf = m_Helper.MyLoadString(IDC_WIDTH);
					cstrWidth.Format(_T("%s %d"), buf, uiWidth);
					m_stWidth.SetWindowText(cstrWidth);
					buf = m_Helper.MyLoadString(IDC_HEIGHT);
					cstrHeight.Format(_T("%s %d"), buf, uiHeight);
					m_stHeight.SetWindowText(cstrHeight);

					// Display the image bits per pixel (color depth).
					buf = m_Helper.MyLoadString(IDC_TYPE);
					switch (image.GetPixelFormat())
					{
					case PixelFormat1bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_1BPP));
						break;
					case PixelFormat4bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_4BPP));
						break;
					case PixelFormat8bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_8BPP));
						break;
					case PixelFormat16bppARGB1555:
					case PixelFormat16bppGrayScale:
					case PixelFormat16bppRGB555:
					case PixelFormat16bppRGB565:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_16BPP));
						break;
					case PixelFormat24bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_24BPP));
						break;
					case PixelFormat32bppARGB:
					case PixelFormat32bppPARGB:
					case PixelFormat32bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_32BPP));
						break;
					case PixelFormat48bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_48BPP));
						break;
					case PixelFormat64bppARGB:
					case PixelFormat64bppPARGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_64BPP));
						break;
					default:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_UNKNOWN));
						break;
					}

					m_stType.SetWindowText(cstrType);
				}
				catch (...)
				{
				}
			}
			else
			{
				COLORREF oldCR = pDC->SetTextColor(RGB(255, 0, 0));
				//TODO
				pDC->DrawText(m_Helper.MyLoadString(IDS_FILENOTEXIST), &rect, DT_CENTER | DT_WORDBREAK);
				pDC->SetTextColor(oldCR);

				// No image to preview.
				CString buf = m_Helper.MyLoadString(IDC_WIDTH);
				m_stWidth.SetWindowText(buf);
				buf = m_Helper.MyLoadString(IDC_HEIGHT);
				m_stHeight.SetWindowText(buf);
				buf = m_Helper.MyLoadString(IDC_TYPE);
				m_stType.SetWindowText(buf);
			}
		}
		else
		{
			// No image to preview.
			CString buf = m_Helper.MyLoadString(IDC_WIDTH);
			m_stWidth.SetWindowText(buf);
			buf = m_Helper.MyLoadString(IDC_HEIGHT);
			m_stHeight.SetWindowText(buf);
			buf = m_Helper.MyLoadString(IDC_TYPE);
			m_stType.SetWindowText(buf);
		}
	}

	if (pDC)
		ReleaseDC(pDC);  // Release the device context retrieved earlier.
}


void CSettingsDialogImagesQueued::OnOK()
{
	UpdateData(TRUE);

	CPropertyPage::OnOK();
}

void CSettingsDialogImagesQueued::OnLbnSelchangeQlist()
{
	// Show the last selected item in the queued image list in the preview window.
	DrawPreview();

	if (m_lbQList.GetCount() > 0)
	{
		// If there are items in the queue list, enable the position combo box. 
		m_comboPosition.SetCurSel((int)m_lbQList.GetItemData(m_lbQList.GetCurSel()));
		m_comboPosition.EnableWindow();

		// Change the up and down arrow images.
		if (m_lbQList.GetSelCount() > 1)
		{
			m_btnArrowUp.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWUP)));
			m_btnArrowDown.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_DARROWDOWN)));
		}
		else if (m_lbQList.GetSelCount() == 1)
		{
			m_btnArrowUp.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_ARROWUP)));
			m_btnArrowDown.SetBitmap(::LoadBitmap(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDB_ARROWDOWN)));
		}
	}
}

void CSettingsDialogImagesQueued::OnBnClickedArrowup()
{
	int iIndex;
	DWORD dwData;
	CString cstrPath;

	// Move the selected item (single selection only) up one spot in the list.
	if (m_lbQList.GetCount() > 0 && m_lbQList.GetSelCount() == 1)
	{
		iIndex = m_lbQList.GetCurSel();

		if (iIndex > 0)
		{
			m_lbQList.GetText(iIndex, cstrPath);
			dwData = (DWORD)m_lbQList.GetItemData(iIndex);

			m_lbQList.DeleteString(iIndex);

			iIndex--;

			m_lbQList.InsertString(iIndex, cstrPath);
			m_lbQList.SetItemData(iIndex, dwData);

			m_lbQList.SetSel(iIndex);
			m_lbQList.SetFocus();
			OnLbnSelchangeQlist();

			SettingsDialog->m_btnApply.EnableWindow();
		}
	}
}

void CSettingsDialogImagesQueued::OnBnClickedArrowdown()
{
	int iIndex;
	DWORD dwData;
	CString cstrPath;

	// Move the selected item (single selection only) down one spot in the list.
	if (m_lbQList.GetSelCount() > 0 && m_lbQList.GetSelCount() == 1)
	{
		iIndex = m_lbQList.GetCurSel();

		if (iIndex < m_lbQList.GetCount())
		{
			m_lbQList.GetText(iIndex, cstrPath);
			dwData = (DWORD)m_lbQList.GetItemData(iIndex);

			m_lbQList.DeleteString(iIndex);

			iIndex++;

			m_lbQList.InsertString(iIndex, cstrPath);
			m_lbQList.SetItemData(iIndex, dwData);

			m_lbQList.SetSel(iIndex);
			m_lbQList.SetFocus();
			OnLbnSelchangeQlist();

			SettingsDialog->m_btnApply.EnableWindow();
		}
	}
}


void CSettingsDialogImagesQueued::OnBnClickedRemoveselected()
{
	int iCount = 0;

	// Get all selected items from the browse list.
	iCount = m_lbQList.GetSelCount();
	CArray<int, int> aryListBoxSel;

	aryListBoxSel.SetSize(iCount);
	m_lbQList.GetSelItems(iCount, aryListBoxSel.GetData());

	// Loop through each selected item and delete it from the Queue list.
	for (int i = 0; i < aryListBoxSel.GetSize(); i++)
		m_lbQList.DeleteString(aryListBoxSel.GetAt(i));

	SettingsDialog->m_btnApply.EnableWindow();
}


void CSettingsDialogImagesQueued::OnBnClickedQlistempty()
{
	// Empty the Queued images list.
	m_lbQList.ResetContent();

	SettingsDialog->m_btnApply.EnableWindow();
}

void CSettingsDialogImagesQueued::OnCbnSelchangePosition()
{
	int iPosition = 1;
	int iCount = 0;

	iPosition = m_comboPosition.GetCurSel();

	// Get all selected items from the queued image list.
	iCount = m_lbQList.GetSelCount();
	CArray<int, int> aryListBoxSel;

	aryListBoxSel.SetSize(iCount);
	m_lbQList.GetSelItems(iCount, aryListBoxSel.GetData());

	// Loop through each selected item and change its wallpaper position.
	for (int i = 0; i < aryListBoxSel.GetSize(); i++)
		m_lbQList.SetItemData(aryListBoxSel.GetAt(i), (DWORD)iPosition);

	SettingsDialog->m_btnApply.EnableWindow();
}

void CSettingsDialogImagesQueued::OnEnableApply()
{
	if(IsWindowVisible())
		SettingsDialog->m_btnApply.EnableWindow();
}

BEGIN_MESSAGE_MAP(CSettingsDialogImagesQueued, CPropertyPage)
	ON_WM_ERASEBKGND()
	ON_LBN_SELCHANGE(IDC_QLIST, OnLbnSelchangeQlist)
	ON_CBN_SELCHANGE(IDC_POSITION, OnCbnSelchangePosition)
	ON_BN_CLICKED(IDC_SETCURRENT, OnBnClickedSetCurrent)
	ON_BN_CLICKED(IDC_RANDOMIZE, OnEnableApply)
	ON_BN_CLICKED(IDC_ARROWUP, OnBnClickedArrowup)
	ON_BN_CLICKED(IDC_ARROWDOWN, OnBnClickedArrowdown)
	ON_BN_CLICKED(IDC_REMOVESELECTED, OnBnClickedRemoveselected)
	ON_BN_CLICKED(IDC_QLISTEMPTY, OnBnClickedQlistempty)
	ON_EN_CHANGE(IDC_NBROWS, OnEnableApply)
	ON_EN_CHANGE(IDC_NBCOLS, OnEnableApply)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN1, OnDeltaposSpin)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN2, OnDeltaposSpin)
END_MESSAGE_MAP()

void CSettingsDialogImagesQueued::ResetMainDialog()
{
	// Set the randomize checkbox.
	m_chkRandomize.SetCheck((int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Randomize")));
	m_nbRows = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NbRows"), 1);
	m_nbCols = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NbCols"), 1);
	m_spin1.SetPos(m_nbRows);
	m_spin2.SetPos(m_nbCols);

	// Disable the position combobox since nothing is selected in the queue list. 
	m_comboPosition.EnableWindow(FALSE);

	// Fill the queue list with currently queued images.
	FillQueuedImages();

}

void CSettingsDialogImagesQueued::FillQueuedImages()
{
	DWORD dwNumImagesInQueue = 0;
	CString cstrImageInfoPath = _T("");
	CString cstrImageInfo = _T("");
	int iQueuedWallpaperPosition = 0;

	// Get the total number of already queued images from the registry.
	dwNumImagesInQueue = CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NumImages"));

	// Clear the queued image list.
	m_lbQList.ResetContent();

	// Fill the queued images list with already queued images from the registry.
	for (DWORD dwLoop = 0; dwLoop < dwNumImagesInQueue; dwLoop++)
	{
		cstrImageInfoPath.Format(_T("%d"), dwLoop);

		cstrImageInfo = CRegistry::GetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), cstrImageInfoPath);

		// We save the image path and also the "position" of the wallpaper in the same string (comma delimited).
		int iIndex = m_lbQList.InsertString((int)dwLoop, cstrImageInfo.Left(cstrImageInfo.Find(_T(","))));
		int iWPPos = _tstoi(cstrImageInfo.Right(cstrImageInfo.GetLength() - cstrImageInfo.Find(_T(",")) - 1));
		m_lbQList.SetItemData(iIndex, iWPPos);
	}

	// Set the currently displayed wallpaper item.
	iCurrentImage = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("CurrentImage"));

	// Select the first item.
	if (dwNumImagesInQueue > 0)
	{
		for (int i = 0; i < m_lbQList.GetCount(); i++)
			m_lbQList.SetSel(i, i == iCurrentImage);
		m_lbQList.SetFocus();
		OnLbnSelchangeQlist();
	}
}

void CSettingsDialogImagesQueued::ChangeWallpaper(int iMode)
{
	USES_CONVERSION;

	int iImageNum = 0;
	int iPosition = 1;
	CString cstrTileWallpaper;
	CString cstrWallpaperStyle;
	TCHAR szBuf[MAX_PATH];
	m_buildDesktopImage->ReadRegistry();
	m_buildDesktopImage->StartBitmap();

	// Choose then next image in the list, selected image, or choose a random image.
	if (iMode == 1)  // random
	{
		srand((unsigned)time(NULL));

		iImageNum = rand() % m_lbQList.GetCount();

		iCurrentImage = iImageNum;
	}
	else if (iMode == 2)
	{
		iCurrentImage = m_lbQList.GetCurSel();

		iImageNum = iCurrentImage;
	}
	else
	{
		iCurrentImage++;
		if (iCurrentImage + 1 > m_lbQList.GetCount())
			iCurrentImage = 0;

		iImageNum = iCurrentImage;
	}
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("CurrentImage"), (DWORD)iCurrentImage);

	int xCount = m_nbCols;
	int yCount = m_nbRows;
	for (int i = 0; i< m_buildDesktopImage->m_MonitorArray.GetSize(); i++)
	{
		for (int k = 0; k < xCount; k++)
		{
			for (int l = 0; l < yCount; l++)
			{
				if (i > 0 || k > 0 || l > 0)
				{
					// Choose then next image in the list, selected image, or choose a random image.
					if (iMode == 1)  // random
					{
						iImageNum = rand() % m_lbQList.GetCount();
					}
					else
					{
						iImageNum++;
						if (iImageNum + 1 > m_lbQList.GetCount())
							iImageNum = 0;
					}
				}
				// Get the image path from the registry by the image number.
				iPosition = (int)m_lbQList.GetItemData(iImageNum);
				m_lbQList.GetText(iImageNum, szBuf);
				CString tmp;
				tmp.Format(_T("%s"), szBuf);
				m_buildDesktopImage->SetImage(tmp, iPosition, i, k, l, xCount, yCount);
			}
		}
	}

	m_buildDesktopImage->StopBitmap();

	TCHAR path[MAX_PATH];
	GetModuleFileName(AfxGetInstanceHandle(), path, MAX_PATH);
	TCHAR drive[MAX_PATH];
	TCHAR dir[MAX_PATH];
	_tsplitpath(path, drive, dir, NULL, NULL);
	CString szNewFile;
	szNewFile.Format(_T("%s%sDesktop Calendar.bmp"), drive, dir);
	_tcscpy(szBuf, szNewFile.operator LPCTSTR());

	if (szBuf[0] != '/0')
	{
		// If Active Desktop is not available, use the SystemParametersInfo function.
		if (pIActiveDesktop == NULL)
		{
			if (iPosition == 0)
			{
				// Tiled
				cstrTileWallpaper = _T("1");
				cstrWallpaperStyle = _T("1");
			}
			else if (iPosition == 1)
			{
				// Center
				cstrTileWallpaper = _T("0");
				cstrWallpaperStyle = _T("1");
			}
			else
			{
				// Stretch
				cstrTileWallpaper = _T("0");
				cstrWallpaperStyle = _T("2");
			}
			cstrTileWallpaper = _T("1");
			cstrWallpaperStyle = _T("1");

			// Set registry values for the wallpaper position.
			CRegistry::SetRegString(_T("Control Panel\\Desktop"), _T("TileWallpaper"), cstrTileWallpaper);
			CRegistry::SetRegString(_T("Control Panel\\Desktop"), _T("WallpaperStyle"), cstrWallpaperStyle);

			// Change the wallpaper.
			SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, (void*)szBuf, SPIF_UPDATEINIFILE | SPIF_SENDWININICHANGE);
		}
		else
		{
			WALLPAPEROPT wpOptions;
			COMPONENTSOPT compOptions;

			compOptions.dwSize = sizeof(COMPONENTSOPT);
			compOptions.fActiveDesktop = TRUE;
			compOptions.fEnableComponents = TRUE;

			pIActiveDesktop->SetDesktopItemOptions(&compOptions, 0);

			// Set the wallpaper position.
			wpOptions.dwSize = sizeof(WALLPAPEROPT);
			if (iPosition == 0)
				wpOptions.dwStyle = WPSTYLE_TILE;
			else if (iPosition == 1)
				wpOptions.dwStyle = WPSTYLE_CENTER;
			else
				wpOptions.dwStyle = WPSTYLE_STRETCH;
			wpOptions.dwStyle = WPSTYLE_TILE;

			pIActiveDesktop->SetWallpaperOptions(&wpOptions, 0);

			// Set the wallpaper image.
			pIActiveDesktop->SetWallpaper(T2CW(szBuf), 0);

			// Apply all changes.
			pIActiveDesktop->ApplyChanges(AD_APPLY_ALL);
		}
	}
}

BOOL CSettingsDialogImagesQueued::OnSetActive()
{
	// Set the currently displayed wallpaper item.
	iCurrentImage = (int)CRegistry::GetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("CurrentImage"));

	// Select it in the list
	for (int i = 0; i < m_lbQList.GetCount(); i++)
		m_lbQList.SetSel(i, i == iCurrentImage);
	m_lbQList.SetFocus();
	OnLbnSelchangeQlist();

	return TRUE;
}

void CSettingsDialogImagesQueued::OnBnClickedSetCurrent()
{
	// tell the ChangeWallpaper function to set the wallpaper to the current selection inthe Q list
	UpdateData(true);
	ApplyChanges(FALSE);
	ChangeWallpaper(2);

	m_lbQList.SetFocus();
	Invalidate(FALSE);
}

void CSettingsDialogImagesQueued::translate()
{
	if(this->m_hWnd != NULL)
	{
		m_Helper.translateWindowText(this, IDD_SETTINGS_IMAGES_QUEUED);
		m_Helper.translateDlgItemText(this, IDC_POS);
		m_Helper.translateDlgItemText(this, IDC_SETCURRENT);
		m_Helper.translateDlgItemText(this, IDC_REMOVESELECTED);
		m_Helper.translateDlgItemText(this, IDC_QLISTEMPTY);
		m_Helper.translateDlgItemText(this, IDC_INFOIMG);
		m_Helper.translateDlgItemText(this, IDC_WIDTH);
		m_Helper.translateDlgItemText(this, IDC_HEIGHT);
		m_Helper.translateDlgItemText(this, IDC_TYPE);
		m_Helper.translateDlgItemText(this, IDC_RANDOMIZE);
		m_Helper.translateDlgItemText(this, IDC_LNBROWS);
		m_Helper.translateDlgItemText(this, IDC_LNBCOLS);

		m_comboPosition.ResetContent();
		m_comboPosition.AddString(m_Helper.MyLoadString(IDS_TILED));
		m_comboPosition.AddString(m_Helper.MyLoadString(IDS_CENTER));
		m_comboPosition.AddString(m_Helper.MyLoadString(IDS_STRETCH));
		m_comboPosition.AddString(m_Helper.MyLoadString(IDS_ADJUST));
	}
}

void CSettingsDialogImagesQueued::ApplyChanges(BOOL bChangeWallpaper /*= TRUE*/)
{
	// Set the number of images saved in the registry from the queue list.
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NumImages"), m_lbQList.GetCount());

	// Loop through the queued image list and save each image in the registry.
	for (int iLoop = 0; iLoop < m_lbQList.GetCount(); iLoop++)
	{
		CString cstrPath;
		CString cstrRegValue = _T("");
		CString cstrRegPath = _T("");

		m_lbQList.GetText(iLoop, cstrPath);
		cstrRegPath.Format(_T("%s,%d"), cstrPath, (int)m_lbQList.GetItemData(iLoop));

		cstrRegValue.Format(_T("%d"), iLoop);

		CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), cstrRegValue, cstrRegPath);
	}

	// Set whether to display a random image when run.
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("Randomize"), (DWORD)m_chkRandomize.GetCheck());
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NbRows"), (DWORD)m_nbRows);
	CRegistry::SetRegDWORD(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("NbCols"), (DWORD)m_nbCols);

	// Change the wallpaper.
	if (bChangeWallpaper)
		ChangeWallpaper(m_chkRandomize.GetCheck());
}

void CSettingsDialogImagesQueued::OnDeltaposSpin(NMHDR *pNMHDR, LRESULT *pResult)
{
	SettingsDialog->m_btnApply.EnableWindow();
}

