// SettingsDialogQueueImages.cpp : fichier d'implmentation
//

#include "stdafx.h"
#include "resource.h"   	// main symbols
#include "aerosubc.h"
#include "aeroglss.h"
#include "SettingsDialog.h"
#include "registry.h"
#include "MainHelperQ.h"
#include "GdiplusWrapper.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define NETWORK_NEIGHBOUR   _T("Network Neighborhood")

bool ParsePath(CString sPath, int nLevel, CString *const psDirName)
{
	ASSERT(psDirName);
	//find first match
	int nStart = 0;									//Location in sPath to search from
	while (nLevel)
	{
		nStart = sPath.Find(_T('\\'), nStart);
		if (nStart < 0)
			return false;
		nLevel--;
		nStart++;
	}
	//We now have the start point to find the end.
	int nEnd = sPath.Find(_T('\\'), nStart);
	if (nEnd < 0)
		*psDirName = sPath.Mid(nStart);
	else
		*psDirName = sPath.Mid(nStart, nEnd - nStart);
	return true;
}

TCHAR* MakeObjectDynamic(LPTSTR szData)
{
	//Assume a NULL = empty string
	TCHAR* szRet = NULL;
	int nLength = 0;
	if (szData)
		nLength = _tcslen(szData) + 1;
	if (nLength > 0)
	{
		szRet = new TCHAR[nLength];
		ASSERT(szRet);
		_tcscpy(szRet, szData);
	}
	return szRet;
}

// Bote de dialogue CSettingsDialogQueueImages
IMPLEMENT_DYNAMIC(CSettingsDialogQueueImages, CPropertyPage)

CSettingsDialogQueueImages::CSettingsDialogQueueImages() : CPropertyPage(CSettingsDialogQueueImages::IDD)
{
}

CSettingsDialogQueueImages::~CSettingsDialogQueueImages()
{
}

void CSettingsDialogQueueImages::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);

	DDX_Control(pDX, IDC_BROWSELIST, m_lbBrowseList);
	DDX_Control(pDX, IDC_PREVIEW2, m_picPreview2);
	DDX_Control(pDX, IDC_TYPE2, m_stType2);
	DDX_Control(pDX, IDC_HEIGHT2, m_stHeight2);
	DDX_Control(pDX, IDC_WIDTH2, m_stWidth2);
	DDX_Control(pDX, IDC_TREE1, m_Tree);
}

BOOL CSettingsDialogQueueImages::OnInitDialog()
{
	CPropertyPage::OnInitDialog();

	AeroAutoSubclass(m_hWnd, ASC_SUBCLASS_ALL_CONTROLS, 0L);

	translate();

	// Initialize or reset dialog controls.
	ResetMainDialog();

	//
	//Setup the Image lists
	//
	m_ImageListTree.Create(IDB_FILE_TYPES, 17, 1, RGB(255, 255, 255));
	m_Tree.SetImageList(&m_ImageListTree, TVSIL_NORMAL);

	//
	//List the local drives
	//
	for (TCHAR cLetter = _T('A'); cLetter <= _T('Z'); cLetter++)
	{
		CString sDrive;
		sDrive.Format(_T("%c:"), cLetter);						
		UINT nType = GetDriveType(sDrive + _T("\\"));
		if (DRIVE_REMOVABLE <= nType && nType <= DRIVE_RAMDISK)
			InsertItem(TVI_ROOT, NULL, sDrive, nType);
	}

	//
	//Expand the Tree to the Inital Path.
	//
	int nLevel = 0;
	CString sDirName;
	HTREEITEM hCurrent = TVI_ROOT;					//Current item in the list being expanded.
	CString tmpstrBrowsePath = cstrBrowsePath;
	while (ParsePath(tmpstrBrowsePath, nLevel, &sDirName))
	{
		//Network or Drive expand
		if (!nLevel && sDirName.IsEmpty())
			sDirName = NETWORK_NEIGHBOUR;
		//Search for the matching tree item and expand
		HTREEITEM hItem = m_Tree.GetChildItem(hCurrent);	
		while (hItem)
		{
			if (sDirName.CompareNoCase(m_Tree.GetItemText(hItem)) == 0)
			{
				hCurrent = hItem;
				m_Tree.Expand(hCurrent, TVE_EXPAND);
				break;
			}
			hItem = m_Tree.GetNextSiblingItem(hItem);
		}
		nLevel++;
	}

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

BOOL CSettingsDialogQueueImages::OnEraseBkgnd(CDC* pDC) 
{
	CDwmApiImpl dwmApi;
	if(dwmApi.Initialize() && dwmApi.IsDwmCompositionEnabled())
	{
		CRect rcClient;
		GetClientRect(&rcClient);

		VERIFY(pDC->PatBlt(0, 0, rcClient.Width(), rcClient.Height(), BLACKNESS));
		return TRUE;
	}
	return CPropertyPage::OnEraseBkgnd(pDC);
}
void CSettingsDialogQueueImages::OnOK()
{
	UpdateData(TRUE);

	CPropertyPage::OnOK();
}
BEGIN_MESSAGE_MAP(CSettingsDialogQueueImages, CPropertyPage)
	ON_WM_ERASEBKGND()
	ON_NOTIFY(TVN_SELCHANGED, IDC_TREE1, OnSelchangedTree)
	ON_NOTIFY(TVN_ITEMEXPANDING, IDC_TREE1, OnItemexpandingTree)
	ON_LBN_SELCHANGE(IDC_BROWSELIST, DrawPreview2)	
	ON_BN_CLICKED(IDC_ADD, OnBnClickedAdd)
END_MESSAGE_MAP()

void CSettingsDialogQueueImages::DrawPreview2()
{
	USES_CONVERSION;

	CString cstrWidth, cstrHeight, cstrType;
	UINT uiWidth, uiHeight;
	UINT thumbWidth, thumbHeight;
	int iAdjTop, iAdjLeft;
	int iCount = 0;
	double dRatio;
	CDC* pDC;
	TCHAR szBuf[MAX_PATH];
	RECT rect;
	CWnd* pWndTemp = NULL;
	BOOL bExists = TRUE;

	if (!IsWindowVisible())
		return;

	// Initialize the Graphics class in GDI+.
	pDC = m_picPreview2.GetWindowDC();
	Graphics graphics(pDC->m_hDC);

	// Fill the preview ara with a WHITE background.
	m_picPreview2.GetClientRect(&rect);
	rect.left += 1;
	rect.top += 1;
	rect.right -= 1;
	rect.bottom -= 1;
	pDC->FillSolidRect(&rect, RGB(255, 255, 255));

	iCount = m_lbBrowseList.GetCount();  // LeftList has focus, so get its count.

	if (iCount > 0)
	{
		// Get the path to the image from the list with focus.
		_tcscpy(szBuf, cstrBrowsePath.GetString());
		TCHAR* tmp = szBuf + cstrBrowsePath.GetLength();
		m_lbBrowseList.GetText(m_lbBrowseList.GetCurSel(), tmp);

		// Make sure there is an image chosen.
		if (szBuf[0] != '/0')
		{
			// Make sure the image exists
			CFile file;
			if (file.Open(szBuf, CFile::modeRead))
				file.Close();
			else
				bExists = FALSE;

			if (bExists)
			{
				try
				{
					// Use the Image class to display a thumbnail of the image.
					Image image(T2CW(szBuf));

					// Determine the appropriate size of the thumbnail preview
					// given the image size ratio and the preview window size ratio.
					uiWidth = image.GetWidth();
					uiHeight = image.GetHeight();
					dRatio = ((double)uiWidth) / ((double)uiHeight);

					// If the width is larger than the height of the image, 
					// set the width of the thumbnail to the width of the preview area
					// and calculate the thumbnail height by using the ratios.
					// If the height is larger than the width of the image, 
					// set the height of the thumbnail to the height of the preview area
					// and calculate the thumbnail width by using the ratios.
					if (uiWidth > uiHeight)
					{
						thumbWidth = rect.right - rect.left;
						thumbHeight = (UINT)(thumbWidth / dRatio);

						if (thumbHeight == 0)
							thumbHeight = 1;
						if (thumbHeight > (UINT)(rect.bottom - rect.top))
							thumbHeight = rect.bottom - rect.top;

						// Adjust things to make the preview not paint over the control border.
						iAdjTop = 1 + ((rect.bottom - rect.top) / 2) - (thumbHeight / 2);
						iAdjLeft = 1;
					}
					else
					{
						thumbHeight = rect.bottom - rect.top;
						thumbWidth = (UINT)(uiWidth * thumbHeight / uiHeight);

						if (thumbWidth == 0)
							thumbWidth = 1;
						if (thumbWidth > (UINT)(rect.right - rect.left))
							thumbWidth = rect.right - rect.left;

						// Adjust things to make the preview not paint over the control border.
						iAdjTop = 1;
						iAdjLeft = 1 + ((rect.right - rect.left) / 2) - (thumbWidth / 2);
					}

					// Get the thumbnail and display it in the preview control.
					Image* pThumbnail = image.GetThumbnailImage(thumbWidth, thumbHeight, NULL, NULL);
					graphics.DrawImage(pThumbnail, iAdjLeft, iAdjTop, pThumbnail->GetWidth(), pThumbnail->GetHeight());
					delete pThumbnail;

					// Display the image width and height.
					CString buf = m_Helper.MyLoadString(IDC_WIDTH2);
					cstrWidth.Format(_T("%s %d"), buf, uiWidth);
					m_stWidth2.SetWindowText(cstrWidth);
					buf = m_Helper.MyLoadString(IDC_HEIGHT2);
					cstrHeight.Format(_T("%s %d"), buf, uiHeight);
					m_stHeight2.SetWindowText(cstrHeight);

					// Display the image bits per pixel (color depth).
					buf = m_Helper.MyLoadString(IDC_TYPE2);
					switch (image.GetPixelFormat())
					{
					case PixelFormat1bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_1BPP));
						break;
					case PixelFormat4bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_4BPP));
						break;
					case PixelFormat8bppIndexed:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_8BPP));
						break;
					case PixelFormat16bppARGB1555:
					case PixelFormat16bppGrayScale:
					case PixelFormat16bppRGB555:
					case PixelFormat16bppRGB565:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_16BPP));
						break;
					case PixelFormat24bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_24BPP));
						break;
					case PixelFormat32bppARGB:
					case PixelFormat32bppPARGB:
					case PixelFormat32bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_32BPP));
						break;
					case PixelFormat48bppRGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_48BPP));
						break;
					case PixelFormat64bppARGB:
					case PixelFormat64bppPARGB:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_64BPP));
						break;
					default:
						cstrType.Format(_T("%s %s"), buf, m_Helper.MyLoadString(IDS_UNKNOWN));
						break;
					}

					m_stType2.SetWindowText(cstrType);
				}
				catch (...)
				{
				}
			}
			else
			{
				COLORREF oldCR = pDC->SetTextColor(RGB(255, 0, 0));
				//TODO
				pDC->DrawText(m_Helper.MyLoadString(IDS_FILENOTEXIST), &rect, DT_CENTER | DT_WORDBREAK);
				pDC->SetTextColor(oldCR);

				// No image to preview.
				CString buf = m_Helper.MyLoadString(IDC_WIDTH2);
				m_stWidth2.SetWindowText(buf);
				buf = m_Helper.MyLoadString(IDC_HEIGHT2);
				m_stHeight2.SetWindowText(buf);
				buf = m_Helper.MyLoadString(IDC_TYPE2);
				m_stType2.SetWindowText(buf);
			}
		}
		else
		{
			// No image to preview.
			CString buf = m_Helper.MyLoadString(IDC_WIDTH2);
			m_stWidth2.SetWindowText(buf);
			buf = m_Helper.MyLoadString(IDC_HEIGHT2);
			m_stHeight2.SetWindowText(buf);
			buf = m_Helper.MyLoadString(IDC_TYPE2);
			m_stType2.SetWindowText(buf);
		}
	}

	if (pDC)
		ReleaseDC(pDC);  // Release the device context retrieved earlier.
}

BOOL CSettingsDialogQueueImages::DestroyWindow()
{
	//Start at the root deleting.
	HTREEITEM hItemCur = m_Tree.GetRootItem();		//Current item under inspection

	//
	//While their is still data in the tree
	//
	while (hItemCur)
	{
		HTREEITEM hItem = m_Tree.GetChildItem(hItemCur);
		//Has children then make the child current
		if (hItem)
		{
			hItemCur = hItem;
			continue;
		}

		//Item has not children so we shall destroy it.
		//but first we must decide who is to take its place.
		HTREEITEM hNextSibItem = m_Tree.GetNextSiblingItem(hItemCur);
		HTREEITEM hPrevSibItem = m_Tree.GetPrevSiblingItem(hItemCur);
		HTREEITEM hParentItem = m_Tree.GetParentItem(hItemCur);
		//Get item data to check if lparam is to be destroyed
		NETRESOURCE *const pNetResource = (NETRESOURCE*)m_Tree.GetItemData(hItemCur);//(item.lParam);
		if (pNetResource)
		{
			delete[] (pNetResource->lpLocalName);
			delete[] (pNetResource->lpRemoteName);
			delete[] (pNetResource->lpComment);
			delete[] (pNetResource->lpProvider);
			delete pNetResource;
		}
		m_Tree.DeleteItem(hItemCur);
		hItemCur = NULL;
		//Determine which item is next to recieve the focus
		if (hParentItem)
			hItemCur = hParentItem;
		if (hPrevSibItem)
			hItemCur = hPrevSibItem;
		if (hNextSibItem)
			hItemCur = hNextSibItem;
	}	
	//All items removed from list now. Lets trash this place and go home.
	return CDialog::DestroyWindow();
}

HTREEITEM CSettingsDialogQueueImages::InsertItem(HTREEITEM hParent, NETRESOURCE *const pNetResource, CString sText, int iImage, int iImageSelected /* = -1 */)
{
	TVINSERTSTRUCT InsertStruct;
	InsertStruct.hParent = hParent;	
	InsertStruct.hInsertAfter = TVI_LAST;
	InsertStruct.itemex.mask = TVIF_IMAGE | TVIF_TEXT | TVIF_CHILDREN | TVIF_SELECTEDIMAGE | TVIF_PARAM;
	InsertStruct.itemex.pszText = sText.GetBuffer(sText.GetLength());  sText.ReleaseBuffer();
	InsertStruct.itemex.iImage = iImage;
	InsertStruct.itemex.cChildren = 1;
	InsertStruct.itemex.lParam = (LPARAM)pNetResource;
	if (iImageSelected == -1)
		InsertStruct.itemex.iSelectedImage = iImage;
	else
		InsertStruct.itemex.iSelectedImage = iImageSelected;
	return m_Tree.InsertItem(&InsertStruct);
}

bool CSettingsDialogQueueImages::PopulateTree(CString sPath, HTREEITEM hParent)
{
	bool bGotChildren = false;						//True if a child is added.	

	//
	//Search for files and populate the CTreeCtrl
	//
	CFileFind finder;
	BOOL bWorking = finder.FindFile(sPath + _T("*.*"));
	while (bWorking)
	{
		bWorking = finder.FindNextFile();
		if (finder.IsDots())
			continue;
		if (finder.IsDirectory())
		{
			InsertItem(hParent, NULL, finder.GetFileName(), DRIVE_NO_ROOT_DIR, DRIVE_UNKNOWN);
			bGotChildren = true;
		}
	}

	//
	//Remove the [+] if no children
	//
	if (!bGotChildren)
	{
		TVITEM item =
		{
			0
		};
		item.mask = TVIF_HANDLE | TVIF_CHILDREN;
		item.hItem = hParent;
		item.cChildren = 0;
		m_Tree.SetItem(&item);
	}
	return bGotChildren;
}

CString CSettingsDialogQueueImages::GetItemPath(HTREEITEM hItem)
{
	CString sRet;
	do
	{
		//End with a share name.
		NETRESOURCE *const pNetResource = (NETRESOURCE*)(m_Tree.GetItemData(hItem));
		if (pNetResource)
		{
			sRet = CString(pNetResource->lpRemoteName) + _T('\\') + sRet; 		
			break;
		}
		//Add the directory name to the path.
		sRet = m_Tree.GetItemText(hItem) + _T('\\') + sRet; 
		hItem = m_Tree.GetParentItem(hItem);
	}
	while (hItem);
	return sRet;
}

void CSettingsDialogQueueImages::OnItemexpandingTree(NMHDR* pNMHDR, LRESULT* pResult)
{
	CWaitCursor CursorWaiting;						//Show the wait cursor while expanding
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	ASSERT(pNMTreeView);
	ASSERT(pResult);
	//Only action 2 notifications
	if (pNMTreeView->action == 2)
	{
		//Update location display
		CString sPath = GetItemPath(pNMTreeView->itemNew.hItem);
		//Refresh children
		if (!m_Tree.GetChildItem(pNMTreeView->itemNew.hItem))
		{
			PopulateTree(sPath, pNMTreeView->itemNew.hItem);
			if (m_Tree.GetSelectedItem() != pNMTreeView->itemNew.hItem)
				m_Tree.SelectItem(pNMTreeView->itemNew.hItem);
		}
	}
	*pResult = 0;
}

void CSettingsDialogQueueImages::OnSelchangedTree(NMHDR* pNMHDR, LRESULT* pResult)
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	ASSERT(pNMTreeView);
	ASSERT(pResult);
	//Disable search on Workstation roots
	if (m_Tree.GetItemData(pNMTreeView->itemNew.hItem))
	{
//		GetDlgItem(IDOK)->EnableWindow(false);
	}
	else
	{
//		GetDlgItem(IDOK)->EnableWindow(true);
		cstrBrowsePath = GetItemPath(pNMTreeView->itemNew.hItem);
	}
	*pResult = 0;

	FillList();
	//
	//TODO add code here to process users selection
	//
	//	ASSERT( GetDlgItem( IDC_STATIC_NOTE ) );
	//	SetDlgItemText( IDC_STATIC_NOTE, sPath );
}

void CSettingsDialogQueueImages::FillList()
{
	CFileFind finder;

	// Clear the list box.
	m_lbBrowseList.ResetContent();

	// Build the search string.
	if (cstrBrowsePath.GetAt(cstrBrowsePath.GetLength() - 1) != '\\')
		cstrBrowsePath += "\\";

	CString cstrSearch[6] =
	{
		cstrBrowsePath + _T("*.bmp"), cstrBrowsePath + _T("*.jpg"), cstrBrowsePath + _T("*.jpeg"), cstrBrowsePath + _T("*.gif"), cstrBrowsePath + _T("*.dib"), cstrBrowsePath + _T("*.png")
	};

	// Get all image files into the list.
	for (int i = 0; i < 6; i++)
	{
		BOOL bWorking = finder.FindFile(cstrSearch[i]);
		while (bWorking)
		{
			bWorking = finder.FindNextFile();
			m_lbBrowseList.AddString(finder.GetFileName());
		}

		finder.Close();
	}

	// Select the first image.
	if (m_lbBrowseList.GetCount() > 0)
	{
		m_lbBrowseList.SetSel(0);
		m_lbBrowseList.SetFocus();
		DrawPreview2();
	}
}

void CSettingsDialogQueueImages::ResetMainDialog()
{
	CString cstrAppPath;
	TCHAR strBrowsePath[MAX_PATH + 1];
	// Initialize the strBrowsePath string to zeros.
	ZeroMemory(strBrowsePath, sizeof(char) * (MAX_PATH + 1));

	// Get the application path.
	GetModuleFileName(AfxGetInstanceHandle(), strBrowsePath, MAX_PATH + 1);
	cstrAppPath.Format(_T("%s"), strBrowsePath);
	cstrAppPath = cstrAppPath.Left(cstrAppPath.ReverseFind('\\') + 1);

	// Get the browse path for the top list box.
	cstrBrowsePath = CRegistry::GetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("LastPath"), cstrAppPath);

	RECT rect;

	// Set the scroll size of the browse list box.
	m_lbBrowseList.GetWindowRect(&rect);
	m_lbBrowseList.SetHorizontalExtent(rect.right - rect.left + 100);

	// Fill the top list box with all image files in the specified location.
	m_lbBrowseList.ResetContent();
	FillList();
}

void CSettingsDialogQueueImages::ApplyChanges()
{
	// Set the last browse path in the registry.
	CRegistry::SetRegString(_T("Software\\Desktop Calendar_V1.0\\DesktopCalendar\\Configuration"), _T("LastPath"), cstrBrowsePath);
}

void CSettingsDialogQueueImages::translate()
{
	if(this->m_hWnd != NULL)
	{
		m_Helper.translateWindowText(this, IDD_SETTINGS_QUEUE_IMAGES);
		m_Helper.translateDlgItemText(this, IDC_ADD);
		m_Helper.translateDlgItemText(this, IDC_INFOIMG2);
		m_Helper.translateDlgItemText(this, IDC_WIDTH2);
		m_Helper.translateDlgItemText(this, IDC_HEIGHT2);
		m_Helper.translateDlgItemText(this, IDC_TYPE2);
	}
}

BOOL CSettingsDialogQueueImages::OnSetActive()
{
	CString buf = m_Helper.MyLoadString(IDC_LIMAGES_QUEUED);
	if(SettingsDialog->m_SettingsDialogImagesQueued.m_lbQList.m_hWnd!= NULL)
	{
		CString tmp;
		tmp.Format(_T("%d %s"), SettingsDialog->m_SettingsDialogImagesQueued.m_lbQList.GetCount(), buf);
		GetDlgItem(IDC_IMAGES_QUEUED)->SetWindowText(tmp);
	}
	return TRUE;
}

void CSettingsDialogQueueImages::OnBnClickedAdd()
{
	int iCount;
	CString cstrTemp;

	// Get all selected items from the browse list.
	iCount = m_lbBrowseList.GetSelCount();
	CArray<int, int> aryListBoxSel;

	aryListBoxSel.SetSize(iCount);
	m_lbBrowseList.GetSelItems(iCount, aryListBoxSel.GetData());

	// Loop through each selected item and add it to the Queue list.
	for (int i = 0; i < aryListBoxSel.GetSize(); i++)
	{
		m_lbBrowseList.GetText(aryListBoxSel.GetAt(i), cstrTemp);

		// Select each newly added item.
		if (SettingsDialog->m_SettingsDialogImagesQueued.m_lbQList.FindStringExact(0, cstrBrowsePath + cstrTemp) > -1)
		{
			// If its already in the list, ask if they want to add it again.
			if (AfxMessageBox(m_Helper.MyLoadString(IDS_ALREADYEXIST), MB_YESNO) == IDYES)
			{
				int index = SettingsDialog->m_SettingsDialogImagesQueued.m_lbQList.InsertString(SettingsDialog->m_SettingsDialogImagesQueued.m_lbQList.GetCount() + 1, cstrBrowsePath + cstrTemp);
				SettingsDialog->m_SettingsDialogImagesQueued.m_lbQList.SetItemData(index, GetWPPositionByImageSize(cstrBrowsePath + cstrTemp));
			}
		}
		else
		{
			//int index = m_lbQList.InsertString(m_lbQList.GetCount() + 1, cstrTemp);
			int index = SettingsDialog->m_SettingsDialogImagesQueued.m_lbQList.AddString(cstrBrowsePath + cstrTemp);
			SettingsDialog->m_SettingsDialogImagesQueued.m_lbQList.SetItemData(index, GetWPPositionByImageSize(cstrBrowsePath + cstrTemp));
		}
	}

	CString buf = m_Helper.MyLoadString(IDC_LIMAGES_QUEUED);
	CString tmp;
	tmp.Format(_T("%d %s"), SettingsDialog->m_SettingsDialogImagesQueued.m_lbQList.GetCount(), buf);
	GetDlgItem(IDC_IMAGES_QUEUED)->SetWindowText(tmp);

	SettingsDialog->m_btnApply.EnableWindow();
}

int CSettingsDialogQueueImages::GetWPPositionByImageSize(CString cstrImagePath)
{
	USES_CONVERSION;

	CDC* pDC = NULL;
	UINT uiWidth;
	UINT uiHeight;
	int iVertRes;
	int iHorzRes;
	int iTemp;
	double dRatio;
	double dScreenRatio;
	int iCalcPosition = 2;

	// Initialize the Graphics class in GDI+.
	pDC = GetDC();
	Graphics graphics(pDC->m_hDC);

	// Use the Image class to get dimensions of the image.
	Image image(T2CW((LPCTSTR)cstrImagePath));

	uiWidth = image.GetWidth();
	uiHeight = image.GetHeight();
	dRatio = ((double)uiWidth / (double)uiHeight);
	// round it
	iTemp = (int)((dRatio * 100.0) + 0.51);
	dRatio = ((double)iTemp) / 100.0;

	iHorzRes = pDC->GetDeviceCaps(HORZRES);
	iVertRes = pDC->GetDeviceCaps(VERTRES);
	dScreenRatio = ((double)iHorzRes / (double)iVertRes);
	// round it
	iTemp = (int)((dScreenRatio * 100.0) + 0.51);
	dScreenRatio = ((double)iTemp) / 100.0;

	if (dRatio == dScreenRatio || dRatio > 1.0)
		iCalcPosition = 2; // stretch
	else
		iCalcPosition = 1; // center

	ReleaseDC(pDC);

	return iCalcPosition;
}
