#include "StdAfx.h"
#include "GdiplusWrapper.h"

//#pragma warning(disable:4482) // warning C4482

GdiplusWrapper gdiplusWrapper;

GdiplusWrapper::GdiplusWrapper(void) : m_hGdiplusLib(NULL)
{
	m_GdiPlusStatus = GenericError;
	Initialize();
}

BOOL GdiplusWrapper::Initialize(void)
{
	if (m_hGdiplusLib)
	{
		SetLastError(ERROR_ALREADY_INITIALIZED);
		return FALSE;
	}

	m_hGdiplusLib = LoadLibrary(_T("gdiplus.dll"));
	return IsInitialized();
}

BOOL GdiplusWrapper::IsLoaded(void)
{
	return (NULL != m_hGdiplusLib);
}

BOOL GdiplusWrapper::IsInitialized(void)
{
	return (IsLoaded() && m_GdiPlusStatus == Ok);
}


GdiplusWrapper::~GdiplusWrapper(void)
{
	if (IsLoaded())
	{
		VERIFY(FreeLibrary(m_hGdiplusLib));
		m_hGdiplusLib = NULL;
	}
}

Status WINAPI GdiplusWrapper::GdiplusStartup(OUT ULONG_PTR* token, const GdiplusStartupInput* input, OUT GdiplusStartupOutput* output)
{
	if (!IsLoaded())
	{
		return GenericError;
	}

	GDI_PLUS_STARTUP pfnGdiplusStartup = (GDI_PLUS_STARTUP) GetProcAddress(m_hGdiplusLib, "GdiplusStartup");
	if (!pfnGdiplusStartup)
	{
		return GenericError;
	}

	m_GdiPlusStatus = pfnGdiplusStartup(token, input, output);
	return m_GdiPlusStatus;
}

VOID WINAPI GdiplusWrapper::GdiplusShutdown(ULONG_PTR token)
{
	if (!IsInitialized())
	{
		return;
	}

	GDI_PLUS_SHUTDOWN pfnGdiplusShutdown = (GDI_PLUS_SHUTDOWN) GetProcAddress(m_hGdiplusLib, "GdiplusShutdown");
	if (!pfnGdiplusShutdown)
	{
		return;
	}

	pfnGdiplusShutdown(token);
}

void* WINAPI GdiplusWrapper::GdipAlloc(size_t size)
{
	if (!IsInitialized())
	{
		return NULL;
	}

	GDIP_ALLOC pfnGdipAlloc = (GDIP_ALLOC) GetProcAddress(m_hGdiplusLib, "GdipAlloc");
	if (!pfnGdipAlloc)
	{
		return NULL;
	}

	return pfnGdipAlloc(size);
}

void WINAPI GdiplusWrapper::GdipFree(void* ptr)
{
	if (!IsInitialized())
	{
		return;
	}

	GDIP_FREE pfnGdipFree = (GDIP_FREE) GetProcAddress(m_hGdiplusLib, "GdipFree");
	if (!pfnGdipFree)
	{
		return;
	}

	pfnGdipFree(ptr);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
GpStatus WINAPI GdiplusWrapper::GdipCreatePath(GpFillMode brushMode, GpPath** path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreatePath pfnGdipCreatePath = (pGdipCreatePath) GetProcAddress(m_hGdiplusLib, "GdipCreatePath");
	if(!pfnGdipCreatePath)
	{
		return GenericError;
	}
	return pfnGdipCreatePath(brushMode,path);
}

GpStatus WINAPI GdiplusWrapper::GdipCreatePath2(GDIPCONST GpPointF* points, GDIPCONST BYTE* types, INT count, GpFillMode fillMode, GpPath** path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreatePath2 pfnGdipCreatePath2 = (pGdipCreatePath2) GetProcAddress(m_hGdiplusLib, "GdipCreatePath2");
	if(!pfnGdipCreatePath2)
	{
		return GenericError;
	}
	return pfnGdipCreatePath2(points,types,count,fillMode,path);
}

GpStatus WINAPI GdiplusWrapper::GdipCreatePath2I(GDIPCONST GpPoint* points, GDIPCONST BYTE* types, INT count, GpFillMode fillMode, GpPath** path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreatePath2I pfnGdipCreatePath2I = (pGdipCreatePath2I) GetProcAddress(m_hGdiplusLib, "GdipCreatePath2I");
	if(!pfnGdipCreatePath2I)
	{
		return GenericError;
	}
	return pfnGdipCreatePath2I(points,types,count,fillMode,path);
}

GpStatus WINAPI GdiplusWrapper::GdipClonePath(GpPath* path, GpPath** clonePath)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipClonePath pfnGdipClonePath = (pGdipClonePath) GetProcAddress(m_hGdiplusLib, "GdipClonePath");
	if(!pfnGdipClonePath)
	{
		return GenericError;
	}
	return pfnGdipClonePath(path,clonePath);
}

GpStatus WINAPI GdiplusWrapper::GdipDeletePath(GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeletePath pfnGdipDeletePath = (pGdipDeletePath) GetProcAddress(m_hGdiplusLib, "GdipDeletePath");
	if(!pfnGdipDeletePath)
	{
		return GenericError;
	}
	return pfnGdipDeletePath(path);
}

GpStatus WINAPI GdiplusWrapper::GdipResetPath(GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipResetPath pfnGdipResetPath = (pGdipResetPath) GetProcAddress(m_hGdiplusLib, "GdipResetPath");
	if(!pfnGdipResetPath)
	{
		return GenericError;
	}
	return pfnGdipResetPath(path);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPointCount(GpPath* path, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPointCount pfnGdipGetPointCount = (pGdipGetPointCount) GetProcAddress(m_hGdiplusLib, "GdipGetPointCount");
	if(!pfnGdipGetPointCount)
	{
		return GenericError;
	}
	return pfnGdipGetPointCount(path,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathTypes(GpPath* path, BYTE* types, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathTypes pfnGdipGetPathTypes = (pGdipGetPathTypes) GetProcAddress(m_hGdiplusLib, "GdipGetPathTypes");
	if(!pfnGdipGetPathTypes)
	{
		return GenericError;
	}
	return pfnGdipGetPathTypes(path,types,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathPoints(GpPath* path, GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathPoints pfnGdipGetPathPoints = (pGdipGetPathPoints) GetProcAddress(m_hGdiplusLib, "GdipGetPathPoints");
	if(!pfnGdipGetPathPoints)
	{
		return GenericError;
	}
	return pfnGdipGetPathPoints(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathPointsI(GpPath* path, GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathPointsI pfnGdipGetPathPointsI = (pGdipGetPathPointsI) GetProcAddress(m_hGdiplusLib, "GdipGetPathPointsI");
	if(!pfnGdipGetPathPointsI)
	{
		return GenericError;
	}
	return pfnGdipGetPathPointsI(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathFillMode(GpPath* path, GpFillMode* fillmode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathFillMode pfnGdipGetPathFillMode = (pGdipGetPathFillMode) GetProcAddress(m_hGdiplusLib, "GdipGetPathFillMode");
	if(!pfnGdipGetPathFillMode)
	{
		return GenericError;
	}
	return pfnGdipGetPathFillMode(path,fillmode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathFillMode(GpPath* path, GpFillMode fillmode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathFillMode pfnGdipSetPathFillMode = (pGdipSetPathFillMode) GetProcAddress(m_hGdiplusLib, "GdipSetPathFillMode");
	if(!pfnGdipSetPathFillMode)
	{
		return GenericError;
	}
	return pfnGdipSetPathFillMode(path,fillmode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathData(GpPath* path, GpPathData* pathData)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathData pfnGdipGetPathData = (pGdipGetPathData) GetProcAddress(m_hGdiplusLib, "GdipGetPathData");
	if(!pfnGdipGetPathData)
	{
		return GenericError;
	}
	return pfnGdipGetPathData(path,pathData);
}

GpStatus WINAPI GdiplusWrapper::GdipStartPathFigure(GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipStartPathFigure pfnGdipStartPathFigure = (pGdipStartPathFigure) GetProcAddress(m_hGdiplusLib, "GdipStartPathFigure");
	if(!pfnGdipStartPathFigure)
	{
		return GenericError;
	}
	return pfnGdipStartPathFigure(path);
}

GpStatus WINAPI GdiplusWrapper::GdipClosePathFigure(GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipClosePathFigure pfnGdipClosePathFigure = (pGdipClosePathFigure) GetProcAddress(m_hGdiplusLib, "GdipClosePathFigure");
	if(!pfnGdipClosePathFigure)
	{
		return GenericError;
	}
	return pfnGdipClosePathFigure(path);
}

GpStatus WINAPI GdiplusWrapper::GdipClosePathFigures(GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipClosePathFigures pfnGdipClosePathFigures = (pGdipClosePathFigures) GetProcAddress(m_hGdiplusLib, "GdipClosePathFigures");
	if(!pfnGdipClosePathFigures)
	{
		return GenericError;
	}
	return pfnGdipClosePathFigures(path);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathMarker(GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathMarker pfnGdipSetPathMarker = (pGdipSetPathMarker) GetProcAddress(m_hGdiplusLib, "GdipSetPathMarker");
	if(!pfnGdipSetPathMarker)
	{
		return GenericError;
	}
	return pfnGdipSetPathMarker(path);
}

GpStatus WINAPI GdiplusWrapper::GdipClearPathMarkers(GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipClearPathMarkers pfnGdipClearPathMarkers = (pGdipClearPathMarkers) GetProcAddress(m_hGdiplusLib, "GdipClearPathMarkers");
	if(!pfnGdipClearPathMarkers)
	{
		return GenericError;
	}
	return pfnGdipClearPathMarkers(path);
}

GpStatus WINAPI GdiplusWrapper::GdipReversePath(GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipReversePath pfnGdipReversePath = (pGdipReversePath) GetProcAddress(m_hGdiplusLib, "GdipReversePath");
	if(!pfnGdipReversePath)
	{
		return GenericError;
	}
	return pfnGdipReversePath(path);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathLastPoint(GpPath* path, GpPointF* lastPoint)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathLastPoint pfnGdipGetPathLastPoint = (pGdipGetPathLastPoint) GetProcAddress(m_hGdiplusLib, "GdipGetPathLastPoint");
	if(!pfnGdipGetPathLastPoint)
	{
		return GenericError;
	}
	return pfnGdipGetPathLastPoint(path,lastPoint);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathLine(GpPath* path, REAL x1, REAL y1, REAL x2, REAL y2)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathLine pfnGdipAddPathLine = (pGdipAddPathLine) GetProcAddress(m_hGdiplusLib, "GdipAddPathLine");
	if(!pfnGdipAddPathLine)
	{
		return GenericError;
	}
	return pfnGdipAddPathLine(path,x1,y1,x2,y2);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathLine2(GpPath* path, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathLine2 pfnGdipAddPathLine2 = (pGdipAddPathLine2) GetProcAddress(m_hGdiplusLib, "GdipAddPathLine2");
	if(!pfnGdipAddPathLine2)
	{
		return GenericError;
	}
	return pfnGdipAddPathLine2(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathArc(GpPath* path, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathArc pfnGdipAddPathArc = (pGdipAddPathArc) GetProcAddress(m_hGdiplusLib, "GdipAddPathArc");
	if(!pfnGdipAddPathArc)
	{
		return GenericError;
	}
	return pfnGdipAddPathArc(path,x,y,width,height,startAngle,sweepAngle);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathBezier(GpPath* path, REAL x1, REAL y1, REAL x2, REAL y2, REAL x3, REAL y3, REAL x4, REAL y4)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathBezier pfnGdipAddPathBezier = (pGdipAddPathBezier) GetProcAddress(m_hGdiplusLib, "GdipAddPathBezier");
	if(!pfnGdipAddPathBezier)
	{
		return GenericError;
	}
	return pfnGdipAddPathBezier(path,x1,y1,x2,y2,x3,y3,x4,y4);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathBeziers(GpPath* path, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathBeziers pfnGdipAddPathBeziers = (pGdipAddPathBeziers) GetProcAddress(m_hGdiplusLib, "GdipAddPathBeziers");
	if(!pfnGdipAddPathBeziers)
	{
		return GenericError;
	}
	return pfnGdipAddPathBeziers(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathCurve(GpPath* path, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathCurve pfnGdipAddPathCurve = (pGdipAddPathCurve) GetProcAddress(m_hGdiplusLib, "GdipAddPathCurve");
	if(!pfnGdipAddPathCurve)
	{
		return GenericError;
	}
	return pfnGdipAddPathCurve(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathCurve2(GpPath* path, GDIPCONST GpPointF* points, INT count, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathCurve2 pfnGdipAddPathCurve2 = (pGdipAddPathCurve2) GetProcAddress(m_hGdiplusLib, "GdipAddPathCurve2");
	if(!pfnGdipAddPathCurve2)
	{
		return GenericError;
	}
	return pfnGdipAddPathCurve2(path,points,count,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathCurve3(GpPath* path, GDIPCONST GpPointF* points, INT count, INT offset, INT numberOfSegments, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathCurve3 pfnGdipAddPathCurve3 = (pGdipAddPathCurve3) GetProcAddress(m_hGdiplusLib, "GdipAddPathCurve3");
	if(!pfnGdipAddPathCurve3)
	{
		return GenericError;
	}
	return pfnGdipAddPathCurve3(path,points,count,offset,numberOfSegments,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathClosedCurve(GpPath* path, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathClosedCurve pfnGdipAddPathClosedCurve = (pGdipAddPathClosedCurve) GetProcAddress(m_hGdiplusLib, "GdipAddPathClosedCurve");
	if(!pfnGdipAddPathClosedCurve)
	{
		return GenericError;
	}
	return pfnGdipAddPathClosedCurve(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathClosedCurve2(GpPath* path, GDIPCONST GpPointF* points, INT count, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathClosedCurve2 pfnGdipAddPathClosedCurve2 = (pGdipAddPathClosedCurve2) GetProcAddress(m_hGdiplusLib, "GdipAddPathClosedCurve2");
	if(!pfnGdipAddPathClosedCurve2)
	{
		return GenericError;
	}
	return pfnGdipAddPathClosedCurve2(path,points,count,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathRectangle(GpPath* path, REAL x, REAL y, REAL width, REAL height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathRectangle pfnGdipAddPathRectangle = (pGdipAddPathRectangle) GetProcAddress(m_hGdiplusLib, "GdipAddPathRectangle");
	if(!pfnGdipAddPathRectangle)
	{
		return GenericError;
	}
	return pfnGdipAddPathRectangle(path,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathRectangles(GpPath* path, GDIPCONST GpRectF* rects, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathRectangles pfnGdipAddPathRectangles = (pGdipAddPathRectangles) GetProcAddress(m_hGdiplusLib, "GdipAddPathRectangles");
	if(!pfnGdipAddPathRectangles)
	{
		return GenericError;
	}
	return pfnGdipAddPathRectangles(path,rects,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathEllipse(GpPath* path, REAL x, REAL y, REAL width, REAL height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathEllipse pfnGdipAddPathEllipse = (pGdipAddPathEllipse) GetProcAddress(m_hGdiplusLib, "GdipAddPathEllipse");
	if(!pfnGdipAddPathEllipse)
	{
		return GenericError;
	}
	return pfnGdipAddPathEllipse(path,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathPie(GpPath* path, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathPie pfnGdipAddPathPie = (pGdipAddPathPie) GetProcAddress(m_hGdiplusLib, "GdipAddPathPie");
	if(!pfnGdipAddPathPie)
	{
		return GenericError;
	}
	return pfnGdipAddPathPie(path,x,y,width,height,startAngle,sweepAngle);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathPolygon(GpPath* path, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathPolygon pfnGdipAddPathPolygon = (pGdipAddPathPolygon) GetProcAddress(m_hGdiplusLib, "GdipAddPathPolygon");
	if(!pfnGdipAddPathPolygon)
	{
		return GenericError;
	}
	return pfnGdipAddPathPolygon(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathPath(GpPath* path, GDIPCONST GpPath* addingPath, BOOL connect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathPath pfnGdipAddPathPath = (pGdipAddPathPath) GetProcAddress(m_hGdiplusLib, "GdipAddPathPath");
	if(!pfnGdipAddPathPath)
	{
		return GenericError;
	}
	return pfnGdipAddPathPath(path,addingPath,connect);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathString(GpPath* path, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFontFamily* family, INT style, REAL emSize, GDIPCONST RectF* layoutRect, GDIPCONST GpStringFormat* format)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathString pfnGdipAddPathString = (pGdipAddPathString) GetProcAddress(m_hGdiplusLib, "GdipAddPathString");
	if(!pfnGdipAddPathString)
	{
		return GenericError;
	}
	return pfnGdipAddPathString(path,string,length,family,style,emSize,layoutRect,format);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathStringI(GpPath* path, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFontFamily* family, INT style, REAL emSize, GDIPCONST Rect* layoutRect, GDIPCONST GpStringFormat* format)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathStringI pfnGdipAddPathStringI = (pGdipAddPathStringI) GetProcAddress(m_hGdiplusLib, "GdipAddPathStringI");
	if(!pfnGdipAddPathStringI)
	{
		return GenericError;
	}
	return pfnGdipAddPathStringI(path,string,length,family,style,emSize,layoutRect,format);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathLineI(GpPath* path, INT x1, INT y1, INT x2, INT y2)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathLineI pfnGdipAddPathLineI = (pGdipAddPathLineI) GetProcAddress(m_hGdiplusLib, "GdipAddPathLineI");
	if(!pfnGdipAddPathLineI)
	{
		return GenericError;
	}
	return pfnGdipAddPathLineI(path,x1,y1,x2,y2);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathLine2I(GpPath* path, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathLine2I pfnGdipAddPathLine2I = (pGdipAddPathLine2I) GetProcAddress(m_hGdiplusLib, "GdipAddPathLine2I");
	if(!pfnGdipAddPathLine2I)
	{
		return GenericError;
	}
	return pfnGdipAddPathLine2I(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathArcI(GpPath* path, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathArcI pfnGdipAddPathArcI = (pGdipAddPathArcI) GetProcAddress(m_hGdiplusLib, "GdipAddPathArcI");
	if(!pfnGdipAddPathArcI)
	{
		return GenericError;
	}
	return pfnGdipAddPathArcI(path,x,y,width,height,startAngle,sweepAngle);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathBezierI(GpPath* path, INT x1, INT y1, INT x2, INT y2, INT x3, INT y3, INT x4, INT y4)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathBezierI pfnGdipAddPathBezierI = (pGdipAddPathBezierI) GetProcAddress(m_hGdiplusLib, "GdipAddPathBezierI");
	if(!pfnGdipAddPathBezierI)
	{
		return GenericError;
	}
	return pfnGdipAddPathBezierI(path,x1,y1,x2,y2,x3,y3,x4,y4);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathBeziersI(GpPath* path, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathBeziersI pfnGdipAddPathBeziersI = (pGdipAddPathBeziersI) GetProcAddress(m_hGdiplusLib, "GdipAddPathBeziersI");
	if(!pfnGdipAddPathBeziersI)
	{
		return GenericError;
	}
	return pfnGdipAddPathBeziersI(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathCurveI(GpPath* path, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathCurveI pfnGdipAddPathCurveI = (pGdipAddPathCurveI) GetProcAddress(m_hGdiplusLib, "GdipAddPathCurveI");
	if(!pfnGdipAddPathCurveI)
	{
		return GenericError;
	}
	return pfnGdipAddPathCurveI(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathCurve2I(GpPath* path, GDIPCONST GpPoint* points, INT count, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathCurve2I pfnGdipAddPathCurve2I = (pGdipAddPathCurve2I) GetProcAddress(m_hGdiplusLib, "GdipAddPathCurve2I");
	if(!pfnGdipAddPathCurve2I)
	{
		return GenericError;
	}
	return pfnGdipAddPathCurve2I(path,points,count,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathCurve3I(GpPath* path, GDIPCONST GpPoint* points, INT count, INT offset, INT numberOfSegments, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathCurve3I pfnGdipAddPathCurve3I = (pGdipAddPathCurve3I) GetProcAddress(m_hGdiplusLib, "GdipAddPathCurve3I");
	if(!pfnGdipAddPathCurve3I)
	{
		return GenericError;
	}
	return pfnGdipAddPathCurve3I(path,points,count,offset,numberOfSegments,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathClosedCurveI(GpPath* path, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathClosedCurveI pfnGdipAddPathClosedCurveI = (pGdipAddPathClosedCurveI) GetProcAddress(m_hGdiplusLib, "GdipAddPathClosedCurveI");
	if(!pfnGdipAddPathClosedCurveI)
	{
		return GenericError;
	}
	return pfnGdipAddPathClosedCurveI(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathClosedCurve2I(GpPath* path, GDIPCONST GpPoint* points, INT count, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathClosedCurve2I pfnGdipAddPathClosedCurve2I = (pGdipAddPathClosedCurve2I) GetProcAddress(m_hGdiplusLib, "GdipAddPathClosedCurve2I");
	if(!pfnGdipAddPathClosedCurve2I)
	{
		return GenericError;
	}
	return pfnGdipAddPathClosedCurve2I(path,points,count,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathRectangleI(GpPath* path, INT x, INT y, INT width, INT height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathRectangleI pfnGdipAddPathRectangleI = (pGdipAddPathRectangleI) GetProcAddress(m_hGdiplusLib, "GdipAddPathRectangleI");
	if(!pfnGdipAddPathRectangleI)
	{
		return GenericError;
	}
	return pfnGdipAddPathRectangleI(path,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathRectanglesI(GpPath* path, GDIPCONST GpRect* rects, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathRectanglesI pfnGdipAddPathRectanglesI = (pGdipAddPathRectanglesI) GetProcAddress(m_hGdiplusLib, "GdipAddPathRectanglesI");
	if(!pfnGdipAddPathRectanglesI)
	{
		return GenericError;
	}
	return pfnGdipAddPathRectanglesI(path,rects,count);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathEllipseI(GpPath* path, INT x, INT y, INT width, INT height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathEllipseI pfnGdipAddPathEllipseI = (pGdipAddPathEllipseI) GetProcAddress(m_hGdiplusLib, "GdipAddPathEllipseI");
	if(!pfnGdipAddPathEllipseI)
	{
		return GenericError;
	}
	return pfnGdipAddPathEllipseI(path,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathPieI(GpPath* path, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathPieI pfnGdipAddPathPieI = (pGdipAddPathPieI) GetProcAddress(m_hGdiplusLib, "GdipAddPathPieI");
	if(!pfnGdipAddPathPieI)
	{
		return GenericError;
	}
	return pfnGdipAddPathPieI(path,x,y,width,height,startAngle,sweepAngle);
}

GpStatus WINAPI GdiplusWrapper::GdipAddPathPolygonI(GpPath* path, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipAddPathPolygonI pfnGdipAddPathPolygonI = (pGdipAddPathPolygonI) GetProcAddress(m_hGdiplusLib, "GdipAddPathPolygonI");
	if(!pfnGdipAddPathPolygonI)
	{
		return GenericError;
	}
	return pfnGdipAddPathPolygonI(path,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipFlattenPath(GpPath* path, GpMatrix* matrix, REAL flatness)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFlattenPath pfnGdipFlattenPath = (pGdipFlattenPath) GetProcAddress(m_hGdiplusLib, "GdipFlattenPath");
	if(!pfnGdipFlattenPath)
	{
		return GenericError;
	}
	return pfnGdipFlattenPath(path,matrix,flatness);
}

GpStatus WINAPI GdiplusWrapper::GdipWindingModeOutline(GpPath* path, GpMatrix* matrix, REAL flatness)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipWindingModeOutline pfnGdipWindingModeOutline = (pGdipWindingModeOutline) GetProcAddress(m_hGdiplusLib, "GdipWindingModeOutline");
	if(!pfnGdipWindingModeOutline)
	{
		return GenericError;
	}
	return pfnGdipWindingModeOutline(path,matrix,flatness);
}

GpStatus WINAPI GdiplusWrapper::GdipWidenPath(GpPath* nativePath, GpPen* pen, GpMatrix* matrix, REAL flatness)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipWidenPath pfnGdipWidenPath = (pGdipWidenPath) GetProcAddress(m_hGdiplusLib, "GdipWidenPath");
	if(!pfnGdipWidenPath)
	{
		return GenericError;
	}
	return pfnGdipWidenPath(nativePath,pen,matrix,flatness);
}

GpStatus WINAPI GdiplusWrapper::GdipWarpPath(GpPath* path, GpMatrix* matrix, GDIPCONST GpPointF* points, INT count, REAL srcx, REAL srcy, REAL srcwidth, REAL srcheight, WarpMode warpMode, REAL flatness)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipWarpPath pfnGdipWarpPath = (pGdipWarpPath) GetProcAddress(m_hGdiplusLib, "GdipWarpPath");
	if(!pfnGdipWarpPath)
	{
		return GenericError;
	}
	return pfnGdipWarpPath(path,matrix,points,count,srcx,srcy,srcwidth,srcheight,warpMode,flatness);
}

GpStatus WINAPI GdiplusWrapper::GdipTransformPath(GpPath* path, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTransformPath pfnGdipTransformPath = (pGdipTransformPath) GetProcAddress(m_hGdiplusLib, "GdipTransformPath");
	if(!pfnGdipTransformPath)
	{
		return GenericError;
	}
	return pfnGdipTransformPath(path,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathWorldBounds(GpPath* path, GpRectF* bounds, GDIPCONST GpMatrix* matrix, GDIPCONST GpPen* pen)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathWorldBounds pfnGdipGetPathWorldBounds = (pGdipGetPathWorldBounds) GetProcAddress(m_hGdiplusLib, "GdipGetPathWorldBounds");
	if(!pfnGdipGetPathWorldBounds)
	{
		return GenericError;
	}
	return pfnGdipGetPathWorldBounds(path,bounds,matrix,pen);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathWorldBoundsI(GpPath* path, GpRect* bounds, GDIPCONST GpMatrix* matrix, GDIPCONST GpPen* pen)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathWorldBoundsI pfnGdipGetPathWorldBoundsI = (pGdipGetPathWorldBoundsI) GetProcAddress(m_hGdiplusLib, "GdipGetPathWorldBoundsI");
	if(!pfnGdipGetPathWorldBoundsI)
	{
		return GenericError;
	}
	return pfnGdipGetPathWorldBoundsI(path,bounds,matrix,pen);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisiblePathPoint(GpPath* path, REAL x, REAL y, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisiblePathPoint pfnGdipIsVisiblePathPoint = (pGdipIsVisiblePathPoint) GetProcAddress(m_hGdiplusLib, "GdipIsVisiblePathPoint");
	if(!pfnGdipIsVisiblePathPoint)
	{
		return GenericError;
	}
	return pfnGdipIsVisiblePathPoint(path,x,y,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisiblePathPointI(GpPath* path, INT x, INT y, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisiblePathPointI pfnGdipIsVisiblePathPointI = (pGdipIsVisiblePathPointI) GetProcAddress(m_hGdiplusLib, "GdipIsVisiblePathPointI");
	if(!pfnGdipIsVisiblePathPointI)
	{
		return GenericError;
	}
	return pfnGdipIsVisiblePathPointI(path,x,y,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsOutlineVisiblePathPoint(GpPath* path, REAL x, REAL y, GpPen* pen, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsOutlineVisiblePathPoint pfnGdipIsOutlineVisiblePathPoint = (pGdipIsOutlineVisiblePathPoint) GetProcAddress(m_hGdiplusLib, "GdipIsOutlineVisiblePathPoint");
	if(!pfnGdipIsOutlineVisiblePathPoint)
	{
		return GenericError;
	}
	return pfnGdipIsOutlineVisiblePathPoint(path,x,y,pen,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsOutlineVisiblePathPointI(GpPath* path, INT x, INT y, GpPen* pen, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsOutlineVisiblePathPointI pfnGdipIsOutlineVisiblePathPointI = (pGdipIsOutlineVisiblePathPointI) GetProcAddress(m_hGdiplusLib, "GdipIsOutlineVisiblePathPointI");
	if(!pfnGdipIsOutlineVisiblePathPointI)
	{
		return GenericError;
	}
	return pfnGdipIsOutlineVisiblePathPointI(path,x,y,pen,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipCreatePathIter(GpPathIterator** iterator, GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreatePathIter pfnGdipCreatePathIter = (pGdipCreatePathIter) GetProcAddress(m_hGdiplusLib, "GdipCreatePathIter");
	if(!pfnGdipCreatePathIter)
	{
		return GenericError;
	}
	return pfnGdipCreatePathIter(iterator,path);
}

GpStatus WINAPI GdiplusWrapper::GdipDeletePathIter(GpPathIterator* iterator)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeletePathIter pfnGdipDeletePathIter = (pGdipDeletePathIter) GetProcAddress(m_hGdiplusLib, "GdipDeletePathIter");
	if(!pfnGdipDeletePathIter)
	{
		return GenericError;
	}
	return pfnGdipDeletePathIter(iterator);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterNextSubpath(GpPathIterator* iterator, INT* resultCount, INT* startIndex, INT* endIndex, BOOL* isClosed)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterNextSubpath pfnGdipPathIterNextSubpath = (pGdipPathIterNextSubpath) GetProcAddress(m_hGdiplusLib, "GdipPathIterNextSubpath");
	if(!pfnGdipPathIterNextSubpath)
	{
		return GenericError;
	}
	return pfnGdipPathIterNextSubpath(iterator,resultCount,startIndex,endIndex,isClosed);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterNextSubpathPath(GpPathIterator* iterator, INT* resultCount, GpPath* path, BOOL* isClosed)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterNextSubpathPath pfnGdipPathIterNextSubpathPath = (pGdipPathIterNextSubpathPath) GetProcAddress(m_hGdiplusLib, "GdipPathIterNextSubpathPath");
	if(!pfnGdipPathIterNextSubpathPath)
	{
		return GenericError;
	}
	return pfnGdipPathIterNextSubpathPath(iterator,resultCount,path,isClosed);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterNextPathType(GpPathIterator* iterator, INT* resultCount, BYTE* pathType, INT* startIndex, INT* endIndex)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterNextPathType pfnGdipPathIterNextPathType = (pGdipPathIterNextPathType) GetProcAddress(m_hGdiplusLib, "GdipPathIterNextPathType");
	if(!pfnGdipPathIterNextPathType)
	{
		return GenericError;
	}
	return pfnGdipPathIterNextPathType(iterator,resultCount,pathType,startIndex,endIndex);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterNextMarker(GpPathIterator* iterator, INT* resultCount, INT* startIndex, INT* endIndex)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterNextMarker pfnGdipPathIterNextMarker = (pGdipPathIterNextMarker) GetProcAddress(m_hGdiplusLib, "GdipPathIterNextMarker");
	if(!pfnGdipPathIterNextMarker)
	{
		return GenericError;
	}
	return pfnGdipPathIterNextMarker(iterator,resultCount,startIndex,endIndex);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterNextMarkerPath(GpPathIterator* iterator, INT* resultCount, GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterNextMarkerPath pfnGdipPathIterNextMarkerPath = (pGdipPathIterNextMarkerPath) GetProcAddress(m_hGdiplusLib, "GdipPathIterNextMarkerPath");
	if(!pfnGdipPathIterNextMarkerPath)
	{
		return GenericError;
	}
	return pfnGdipPathIterNextMarkerPath(iterator,resultCount,path);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterGetCount(GpPathIterator* iterator, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterGetCount pfnGdipPathIterGetCount = (pGdipPathIterGetCount) GetProcAddress(m_hGdiplusLib, "GdipPathIterGetCount");
	if(!pfnGdipPathIterGetCount)
	{
		return GenericError;
	}
	return pfnGdipPathIterGetCount(iterator,count);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterGetSubpathCount(GpPathIterator* iterator, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterGetSubpathCount pfnGdipPathIterGetSubpathCount = (pGdipPathIterGetSubpathCount) GetProcAddress(m_hGdiplusLib, "GdipPathIterGetSubpathCount");
	if(!pfnGdipPathIterGetSubpathCount)
	{
		return GenericError;
	}
	return pfnGdipPathIterGetSubpathCount(iterator,count);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterIsValid(GpPathIterator* iterator, BOOL* valid)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterIsValid pfnGdipPathIterIsValid = (pGdipPathIterIsValid) GetProcAddress(m_hGdiplusLib, "GdipPathIterIsValid");
	if(!pfnGdipPathIterIsValid)
	{
		return GenericError;
	}
	return pfnGdipPathIterIsValid(iterator,valid);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterHasCurve(GpPathIterator* iterator, BOOL* hasCurve)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterHasCurve pfnGdipPathIterHasCurve = (pGdipPathIterHasCurve) GetProcAddress(m_hGdiplusLib, "GdipPathIterHasCurve");
	if(!pfnGdipPathIterHasCurve)
	{
		return GenericError;
	}
	return pfnGdipPathIterHasCurve(iterator,hasCurve);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterRewind(GpPathIterator* iterator)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterRewind pfnGdipPathIterRewind = (pGdipPathIterRewind) GetProcAddress(m_hGdiplusLib, "GdipPathIterRewind");
	if(!pfnGdipPathIterRewind)
	{
		return GenericError;
	}
	return pfnGdipPathIterRewind(iterator);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterEnumerate(GpPathIterator* iterator, INT* resultCount, GpPointF* points, BYTE* types, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterEnumerate pfnGdipPathIterEnumerate = (pGdipPathIterEnumerate) GetProcAddress(m_hGdiplusLib, "GdipPathIterEnumerate");
	if(!pfnGdipPathIterEnumerate)
	{
		return GenericError;
	}
	return pfnGdipPathIterEnumerate(iterator,resultCount,points,types,count);
}

GpStatus WINAPI GdiplusWrapper::GdipPathIterCopyData(GpPathIterator* iterator, INT* resultCount, GpPointF* points, BYTE* types, INT startIndex, INT endIndex)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPathIterCopyData pfnGdipPathIterCopyData = (pGdipPathIterCopyData) GetProcAddress(m_hGdiplusLib, "GdipPathIterCopyData");
	if(!pfnGdipPathIterCopyData)
	{
		return GenericError;
	}
	return pfnGdipPathIterCopyData(iterator,resultCount,points,types,startIndex,endIndex);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateMatrix(GpMatrix** matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateMatrix pfnGdipCreateMatrix = (pGdipCreateMatrix) GetProcAddress(m_hGdiplusLib, "GdipCreateMatrix");
	if(!pfnGdipCreateMatrix)
	{
		return GenericError;
	}
	return pfnGdipCreateMatrix(matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateMatrix2(REAL m11, REAL m12, REAL m21, REAL m22, REAL dx, REAL dy, GpMatrix** matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateMatrix2 pfnGdipCreateMatrix2 = (pGdipCreateMatrix2) GetProcAddress(m_hGdiplusLib, "GdipCreateMatrix2");
	if(!pfnGdipCreateMatrix2)
	{
		return GenericError;
	}
	return pfnGdipCreateMatrix2(m11,m12,m21,m22,dx,dy,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateMatrix3(GDIPCONST GpRectF* rect, GDIPCONST GpPointF* dstplg, GpMatrix** matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateMatrix3 pfnGdipCreateMatrix3 = (pGdipCreateMatrix3) GetProcAddress(m_hGdiplusLib, "GdipCreateMatrix3");
	if(!pfnGdipCreateMatrix3)
	{
		return GenericError;
	}
	return pfnGdipCreateMatrix3(rect,dstplg,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateMatrix3I(GDIPCONST GpRect* rect, GDIPCONST GpPoint* dstplg, GpMatrix** matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateMatrix3I pfnGdipCreateMatrix3I = (pGdipCreateMatrix3I) GetProcAddress(m_hGdiplusLib, "GdipCreateMatrix3I");
	if(!pfnGdipCreateMatrix3I)
	{
		return GenericError;
	}
	return pfnGdipCreateMatrix3I(rect,dstplg,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneMatrix(GpMatrix* matrix, GpMatrix** cloneMatrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneMatrix pfnGdipCloneMatrix = (pGdipCloneMatrix) GetProcAddress(m_hGdiplusLib, "GdipCloneMatrix");
	if(!pfnGdipCloneMatrix)
	{
		return GenericError;
	}
	return pfnGdipCloneMatrix(matrix,cloneMatrix);
}

GpStatus WINAPI GdiplusWrapper::GdipDeleteMatrix(GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeleteMatrix pfnGdipDeleteMatrix = (pGdipDeleteMatrix) GetProcAddress(m_hGdiplusLib, "GdipDeleteMatrix");
	if(!pfnGdipDeleteMatrix)
	{
		return GenericError;
	}
	return pfnGdipDeleteMatrix(matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipSetMatrixElements(GpMatrix* matrix, REAL m11, REAL m12, REAL m21, REAL m22, REAL dx, REAL dy)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetMatrixElements pfnGdipSetMatrixElements = (pGdipSetMatrixElements) GetProcAddress(m_hGdiplusLib, "GdipSetMatrixElements");
	if(!pfnGdipSetMatrixElements)
	{
		return GenericError;
	}
	return pfnGdipSetMatrixElements(matrix,m11,m12,m21,m22,dx,dy);
}

GpStatus WINAPI GdiplusWrapper::GdipMultiplyMatrix(GpMatrix* matrix, GpMatrix* matrix2, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipMultiplyMatrix pfnGdipMultiplyMatrix = (pGdipMultiplyMatrix) GetProcAddress(m_hGdiplusLib, "GdipMultiplyMatrix");
	if(!pfnGdipMultiplyMatrix)
	{
		return GenericError;
	}
	return pfnGdipMultiplyMatrix(matrix,matrix2,order);
}

GpStatus WINAPI GdiplusWrapper::GdipTranslateMatrix(GpMatrix* matrix, REAL offsetX, REAL offsetY, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTranslateMatrix pfnGdipTranslateMatrix = (pGdipTranslateMatrix) GetProcAddress(m_hGdiplusLib, "GdipTranslateMatrix");
	if(!pfnGdipTranslateMatrix)
	{
		return GenericError;
	}
	return pfnGdipTranslateMatrix(matrix,offsetX,offsetY,order);
}

GpStatus WINAPI GdiplusWrapper::GdipScaleMatrix(GpMatrix* matrix, REAL scaleX, REAL scaleY, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipScaleMatrix pfnGdipScaleMatrix = (pGdipScaleMatrix) GetProcAddress(m_hGdiplusLib, "GdipScaleMatrix");
	if(!pfnGdipScaleMatrix)
	{
		return GenericError;
	}
	return pfnGdipScaleMatrix(matrix,scaleX,scaleY,order);
}

GpStatus WINAPI GdiplusWrapper::GdipRotateMatrix(GpMatrix* matrix, REAL angle, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRotateMatrix pfnGdipRotateMatrix = (pGdipRotateMatrix) GetProcAddress(m_hGdiplusLib, "GdipRotateMatrix");
	if(!pfnGdipRotateMatrix)
	{
		return GenericError;
	}
	return pfnGdipRotateMatrix(matrix,angle,order);
}

GpStatus WINAPI GdiplusWrapper::GdipShearMatrix(GpMatrix* matrix, REAL shearX, REAL shearY, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipShearMatrix pfnGdipShearMatrix = (pGdipShearMatrix) GetProcAddress(m_hGdiplusLib, "GdipShearMatrix");
	if(!pfnGdipShearMatrix)
	{
		return GenericError;
	}
	return pfnGdipShearMatrix(matrix,shearX,shearY,order);
}

GpStatus WINAPI GdiplusWrapper::GdipInvertMatrix(GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipInvertMatrix pfnGdipInvertMatrix = (pGdipInvertMatrix) GetProcAddress(m_hGdiplusLib, "GdipInvertMatrix");
	if(!pfnGdipInvertMatrix)
	{
		return GenericError;
	}
	return pfnGdipInvertMatrix(matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipTransformMatrixPoints(GpMatrix* matrix, GpPointF* pts, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTransformMatrixPoints pfnGdipTransformMatrixPoints = (pGdipTransformMatrixPoints) GetProcAddress(m_hGdiplusLib, "GdipTransformMatrixPoints");
	if(!pfnGdipTransformMatrixPoints)
	{
		return GenericError;
	}
	return pfnGdipTransformMatrixPoints(matrix,pts,count);
}

GpStatus WINAPI GdiplusWrapper::GdipTransformMatrixPointsI(GpMatrix* matrix, GpPoint* pts, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTransformMatrixPointsI pfnGdipTransformMatrixPointsI = (pGdipTransformMatrixPointsI) GetProcAddress(m_hGdiplusLib, "GdipTransformMatrixPointsI");
	if(!pfnGdipTransformMatrixPointsI)
	{
		return GenericError;
	}
	return pfnGdipTransformMatrixPointsI(matrix,pts,count);
}

GpStatus WINAPI GdiplusWrapper::GdipVectorTransformMatrixPoints(GpMatrix* matrix, GpPointF* pts, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipVectorTransformMatrixPoints pfnGdipVectorTransformMatrixPoints = (pGdipVectorTransformMatrixPoints) GetProcAddress(m_hGdiplusLib, "GdipVectorTransformMatrixPoints");
	if(!pfnGdipVectorTransformMatrixPoints)
	{
		return GenericError;
	}
	return pfnGdipVectorTransformMatrixPoints(matrix,pts,count);
}

GpStatus WINAPI GdiplusWrapper::GdipVectorTransformMatrixPointsI(GpMatrix* matrix, GpPoint* pts, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipVectorTransformMatrixPointsI pfnGdipVectorTransformMatrixPointsI = (pGdipVectorTransformMatrixPointsI) GetProcAddress(m_hGdiplusLib, "GdipVectorTransformMatrixPointsI");
	if(!pfnGdipVectorTransformMatrixPointsI)
	{
		return GenericError;
	}
	return pfnGdipVectorTransformMatrixPointsI(matrix,pts,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetMatrixElements(GDIPCONST GpMatrix* matrix, REAL* matrixOut)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetMatrixElements pfnGdipGetMatrixElements = (pGdipGetMatrixElements) GetProcAddress(m_hGdiplusLib, "GdipGetMatrixElements");
	if(!pfnGdipGetMatrixElements)
	{
		return GenericError;
	}
	return pfnGdipGetMatrixElements(matrix,matrixOut);
}

GpStatus WINAPI GdiplusWrapper::GdipIsMatrixInvertible(GDIPCONST GpMatrix* matrix, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsMatrixInvertible pfnGdipIsMatrixInvertible = (pGdipIsMatrixInvertible) GetProcAddress(m_hGdiplusLib, "GdipIsMatrixInvertible");
	if(!pfnGdipIsMatrixInvertible)
	{
		return GenericError;
	}
	return pfnGdipIsMatrixInvertible(matrix,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsMatrixIdentity(GDIPCONST GpMatrix* matrix, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsMatrixIdentity pfnGdipIsMatrixIdentity = (pGdipIsMatrixIdentity) GetProcAddress(m_hGdiplusLib, "GdipIsMatrixIdentity");
	if(!pfnGdipIsMatrixIdentity)
	{
		return GenericError;
	}
	return pfnGdipIsMatrixIdentity(matrix,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsMatrixEqual(GDIPCONST GpMatrix* matrix, GDIPCONST GpMatrix* matrix2, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsMatrixEqual pfnGdipIsMatrixEqual = (pGdipIsMatrixEqual) GetProcAddress(m_hGdiplusLib, "GdipIsMatrixEqual");
	if(!pfnGdipIsMatrixEqual)
	{
		return GenericError;
	}
	return pfnGdipIsMatrixEqual(matrix,matrix2,result);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateRegion(GpRegion** region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateRegion pfnGdipCreateRegion = (pGdipCreateRegion) GetProcAddress(m_hGdiplusLib, "GdipCreateRegion");
	if(!pfnGdipCreateRegion)
	{
		return GenericError;
	}
	return pfnGdipCreateRegion(region);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateRegionRect(GDIPCONST GpRectF* rect, GpRegion** region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateRegionRect pfnGdipCreateRegionRect = (pGdipCreateRegionRect) GetProcAddress(m_hGdiplusLib, "GdipCreateRegionRect");
	if(!pfnGdipCreateRegionRect)
	{
		return GenericError;
	}
	return pfnGdipCreateRegionRect(rect,region);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateRegionRectI(GDIPCONST GpRect* rect, GpRegion** region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateRegionRectI pfnGdipCreateRegionRectI = (pGdipCreateRegionRectI) GetProcAddress(m_hGdiplusLib, "GdipCreateRegionRectI");
	if(!pfnGdipCreateRegionRectI)
	{
		return GenericError;
	}
	return pfnGdipCreateRegionRectI(rect,region);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateRegionPath(GpPath* path, GpRegion** region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateRegionPath pfnGdipCreateRegionPath = (pGdipCreateRegionPath) GetProcAddress(m_hGdiplusLib, "GdipCreateRegionPath");
	if(!pfnGdipCreateRegionPath)
	{
		return GenericError;
	}
	return pfnGdipCreateRegionPath(path,region);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateRegionRgnData(GDIPCONST BYTE* regionData, INT size, GpRegion** region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateRegionRgnData pfnGdipCreateRegionRgnData = (pGdipCreateRegionRgnData) GetProcAddress(m_hGdiplusLib, "GdipCreateRegionRgnData");
	if(!pfnGdipCreateRegionRgnData)
	{
		return GenericError;
	}
	return pfnGdipCreateRegionRgnData(regionData,size,region);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateRegionHrgn(HRGN hRgn, GpRegion** region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateRegionHrgn pfnGdipCreateRegionHrgn = (pGdipCreateRegionHrgn) GetProcAddress(m_hGdiplusLib, "GdipCreateRegionHrgn");
	if(!pfnGdipCreateRegionHrgn)
	{
		return GenericError;
	}
	return pfnGdipCreateRegionHrgn(hRgn,region);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneRegion(GpRegion* region, GpRegion** cloneRegion)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneRegion pfnGdipCloneRegion = (pGdipCloneRegion) GetProcAddress(m_hGdiplusLib, "GdipCloneRegion");
	if(!pfnGdipCloneRegion)
	{
		return GenericError;
	}
	return pfnGdipCloneRegion(region,cloneRegion);
}

GpStatus WINAPI GdiplusWrapper::GdipDeleteRegion(GpRegion* region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeleteRegion pfnGdipDeleteRegion = (pGdipDeleteRegion) GetProcAddress(m_hGdiplusLib, "GdipDeleteRegion");
	if(!pfnGdipDeleteRegion)
	{
		return GenericError;
	}
	return pfnGdipDeleteRegion(region);
}

GpStatus WINAPI GdiplusWrapper::GdipSetInfinite(GpRegion* region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetInfinite pfnGdipSetInfinite = (pGdipSetInfinite) GetProcAddress(m_hGdiplusLib, "GdipSetInfinite");
	if(!pfnGdipSetInfinite)
	{
		return GenericError;
	}
	return pfnGdipSetInfinite(region);
}

GpStatus WINAPI GdiplusWrapper::GdipSetEmpty(GpRegion* region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetEmpty pfnGdipSetEmpty = (pGdipSetEmpty) GetProcAddress(m_hGdiplusLib, "GdipSetEmpty");
	if(!pfnGdipSetEmpty)
	{
		return GenericError;
	}
	return pfnGdipSetEmpty(region);
}

GpStatus WINAPI GdiplusWrapper::GdipCombineRegionRect(GpRegion* region, GDIPCONST GpRectF* rect, CombineMode combineMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCombineRegionRect pfnGdipCombineRegionRect = (pGdipCombineRegionRect) GetProcAddress(m_hGdiplusLib, "GdipCombineRegionRect");
	if(!pfnGdipCombineRegionRect)
	{
		return GenericError;
	}
	return pfnGdipCombineRegionRect(region,rect,combineMode);
}

GpStatus WINAPI GdiplusWrapper::GdipCombineRegionRectI(GpRegion* region, GDIPCONST GpRect* rect, CombineMode combineMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCombineRegionRectI pfnGdipCombineRegionRectI = (pGdipCombineRegionRectI) GetProcAddress(m_hGdiplusLib, "GdipCombineRegionRectI");
	if(!pfnGdipCombineRegionRectI)
	{
		return GenericError;
	}
	return pfnGdipCombineRegionRectI(region,rect,combineMode);
}

GpStatus WINAPI GdiplusWrapper::GdipCombineRegionPath(GpRegion* region, GpPath* path, CombineMode combineMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCombineRegionPath pfnGdipCombineRegionPath = (pGdipCombineRegionPath) GetProcAddress(m_hGdiplusLib, "GdipCombineRegionPath");
	if(!pfnGdipCombineRegionPath)
	{
		return GenericError;
	}
	return pfnGdipCombineRegionPath(region,path,combineMode);
}

GpStatus WINAPI GdiplusWrapper::GdipCombineRegionRegion(GpRegion* region, GpRegion* region2, CombineMode combineMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCombineRegionRegion pfnGdipCombineRegionRegion = (pGdipCombineRegionRegion) GetProcAddress(m_hGdiplusLib, "GdipCombineRegionRegion");
	if(!pfnGdipCombineRegionRegion)
	{
		return GenericError;
	}
	return pfnGdipCombineRegionRegion(region,region2,combineMode);
}

GpStatus WINAPI GdiplusWrapper::GdipTranslateRegion(GpRegion* region, REAL dx, REAL dy)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTranslateRegion pfnGdipTranslateRegion = (pGdipTranslateRegion) GetProcAddress(m_hGdiplusLib, "GdipTranslateRegion");
	if(!pfnGdipTranslateRegion)
	{
		return GenericError;
	}
	return pfnGdipTranslateRegion(region,dx,dy);
}

GpStatus WINAPI GdiplusWrapper::GdipTranslateRegionI(GpRegion* region, INT dx, INT dy)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTranslateRegionI pfnGdipTranslateRegionI = (pGdipTranslateRegionI) GetProcAddress(m_hGdiplusLib, "GdipTranslateRegionI");
	if(!pfnGdipTranslateRegionI)
	{
		return GenericError;
	}
	return pfnGdipTranslateRegionI(region,dx,dy);
}

GpStatus WINAPI GdiplusWrapper::GdipTransformRegion(GpRegion* region, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTransformRegion pfnGdipTransformRegion = (pGdipTransformRegion) GetProcAddress(m_hGdiplusLib, "GdipTransformRegion");
	if(!pfnGdipTransformRegion)
	{
		return GenericError;
	}
	return pfnGdipTransformRegion(region,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipGetRegionBounds(GpRegion* region, GpGraphics* graphics, GpRectF* rect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetRegionBounds pfnGdipGetRegionBounds = (pGdipGetRegionBounds) GetProcAddress(m_hGdiplusLib, "GdipGetRegionBounds");
	if(!pfnGdipGetRegionBounds)
	{
		return GenericError;
	}
	return pfnGdipGetRegionBounds(region,graphics,rect);
}

GpStatus WINAPI GdiplusWrapper::GdipGetRegionBoundsI(GpRegion* region, GpGraphics* graphics, GpRect* rect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetRegionBoundsI pfnGdipGetRegionBoundsI = (pGdipGetRegionBoundsI) GetProcAddress(m_hGdiplusLib, "GdipGetRegionBoundsI");
	if(!pfnGdipGetRegionBoundsI)
	{
		return GenericError;
	}
	return pfnGdipGetRegionBoundsI(region,graphics,rect);
}

GpStatus WINAPI GdiplusWrapper::GdipGetRegionHRgn(GpRegion* region, GpGraphics* graphics, HRGN* hRgn)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetRegionHRgn pfnGdipGetRegionHRgn = (pGdipGetRegionHRgn) GetProcAddress(m_hGdiplusLib, "GdipGetRegionHRgn");
	if(!pfnGdipGetRegionHRgn)
	{
		return GenericError;
	}
	return pfnGdipGetRegionHRgn(region,graphics,hRgn);
}

GpStatus WINAPI GdiplusWrapper::GdipIsEmptyRegion(GpRegion* region, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsEmptyRegion pfnGdipIsEmptyRegion = (pGdipIsEmptyRegion) GetProcAddress(m_hGdiplusLib, "GdipIsEmptyRegion");
	if(!pfnGdipIsEmptyRegion)
	{
		return GenericError;
	}
	return pfnGdipIsEmptyRegion(region,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsInfiniteRegion(GpRegion* region, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsInfiniteRegion pfnGdipIsInfiniteRegion = (pGdipIsInfiniteRegion) GetProcAddress(m_hGdiplusLib, "GdipIsInfiniteRegion");
	if(!pfnGdipIsInfiniteRegion)
	{
		return GenericError;
	}
	return pfnGdipIsInfiniteRegion(region,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsEqualRegion(GpRegion* region, GpRegion* region2, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsEqualRegion pfnGdipIsEqualRegion = (pGdipIsEqualRegion) GetProcAddress(m_hGdiplusLib, "GdipIsEqualRegion");
	if(!pfnGdipIsEqualRegion)
	{
		return GenericError;
	}
	return pfnGdipIsEqualRegion(region,region2,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipGetRegionDataSize(GpRegion* region, UINT* bufferSize)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetRegionDataSize pfnGdipGetRegionDataSize = (pGdipGetRegionDataSize) GetProcAddress(m_hGdiplusLib, "GdipGetRegionDataSize");
	if(!pfnGdipGetRegionDataSize)
	{
		return GenericError;
	}
	return pfnGdipGetRegionDataSize(region,bufferSize);
}

GpStatus WINAPI GdiplusWrapper::GdipGetRegionData(GpRegion* region, BYTE* buffer, UINT bufferSize, UINT* sizeFilled)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetRegionData pfnGdipGetRegionData = (pGdipGetRegionData) GetProcAddress(m_hGdiplusLib, "GdipGetRegionData");
	if(!pfnGdipGetRegionData)
	{
		return GenericError;
	}
	return pfnGdipGetRegionData(region,buffer,bufferSize,sizeFilled);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisibleRegionPoint(GpRegion* region, REAL x, REAL y, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisibleRegionPoint pfnGdipIsVisibleRegionPoint = (pGdipIsVisibleRegionPoint) GetProcAddress(m_hGdiplusLib, "GdipIsVisibleRegionPoint");
	if(!pfnGdipIsVisibleRegionPoint)
	{
		return GenericError;
	}
	return pfnGdipIsVisibleRegionPoint(region,x,y,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisibleRegionPointI(GpRegion* region, INT x, INT y, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisibleRegionPointI pfnGdipIsVisibleRegionPointI = (pGdipIsVisibleRegionPointI) GetProcAddress(m_hGdiplusLib, "GdipIsVisibleRegionPointI");
	if(!pfnGdipIsVisibleRegionPointI)
	{
		return GenericError;
	}
	return pfnGdipIsVisibleRegionPointI(region,x,y,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisibleRegionRect(GpRegion* region, REAL x, REAL y, REAL width, REAL height, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisibleRegionRect pfnGdipIsVisibleRegionRect = (pGdipIsVisibleRegionRect) GetProcAddress(m_hGdiplusLib, "GdipIsVisibleRegionRect");
	if(!pfnGdipIsVisibleRegionRect)
	{
		return GenericError;
	}
	return pfnGdipIsVisibleRegionRect(region,x,y,width,height,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisibleRegionRectI(GpRegion* region, INT x, INT y, INT width, INT height, GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisibleRegionRectI pfnGdipIsVisibleRegionRectI = (pGdipIsVisibleRegionRectI) GetProcAddress(m_hGdiplusLib, "GdipIsVisibleRegionRectI");
	if(!pfnGdipIsVisibleRegionRectI)
	{
		return GenericError;
	}
	return pfnGdipIsVisibleRegionRectI(region,x,y,width,height,graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipGetRegionScansCount(GpRegion* region, UINT* count, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetRegionScansCount pfnGdipGetRegionScansCount = (pGdipGetRegionScansCount) GetProcAddress(m_hGdiplusLib, "GdipGetRegionScansCount");
	if(!pfnGdipGetRegionScansCount)
	{
		return GenericError;
	}
	return pfnGdipGetRegionScansCount(region,count,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipGetRegionScans(GpRegion* region, GpRectF* rects, INT* count, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetRegionScans pfnGdipGetRegionScans = (pGdipGetRegionScans) GetProcAddress(m_hGdiplusLib, "GdipGetRegionScans");
	if(!pfnGdipGetRegionScans)
	{
		return GenericError;
	}
	return pfnGdipGetRegionScans(region,rects,count,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipGetRegionScansI(GpRegion* region, GpRect* rects, INT* count, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetRegionScansI pfnGdipGetRegionScansI = (pGdipGetRegionScansI) GetProcAddress(m_hGdiplusLib, "GdipGetRegionScansI");
	if(!pfnGdipGetRegionScansI)
	{
		return GenericError;
	}
	return pfnGdipGetRegionScansI(region,rects,count,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneBrush(GpBrush* brush, GpBrush** cloneBrush)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneBrush pfnGdipCloneBrush = (pGdipCloneBrush) GetProcAddress(m_hGdiplusLib, "GdipCloneBrush");
	if(!pfnGdipCloneBrush)
	{
		return GenericError;
	}
	return pfnGdipCloneBrush(brush,cloneBrush);
}

GpStatus WINAPI GdiplusWrapper::GdipDeleteBrush(GpBrush* brush)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeleteBrush pfnGdipDeleteBrush = (pGdipDeleteBrush) GetProcAddress(m_hGdiplusLib, "GdipDeleteBrush");
	if(!pfnGdipDeleteBrush)
	{
		return GenericError;
	}
	return pfnGdipDeleteBrush(brush);
}

GpStatus WINAPI GdiplusWrapper::GdipGetBrushType(GpBrush* brush, GpBrushType* type)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetBrushType pfnGdipGetBrushType = (pGdipGetBrushType) GetProcAddress(m_hGdiplusLib, "GdipGetBrushType");
	if(!pfnGdipGetBrushType)
	{
		return GenericError;
	}
	return pfnGdipGetBrushType(brush,type);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateHatchBrush(GpHatchStyle hatchstyle, ARGB forecol, ARGB backcol, GpHatch** brush)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateHatchBrush pfnGdipCreateHatchBrush = (pGdipCreateHatchBrush) GetProcAddress(m_hGdiplusLib, "GdipCreateHatchBrush");
	if(!pfnGdipCreateHatchBrush)
	{
		return GenericError;
	}
	return pfnGdipCreateHatchBrush(hatchstyle,forecol,backcol,brush);
}

GpStatus WINAPI GdiplusWrapper::GdipGetHatchStyle(GpHatch* brush, GpHatchStyle* hatchstyle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetHatchStyle pfnGdipGetHatchStyle = (pGdipGetHatchStyle) GetProcAddress(m_hGdiplusLib, "GdipGetHatchStyle");
	if(!pfnGdipGetHatchStyle)
	{
		return GenericError;
	}
	return pfnGdipGetHatchStyle(brush,hatchstyle);
}

GpStatus WINAPI GdiplusWrapper::GdipGetHatchForegroundColor(GpHatch* brush, ARGB* forecol)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetHatchForegroundColor pfnGdipGetHatchForegroundColor = (pGdipGetHatchForegroundColor) GetProcAddress(m_hGdiplusLib, "GdipGetHatchForegroundColor");
	if(!pfnGdipGetHatchForegroundColor)
	{
		return GenericError;
	}
	return pfnGdipGetHatchForegroundColor(brush,forecol);
}

GpStatus WINAPI GdiplusWrapper::GdipGetHatchBackgroundColor(GpHatch* brush, ARGB* backcol)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetHatchBackgroundColor pfnGdipGetHatchBackgroundColor = (pGdipGetHatchBackgroundColor) GetProcAddress(m_hGdiplusLib, "GdipGetHatchBackgroundColor");
	if(!pfnGdipGetHatchBackgroundColor)
	{
		return GenericError;
	}
	return pfnGdipGetHatchBackgroundColor(brush,backcol);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateTexture(GpImage* image, GpWrapMode wrapmode, GpTexture** texture)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateTexture pfnGdipCreateTexture = (pGdipCreateTexture) GetProcAddress(m_hGdiplusLib, "GdipCreateTexture");
	if(!pfnGdipCreateTexture)
	{
		return GenericError;
	}
	return pfnGdipCreateTexture(image,wrapmode,texture);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateTexture2(GpImage* image, GpWrapMode wrapmode, REAL x, REAL y, REAL width, REAL height, GpTexture** texture)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateTexture2 pfnGdipCreateTexture2 = (pGdipCreateTexture2) GetProcAddress(m_hGdiplusLib, "GdipCreateTexture2");
	if(!pfnGdipCreateTexture2)
	{
		return GenericError;
	}
	return pfnGdipCreateTexture2(image,wrapmode,x,y,width,height,texture);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateTextureIA(GpImage* image, GDIPCONST GpImageAttributes* imageAttributes, REAL x, REAL y, REAL width, REAL height, GpTexture** texture)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateTextureIA pfnGdipCreateTextureIA = (pGdipCreateTextureIA) GetProcAddress(m_hGdiplusLib, "GdipCreateTextureIA");
	if(!pfnGdipCreateTextureIA)
	{
		return GenericError;
	}
	return pfnGdipCreateTextureIA(image,imageAttributes,x,y,width,height,texture);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateTexture2I(GpImage* image, GpWrapMode wrapmode, INT x, INT y, INT width, INT height, GpTexture** texture)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateTexture2I pfnGdipCreateTexture2I = (pGdipCreateTexture2I) GetProcAddress(m_hGdiplusLib, "GdipCreateTexture2I");
	if(!pfnGdipCreateTexture2I)
	{
		return GenericError;
	}
	return pfnGdipCreateTexture2I(image,wrapmode,x,y,width,height,texture);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateTextureIAI(GpImage* image, GDIPCONST GpImageAttributes* imageAttributes, INT x, INT y, INT width, INT height, GpTexture** texture)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateTextureIAI pfnGdipCreateTextureIAI = (pGdipCreateTextureIAI) GetProcAddress(m_hGdiplusLib, "GdipCreateTextureIAI");
	if(!pfnGdipCreateTextureIAI)
	{
		return GenericError;
	}
	return pfnGdipCreateTextureIAI(image,imageAttributes,x,y,width,height,texture);
}

GpStatus WINAPI GdiplusWrapper::GdipGetTextureTransform(GpTexture* brush, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetTextureTransform pfnGdipGetTextureTransform = (pGdipGetTextureTransform) GetProcAddress(m_hGdiplusLib, "GdipGetTextureTransform");
	if(!pfnGdipGetTextureTransform)
	{
		return GenericError;
	}
	return pfnGdipGetTextureTransform(brush,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipSetTextureTransform(GpTexture* brush, GDIPCONST GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetTextureTransform pfnGdipSetTextureTransform = (pGdipSetTextureTransform) GetProcAddress(m_hGdiplusLib, "GdipSetTextureTransform");
	if(!pfnGdipSetTextureTransform)
	{
		return GenericError;
	}
	return pfnGdipSetTextureTransform(brush,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipResetTextureTransform(GpTexture* brush)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipResetTextureTransform pfnGdipResetTextureTransform = (pGdipResetTextureTransform) GetProcAddress(m_hGdiplusLib, "GdipResetTextureTransform");
	if(!pfnGdipResetTextureTransform)
	{
		return GenericError;
	}
	return pfnGdipResetTextureTransform(brush);
}

GpStatus WINAPI GdiplusWrapper::GdipMultiplyTextureTransform(GpTexture* brush, GDIPCONST GpMatrix* matrix, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipMultiplyTextureTransform pfnGdipMultiplyTextureTransform = (pGdipMultiplyTextureTransform) GetProcAddress(m_hGdiplusLib, "GdipMultiplyTextureTransform");
	if(!pfnGdipMultiplyTextureTransform)
	{
		return GenericError;
	}
	return pfnGdipMultiplyTextureTransform(brush,matrix,order);
}

GpStatus WINAPI GdiplusWrapper::GdipTranslateTextureTransform(GpTexture* brush, REAL dx, REAL dy, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTranslateTextureTransform pfnGdipTranslateTextureTransform = (pGdipTranslateTextureTransform) GetProcAddress(m_hGdiplusLib, "GdipTranslateTextureTransform");
	if(!pfnGdipTranslateTextureTransform)
	{
		return GenericError;
	}
	return pfnGdipTranslateTextureTransform(brush,dx,dy,order);
}

GpStatus WINAPI GdiplusWrapper::GdipScaleTextureTransform(GpTexture* brush, REAL sx, REAL sy, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipScaleTextureTransform pfnGdipScaleTextureTransform = (pGdipScaleTextureTransform) GetProcAddress(m_hGdiplusLib, "GdipScaleTextureTransform");
	if(!pfnGdipScaleTextureTransform)
	{
		return GenericError;
	}
	return pfnGdipScaleTextureTransform(brush,sx,sy,order);
}

GpStatus WINAPI GdiplusWrapper::GdipRotateTextureTransform(GpTexture* brush, REAL angle, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRotateTextureTransform pfnGdipRotateTextureTransform = (pGdipRotateTextureTransform) GetProcAddress(m_hGdiplusLib, "GdipRotateTextureTransform");
	if(!pfnGdipRotateTextureTransform)
	{
		return GenericError;
	}
	return pfnGdipRotateTextureTransform(brush,angle,order);
}

GpStatus WINAPI GdiplusWrapper::GdipSetTextureWrapMode(GpTexture* brush, GpWrapMode wrapmode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetTextureWrapMode pfnGdipSetTextureWrapMode = (pGdipSetTextureWrapMode) GetProcAddress(m_hGdiplusLib, "GdipSetTextureWrapMode");
	if(!pfnGdipSetTextureWrapMode)
	{
		return GenericError;
	}
	return pfnGdipSetTextureWrapMode(brush,wrapmode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetTextureWrapMode(GpTexture* brush, GpWrapMode* wrapmode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetTextureWrapMode pfnGdipGetTextureWrapMode = (pGdipGetTextureWrapMode) GetProcAddress(m_hGdiplusLib, "GdipGetTextureWrapMode");
	if(!pfnGdipGetTextureWrapMode)
	{
		return GenericError;
	}
	return pfnGdipGetTextureWrapMode(brush,wrapmode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetTextureImage(GpTexture* brush, GpImage** image)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetTextureImage pfnGdipGetTextureImage = (pGdipGetTextureImage) GetProcAddress(m_hGdiplusLib, "GdipGetTextureImage");
	if(!pfnGdipGetTextureImage)
	{
		return GenericError;
	}
	return pfnGdipGetTextureImage(brush,image);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateSolidFill(ARGB color, GpSolidFill** brush)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateSolidFill pfnGdipCreateSolidFill = (pGdipCreateSolidFill) GetProcAddress(m_hGdiplusLib, "GdipCreateSolidFill");
	if(!pfnGdipCreateSolidFill)
	{
		return GenericError;
	}
	return pfnGdipCreateSolidFill(color,brush);
}

GpStatus WINAPI GdiplusWrapper::GdipSetSolidFillColor(GpSolidFill* brush, ARGB color)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetSolidFillColor pfnGdipSetSolidFillColor = (pGdipSetSolidFillColor) GetProcAddress(m_hGdiplusLib, "GdipSetSolidFillColor");
	if(!pfnGdipSetSolidFillColor)
	{
		return GenericError;
	}
	return pfnGdipSetSolidFillColor(brush,color);
}

GpStatus WINAPI GdiplusWrapper::GdipGetSolidFillColor(GpSolidFill* brush, ARGB* color)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetSolidFillColor pfnGdipGetSolidFillColor = (pGdipGetSolidFillColor) GetProcAddress(m_hGdiplusLib, "GdipGetSolidFillColor");
	if(!pfnGdipGetSolidFillColor)
	{
		return GenericError;
	}
	return pfnGdipGetSolidFillColor(brush,color);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateLineBrush(GDIPCONST GpPointF* point1, GDIPCONST GpPointF* point2, ARGB color1, ARGB color2, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateLineBrush pfnGdipCreateLineBrush = (pGdipCreateLineBrush) GetProcAddress(m_hGdiplusLib, "GdipCreateLineBrush");
	if(!pfnGdipCreateLineBrush)
	{
		return GenericError;
	}
	return pfnGdipCreateLineBrush(point1,point2,color1,color2,wrapMode,lineGradient);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateLineBrushI(GDIPCONST GpPoint* point1, GDIPCONST GpPoint* point2, ARGB color1, ARGB color2, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateLineBrushI pfnGdipCreateLineBrushI = (pGdipCreateLineBrushI) GetProcAddress(m_hGdiplusLib, "GdipCreateLineBrushI");
	if(!pfnGdipCreateLineBrushI)
	{
		return GenericError;
	}
	return pfnGdipCreateLineBrushI(point1,point2,color1,color2,wrapMode,lineGradient);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateLineBrushFromRect(GDIPCONST GpRectF* rect, ARGB color1, ARGB color2, LinearGradientMode mode, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateLineBrushFromRect pfnGdipCreateLineBrushFromRect = (pGdipCreateLineBrushFromRect) GetProcAddress(m_hGdiplusLib, "GdipCreateLineBrushFromRect");
	if(!pfnGdipCreateLineBrushFromRect)
	{
		return GenericError;
	}
	return pfnGdipCreateLineBrushFromRect(rect,color1,color2,mode,wrapMode,lineGradient);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateLineBrushFromRectI(GDIPCONST GpRect* rect, ARGB color1, ARGB color2, LinearGradientMode mode, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateLineBrushFromRectI pfnGdipCreateLineBrushFromRectI = (pGdipCreateLineBrushFromRectI) GetProcAddress(m_hGdiplusLib, "GdipCreateLineBrushFromRectI");
	if(!pfnGdipCreateLineBrushFromRectI)
	{
		return GenericError;
	}
	return pfnGdipCreateLineBrushFromRectI(rect,color1,color2,mode,wrapMode,lineGradient);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateLineBrushFromRectWithAngle(GDIPCONST GpRectF* rect, ARGB color1, ARGB color2, REAL angle, BOOL isAngleScalable, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateLineBrushFromRectWithAngle pfnGdipCreateLineBrushFromRectWithAngle = (pGdipCreateLineBrushFromRectWithAngle) GetProcAddress(m_hGdiplusLib, "GdipCreateLineBrushFromRectWithAngle");
	if(!pfnGdipCreateLineBrushFromRectWithAngle)
	{
		return GenericError;
	}
	return pfnGdipCreateLineBrushFromRectWithAngle(rect,color1,color2,angle,isAngleScalable,wrapMode,lineGradient);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateLineBrushFromRectWithAngleI(GDIPCONST GpRect* rect, ARGB color1, ARGB color2, REAL angle, BOOL isAngleScalable, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateLineBrushFromRectWithAngleI pfnGdipCreateLineBrushFromRectWithAngleI = (pGdipCreateLineBrushFromRectWithAngleI) GetProcAddress(m_hGdiplusLib, "GdipCreateLineBrushFromRectWithAngleI");
	if(!pfnGdipCreateLineBrushFromRectWithAngleI)
	{
		return GenericError;
	}
	return pfnGdipCreateLineBrushFromRectWithAngleI(rect,color1,color2,angle,isAngleScalable,wrapMode,lineGradient);
}

GpStatus WINAPI GdiplusWrapper::GdipSetLineColors(GpLineGradient* brush, ARGB color1, ARGB color2)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetLineColors pfnGdipSetLineColors = (pGdipSetLineColors) GetProcAddress(m_hGdiplusLib, "GdipSetLineColors");
	if(!pfnGdipSetLineColors)
	{
		return GenericError;
	}
	return pfnGdipSetLineColors(brush,color1,color2);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLineColors(GpLineGradient* brush, ARGB* colors)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLineColors pfnGdipGetLineColors = (pGdipGetLineColors) GetProcAddress(m_hGdiplusLib, "GdipGetLineColors");
	if(!pfnGdipGetLineColors)
	{
		return GenericError;
	}
	return pfnGdipGetLineColors(brush,colors);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLineRect(GpLineGradient* brush, GpRectF* rect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLineRect pfnGdipGetLineRect = (pGdipGetLineRect) GetProcAddress(m_hGdiplusLib, "GdipGetLineRect");
	if(!pfnGdipGetLineRect)
	{
		return GenericError;
	}
	return pfnGdipGetLineRect(brush,rect);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLineRectI(GpLineGradient* brush, GpRect* rect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLineRectI pfnGdipGetLineRectI = (pGdipGetLineRectI) GetProcAddress(m_hGdiplusLib, "GdipGetLineRectI");
	if(!pfnGdipGetLineRectI)
	{
		return GenericError;
	}
	return pfnGdipGetLineRectI(brush,rect);
}

GpStatus WINAPI GdiplusWrapper::GdipSetLineGammaCorrection(GpLineGradient* brush, BOOL useGammaCorrection)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetLineGammaCorrection pfnGdipSetLineGammaCorrection = (pGdipSetLineGammaCorrection) GetProcAddress(m_hGdiplusLib, "GdipSetLineGammaCorrection");
	if(!pfnGdipSetLineGammaCorrection)
	{
		return GenericError;
	}
	return pfnGdipSetLineGammaCorrection(brush,useGammaCorrection);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLineGammaCorrection(GpLineGradient* brush, BOOL* useGammaCorrection)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLineGammaCorrection pfnGdipGetLineGammaCorrection = (pGdipGetLineGammaCorrection) GetProcAddress(m_hGdiplusLib, "GdipGetLineGammaCorrection");
	if(!pfnGdipGetLineGammaCorrection)
	{
		return GenericError;
	}
	return pfnGdipGetLineGammaCorrection(brush,useGammaCorrection);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLineBlendCount(GpLineGradient* brush, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLineBlendCount pfnGdipGetLineBlendCount = (pGdipGetLineBlendCount) GetProcAddress(m_hGdiplusLib, "GdipGetLineBlendCount");
	if(!pfnGdipGetLineBlendCount)
	{
		return GenericError;
	}
	return pfnGdipGetLineBlendCount(brush,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLineBlend(GpLineGradient* brush, REAL* blend, REAL* positions, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLineBlend pfnGdipGetLineBlend = (pGdipGetLineBlend) GetProcAddress(m_hGdiplusLib, "GdipGetLineBlend");
	if(!pfnGdipGetLineBlend)
	{
		return GenericError;
	}
	return pfnGdipGetLineBlend(brush,blend,positions,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetLineBlend(GpLineGradient* brush, GDIPCONST REAL* blend, GDIPCONST REAL* positions, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetLineBlend pfnGdipSetLineBlend = (pGdipSetLineBlend) GetProcAddress(m_hGdiplusLib, "GdipSetLineBlend");
	if(!pfnGdipSetLineBlend)
	{
		return GenericError;
	}
	return pfnGdipSetLineBlend(brush,blend,positions,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLinePresetBlendCount(GpLineGradient* brush, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLinePresetBlendCount pfnGdipGetLinePresetBlendCount = (pGdipGetLinePresetBlendCount) GetProcAddress(m_hGdiplusLib, "GdipGetLinePresetBlendCount");
	if(!pfnGdipGetLinePresetBlendCount)
	{
		return GenericError;
	}
	return pfnGdipGetLinePresetBlendCount(brush,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLinePresetBlend(GpLineGradient* brush, ARGB* blend, REAL* positions, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLinePresetBlend pfnGdipGetLinePresetBlend = (pGdipGetLinePresetBlend) GetProcAddress(m_hGdiplusLib, "GdipGetLinePresetBlend");
	if(!pfnGdipGetLinePresetBlend)
	{
		return GenericError;
	}
	return pfnGdipGetLinePresetBlend(brush,blend,positions,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetLinePresetBlend(GpLineGradient* brush, GDIPCONST ARGB* blend, GDIPCONST REAL* positions, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetLinePresetBlend pfnGdipSetLinePresetBlend = (pGdipSetLinePresetBlend) GetProcAddress(m_hGdiplusLib, "GdipSetLinePresetBlend");
	if(!pfnGdipSetLinePresetBlend)
	{
		return GenericError;
	}
	return pfnGdipSetLinePresetBlend(brush,blend,positions,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetLineSigmaBlend(GpLineGradient* brush, REAL focus, REAL scale)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetLineSigmaBlend pfnGdipSetLineSigmaBlend = (pGdipSetLineSigmaBlend) GetProcAddress(m_hGdiplusLib, "GdipSetLineSigmaBlend");
	if(!pfnGdipSetLineSigmaBlend)
	{
		return GenericError;
	}
	return pfnGdipSetLineSigmaBlend(brush,focus,scale);
}

GpStatus WINAPI GdiplusWrapper::GdipSetLineLinearBlend(GpLineGradient* brush, REAL focus, REAL scale)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetLineLinearBlend pfnGdipSetLineLinearBlend = (pGdipSetLineLinearBlend) GetProcAddress(m_hGdiplusLib, "GdipSetLineLinearBlend");
	if(!pfnGdipSetLineLinearBlend)
	{
		return GenericError;
	}
	return pfnGdipSetLineLinearBlend(brush,focus,scale);
}

GpStatus WINAPI GdiplusWrapper::GdipSetLineWrapMode(GpLineGradient* brush, GpWrapMode wrapmode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetLineWrapMode pfnGdipSetLineWrapMode = (pGdipSetLineWrapMode) GetProcAddress(m_hGdiplusLib, "GdipSetLineWrapMode");
	if(!pfnGdipSetLineWrapMode)
	{
		return GenericError;
	}
	return pfnGdipSetLineWrapMode(brush,wrapmode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLineWrapMode(GpLineGradient* brush, GpWrapMode* wrapmode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLineWrapMode pfnGdipGetLineWrapMode = (pGdipGetLineWrapMode) GetProcAddress(m_hGdiplusLib, "GdipGetLineWrapMode");
	if(!pfnGdipGetLineWrapMode)
	{
		return GenericError;
	}
	return pfnGdipGetLineWrapMode(brush,wrapmode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLineTransform(GpLineGradient* brush, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLineTransform pfnGdipGetLineTransform = (pGdipGetLineTransform) GetProcAddress(m_hGdiplusLib, "GdipGetLineTransform");
	if(!pfnGdipGetLineTransform)
	{
		return GenericError;
	}
	return pfnGdipGetLineTransform(brush,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipSetLineTransform(GpLineGradient* brush, GDIPCONST GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetLineTransform pfnGdipSetLineTransform = (pGdipSetLineTransform) GetProcAddress(m_hGdiplusLib, "GdipSetLineTransform");
	if(!pfnGdipSetLineTransform)
	{
		return GenericError;
	}
	return pfnGdipSetLineTransform(brush,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipResetLineTransform(GpLineGradient* brush)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipResetLineTransform pfnGdipResetLineTransform = (pGdipResetLineTransform) GetProcAddress(m_hGdiplusLib, "GdipResetLineTransform");
	if(!pfnGdipResetLineTransform)
	{
		return GenericError;
	}
	return pfnGdipResetLineTransform(brush);
}

GpStatus WINAPI GdiplusWrapper::GdipMultiplyLineTransform(GpLineGradient* brush, GDIPCONST GpMatrix* matrix, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipMultiplyLineTransform pfnGdipMultiplyLineTransform = (pGdipMultiplyLineTransform) GetProcAddress(m_hGdiplusLib, "GdipMultiplyLineTransform");
	if(!pfnGdipMultiplyLineTransform)
	{
		return GenericError;
	}
	return pfnGdipMultiplyLineTransform(brush,matrix,order);
}

GpStatus WINAPI GdiplusWrapper::GdipTranslateLineTransform(GpLineGradient* brush, REAL dx, REAL dy, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTranslateLineTransform pfnGdipTranslateLineTransform = (pGdipTranslateLineTransform) GetProcAddress(m_hGdiplusLib, "GdipTranslateLineTransform");
	if(!pfnGdipTranslateLineTransform)
	{
		return GenericError;
	}
	return pfnGdipTranslateLineTransform(brush,dx,dy,order);
}

GpStatus WINAPI GdiplusWrapper::GdipScaleLineTransform(GpLineGradient* brush, REAL sx, REAL sy, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipScaleLineTransform pfnGdipScaleLineTransform = (pGdipScaleLineTransform) GetProcAddress(m_hGdiplusLib, "GdipScaleLineTransform");
	if(!pfnGdipScaleLineTransform)
	{
		return GenericError;
	}
	return pfnGdipScaleLineTransform(brush,sx,sy,order);
}

GpStatus WINAPI GdiplusWrapper::GdipRotateLineTransform(GpLineGradient* brush, REAL angle, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRotateLineTransform pfnGdipRotateLineTransform = (pGdipRotateLineTransform) GetProcAddress(m_hGdiplusLib, "GdipRotateLineTransform");
	if(!pfnGdipRotateLineTransform)
	{
		return GenericError;
	}
	return pfnGdipRotateLineTransform(brush,angle,order);
}

GpStatus WINAPI GdiplusWrapper::GdipCreatePathGradient(GDIPCONST GpPointF* points, INT count, GpWrapMode wrapMode, GpPathGradient** polyGradient)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreatePathGradient pfnGdipCreatePathGradient = (pGdipCreatePathGradient) GetProcAddress(m_hGdiplusLib, "GdipCreatePathGradient");
	if(!pfnGdipCreatePathGradient)
	{
		return GenericError;
	}
	return pfnGdipCreatePathGradient(points,count,wrapMode,polyGradient);
}

GpStatus WINAPI GdiplusWrapper::GdipCreatePathGradientI(GDIPCONST GpPoint* points, INT count, GpWrapMode wrapMode, GpPathGradient** polyGradient)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreatePathGradientI pfnGdipCreatePathGradientI = (pGdipCreatePathGradientI) GetProcAddress(m_hGdiplusLib, "GdipCreatePathGradientI");
	if(!pfnGdipCreatePathGradientI)
	{
		return GenericError;
	}
	return pfnGdipCreatePathGradientI(points,count,wrapMode,polyGradient);
}

GpStatus WINAPI GdiplusWrapper::GdipCreatePathGradientFromPath(GDIPCONST GpPath* path, GpPathGradient** polyGradient)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreatePathGradientFromPath pfnGdipCreatePathGradientFromPath = (pGdipCreatePathGradientFromPath) GetProcAddress(m_hGdiplusLib, "GdipCreatePathGradientFromPath");
	if(!pfnGdipCreatePathGradientFromPath)
	{
		return GenericError;
	}
	return pfnGdipCreatePathGradientFromPath(path,polyGradient);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientCenterColor(GpPathGradient* brush, ARGB* colors)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientCenterColor pfnGdipGetPathGradientCenterColor = (pGdipGetPathGradientCenterColor) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientCenterColor");
	if(!pfnGdipGetPathGradientCenterColor)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientCenterColor(brush,colors);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientCenterColor(GpPathGradient* brush, ARGB colors)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientCenterColor pfnGdipSetPathGradientCenterColor = (pGdipSetPathGradientCenterColor) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientCenterColor");
	if(!pfnGdipSetPathGradientCenterColor)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientCenterColor(brush,colors);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientSurroundColorsWithCount(GpPathGradient* brush, ARGB* color, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientSurroundColorsWithCount pfnGdipGetPathGradientSurroundColorsWithCount = (pGdipGetPathGradientSurroundColorsWithCount) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientSurroundColorsWithCount");
	if(!pfnGdipGetPathGradientSurroundColorsWithCount)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientSurroundColorsWithCount(brush,color,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientSurroundColorsWithCount(GpPathGradient* brush, GDIPCONST ARGB* color, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientSurroundColorsWithCount pfnGdipSetPathGradientSurroundColorsWithCount = (pGdipSetPathGradientSurroundColorsWithCount) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientSurroundColorsWithCount");
	if(!pfnGdipSetPathGradientSurroundColorsWithCount)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientSurroundColorsWithCount(brush,color,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientPath(GpPathGradient* brush, GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientPath pfnGdipGetPathGradientPath = (pGdipGetPathGradientPath) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientPath");
	if(!pfnGdipGetPathGradientPath)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientPath(brush,path);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientPath(GpPathGradient* brush, GDIPCONST GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientPath pfnGdipSetPathGradientPath = (pGdipSetPathGradientPath) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientPath");
	if(!pfnGdipSetPathGradientPath)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientPath(brush,path);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientCenterPoint(GpPathGradient* brush, GpPointF* points)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientCenterPoint pfnGdipGetPathGradientCenterPoint = (pGdipGetPathGradientCenterPoint) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientCenterPoint");
	if(!pfnGdipGetPathGradientCenterPoint)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientCenterPoint(brush,points);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientCenterPointI(GpPathGradient* brush, GpPoint* points)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientCenterPointI pfnGdipGetPathGradientCenterPointI = (pGdipGetPathGradientCenterPointI) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientCenterPointI");
	if(!pfnGdipGetPathGradientCenterPointI)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientCenterPointI(brush,points);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientCenterPoint(GpPathGradient* brush, GDIPCONST GpPointF* points)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientCenterPoint pfnGdipSetPathGradientCenterPoint = (pGdipSetPathGradientCenterPoint) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientCenterPoint");
	if(!pfnGdipSetPathGradientCenterPoint)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientCenterPoint(brush,points);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientCenterPointI(GpPathGradient* brush, GDIPCONST GpPoint* points)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientCenterPointI pfnGdipSetPathGradientCenterPointI = (pGdipSetPathGradientCenterPointI) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientCenterPointI");
	if(!pfnGdipSetPathGradientCenterPointI)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientCenterPointI(brush,points);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientRect(GpPathGradient* brush, GpRectF* rect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientRect pfnGdipGetPathGradientRect = (pGdipGetPathGradientRect) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientRect");
	if(!pfnGdipGetPathGradientRect)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientRect(brush,rect);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientRectI(GpPathGradient* brush, GpRect* rect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientRectI pfnGdipGetPathGradientRectI = (pGdipGetPathGradientRectI) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientRectI");
	if(!pfnGdipGetPathGradientRectI)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientRectI(brush,rect);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientPointCount(GpPathGradient* brush, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientPointCount pfnGdipGetPathGradientPointCount = (pGdipGetPathGradientPointCount) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientPointCount");
	if(!pfnGdipGetPathGradientPointCount)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientPointCount(brush,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientSurroundColorCount(GpPathGradient* brush, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientSurroundColorCount pfnGdipGetPathGradientSurroundColorCount = (pGdipGetPathGradientSurroundColorCount) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientSurroundColorCount");
	if(!pfnGdipGetPathGradientSurroundColorCount)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientSurroundColorCount(brush,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientGammaCorrection(GpPathGradient* brush, BOOL useGammaCorrection)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientGammaCorrection pfnGdipSetPathGradientGammaCorrection = (pGdipSetPathGradientGammaCorrection) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientGammaCorrection");
	if(!pfnGdipSetPathGradientGammaCorrection)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientGammaCorrection(brush,useGammaCorrection);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientGammaCorrection(GpPathGradient* brush, BOOL* useGammaCorrection)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientGammaCorrection pfnGdipGetPathGradientGammaCorrection = (pGdipGetPathGradientGammaCorrection) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientGammaCorrection");
	if(!pfnGdipGetPathGradientGammaCorrection)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientGammaCorrection(brush,useGammaCorrection);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientBlendCount(GpPathGradient* brush, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientBlendCount pfnGdipGetPathGradientBlendCount = (pGdipGetPathGradientBlendCount) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientBlendCount");
	if(!pfnGdipGetPathGradientBlendCount)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientBlendCount(brush,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientBlend(GpPathGradient* brush, REAL* blend, REAL* positions, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientBlend pfnGdipGetPathGradientBlend = (pGdipGetPathGradientBlend) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientBlend");
	if(!pfnGdipGetPathGradientBlend)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientBlend(brush,blend,positions,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientBlend(GpPathGradient* brush, GDIPCONST REAL* blend, GDIPCONST REAL* positions, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientBlend pfnGdipSetPathGradientBlend = (pGdipSetPathGradientBlend) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientBlend");
	if(!pfnGdipSetPathGradientBlend)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientBlend(brush,blend,positions,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientPresetBlendCount(GpPathGradient* brush, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientPresetBlendCount pfnGdipGetPathGradientPresetBlendCount = (pGdipGetPathGradientPresetBlendCount) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientPresetBlendCount");
	if(!pfnGdipGetPathGradientPresetBlendCount)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientPresetBlendCount(brush,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientPresetBlend(GpPathGradient* brush, ARGB* blend, REAL* positions, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientPresetBlend pfnGdipGetPathGradientPresetBlend = (pGdipGetPathGradientPresetBlend) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientPresetBlend");
	if(!pfnGdipGetPathGradientPresetBlend)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientPresetBlend(brush,blend,positions,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientPresetBlend(GpPathGradient* brush, GDIPCONST ARGB* blend, GDIPCONST REAL* positions, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientPresetBlend pfnGdipSetPathGradientPresetBlend = (pGdipSetPathGradientPresetBlend) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientPresetBlend");
	if(!pfnGdipSetPathGradientPresetBlend)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientPresetBlend(brush,blend,positions,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientSigmaBlend(GpPathGradient* brush, REAL focus, REAL scale)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientSigmaBlend pfnGdipSetPathGradientSigmaBlend = (pGdipSetPathGradientSigmaBlend) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientSigmaBlend");
	if(!pfnGdipSetPathGradientSigmaBlend)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientSigmaBlend(brush,focus,scale);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientLinearBlend(GpPathGradient* brush, REAL focus, REAL scale)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientLinearBlend pfnGdipSetPathGradientLinearBlend = (pGdipSetPathGradientLinearBlend) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientLinearBlend");
	if(!pfnGdipSetPathGradientLinearBlend)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientLinearBlend(brush,focus,scale);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientWrapMode(GpPathGradient* brush, GpWrapMode* wrapmode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientWrapMode pfnGdipGetPathGradientWrapMode = (pGdipGetPathGradientWrapMode) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientWrapMode");
	if(!pfnGdipGetPathGradientWrapMode)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientWrapMode(brush,wrapmode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientWrapMode(GpPathGradient* brush, GpWrapMode wrapmode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientWrapMode pfnGdipSetPathGradientWrapMode = (pGdipSetPathGradientWrapMode) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientWrapMode");
	if(!pfnGdipSetPathGradientWrapMode)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientWrapMode(brush,wrapmode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientTransform(GpPathGradient* brush, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientTransform pfnGdipGetPathGradientTransform = (pGdipGetPathGradientTransform) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientTransform");
	if(!pfnGdipGetPathGradientTransform)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientTransform(brush,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientTransform(GpPathGradient* brush, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientTransform pfnGdipSetPathGradientTransform = (pGdipSetPathGradientTransform) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientTransform");
	if(!pfnGdipSetPathGradientTransform)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientTransform(brush,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipResetPathGradientTransform(GpPathGradient* brush)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipResetPathGradientTransform pfnGdipResetPathGradientTransform = (pGdipResetPathGradientTransform) GetProcAddress(m_hGdiplusLib, "GdipResetPathGradientTransform");
	if(!pfnGdipResetPathGradientTransform)
	{
		return GenericError;
	}
	return pfnGdipResetPathGradientTransform(brush);
}

GpStatus WINAPI GdiplusWrapper::GdipMultiplyPathGradientTransform(GpPathGradient* brush, GDIPCONST GpMatrix* matrix, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipMultiplyPathGradientTransform pfnGdipMultiplyPathGradientTransform = (pGdipMultiplyPathGradientTransform) GetProcAddress(m_hGdiplusLib, "GdipMultiplyPathGradientTransform");
	if(!pfnGdipMultiplyPathGradientTransform)
	{
		return GenericError;
	}
	return pfnGdipMultiplyPathGradientTransform(brush,matrix,order);
}

GpStatus WINAPI GdiplusWrapper::GdipTranslatePathGradientTransform(GpPathGradient* brush, REAL dx, REAL dy, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTranslatePathGradientTransform pfnGdipTranslatePathGradientTransform = (pGdipTranslatePathGradientTransform) GetProcAddress(m_hGdiplusLib, "GdipTranslatePathGradientTransform");
	if(!pfnGdipTranslatePathGradientTransform)
	{
		return GenericError;
	}
	return pfnGdipTranslatePathGradientTransform(brush,dx,dy,order);
}

GpStatus WINAPI GdiplusWrapper::GdipScalePathGradientTransform(GpPathGradient* brush, REAL sx, REAL sy, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipScalePathGradientTransform pfnGdipScalePathGradientTransform = (pGdipScalePathGradientTransform) GetProcAddress(m_hGdiplusLib, "GdipScalePathGradientTransform");
	if(!pfnGdipScalePathGradientTransform)
	{
		return GenericError;
	}
	return pfnGdipScalePathGradientTransform(brush,sx,sy,order);
}

GpStatus WINAPI GdiplusWrapper::GdipRotatePathGradientTransform(GpPathGradient* brush, REAL angle, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRotatePathGradientTransform pfnGdipRotatePathGradientTransform = (pGdipRotatePathGradientTransform) GetProcAddress(m_hGdiplusLib, "GdipRotatePathGradientTransform");
	if(!pfnGdipRotatePathGradientTransform)
	{
		return GenericError;
	}
	return pfnGdipRotatePathGradientTransform(brush,angle,order);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPathGradientFocusScales(GpPathGradient* brush, REAL* xScale, REAL* yScale)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPathGradientFocusScales pfnGdipGetPathGradientFocusScales = (pGdipGetPathGradientFocusScales) GetProcAddress(m_hGdiplusLib, "GdipGetPathGradientFocusScales");
	if(!pfnGdipGetPathGradientFocusScales)
	{
		return GenericError;
	}
	return pfnGdipGetPathGradientFocusScales(brush,xScale,yScale);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPathGradientFocusScales(GpPathGradient* brush, REAL xScale, REAL yScale)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPathGradientFocusScales pfnGdipSetPathGradientFocusScales = (pGdipSetPathGradientFocusScales) GetProcAddress(m_hGdiplusLib, "GdipSetPathGradientFocusScales");
	if(!pfnGdipSetPathGradientFocusScales)
	{
		return GenericError;
	}
	return pfnGdipSetPathGradientFocusScales(brush,xScale,yScale);
}

GpStatus WINAPI GdiplusWrapper::GdipCreatePen1(ARGB color, REAL width, GpUnit unit, GpPen** pen)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreatePen1 pfnGdipCreatePen1 = (pGdipCreatePen1) GetProcAddress(m_hGdiplusLib, "GdipCreatePen1");
	if(!pfnGdipCreatePen1)
	{
		return GenericError;
	}
	return pfnGdipCreatePen1(color,width,unit,pen);
}

GpStatus WINAPI GdiplusWrapper::GdipCreatePen2(GpBrush* brush, REAL width, GpUnit unit, GpPen** pen)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreatePen2 pfnGdipCreatePen2 = (pGdipCreatePen2) GetProcAddress(m_hGdiplusLib, "GdipCreatePen2");
	if(!pfnGdipCreatePen2)
	{
		return GenericError;
	}
	return pfnGdipCreatePen2(brush,width,unit,pen);
}

GpStatus WINAPI GdiplusWrapper::GdipClonePen(GpPen* pen, GpPen** clonepen)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipClonePen pfnGdipClonePen = (pGdipClonePen) GetProcAddress(m_hGdiplusLib, "GdipClonePen");
	if(!pfnGdipClonePen)
	{
		return GenericError;
	}
	return pfnGdipClonePen(pen,clonepen);
}

GpStatus WINAPI GdiplusWrapper::GdipDeletePen(GpPen* pen)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeletePen pfnGdipDeletePen = (pGdipDeletePen) GetProcAddress(m_hGdiplusLib, "GdipDeletePen");
	if(!pfnGdipDeletePen)
	{
		return GenericError;
	}
	return pfnGdipDeletePen(pen);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenWidth(GpPen* pen, REAL width)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenWidth pfnGdipSetPenWidth = (pGdipSetPenWidth) GetProcAddress(m_hGdiplusLib, "GdipSetPenWidth");
	if(!pfnGdipSetPenWidth)
	{
		return GenericError;
	}
	return pfnGdipSetPenWidth(pen,width);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenWidth(GpPen* pen, REAL* width)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenWidth pfnGdipGetPenWidth = (pGdipGetPenWidth) GetProcAddress(m_hGdiplusLib, "GdipGetPenWidth");
	if(!pfnGdipGetPenWidth)
	{
		return GenericError;
	}
	return pfnGdipGetPenWidth(pen,width);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenUnit(GpPen* pen, GpUnit unit)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenUnit pfnGdipSetPenUnit = (pGdipSetPenUnit) GetProcAddress(m_hGdiplusLib, "GdipSetPenUnit");
	if(!pfnGdipSetPenUnit)
	{
		return GenericError;
	}
	return pfnGdipSetPenUnit(pen,unit);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenUnit(GpPen* pen, GpUnit* unit)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenUnit pfnGdipGetPenUnit = (pGdipGetPenUnit) GetProcAddress(m_hGdiplusLib, "GdipGetPenUnit");
	if(!pfnGdipGetPenUnit)
	{
		return GenericError;
	}
	return pfnGdipGetPenUnit(pen,unit);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenLineCap197819(GpPen* pen, GpLineCap startCap, GpLineCap endCap, GpDashCap dashCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenLineCap197819 pfnGdipSetPenLineCap197819 = (pGdipSetPenLineCap197819) GetProcAddress(m_hGdiplusLib, "GdipSetPenLineCap197819");
	if(!pfnGdipSetPenLineCap197819)
	{
		return GenericError;
	}
	return pfnGdipSetPenLineCap197819(pen,startCap,endCap,dashCap);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenStartCap(GpPen* pen, GpLineCap startCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenStartCap pfnGdipSetPenStartCap = (pGdipSetPenStartCap) GetProcAddress(m_hGdiplusLib, "GdipSetPenStartCap");
	if(!pfnGdipSetPenStartCap)
	{
		return GenericError;
	}
	return pfnGdipSetPenStartCap(pen,startCap);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenEndCap(GpPen* pen, GpLineCap endCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenEndCap pfnGdipSetPenEndCap = (pGdipSetPenEndCap) GetProcAddress(m_hGdiplusLib, "GdipSetPenEndCap");
	if(!pfnGdipSetPenEndCap)
	{
		return GenericError;
	}
	return pfnGdipSetPenEndCap(pen,endCap);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenDashCap197819(GpPen* pen, GpDashCap dashCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenDashCap197819 pfnGdipSetPenDashCap197819 = (pGdipSetPenDashCap197819) GetProcAddress(m_hGdiplusLib, "GdipSetPenDashCap197819");
	if(!pfnGdipSetPenDashCap197819)
	{
		return GenericError;
	}
	return pfnGdipSetPenDashCap197819(pen,dashCap);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenStartCap(GpPen* pen, GpLineCap* startCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenStartCap pfnGdipGetPenStartCap = (pGdipGetPenStartCap) GetProcAddress(m_hGdiplusLib, "GdipGetPenStartCap");
	if(!pfnGdipGetPenStartCap)
	{
		return GenericError;
	}
	return pfnGdipGetPenStartCap(pen,startCap);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenEndCap(GpPen* pen, GpLineCap* endCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenEndCap pfnGdipGetPenEndCap = (pGdipGetPenEndCap) GetProcAddress(m_hGdiplusLib, "GdipGetPenEndCap");
	if(!pfnGdipGetPenEndCap)
	{
		return GenericError;
	}
	return pfnGdipGetPenEndCap(pen,endCap);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenDashCap197819(GpPen* pen, GpDashCap* dashCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenDashCap197819 pfnGdipGetPenDashCap197819 = (pGdipGetPenDashCap197819) GetProcAddress(m_hGdiplusLib, "GdipGetPenDashCap197819");
	if(!pfnGdipGetPenDashCap197819)
	{
		return GenericError;
	}
	return pfnGdipGetPenDashCap197819(pen,dashCap);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenLineJoin(GpPen* pen, GpLineJoin lineJoin)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenLineJoin pfnGdipSetPenLineJoin = (pGdipSetPenLineJoin) GetProcAddress(m_hGdiplusLib, "GdipSetPenLineJoin");
	if(!pfnGdipSetPenLineJoin)
	{
		return GenericError;
	}
	return pfnGdipSetPenLineJoin(pen,lineJoin);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenLineJoin(GpPen* pen, GpLineJoin* lineJoin)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenLineJoin pfnGdipGetPenLineJoin = (pGdipGetPenLineJoin) GetProcAddress(m_hGdiplusLib, "GdipGetPenLineJoin");
	if(!pfnGdipGetPenLineJoin)
	{
		return GenericError;
	}
	return pfnGdipGetPenLineJoin(pen,lineJoin);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenCustomStartCap(GpPen* pen, GpCustomLineCap* customCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenCustomStartCap pfnGdipSetPenCustomStartCap = (pGdipSetPenCustomStartCap) GetProcAddress(m_hGdiplusLib, "GdipSetPenCustomStartCap");
	if(!pfnGdipSetPenCustomStartCap)
	{
		return GenericError;
	}
	return pfnGdipSetPenCustomStartCap(pen,customCap);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenCustomStartCap(GpPen* pen, GpCustomLineCap** customCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenCustomStartCap pfnGdipGetPenCustomStartCap = (pGdipGetPenCustomStartCap) GetProcAddress(m_hGdiplusLib, "GdipGetPenCustomStartCap");
	if(!pfnGdipGetPenCustomStartCap)
	{
		return GenericError;
	}
	return pfnGdipGetPenCustomStartCap(pen,customCap);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenCustomEndCap(GpPen* pen, GpCustomLineCap* customCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenCustomEndCap pfnGdipSetPenCustomEndCap = (pGdipSetPenCustomEndCap) GetProcAddress(m_hGdiplusLib, "GdipSetPenCustomEndCap");
	if(!pfnGdipSetPenCustomEndCap)
	{
		return GenericError;
	}
	return pfnGdipSetPenCustomEndCap(pen,customCap);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenCustomEndCap(GpPen* pen, GpCustomLineCap** customCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenCustomEndCap pfnGdipGetPenCustomEndCap = (pGdipGetPenCustomEndCap) GetProcAddress(m_hGdiplusLib, "GdipGetPenCustomEndCap");
	if(!pfnGdipGetPenCustomEndCap)
	{
		return GenericError;
	}
	return pfnGdipGetPenCustomEndCap(pen,customCap);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenMiterLimit(GpPen* pen, REAL miterLimit)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenMiterLimit pfnGdipSetPenMiterLimit = (pGdipSetPenMiterLimit) GetProcAddress(m_hGdiplusLib, "GdipSetPenMiterLimit");
	if(!pfnGdipSetPenMiterLimit)
	{
		return GenericError;
	}
	return pfnGdipSetPenMiterLimit(pen,miterLimit);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenMiterLimit(GpPen* pen, REAL* miterLimit)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenMiterLimit pfnGdipGetPenMiterLimit = (pGdipGetPenMiterLimit) GetProcAddress(m_hGdiplusLib, "GdipGetPenMiterLimit");
	if(!pfnGdipGetPenMiterLimit)
	{
		return GenericError;
	}
	return pfnGdipGetPenMiterLimit(pen,miterLimit);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenMode(GpPen* pen, GpPenAlignment penMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenMode pfnGdipSetPenMode = (pGdipSetPenMode) GetProcAddress(m_hGdiplusLib, "GdipSetPenMode");
	if(!pfnGdipSetPenMode)
	{
		return GenericError;
	}
	return pfnGdipSetPenMode(pen,penMode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenMode(GpPen* pen, GpPenAlignment* penMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenMode pfnGdipGetPenMode = (pGdipGetPenMode) GetProcAddress(m_hGdiplusLib, "GdipGetPenMode");
	if(!pfnGdipGetPenMode)
	{
		return GenericError;
	}
	return pfnGdipGetPenMode(pen,penMode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenTransform(GpPen* pen, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenTransform pfnGdipSetPenTransform = (pGdipSetPenTransform) GetProcAddress(m_hGdiplusLib, "GdipSetPenTransform");
	if(!pfnGdipSetPenTransform)
	{
		return GenericError;
	}
	return pfnGdipSetPenTransform(pen,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenTransform(GpPen* pen, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenTransform pfnGdipGetPenTransform = (pGdipGetPenTransform) GetProcAddress(m_hGdiplusLib, "GdipGetPenTransform");
	if(!pfnGdipGetPenTransform)
	{
		return GenericError;
	}
	return pfnGdipGetPenTransform(pen,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipResetPenTransform(GpPen* pen)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipResetPenTransform pfnGdipResetPenTransform = (pGdipResetPenTransform) GetProcAddress(m_hGdiplusLib, "GdipResetPenTransform");
	if(!pfnGdipResetPenTransform)
	{
		return GenericError;
	}
	return pfnGdipResetPenTransform(pen);
}

GpStatus WINAPI GdiplusWrapper::GdipMultiplyPenTransform(GpPen* pen, GDIPCONST GpMatrix* matrix, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipMultiplyPenTransform pfnGdipMultiplyPenTransform = (pGdipMultiplyPenTransform) GetProcAddress(m_hGdiplusLib, "GdipMultiplyPenTransform");
	if(!pfnGdipMultiplyPenTransform)
	{
		return GenericError;
	}
	return pfnGdipMultiplyPenTransform(pen,matrix,order);
}

GpStatus WINAPI GdiplusWrapper::GdipTranslatePenTransform(GpPen* pen, REAL dx, REAL dy, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTranslatePenTransform pfnGdipTranslatePenTransform = (pGdipTranslatePenTransform) GetProcAddress(m_hGdiplusLib, "GdipTranslatePenTransform");
	if(!pfnGdipTranslatePenTransform)
	{
		return GenericError;
	}
	return pfnGdipTranslatePenTransform(pen,dx,dy,order);
}

GpStatus WINAPI GdiplusWrapper::GdipScalePenTransform(GpPen* pen, REAL sx, REAL sy, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipScalePenTransform pfnGdipScalePenTransform = (pGdipScalePenTransform) GetProcAddress(m_hGdiplusLib, "GdipScalePenTransform");
	if(!pfnGdipScalePenTransform)
	{
		return GenericError;
	}
	return pfnGdipScalePenTransform(pen,sx,sy,order);
}

GpStatus WINAPI GdiplusWrapper::GdipRotatePenTransform(GpPen* pen, REAL angle, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRotatePenTransform pfnGdipRotatePenTransform = (pGdipRotatePenTransform) GetProcAddress(m_hGdiplusLib, "GdipRotatePenTransform");
	if(!pfnGdipRotatePenTransform)
	{
		return GenericError;
	}
	return pfnGdipRotatePenTransform(pen,angle,order);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenColor(GpPen* pen, ARGB argb)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenColor pfnGdipSetPenColor = (pGdipSetPenColor) GetProcAddress(m_hGdiplusLib, "GdipSetPenColor");
	if(!pfnGdipSetPenColor)
	{
		return GenericError;
	}
	return pfnGdipSetPenColor(pen,argb);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenColor(GpPen* pen, ARGB* argb)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenColor pfnGdipGetPenColor = (pGdipGetPenColor) GetProcAddress(m_hGdiplusLib, "GdipGetPenColor");
	if(!pfnGdipGetPenColor)
	{
		return GenericError;
	}
	return pfnGdipGetPenColor(pen,argb);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenBrushFill(GpPen* pen, GpBrush* brush)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenBrushFill pfnGdipSetPenBrushFill = (pGdipSetPenBrushFill) GetProcAddress(m_hGdiplusLib, "GdipSetPenBrushFill");
	if(!pfnGdipSetPenBrushFill)
	{
		return GenericError;
	}
	return pfnGdipSetPenBrushFill(pen,brush);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenBrushFill(GpPen* pen, GpBrush** brush)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenBrushFill pfnGdipGetPenBrushFill = (pGdipGetPenBrushFill) GetProcAddress(m_hGdiplusLib, "GdipGetPenBrushFill");
	if(!pfnGdipGetPenBrushFill)
	{
		return GenericError;
	}
	return pfnGdipGetPenBrushFill(pen,brush);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenFillType(GpPen* pen, GpPenType* type)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenFillType pfnGdipGetPenFillType = (pGdipGetPenFillType) GetProcAddress(m_hGdiplusLib, "GdipGetPenFillType");
	if(!pfnGdipGetPenFillType)
	{
		return GenericError;
	}
	return pfnGdipGetPenFillType(pen,type);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenDashStyle(GpPen* pen, GpDashStyle* dashstyle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenDashStyle pfnGdipGetPenDashStyle = (pGdipGetPenDashStyle) GetProcAddress(m_hGdiplusLib, "GdipGetPenDashStyle");
	if(!pfnGdipGetPenDashStyle)
	{
		return GenericError;
	}
	return pfnGdipGetPenDashStyle(pen,dashstyle);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenDashStyle(GpPen* pen, GpDashStyle dashstyle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenDashStyle pfnGdipSetPenDashStyle = (pGdipSetPenDashStyle) GetProcAddress(m_hGdiplusLib, "GdipSetPenDashStyle");
	if(!pfnGdipSetPenDashStyle)
	{
		return GenericError;
	}
	return pfnGdipSetPenDashStyle(pen,dashstyle);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenDashOffset(GpPen* pen, REAL* offset)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenDashOffset pfnGdipGetPenDashOffset = (pGdipGetPenDashOffset) GetProcAddress(m_hGdiplusLib, "GdipGetPenDashOffset");
	if(!pfnGdipGetPenDashOffset)
	{
		return GenericError;
	}
	return pfnGdipGetPenDashOffset(pen,offset);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenDashOffset(GpPen* pen, REAL offset)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenDashOffset pfnGdipSetPenDashOffset = (pGdipSetPenDashOffset) GetProcAddress(m_hGdiplusLib, "GdipSetPenDashOffset");
	if(!pfnGdipSetPenDashOffset)
	{
		return GenericError;
	}
	return pfnGdipSetPenDashOffset(pen,offset);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenDashCount(GpPen* pen, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenDashCount pfnGdipGetPenDashCount = (pGdipGetPenDashCount) GetProcAddress(m_hGdiplusLib, "GdipGetPenDashCount");
	if(!pfnGdipGetPenDashCount)
	{
		return GenericError;
	}
	return pfnGdipGetPenDashCount(pen,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenDashArray(GpPen* pen, GDIPCONST REAL* dash, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenDashArray pfnGdipSetPenDashArray = (pGdipSetPenDashArray) GetProcAddress(m_hGdiplusLib, "GdipSetPenDashArray");
	if(!pfnGdipSetPenDashArray)
	{
		return GenericError;
	}
	return pfnGdipSetPenDashArray(pen,dash,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenDashArray(GpPen* pen, REAL* dash, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenDashArray pfnGdipGetPenDashArray = (pGdipGetPenDashArray) GetProcAddress(m_hGdiplusLib, "GdipGetPenDashArray");
	if(!pfnGdipGetPenDashArray)
	{
		return GenericError;
	}
	return pfnGdipGetPenDashArray(pen,dash,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenCompoundCount(GpPen* pen, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenCompoundCount pfnGdipGetPenCompoundCount = (pGdipGetPenCompoundCount) GetProcAddress(m_hGdiplusLib, "GdipGetPenCompoundCount");
	if(!pfnGdipGetPenCompoundCount)
	{
		return GenericError;
	}
	return pfnGdipGetPenCompoundCount(pen,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPenCompoundArray(GpPen* pen, GDIPCONST REAL* dash, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPenCompoundArray pfnGdipSetPenCompoundArray = (pGdipSetPenCompoundArray) GetProcAddress(m_hGdiplusLib, "GdipSetPenCompoundArray");
	if(!pfnGdipSetPenCompoundArray)
	{
		return GenericError;
	}
	return pfnGdipSetPenCompoundArray(pen,dash,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPenCompoundArray(GpPen* pen, REAL* dash, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPenCompoundArray pfnGdipGetPenCompoundArray = (pGdipGetPenCompoundArray) GetProcAddress(m_hGdiplusLib, "GdipGetPenCompoundArray");
	if(!pfnGdipGetPenCompoundArray)
	{
		return GenericError;
	}
	return pfnGdipGetPenCompoundArray(pen,dash,count);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateCustomLineCap(GpPath* fillPath, GpPath* strokePath, GpLineCap baseCap, REAL baseInset, GpCustomLineCap** customCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateCustomLineCap pfnGdipCreateCustomLineCap = (pGdipCreateCustomLineCap) GetProcAddress(m_hGdiplusLib, "GdipCreateCustomLineCap");
	if(!pfnGdipCreateCustomLineCap)
	{
		return GenericError;
	}
	return pfnGdipCreateCustomLineCap(fillPath,strokePath,baseCap,baseInset,customCap);
}

GpStatus WINAPI GdiplusWrapper::GdipDeleteCustomLineCap(GpCustomLineCap* customCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeleteCustomLineCap pfnGdipDeleteCustomLineCap = (pGdipDeleteCustomLineCap) GetProcAddress(m_hGdiplusLib, "GdipDeleteCustomLineCap");
	if(!pfnGdipDeleteCustomLineCap)
	{
		return GenericError;
	}
	return pfnGdipDeleteCustomLineCap(customCap);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneCustomLineCap(GpCustomLineCap* customCap, GpCustomLineCap** clonedCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneCustomLineCap pfnGdipCloneCustomLineCap = (pGdipCloneCustomLineCap) GetProcAddress(m_hGdiplusLib, "GdipCloneCustomLineCap");
	if(!pfnGdipCloneCustomLineCap)
	{
		return GenericError;
	}
	return pfnGdipCloneCustomLineCap(customCap,clonedCap);
}

GpStatus WINAPI GdiplusWrapper::GdipGetCustomLineCapType(GpCustomLineCap* customCap, CustomLineCapType* capType)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetCustomLineCapType pfnGdipGetCustomLineCapType = (pGdipGetCustomLineCapType) GetProcAddress(m_hGdiplusLib, "GdipGetCustomLineCapType");
	if(!pfnGdipGetCustomLineCapType)
	{
		return GenericError;
	}
	return pfnGdipGetCustomLineCapType(customCap,capType);
}

GpStatus WINAPI GdiplusWrapper::GdipSetCustomLineCapStrokeCaps(GpCustomLineCap* customCap, GpLineCap startCap, GpLineCap endCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetCustomLineCapStrokeCaps pfnGdipSetCustomLineCapStrokeCaps = (pGdipSetCustomLineCapStrokeCaps) GetProcAddress(m_hGdiplusLib, "GdipSetCustomLineCapStrokeCaps");
	if(!pfnGdipSetCustomLineCapStrokeCaps)
	{
		return GenericError;
	}
	return pfnGdipSetCustomLineCapStrokeCaps(customCap,startCap,endCap);
}

GpStatus WINAPI GdiplusWrapper::GdipGetCustomLineCapStrokeCaps(GpCustomLineCap* customCap, GpLineCap* startCap, GpLineCap* endCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetCustomLineCapStrokeCaps pfnGdipGetCustomLineCapStrokeCaps = (pGdipGetCustomLineCapStrokeCaps) GetProcAddress(m_hGdiplusLib, "GdipGetCustomLineCapStrokeCaps");
	if(!pfnGdipGetCustomLineCapStrokeCaps)
	{
		return GenericError;
	}
	return pfnGdipGetCustomLineCapStrokeCaps(customCap,startCap,endCap);
}

GpStatus WINAPI GdiplusWrapper::GdipSetCustomLineCapStrokeJoin(GpCustomLineCap* customCap, GpLineJoin lineJoin)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetCustomLineCapStrokeJoin pfnGdipSetCustomLineCapStrokeJoin = (pGdipSetCustomLineCapStrokeJoin) GetProcAddress(m_hGdiplusLib, "GdipSetCustomLineCapStrokeJoin");
	if(!pfnGdipSetCustomLineCapStrokeJoin)
	{
		return GenericError;
	}
	return pfnGdipSetCustomLineCapStrokeJoin(customCap,lineJoin);
}

GpStatus WINAPI GdiplusWrapper::GdipGetCustomLineCapStrokeJoin(GpCustomLineCap* customCap, GpLineJoin* lineJoin)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetCustomLineCapStrokeJoin pfnGdipGetCustomLineCapStrokeJoin = (pGdipGetCustomLineCapStrokeJoin) GetProcAddress(m_hGdiplusLib, "GdipGetCustomLineCapStrokeJoin");
	if(!pfnGdipGetCustomLineCapStrokeJoin)
	{
		return GenericError;
	}
	return pfnGdipGetCustomLineCapStrokeJoin(customCap,lineJoin);
}

GpStatus WINAPI GdiplusWrapper::GdipSetCustomLineCapBaseCap(GpCustomLineCap* customCap, GpLineCap baseCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetCustomLineCapBaseCap pfnGdipSetCustomLineCapBaseCap = (pGdipSetCustomLineCapBaseCap) GetProcAddress(m_hGdiplusLib, "GdipSetCustomLineCapBaseCap");
	if(!pfnGdipSetCustomLineCapBaseCap)
	{
		return GenericError;
	}
	return pfnGdipSetCustomLineCapBaseCap(customCap,baseCap);
}

GpStatus WINAPI GdiplusWrapper::GdipGetCustomLineCapBaseCap(GpCustomLineCap* customCap, GpLineCap* baseCap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetCustomLineCapBaseCap pfnGdipGetCustomLineCapBaseCap = (pGdipGetCustomLineCapBaseCap) GetProcAddress(m_hGdiplusLib, "GdipGetCustomLineCapBaseCap");
	if(!pfnGdipGetCustomLineCapBaseCap)
	{
		return GenericError;
	}
	return pfnGdipGetCustomLineCapBaseCap(customCap,baseCap);
}

GpStatus WINAPI GdiplusWrapper::GdipSetCustomLineCapBaseInset(GpCustomLineCap* customCap, REAL inset)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetCustomLineCapBaseInset pfnGdipSetCustomLineCapBaseInset = (pGdipSetCustomLineCapBaseInset) GetProcAddress(m_hGdiplusLib, "GdipSetCustomLineCapBaseInset");
	if(!pfnGdipSetCustomLineCapBaseInset)
	{
		return GenericError;
	}
	return pfnGdipSetCustomLineCapBaseInset(customCap,inset);
}

GpStatus WINAPI GdiplusWrapper::GdipGetCustomLineCapBaseInset(GpCustomLineCap* customCap, REAL* inset)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetCustomLineCapBaseInset pfnGdipGetCustomLineCapBaseInset = (pGdipGetCustomLineCapBaseInset) GetProcAddress(m_hGdiplusLib, "GdipGetCustomLineCapBaseInset");
	if(!pfnGdipGetCustomLineCapBaseInset)
	{
		return GenericError;
	}
	return pfnGdipGetCustomLineCapBaseInset(customCap,inset);
}

GpStatus WINAPI GdiplusWrapper::GdipSetCustomLineCapWidthScale(GpCustomLineCap* customCap, REAL widthScale)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetCustomLineCapWidthScale pfnGdipSetCustomLineCapWidthScale = (pGdipSetCustomLineCapWidthScale) GetProcAddress(m_hGdiplusLib, "GdipSetCustomLineCapWidthScale");
	if(!pfnGdipSetCustomLineCapWidthScale)
	{
		return GenericError;
	}
	return pfnGdipSetCustomLineCapWidthScale(customCap,widthScale);
}

GpStatus WINAPI GdiplusWrapper::GdipGetCustomLineCapWidthScale(GpCustomLineCap* customCap, REAL* widthScale)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetCustomLineCapWidthScale pfnGdipGetCustomLineCapWidthScale = (pGdipGetCustomLineCapWidthScale) GetProcAddress(m_hGdiplusLib, "GdipGetCustomLineCapWidthScale");
	if(!pfnGdipGetCustomLineCapWidthScale)
	{
		return GenericError;
	}
	return pfnGdipGetCustomLineCapWidthScale(customCap,widthScale);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateAdjustableArrowCap(REAL height, REAL width, BOOL isFilled, GpAdjustableArrowCap** cap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateAdjustableArrowCap pfnGdipCreateAdjustableArrowCap = (pGdipCreateAdjustableArrowCap) GetProcAddress(m_hGdiplusLib, "GdipCreateAdjustableArrowCap");
	if(!pfnGdipCreateAdjustableArrowCap)
	{
		return GenericError;
	}
	return pfnGdipCreateAdjustableArrowCap(height,width,isFilled,cap);
}

GpStatus WINAPI GdiplusWrapper::GdipSetAdjustableArrowCapHeight(GpAdjustableArrowCap* cap, REAL height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetAdjustableArrowCapHeight pfnGdipSetAdjustableArrowCapHeight = (pGdipSetAdjustableArrowCapHeight) GetProcAddress(m_hGdiplusLib, "GdipSetAdjustableArrowCapHeight");
	if(!pfnGdipSetAdjustableArrowCapHeight)
	{
		return GenericError;
	}
	return pfnGdipSetAdjustableArrowCapHeight(cap,height);
}

GpStatus WINAPI GdiplusWrapper::GdipGetAdjustableArrowCapHeight(GpAdjustableArrowCap* cap, REAL* height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetAdjustableArrowCapHeight pfnGdipGetAdjustableArrowCapHeight = (pGdipGetAdjustableArrowCapHeight) GetProcAddress(m_hGdiplusLib, "GdipGetAdjustableArrowCapHeight");
	if(!pfnGdipGetAdjustableArrowCapHeight)
	{
		return GenericError;
	}
	return pfnGdipGetAdjustableArrowCapHeight(cap,height);
}

GpStatus WINAPI GdiplusWrapper::GdipSetAdjustableArrowCapWidth(GpAdjustableArrowCap* cap, REAL width)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetAdjustableArrowCapWidth pfnGdipSetAdjustableArrowCapWidth = (pGdipSetAdjustableArrowCapWidth) GetProcAddress(m_hGdiplusLib, "GdipSetAdjustableArrowCapWidth");
	if(!pfnGdipSetAdjustableArrowCapWidth)
	{
		return GenericError;
	}
	return pfnGdipSetAdjustableArrowCapWidth(cap,width);
}

GpStatus WINAPI GdiplusWrapper::GdipGetAdjustableArrowCapWidth(GpAdjustableArrowCap* cap, REAL* width)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetAdjustableArrowCapWidth pfnGdipGetAdjustableArrowCapWidth = (pGdipGetAdjustableArrowCapWidth) GetProcAddress(m_hGdiplusLib, "GdipGetAdjustableArrowCapWidth");
	if(!pfnGdipGetAdjustableArrowCapWidth)
	{
		return GenericError;
	}
	return pfnGdipGetAdjustableArrowCapWidth(cap,width);
}

GpStatus WINAPI GdiplusWrapper::GdipSetAdjustableArrowCapMiddleInset(GpAdjustableArrowCap* cap, REAL middleInset)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetAdjustableArrowCapMiddleInset pfnGdipSetAdjustableArrowCapMiddleInset = (pGdipSetAdjustableArrowCapMiddleInset) GetProcAddress(m_hGdiplusLib, "GdipSetAdjustableArrowCapMiddleInset");
	if(!pfnGdipSetAdjustableArrowCapMiddleInset)
	{
		return GenericError;
	}
	return pfnGdipSetAdjustableArrowCapMiddleInset(cap,middleInset);
}

GpStatus WINAPI GdiplusWrapper::GdipGetAdjustableArrowCapMiddleInset(GpAdjustableArrowCap* cap, REAL* middleInset)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetAdjustableArrowCapMiddleInset pfnGdipGetAdjustableArrowCapMiddleInset = (pGdipGetAdjustableArrowCapMiddleInset) GetProcAddress(m_hGdiplusLib, "GdipGetAdjustableArrowCapMiddleInset");
	if(!pfnGdipGetAdjustableArrowCapMiddleInset)
	{
		return GenericError;
	}
	return pfnGdipGetAdjustableArrowCapMiddleInset(cap,middleInset);
}

GpStatus WINAPI GdiplusWrapper::GdipSetAdjustableArrowCapFillState(GpAdjustableArrowCap* cap, BOOL fillState)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetAdjustableArrowCapFillState pfnGdipSetAdjustableArrowCapFillState = (pGdipSetAdjustableArrowCapFillState) GetProcAddress(m_hGdiplusLib, "GdipSetAdjustableArrowCapFillState");
	if(!pfnGdipSetAdjustableArrowCapFillState)
	{
		return GenericError;
	}
	return pfnGdipSetAdjustableArrowCapFillState(cap,fillState);
}

GpStatus WINAPI GdiplusWrapper::GdipGetAdjustableArrowCapFillState(GpAdjustableArrowCap* cap, BOOL* fillState)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetAdjustableArrowCapFillState pfnGdipGetAdjustableArrowCapFillState = (pGdipGetAdjustableArrowCapFillState) GetProcAddress(m_hGdiplusLib, "GdipGetAdjustableArrowCapFillState");
	if(!pfnGdipGetAdjustableArrowCapFillState)
	{
		return GenericError;
	}
	return pfnGdipGetAdjustableArrowCapFillState(cap,fillState);
}

GpStatus WINAPI GdiplusWrapper::GdipLoadImageFromStream(IStream* stream, GpImage** image)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipLoadImageFromStream pfnGdipLoadImageFromStream = (pGdipLoadImageFromStream) GetProcAddress(m_hGdiplusLib, "GdipLoadImageFromStream");
	if(!pfnGdipLoadImageFromStream)
	{
		return GenericError;
	}
	return pfnGdipLoadImageFromStream(stream,image);
}

GpStatus WINAPI GdiplusWrapper::GdipLoadImageFromFile(GDIPCONST WCHAR* filename, GpImage** image)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipLoadImageFromFile pfnGdipLoadImageFromFile = (pGdipLoadImageFromFile) GetProcAddress(m_hGdiplusLib, "GdipLoadImageFromFile");
	if(!pfnGdipLoadImageFromFile)
	{
		return GenericError;
	}
	return pfnGdipLoadImageFromFile(filename,image);
}

GpStatus WINAPI GdiplusWrapper::GdipLoadImageFromStreamICM(IStream* stream, GpImage** image)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipLoadImageFromStreamICM pfnGdipLoadImageFromStreamICM = (pGdipLoadImageFromStreamICM) GetProcAddress(m_hGdiplusLib, "GdipLoadImageFromStreamICM");
	if(!pfnGdipLoadImageFromStreamICM)
	{
		return GenericError;
	}
	return pfnGdipLoadImageFromStreamICM(stream,image);
}

GpStatus WINAPI GdiplusWrapper::GdipLoadImageFromFileICM(GDIPCONST WCHAR* filename, GpImage** image)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipLoadImageFromFileICM pfnGdipLoadImageFromFileICM = (pGdipLoadImageFromFileICM) GetProcAddress(m_hGdiplusLib, "GdipLoadImageFromFileICM");
	if(!pfnGdipLoadImageFromFileICM)
	{
		return GenericError;
	}
	return pfnGdipLoadImageFromFileICM(filename,image);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneImage(GpImage* image, GpImage** cloneImage)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneImage pfnGdipCloneImage = (pGdipCloneImage) GetProcAddress(m_hGdiplusLib, "GdipCloneImage");
	if(!pfnGdipCloneImage)
	{
		return GenericError;
	}
	return pfnGdipCloneImage(image,cloneImage);
}

GpStatus WINAPI GdiplusWrapper::GdipDisposeImage(GpImage* image)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDisposeImage pfnGdipDisposeImage = (pGdipDisposeImage) GetProcAddress(m_hGdiplusLib, "GdipDisposeImage");
	if(!pfnGdipDisposeImage)
	{
		return GenericError;
	}
	return pfnGdipDisposeImage(image);
}

GpStatus WINAPI GdiplusWrapper::GdipSaveImageToFile(GpImage* image, GDIPCONST WCHAR* filename, GDIPCONST CLSID* clsidEncoder, GDIPCONST EncoderParameters* encoderParams)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSaveImageToFile pfnGdipSaveImageToFile = (pGdipSaveImageToFile) GetProcAddress(m_hGdiplusLib, "GdipSaveImageToFile");
	if(!pfnGdipSaveImageToFile)
	{
		return GenericError;
	}
	return pfnGdipSaveImageToFile(image,filename,clsidEncoder,encoderParams);
}

GpStatus WINAPI GdiplusWrapper::GdipSaveImageToStream(GpImage* image, IStream* stream, GDIPCONST CLSID* clsidEncoder, GDIPCONST EncoderParameters* encoderParams)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSaveImageToStream pfnGdipSaveImageToStream = (pGdipSaveImageToStream) GetProcAddress(m_hGdiplusLib, "GdipSaveImageToStream");
	if(!pfnGdipSaveImageToStream)
	{
		return GenericError;
	}
	return pfnGdipSaveImageToStream(image,stream,clsidEncoder,encoderParams);
}

GpStatus WINAPI GdiplusWrapper::GdipSaveAdd(GpImage* image, GDIPCONST EncoderParameters* encoderParams)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSaveAdd pfnGdipSaveAdd = (pGdipSaveAdd) GetProcAddress(m_hGdiplusLib, "GdipSaveAdd");
	if(!pfnGdipSaveAdd)
	{
		return GenericError;
	}
	return pfnGdipSaveAdd(image,encoderParams);
}

GpStatus WINAPI GdiplusWrapper::GdipSaveAddImage(GpImage* image, GpImage* newImage, GDIPCONST EncoderParameters* encoderParams)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSaveAddImage pfnGdipSaveAddImage = (pGdipSaveAddImage) GetProcAddress(m_hGdiplusLib, "GdipSaveAddImage");
	if(!pfnGdipSaveAddImage)
	{
		return GenericError;
	}
	return pfnGdipSaveAddImage(image,newImage,encoderParams);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageGraphicsContext(GpImage* image, GpGraphics** graphics)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageGraphicsContext pfnGdipGetImageGraphicsContext = (pGdipGetImageGraphicsContext) GetProcAddress(m_hGdiplusLib, "GdipGetImageGraphicsContext");
	if(!pfnGdipGetImageGraphicsContext)
	{
		return GenericError;
	}
	return pfnGdipGetImageGraphicsContext(image,graphics);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageBounds(GpImage* image, GpRectF* srcRect, GpUnit* srcUnit)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageBounds pfnGdipGetImageBounds = (pGdipGetImageBounds) GetProcAddress(m_hGdiplusLib, "GdipGetImageBounds");
	if(!pfnGdipGetImageBounds)
	{
		return GenericError;
	}
	return pfnGdipGetImageBounds(image,srcRect,srcUnit);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageDimension(GpImage* image, REAL* width, REAL* height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageDimension pfnGdipGetImageDimension = (pGdipGetImageDimension) GetProcAddress(m_hGdiplusLib, "GdipGetImageDimension");
	if(!pfnGdipGetImageDimension)
	{
		return GenericError;
	}
	return pfnGdipGetImageDimension(image,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageType(GpImage* image, ImageType* type)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageType pfnGdipGetImageType = (pGdipGetImageType) GetProcAddress(m_hGdiplusLib, "GdipGetImageType");
	if(!pfnGdipGetImageType)
	{
		return GenericError;
	}
	return pfnGdipGetImageType(image,type);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageWidth(GpImage* image, UINT* width)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageWidth pfnGdipGetImageWidth = (pGdipGetImageWidth) GetProcAddress(m_hGdiplusLib, "GdipGetImageWidth");
	if(!pfnGdipGetImageWidth)
	{
		return GenericError;
	}
	return pfnGdipGetImageWidth(image,width);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageHeight(GpImage* image, UINT* height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageHeight pfnGdipGetImageHeight = (pGdipGetImageHeight) GetProcAddress(m_hGdiplusLib, "GdipGetImageHeight");
	if(!pfnGdipGetImageHeight)
	{
		return GenericError;
	}
	return pfnGdipGetImageHeight(image,height);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageHorizontalResolution(GpImage* image, REAL* resolution)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageHorizontalResolution pfnGdipGetImageHorizontalResolution = (pGdipGetImageHorizontalResolution) GetProcAddress(m_hGdiplusLib, "GdipGetImageHorizontalResolution");
	if(!pfnGdipGetImageHorizontalResolution)
	{
		return GenericError;
	}
	return pfnGdipGetImageHorizontalResolution(image,resolution);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageVerticalResolution(GpImage* image, REAL* resolution)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageVerticalResolution pfnGdipGetImageVerticalResolution = (pGdipGetImageVerticalResolution) GetProcAddress(m_hGdiplusLib, "GdipGetImageVerticalResolution");
	if(!pfnGdipGetImageVerticalResolution)
	{
		return GenericError;
	}
	return pfnGdipGetImageVerticalResolution(image,resolution);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageFlags(GpImage* image, UINT* flags)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageFlags pfnGdipGetImageFlags = (pGdipGetImageFlags) GetProcAddress(m_hGdiplusLib, "GdipGetImageFlags");
	if(!pfnGdipGetImageFlags)
	{
		return GenericError;
	}
	return pfnGdipGetImageFlags(image,flags);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageRawFormat(GpImage* image, GUID* format)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageRawFormat pfnGdipGetImageRawFormat = (pGdipGetImageRawFormat) GetProcAddress(m_hGdiplusLib, "GdipGetImageRawFormat");
	if(!pfnGdipGetImageRawFormat)
	{
		return GenericError;
	}
	return pfnGdipGetImageRawFormat(image,format);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImagePixelFormat(GpImage* image, PixelFormat* format)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImagePixelFormat pfnGdipGetImagePixelFormat = (pGdipGetImagePixelFormat) GetProcAddress(m_hGdiplusLib, "GdipGetImagePixelFormat");
	if(!pfnGdipGetImagePixelFormat)
	{
		return GenericError;
	}
	return pfnGdipGetImagePixelFormat(image,format);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageThumbnail(GpImage* image, UINT thumbWidth, UINT thumbHeight, GpImage** thumbImage, GetThumbnailImageAbort callback, VOID* callbackData)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageThumbnail pfnGdipGetImageThumbnail = (pGdipGetImageThumbnail) GetProcAddress(m_hGdiplusLib, "GdipGetImageThumbnail");
	if(!pfnGdipGetImageThumbnail)
	{
		return GenericError;
	}
	return pfnGdipGetImageThumbnail(image,thumbWidth,thumbHeight,thumbImage,callback,callbackData);
}

GpStatus WINAPI GdiplusWrapper::GdipGetEncoderParameterListSize(GpImage* image, GDIPCONST CLSID* clsidEncoder, UINT* size)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetEncoderParameterListSize pfnGdipGetEncoderParameterListSize = (pGdipGetEncoderParameterListSize) GetProcAddress(m_hGdiplusLib, "GdipGetEncoderParameterListSize");
	if(!pfnGdipGetEncoderParameterListSize)
	{
		return GenericError;
	}
	return pfnGdipGetEncoderParameterListSize(image,clsidEncoder,size);
}

GpStatus WINAPI GdiplusWrapper::GdipGetEncoderParameterList(GpImage* image, GDIPCONST CLSID* clsidEncoder, UINT size, EncoderParameters* buffer)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetEncoderParameterList pfnGdipGetEncoderParameterList = (pGdipGetEncoderParameterList) GetProcAddress(m_hGdiplusLib, "GdipGetEncoderParameterList");
	if(!pfnGdipGetEncoderParameterList)
	{
		return GenericError;
	}
	return pfnGdipGetEncoderParameterList(image,clsidEncoder,size,buffer);
}

GpStatus WINAPI GdiplusWrapper::GdipImageGetFrameDimensionsCount(GpImage* image, UINT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipImageGetFrameDimensionsCount pfnGdipImageGetFrameDimensionsCount = (pGdipImageGetFrameDimensionsCount) GetProcAddress(m_hGdiplusLib, "GdipImageGetFrameDimensionsCount");
	if(!pfnGdipImageGetFrameDimensionsCount)
	{
		return GenericError;
	}
	return pfnGdipImageGetFrameDimensionsCount(image,count);
}

GpStatus WINAPI GdiplusWrapper::GdipImageGetFrameDimensionsList(GpImage* image, GUID* dimensionIDs, UINT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipImageGetFrameDimensionsList pfnGdipImageGetFrameDimensionsList = (pGdipImageGetFrameDimensionsList) GetProcAddress(m_hGdiplusLib, "GdipImageGetFrameDimensionsList");
	if(!pfnGdipImageGetFrameDimensionsList)
	{
		return GenericError;
	}
	return pfnGdipImageGetFrameDimensionsList(image,dimensionIDs,count);
}

GpStatus WINAPI GdiplusWrapper::GdipImageGetFrameCount(GpImage* image, GDIPCONST GUID* dimensionID, UINT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipImageGetFrameCount pfnGdipImageGetFrameCount = (pGdipImageGetFrameCount) GetProcAddress(m_hGdiplusLib, "GdipImageGetFrameCount");
	if(!pfnGdipImageGetFrameCount)
	{
		return GenericError;
	}
	return pfnGdipImageGetFrameCount(image,dimensionID,count);
}

GpStatus WINAPI GdiplusWrapper::GdipImageSelectActiveFrame(GpImage* image, GDIPCONST GUID* dimensionID, UINT frameIndex)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipImageSelectActiveFrame pfnGdipImageSelectActiveFrame = (pGdipImageSelectActiveFrame) GetProcAddress(m_hGdiplusLib, "GdipImageSelectActiveFrame");
	if(!pfnGdipImageSelectActiveFrame)
	{
		return GenericError;
	}
	return pfnGdipImageSelectActiveFrame(image,dimensionID,frameIndex);
}

GpStatus WINAPI GdiplusWrapper::GdipImageRotateFlip(GpImage* image, RotateFlipType rfType)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipImageRotateFlip pfnGdipImageRotateFlip = (pGdipImageRotateFlip) GetProcAddress(m_hGdiplusLib, "GdipImageRotateFlip");
	if(!pfnGdipImageRotateFlip)
	{
		return GenericError;
	}
	return pfnGdipImageRotateFlip(image,rfType);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImagePalette(GpImage* image, ColorPalette* palette, INT size)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImagePalette pfnGdipGetImagePalette = (pGdipGetImagePalette) GetProcAddress(m_hGdiplusLib, "GdipGetImagePalette");
	if(!pfnGdipGetImagePalette)
	{
		return GenericError;
	}
	return pfnGdipGetImagePalette(image,palette,size);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImagePalette(GpImage* image, GDIPCONST ColorPalette* palette)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImagePalette pfnGdipSetImagePalette = (pGdipSetImagePalette) GetProcAddress(m_hGdiplusLib, "GdipSetImagePalette");
	if(!pfnGdipSetImagePalette)
	{
		return GenericError;
	}
	return pfnGdipSetImagePalette(image,palette);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImagePaletteSize(GpImage* image, INT* size)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImagePaletteSize pfnGdipGetImagePaletteSize = (pGdipGetImagePaletteSize) GetProcAddress(m_hGdiplusLib, "GdipGetImagePaletteSize");
	if(!pfnGdipGetImagePaletteSize)
	{
		return GenericError;
	}
	return pfnGdipGetImagePaletteSize(image,size);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPropertyCount(GpImage* image, UINT* numOfProperty)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPropertyCount pfnGdipGetPropertyCount = (pGdipGetPropertyCount) GetProcAddress(m_hGdiplusLib, "GdipGetPropertyCount");
	if(!pfnGdipGetPropertyCount)
	{
		return GenericError;
	}
	return pfnGdipGetPropertyCount(image,numOfProperty);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPropertyIdList(GpImage* image, UINT numOfProperty, PROPID* list)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPropertyIdList pfnGdipGetPropertyIdList = (pGdipGetPropertyIdList) GetProcAddress(m_hGdiplusLib, "GdipGetPropertyIdList");
	if(!pfnGdipGetPropertyIdList)
	{
		return GenericError;
	}
	return pfnGdipGetPropertyIdList(image,numOfProperty,list);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPropertyItemSize(GpImage* image, PROPID propId, UINT* size)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPropertyItemSize pfnGdipGetPropertyItemSize = (pGdipGetPropertyItemSize) GetProcAddress(m_hGdiplusLib, "GdipGetPropertyItemSize");
	if(!pfnGdipGetPropertyItemSize)
	{
		return GenericError;
	}
	return pfnGdipGetPropertyItemSize(image,propId,size);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPropertyItem(GpImage* image, PROPID propId, UINT propSize, PropertyItem* buffer)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPropertyItem pfnGdipGetPropertyItem = (pGdipGetPropertyItem) GetProcAddress(m_hGdiplusLib, "GdipGetPropertyItem");
	if(!pfnGdipGetPropertyItem)
	{
		return GenericError;
	}
	return pfnGdipGetPropertyItem(image,propId,propSize,buffer);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPropertySize(GpImage* image, UINT* totalBufferSize, UINT* numProperties)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPropertySize pfnGdipGetPropertySize = (pGdipGetPropertySize) GetProcAddress(m_hGdiplusLib, "GdipGetPropertySize");
	if(!pfnGdipGetPropertySize)
	{
		return GenericError;
	}
	return pfnGdipGetPropertySize(image,totalBufferSize,numProperties);
}

GpStatus WINAPI GdiplusWrapper::GdipGetAllPropertyItems(GpImage* image, UINT totalBufferSize, UINT numProperties, PropertyItem* allItems)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetAllPropertyItems pfnGdipGetAllPropertyItems = (pGdipGetAllPropertyItems) GetProcAddress(m_hGdiplusLib, "GdipGetAllPropertyItems");
	if(!pfnGdipGetAllPropertyItems)
	{
		return GenericError;
	}
	return pfnGdipGetAllPropertyItems(image,totalBufferSize,numProperties,allItems);
}

GpStatus WINAPI GdiplusWrapper::GdipRemovePropertyItem(GpImage* image, PROPID propId)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRemovePropertyItem pfnGdipRemovePropertyItem = (pGdipRemovePropertyItem) GetProcAddress(m_hGdiplusLib, "GdipRemovePropertyItem");
	if(!pfnGdipRemovePropertyItem)
	{
		return GenericError;
	}
	return pfnGdipRemovePropertyItem(image,propId);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPropertyItem(GpImage* image, GDIPCONST PropertyItem* item)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPropertyItem pfnGdipSetPropertyItem = (pGdipSetPropertyItem) GetProcAddress(m_hGdiplusLib, "GdipSetPropertyItem");
	if(!pfnGdipSetPropertyItem)
	{
		return GenericError;
	}
	return pfnGdipSetPropertyItem(image,item);
}

GpStatus WINAPI GdiplusWrapper::GdipImageForceValidation(GpImage* image)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipImageForceValidation pfnGdipImageForceValidation = (pGdipImageForceValidation) GetProcAddress(m_hGdiplusLib, "GdipImageForceValidation");
	if(!pfnGdipImageForceValidation)
	{
		return GenericError;
	}
	return pfnGdipImageForceValidation(image);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromStream(IStream* stream, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromStream pfnGdipCreateBitmapFromStream = (pGdipCreateBitmapFromStream) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromStream");
	if(!pfnGdipCreateBitmapFromStream)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromStream(stream,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromFile(GDIPCONST WCHAR* filename, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromFile pfnGdipCreateBitmapFromFile = (pGdipCreateBitmapFromFile) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromFile");
	if(!pfnGdipCreateBitmapFromFile)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromFile(filename,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromStreamICM(IStream* stream, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromStreamICM pfnGdipCreateBitmapFromStreamICM = (pGdipCreateBitmapFromStreamICM) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromStreamICM");
	if(!pfnGdipCreateBitmapFromStreamICM)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromStreamICM(stream,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromFileICM(GDIPCONST WCHAR* filename, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromFileICM pfnGdipCreateBitmapFromFileICM = (pGdipCreateBitmapFromFileICM) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromFileICM");
	if(!pfnGdipCreateBitmapFromFileICM)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromFileICM(filename,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromScan0(INT width, INT height, INT stride, PixelFormat format, BYTE* scan0, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromScan0 pfnGdipCreateBitmapFromScan0 = (pGdipCreateBitmapFromScan0) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromScan0");
	if(!pfnGdipCreateBitmapFromScan0)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromScan0(width,height,stride,format,scan0,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromGraphics(INT width, INT height, GpGraphics* target, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromGraphics pfnGdipCreateBitmapFromGraphics = (pGdipCreateBitmapFromGraphics) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromGraphics");
	if(!pfnGdipCreateBitmapFromGraphics)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromGraphics(width,height,target,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromDirectDrawSurface(IDirectDrawSurface7* surface, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromDirectDrawSurface pfnGdipCreateBitmapFromDirectDrawSurface = (pGdipCreateBitmapFromDirectDrawSurface) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromDirectDrawSurface");
	if(!pfnGdipCreateBitmapFromDirectDrawSurface)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromDirectDrawSurface(surface,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromGdiDib(GDIPCONST BITMAPINFO* gdiBitmapInfo, VOID* gdiBitmapData, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromGdiDib pfnGdipCreateBitmapFromGdiDib = (pGdipCreateBitmapFromGdiDib) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromGdiDib");
	if(!pfnGdipCreateBitmapFromGdiDib)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromGdiDib(gdiBitmapInfo,gdiBitmapData,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromHBITMAP(HBITMAP hbm, HPALETTE hpal, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromHBITMAP pfnGdipCreateBitmapFromHBITMAP = (pGdipCreateBitmapFromHBITMAP) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromHBITMAP");
	if(!pfnGdipCreateBitmapFromHBITMAP)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromHBITMAP(hbm,hpal,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateHBITMAPFromBitmap(GpBitmap* bitmap, HBITMAP* hbmReturn, ARGB background)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateHBITMAPFromBitmap pfnGdipCreateHBITMAPFromBitmap = (pGdipCreateHBITMAPFromBitmap) GetProcAddress(m_hGdiplusLib, "GdipCreateHBITMAPFromBitmap");
	if(!pfnGdipCreateHBITMAPFromBitmap)
	{
		return GenericError;
	}
	return pfnGdipCreateHBITMAPFromBitmap(bitmap,hbmReturn,background);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromHICON(HICON hicon, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromHICON pfnGdipCreateBitmapFromHICON = (pGdipCreateBitmapFromHICON) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromHICON");
	if(!pfnGdipCreateBitmapFromHICON)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromHICON(hicon,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateHICONFromBitmap(GpBitmap* bitmap, HICON* hbmReturn)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateHICONFromBitmap pfnGdipCreateHICONFromBitmap = (pGdipCreateHICONFromBitmap) GetProcAddress(m_hGdiplusLib, "GdipCreateHICONFromBitmap");
	if(!pfnGdipCreateHICONFromBitmap)
	{
		return GenericError;
	}
	return pfnGdipCreateHICONFromBitmap(bitmap,hbmReturn);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateBitmapFromResource(HINSTANCE hInstance, GDIPCONST WCHAR* lpBitmapName, GpBitmap** bitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateBitmapFromResource pfnGdipCreateBitmapFromResource = (pGdipCreateBitmapFromResource) GetProcAddress(m_hGdiplusLib, "GdipCreateBitmapFromResource");
	if(!pfnGdipCreateBitmapFromResource)
	{
		return GenericError;
	}
	return pfnGdipCreateBitmapFromResource(hInstance,lpBitmapName,bitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneBitmapArea(REAL x, REAL y, REAL width, REAL height, PixelFormat format, GpBitmap* srcBitmap, GpBitmap** dstBitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneBitmapArea pfnGdipCloneBitmapArea = (pGdipCloneBitmapArea) GetProcAddress(m_hGdiplusLib, "GdipCloneBitmapArea");
	if(!pfnGdipCloneBitmapArea)
	{
		return GenericError;
	}
	return pfnGdipCloneBitmapArea(x,y,width,height,format,srcBitmap,dstBitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneBitmapAreaI(INT x, INT y, INT width, INT height, PixelFormat format, GpBitmap* srcBitmap, GpBitmap** dstBitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneBitmapAreaI pfnGdipCloneBitmapAreaI = (pGdipCloneBitmapAreaI) GetProcAddress(m_hGdiplusLib, "GdipCloneBitmapAreaI");
	if(!pfnGdipCloneBitmapAreaI)
	{
		return GenericError;
	}
	return pfnGdipCloneBitmapAreaI(x,y,width,height,format,srcBitmap,dstBitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipBitmapLockBits(GpBitmap* bitmap, GDIPCONST GpRect* rect, UINT flags, PixelFormat format, BitmapData* lockedBitmapData)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipBitmapLockBits pfnGdipBitmapLockBits = (pGdipBitmapLockBits) GetProcAddress(m_hGdiplusLib, "GdipBitmapLockBits");
	if(!pfnGdipBitmapLockBits)
	{
		return GenericError;
	}
	return pfnGdipBitmapLockBits(bitmap,rect,flags,format,lockedBitmapData);
}

GpStatus WINAPI GdiplusWrapper::GdipBitmapUnlockBits(GpBitmap* bitmap, BitmapData* lockedBitmapData)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipBitmapUnlockBits pfnGdipBitmapUnlockBits = (pGdipBitmapUnlockBits) GetProcAddress(m_hGdiplusLib, "GdipBitmapUnlockBits");
	if(!pfnGdipBitmapUnlockBits)
	{
		return GenericError;
	}
	return pfnGdipBitmapUnlockBits(bitmap,lockedBitmapData);
}

GpStatus WINAPI GdiplusWrapper::GdipBitmapGetPixel(GpBitmap* bitmap, INT x, INT y, ARGB* color)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipBitmapGetPixel pfnGdipBitmapGetPixel = (pGdipBitmapGetPixel) GetProcAddress(m_hGdiplusLib, "GdipBitmapGetPixel");
	if(!pfnGdipBitmapGetPixel)
	{
		return GenericError;
	}
	return pfnGdipBitmapGetPixel(bitmap,x,y,color);
}

GpStatus WINAPI GdiplusWrapper::GdipBitmapSetPixel(GpBitmap* bitmap, INT x, INT y, ARGB color)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipBitmapSetPixel pfnGdipBitmapSetPixel = (pGdipBitmapSetPixel) GetProcAddress(m_hGdiplusLib, "GdipBitmapSetPixel");
	if(!pfnGdipBitmapSetPixel)
	{
		return GenericError;
	}
	return pfnGdipBitmapSetPixel(bitmap,x,y,color);
}

GpStatus WINAPI GdiplusWrapper::GdipBitmapSetResolution(GpBitmap* bitmap, REAL xdpi, REAL ydpi)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipBitmapSetResolution pfnGdipBitmapSetResolution = (pGdipBitmapSetResolution) GetProcAddress(m_hGdiplusLib, "GdipBitmapSetResolution");
	if(!pfnGdipBitmapSetResolution)
	{
		return GenericError;
	}
	return pfnGdipBitmapSetResolution(bitmap,xdpi,ydpi);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateImageAttributes(GpImageAttributes** imageattr)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateImageAttributes pfnGdipCreateImageAttributes = (pGdipCreateImageAttributes) GetProcAddress(m_hGdiplusLib, "GdipCreateImageAttributes");
	if(!pfnGdipCreateImageAttributes)
	{
		return GenericError;
	}
	return pfnGdipCreateImageAttributes(imageattr);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneImageAttributes(GDIPCONST GpImageAttributes* imageattr, GpImageAttributes** cloneImageattr)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneImageAttributes pfnGdipCloneImageAttributes = (pGdipCloneImageAttributes) GetProcAddress(m_hGdiplusLib, "GdipCloneImageAttributes");
	if(!pfnGdipCloneImageAttributes)
	{
		return GenericError;
	}
	return pfnGdipCloneImageAttributes(imageattr,cloneImageattr);
}

GpStatus WINAPI GdiplusWrapper::GdipDisposeImageAttributes(GpImageAttributes* imageattr)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDisposeImageAttributes pfnGdipDisposeImageAttributes = (pGdipDisposeImageAttributes) GetProcAddress(m_hGdiplusLib, "GdipDisposeImageAttributes");
	if(!pfnGdipDisposeImageAttributes)
	{
		return GenericError;
	}
	return pfnGdipDisposeImageAttributes(imageattr);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesToIdentity(GpImageAttributes* imageattr, ColorAdjustType type)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesToIdentity pfnGdipSetImageAttributesToIdentity = (pGdipSetImageAttributesToIdentity) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesToIdentity");
	if(!pfnGdipSetImageAttributesToIdentity)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesToIdentity(imageattr,type);
}

GpStatus WINAPI GdiplusWrapper::GdipResetImageAttributes(GpImageAttributes* imageattr, ColorAdjustType type)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipResetImageAttributes pfnGdipResetImageAttributes = (pGdipResetImageAttributes) GetProcAddress(m_hGdiplusLib, "GdipResetImageAttributes");
	if(!pfnGdipResetImageAttributes)
	{
		return GenericError;
	}
	return pfnGdipResetImageAttributes(imageattr,type);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesColorMatrix(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, GDIPCONST ColorMatrix* colorMatrix, GDIPCONST ColorMatrix* grayMatrix, ColorMatrixFlags flags)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesColorMatrix pfnGdipSetImageAttributesColorMatrix = (pGdipSetImageAttributesColorMatrix) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesColorMatrix");
	if(!pfnGdipSetImageAttributesColorMatrix)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesColorMatrix(imageattr,type,enableFlag,colorMatrix,grayMatrix,flags);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesThreshold(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, REAL threshold)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesThreshold pfnGdipSetImageAttributesThreshold = (pGdipSetImageAttributesThreshold) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesThreshold");
	if(!pfnGdipSetImageAttributesThreshold)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesThreshold(imageattr,type,enableFlag,threshold);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesGamma(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, REAL gamma)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesGamma pfnGdipSetImageAttributesGamma = (pGdipSetImageAttributesGamma) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesGamma");
	if(!pfnGdipSetImageAttributesGamma)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesGamma(imageattr,type,enableFlag,gamma);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesNoOp(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesNoOp pfnGdipSetImageAttributesNoOp = (pGdipSetImageAttributesNoOp) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesNoOp");
	if(!pfnGdipSetImageAttributesNoOp)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesNoOp(imageattr,type,enableFlag);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesColorKeys(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, ARGB colorLow, ARGB colorHigh)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesColorKeys pfnGdipSetImageAttributesColorKeys = (pGdipSetImageAttributesColorKeys) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesColorKeys");
	if(!pfnGdipSetImageAttributesColorKeys)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesColorKeys(imageattr,type,enableFlag,colorLow,colorHigh);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesOutputChannel(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, ColorChannelFlags channelFlags)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesOutputChannel pfnGdipSetImageAttributesOutputChannel = (pGdipSetImageAttributesOutputChannel) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesOutputChannel");
	if(!pfnGdipSetImageAttributesOutputChannel)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesOutputChannel(imageattr,type,enableFlag,channelFlags);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesOutputChannelColorProfile(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, GDIPCONST WCHAR* colorProfileFilename)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesOutputChannelColorProfile pfnGdipSetImageAttributesOutputChannelColorProfile = (pGdipSetImageAttributesOutputChannelColorProfile) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesOutputChannelColorProfile");
	if(!pfnGdipSetImageAttributesOutputChannelColorProfile)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesOutputChannelColorProfile(imageattr,type,enableFlag,colorProfileFilename);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesRemapTable(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, UINT mapSize, GDIPCONST ColorMap* map)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesRemapTable pfnGdipSetImageAttributesRemapTable = (pGdipSetImageAttributesRemapTable) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesRemapTable");
	if(!pfnGdipSetImageAttributesRemapTable)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesRemapTable(imageattr,type,enableFlag,mapSize,map);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesWrapMode(GpImageAttributes* imageAttr, WrapMode wrap, ARGB argb, BOOL clamp)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesWrapMode pfnGdipSetImageAttributesWrapMode = (pGdipSetImageAttributesWrapMode) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesWrapMode");
	if(!pfnGdipSetImageAttributesWrapMode)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesWrapMode(imageAttr,wrap,argb,clamp);
}

GpStatus WINAPI GdiplusWrapper::GdipSetImageAttributesICMMode(GpImageAttributes* imageAttr, BOOL on)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetImageAttributesICMMode pfnGdipSetImageAttributesICMMode = (pGdipSetImageAttributesICMMode) GetProcAddress(m_hGdiplusLib, "GdipSetImageAttributesICMMode");
	if(!pfnGdipSetImageAttributesICMMode)
	{
		return GenericError;
	}
	return pfnGdipSetImageAttributesICMMode(imageAttr,on);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageAttributesAdjustedPalette(GpImageAttributes* imageAttr, ColorPalette* colorPalette, ColorAdjustType colorAdjustType)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageAttributesAdjustedPalette pfnGdipGetImageAttributesAdjustedPalette = (pGdipGetImageAttributesAdjustedPalette) GetProcAddress(m_hGdiplusLib, "GdipGetImageAttributesAdjustedPalette");
	if(!pfnGdipGetImageAttributesAdjustedPalette)
	{
		return GenericError;
	}
	return pfnGdipGetImageAttributesAdjustedPalette(imageAttr,colorPalette,colorAdjustType);
}

GpStatus WINAPI GdiplusWrapper::GdipFlush(GpGraphics* graphics, GpFlushIntention intention)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFlush pfnGdipFlush = (pGdipFlush) GetProcAddress(m_hGdiplusLib, "GdipFlush");
	if(!pfnGdipFlush)
	{
		return GenericError;
	}
	return pfnGdipFlush(graphics,intention);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateFromHDC(HDC hdc, GpGraphics** graphics)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateFromHDC pfnGdipCreateFromHDC = (pGdipCreateFromHDC) GetProcAddress(m_hGdiplusLib, "GdipCreateFromHDC");
	if(!pfnGdipCreateFromHDC)
	{
		return GenericError;
	}
	return pfnGdipCreateFromHDC(hdc,graphics);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateFromHDC2(HDC hdc, HANDLE hDevice, GpGraphics** graphics)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateFromHDC2 pfnGdipCreateFromHDC2 = (pGdipCreateFromHDC2) GetProcAddress(m_hGdiplusLib, "GdipCreateFromHDC2");
	if(!pfnGdipCreateFromHDC2)
	{
		return GenericError;
	}
	return pfnGdipCreateFromHDC2(hdc,hDevice,graphics);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateFromHWND(HWND hwnd, GpGraphics** graphics)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateFromHWND pfnGdipCreateFromHWND = (pGdipCreateFromHWND) GetProcAddress(m_hGdiplusLib, "GdipCreateFromHWND");
	if(!pfnGdipCreateFromHWND)
	{
		return GenericError;
	}
	return pfnGdipCreateFromHWND(hwnd,graphics);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateFromHWNDICM(HWND hwnd, GpGraphics** graphics)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateFromHWNDICM pfnGdipCreateFromHWNDICM = (pGdipCreateFromHWNDICM) GetProcAddress(m_hGdiplusLib, "GdipCreateFromHWNDICM");
	if(!pfnGdipCreateFromHWNDICM)
	{
		return GenericError;
	}
	return pfnGdipCreateFromHWNDICM(hwnd,graphics);
}

GpStatus WINAPI GdiplusWrapper::GdipDeleteGraphics(GpGraphics* graphics)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeleteGraphics pfnGdipDeleteGraphics = (pGdipDeleteGraphics) GetProcAddress(m_hGdiplusLib, "GdipDeleteGraphics");
	if(!pfnGdipDeleteGraphics)
	{
		return GenericError;
	}
	return pfnGdipDeleteGraphics(graphics);
}

GpStatus WINAPI GdiplusWrapper::GdipGetDC(GpGraphics* graphics, HDC* hdc)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetDC pfnGdipGetDC = (pGdipGetDC) GetProcAddress(m_hGdiplusLib, "GdipGetDC");
	if(!pfnGdipGetDC)
	{
		return GenericError;
	}
	return pfnGdipGetDC(graphics,hdc);
}

GpStatus WINAPI GdiplusWrapper::GdipReleaseDC(GpGraphics* graphics, HDC hdc)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipReleaseDC pfnGdipReleaseDC = (pGdipReleaseDC) GetProcAddress(m_hGdiplusLib, "GdipReleaseDC");
	if(!pfnGdipReleaseDC)
	{
		return GenericError;
	}
	return pfnGdipReleaseDC(graphics,hdc);
}

GpStatus WINAPI GdiplusWrapper::GdipSetCompositingMode(GpGraphics* graphics, CompositingMode compositingMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetCompositingMode pfnGdipSetCompositingMode = (pGdipSetCompositingMode) GetProcAddress(m_hGdiplusLib, "GdipSetCompositingMode");
	if(!pfnGdipSetCompositingMode)
	{
		return GenericError;
	}
	return pfnGdipSetCompositingMode(graphics,compositingMode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetCompositingMode(GpGraphics* graphics, CompositingMode* compositingMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetCompositingMode pfnGdipGetCompositingMode = (pGdipGetCompositingMode) GetProcAddress(m_hGdiplusLib, "GdipGetCompositingMode");
	if(!pfnGdipGetCompositingMode)
	{
		return GenericError;
	}
	return pfnGdipGetCompositingMode(graphics,compositingMode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetRenderingOrigin(GpGraphics* graphics, INT x, INT y)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetRenderingOrigin pfnGdipSetRenderingOrigin = (pGdipSetRenderingOrigin) GetProcAddress(m_hGdiplusLib, "GdipSetRenderingOrigin");
	if(!pfnGdipSetRenderingOrigin)
	{
		return GenericError;
	}
	return pfnGdipSetRenderingOrigin(graphics,x,y);
}

GpStatus WINAPI GdiplusWrapper::GdipGetRenderingOrigin(GpGraphics* graphics, INT* x, INT* y)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetRenderingOrigin pfnGdipGetRenderingOrigin = (pGdipGetRenderingOrigin) GetProcAddress(m_hGdiplusLib, "GdipGetRenderingOrigin");
	if(!pfnGdipGetRenderingOrigin)
	{
		return GenericError;
	}
	return pfnGdipGetRenderingOrigin(graphics,x,y);
}

GpStatus WINAPI GdiplusWrapper::GdipSetCompositingQuality(GpGraphics* graphics, CompositingQuality compositingQuality)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetCompositingQuality pfnGdipSetCompositingQuality = (pGdipSetCompositingQuality) GetProcAddress(m_hGdiplusLib, "GdipSetCompositingQuality");
	if(!pfnGdipSetCompositingQuality)
	{
		return GenericError;
	}
	return pfnGdipSetCompositingQuality(graphics,compositingQuality);
}

GpStatus WINAPI GdiplusWrapper::GdipGetCompositingQuality(GpGraphics* graphics, CompositingQuality* compositingQuality)
	{
		if (!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetCompositingQuality pfnGdipGetCompositingQuality = (pGdipGetCompositingQuality) GetProcAddress(m_hGdiplusLib, "GdipGetCompositingQuality");
	if(!pfnGdipGetCompositingQuality)
	{
		return GenericError;
	}
	return pfnGdipGetCompositingQuality(graphics,compositingQuality);
}

GpStatus WINAPI GdiplusWrapper::GdipSetSmoothingMode(GpGraphics* graphics, SmoothingMode smoothingMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetSmoothingMode pfnGdipSetSmoothingMode = (pGdipSetSmoothingMode) GetProcAddress(m_hGdiplusLib, "GdipSetSmoothingMode");
	if(!pfnGdipSetSmoothingMode)
	{
		return GenericError;
	}
	return pfnGdipSetSmoothingMode(graphics,smoothingMode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetSmoothingMode(GpGraphics* graphics, SmoothingMode* smoothingMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetSmoothingMode pfnGdipGetSmoothingMode = (pGdipGetSmoothingMode) GetProcAddress(m_hGdiplusLib, "GdipGetSmoothingMode");
	if(!pfnGdipGetSmoothingMode)
	{
		return GenericError;
	}
	return pfnGdipGetSmoothingMode(graphics,smoothingMode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPixelOffsetMode(GpGraphics* graphics, PixelOffsetMode pixelOffsetMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPixelOffsetMode pfnGdipSetPixelOffsetMode = (pGdipSetPixelOffsetMode) GetProcAddress(m_hGdiplusLib, "GdipSetPixelOffsetMode");
	if(!pfnGdipSetPixelOffsetMode)
	{
		return GenericError;
	}
	return pfnGdipSetPixelOffsetMode(graphics,pixelOffsetMode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPixelOffsetMode(GpGraphics* graphics, PixelOffsetMode* pixelOffsetMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPixelOffsetMode pfnGdipGetPixelOffsetMode = (pGdipGetPixelOffsetMode) GetProcAddress(m_hGdiplusLib, "GdipGetPixelOffsetMode");
	if(!pfnGdipGetPixelOffsetMode)
	{
		return GenericError;
	}
	return pfnGdipGetPixelOffsetMode(graphics,pixelOffsetMode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetTextRenderingHint(GpGraphics* graphics, TextRenderingHint mode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetTextRenderingHint pfnGdipSetTextRenderingHint = (pGdipSetTextRenderingHint) GetProcAddress(m_hGdiplusLib, "GdipSetTextRenderingHint");
	if(!pfnGdipSetTextRenderingHint)
	{
		return GenericError;
	}
	return pfnGdipSetTextRenderingHint(graphics,mode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetTextRenderingHint(GpGraphics* graphics, TextRenderingHint* mode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetTextRenderingHint pfnGdipGetTextRenderingHint = (pGdipGetTextRenderingHint) GetProcAddress(m_hGdiplusLib, "GdipGetTextRenderingHint");
	if(!pfnGdipGetTextRenderingHint)
	{
		return GenericError;
	}
	return pfnGdipGetTextRenderingHint(graphics,mode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetTextContrast(GpGraphics* graphics, UINT contrast)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetTextContrast pfnGdipSetTextContrast = (pGdipSetTextContrast) GetProcAddress(m_hGdiplusLib, "GdipSetTextContrast");
	if(!pfnGdipSetTextContrast)
	{
		return GenericError;
	}
	return pfnGdipSetTextContrast(graphics,contrast);
}

GpStatus WINAPI GdiplusWrapper::GdipGetTextContrast(GpGraphics* graphics, UINT* contrast)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetTextContrast pfnGdipGetTextContrast = (pGdipGetTextContrast) GetProcAddress(m_hGdiplusLib, "GdipGetTextContrast");
	if(!pfnGdipGetTextContrast)
	{
		return GenericError;
	}
	return pfnGdipGetTextContrast(graphics,contrast);
}

GpStatus WINAPI GdiplusWrapper::GdipSetInterpolationMode(GpGraphics* graphics, InterpolationMode interpolationMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetInterpolationMode pfnGdipSetInterpolationMode = (pGdipSetInterpolationMode) GetProcAddress(m_hGdiplusLib, "GdipSetInterpolationMode");
	if(!pfnGdipSetInterpolationMode)
	{
		return GenericError;
	}
	return pfnGdipSetInterpolationMode(graphics,interpolationMode);
}

GpStatus WINAPI GdiplusWrapper::GdipGetInterpolationMode(GpGraphics* graphics, InterpolationMode* interpolationMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetInterpolationMode pfnGdipGetInterpolationMode = (pGdipGetInterpolationMode) GetProcAddress(m_hGdiplusLib, "GdipGetInterpolationMode");
	if(!pfnGdipGetInterpolationMode)
	{
		return GenericError;
	}
	return pfnGdipGetInterpolationMode(graphics,interpolationMode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetWorldTransform(GpGraphics* graphics, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetWorldTransform pfnGdipSetWorldTransform = (pGdipSetWorldTransform) GetProcAddress(m_hGdiplusLib, "GdipSetWorldTransform");
	if(!pfnGdipSetWorldTransform)
	{
		return GenericError;
	}
	return pfnGdipSetWorldTransform(graphics,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipResetWorldTransform(GpGraphics* graphics)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipResetWorldTransform pfnGdipResetWorldTransform = (pGdipResetWorldTransform) GetProcAddress(m_hGdiplusLib, "GdipResetWorldTransform");
	if(!pfnGdipResetWorldTransform)
	{
		return GenericError;
	}
	return pfnGdipResetWorldTransform(graphics);
}

GpStatus WINAPI GdiplusWrapper::GdipMultiplyWorldTransform(GpGraphics* graphics, GDIPCONST GpMatrix* matrix, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipMultiplyWorldTransform pfnGdipMultiplyWorldTransform = (pGdipMultiplyWorldTransform) GetProcAddress(m_hGdiplusLib, "GdipMultiplyWorldTransform");
	if(!pfnGdipMultiplyWorldTransform)
	{
		return GenericError;
	}
	return pfnGdipMultiplyWorldTransform(graphics,matrix,order);
}

GpStatus WINAPI GdiplusWrapper::GdipTranslateWorldTransform(GpGraphics* graphics, REAL dx, REAL dy, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTranslateWorldTransform pfnGdipTranslateWorldTransform = (pGdipTranslateWorldTransform) GetProcAddress(m_hGdiplusLib, "GdipTranslateWorldTransform");
	if(!pfnGdipTranslateWorldTransform)
	{
		return GenericError;
	}
	return pfnGdipTranslateWorldTransform(graphics,dx,dy,order);
}

GpStatus WINAPI GdiplusWrapper::GdipScaleWorldTransform(GpGraphics* graphics, REAL sx, REAL sy, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipScaleWorldTransform pfnGdipScaleWorldTransform = (pGdipScaleWorldTransform) GetProcAddress(m_hGdiplusLib, "GdipScaleWorldTransform");
	if(!pfnGdipScaleWorldTransform)
	{
		return GenericError;
	}
	return pfnGdipScaleWorldTransform(graphics,sx,sy,order);
}

GpStatus WINAPI GdiplusWrapper::GdipRotateWorldTransform(GpGraphics* graphics, REAL angle, GpMatrixOrder order)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRotateWorldTransform pfnGdipRotateWorldTransform = (pGdipRotateWorldTransform) GetProcAddress(m_hGdiplusLib, "GdipRotateWorldTransform");
	if(!pfnGdipRotateWorldTransform)
	{
		return GenericError;
	}
	return pfnGdipRotateWorldTransform(graphics,angle,order);
}

GpStatus WINAPI GdiplusWrapper::GdipGetWorldTransform(GpGraphics* graphics, GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetWorldTransform pfnGdipGetWorldTransform = (pGdipGetWorldTransform) GetProcAddress(m_hGdiplusLib, "GdipGetWorldTransform");
	if(!pfnGdipGetWorldTransform)
	{
		return GenericError;
	}
	return pfnGdipGetWorldTransform(graphics,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipResetPageTransform(GpGraphics* graphics)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipResetPageTransform pfnGdipResetPageTransform = (pGdipResetPageTransform) GetProcAddress(m_hGdiplusLib, "GdipResetPageTransform");
	if(!pfnGdipResetPageTransform)
	{
		return GenericError;
	}
	return pfnGdipResetPageTransform(graphics);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPageUnit(GpGraphics* graphics, GpUnit* unit)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPageUnit pfnGdipGetPageUnit = (pGdipGetPageUnit) GetProcAddress(m_hGdiplusLib, "GdipGetPageUnit");
	if(!pfnGdipGetPageUnit)
	{
		return GenericError;
	}
	return pfnGdipGetPageUnit(graphics,unit);
}

GpStatus WINAPI GdiplusWrapper::GdipGetPageScale(GpGraphics* graphics, REAL* scale)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetPageScale pfnGdipGetPageScale = (pGdipGetPageScale) GetProcAddress(m_hGdiplusLib, "GdipGetPageScale");
	if(!pfnGdipGetPageScale)
	{
		return GenericError;
	}
	return pfnGdipGetPageScale(graphics,scale);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPageUnit(GpGraphics* graphics, GpUnit unit)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPageUnit pfnGdipSetPageUnit = (pGdipSetPageUnit) GetProcAddress(m_hGdiplusLib, "GdipSetPageUnit");
	if(!pfnGdipSetPageUnit)
	{
		return GenericError;
	}
	return pfnGdipSetPageUnit(graphics,unit);
}

GpStatus WINAPI GdiplusWrapper::GdipSetPageScale(GpGraphics* graphics, REAL scale)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetPageScale pfnGdipSetPageScale = (pGdipSetPageScale) GetProcAddress(m_hGdiplusLib, "GdipSetPageScale");
	if(!pfnGdipSetPageScale)
	{
		return GenericError;
	}
	return pfnGdipSetPageScale(graphics,scale);
}

GpStatus WINAPI GdiplusWrapper::GdipGetDpiX(GpGraphics* graphics, REAL* dpi)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetDpiX pfnGdipGetDpiX = (pGdipGetDpiX) GetProcAddress(m_hGdiplusLib, "GdipGetDpiX");
	if(!pfnGdipGetDpiX)
	{
		return GenericError;
	}
	return pfnGdipGetDpiX(graphics,dpi);
}

GpStatus WINAPI GdiplusWrapper::GdipGetDpiY(GpGraphics* graphics, REAL* dpi)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetDpiY pfnGdipGetDpiY = (pGdipGetDpiY) GetProcAddress(m_hGdiplusLib, "GdipGetDpiY");
	if(!pfnGdipGetDpiY)
	{
		return GenericError;
	}
	return pfnGdipGetDpiY(graphics,dpi);
}

GpStatus WINAPI GdiplusWrapper::GdipTransformPoints(GpGraphics* graphics, GpCoordinateSpace destSpace, GpCoordinateSpace srcSpace, GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTransformPoints pfnGdipTransformPoints = (pGdipTransformPoints) GetProcAddress(m_hGdiplusLib, "GdipTransformPoints");
	if(!pfnGdipTransformPoints)
	{
		return GenericError;
	}
	return pfnGdipTransformPoints(graphics,destSpace,srcSpace,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipTransformPointsI(GpGraphics* graphics, GpCoordinateSpace destSpace, GpCoordinateSpace srcSpace, GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTransformPointsI pfnGdipTransformPointsI = (pGdipTransformPointsI) GetProcAddress(m_hGdiplusLib, "GdipTransformPointsI");
	if(!pfnGdipTransformPointsI)
	{
		return GenericError;
	}
	return pfnGdipTransformPointsI(graphics,destSpace,srcSpace,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipGetNearestColor(GpGraphics* graphics, ARGB* argb)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetNearestColor pfnGdipGetNearestColor = (pGdipGetNearestColor) GetProcAddress(m_hGdiplusLib, "GdipGetNearestColor");
	if(!pfnGdipGetNearestColor)
	{
		return GenericError;
	}
	return pfnGdipGetNearestColor(graphics,argb);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawLine(GpGraphics* graphics, GpPen* pen, REAL x1, REAL y1, REAL x2, REAL y2)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawLine pfnGdipDrawLine = (pGdipDrawLine) GetProcAddress(m_hGdiplusLib, "GdipDrawLine");
	if(!pfnGdipDrawLine)
	{
		return GenericError;
	}
	return pfnGdipDrawLine(graphics,pen,x1,y1,x2,y2);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawLineI(GpGraphics* graphics, GpPen* pen, INT x1, INT y1, INT x2, INT y2)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawLineI pfnGdipDrawLineI = (pGdipDrawLineI) GetProcAddress(m_hGdiplusLib, "GdipDrawLineI");
	if(!pfnGdipDrawLineI)
	{
		return GenericError;
	}
	return pfnGdipDrawLineI(graphics,pen,x1,y1,x2,y2);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawLines(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawLines pfnGdipDrawLines = (pGdipDrawLines) GetProcAddress(m_hGdiplusLib, "GdipDrawLines");
	if(!pfnGdipDrawLines)
	{
		return GenericError;
	}
	return pfnGdipDrawLines(graphics,pen,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawLinesI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawLinesI pfnGdipDrawLinesI = (pGdipDrawLinesI) GetProcAddress(m_hGdiplusLib, "GdipDrawLinesI");
	if(!pfnGdipDrawLinesI)
	{
		return GenericError;
	}
	return pfnGdipDrawLinesI(graphics,pen,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawArc(GpGraphics* graphics, GpPen* pen, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawArc pfnGdipDrawArc = (pGdipDrawArc) GetProcAddress(m_hGdiplusLib, "GdipDrawArc");
	if(!pfnGdipDrawArc)
	{
		return GenericError;
	}
	return pfnGdipDrawArc(graphics,pen,x,y,width,height,startAngle,sweepAngle);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawArcI(GpGraphics* graphics, GpPen* pen, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawArcI pfnGdipDrawArcI = (pGdipDrawArcI) GetProcAddress(m_hGdiplusLib, "GdipDrawArcI");
	if(!pfnGdipDrawArcI)
	{
		return GenericError;
	}
	return pfnGdipDrawArcI(graphics,pen,x,y,width,height,startAngle,sweepAngle);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawBezier(GpGraphics* graphics, GpPen* pen, REAL x1, REAL y1, REAL x2, REAL y2, REAL x3, REAL y3, REAL x4, REAL y4)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawBezier pfnGdipDrawBezier = (pGdipDrawBezier) GetProcAddress(m_hGdiplusLib, "GdipDrawBezier");
	if(!pfnGdipDrawBezier)
	{
		return GenericError;
	}
	return pfnGdipDrawBezier(graphics,pen,x1,y1,x2,y2,x3,y3,x4,y4);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawBezierI(GpGraphics* graphics, GpPen* pen, INT x1, INT y1, INT x2, INT y2, INT x3, INT y3, INT x4, INT y4)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawBezierI pfnGdipDrawBezierI = (pGdipDrawBezierI) GetProcAddress(m_hGdiplusLib, "GdipDrawBezierI");
	if(!pfnGdipDrawBezierI)
	{
		return GenericError;
	}
	return pfnGdipDrawBezierI(graphics,pen,x1,y1,x2,y2,x3,y3,x4,y4);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawBeziers(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawBeziers pfnGdipDrawBeziers = (pGdipDrawBeziers) GetProcAddress(m_hGdiplusLib, "GdipDrawBeziers");
	if(!pfnGdipDrawBeziers)
	{
		return GenericError;
	}
	return pfnGdipDrawBeziers(graphics,pen,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawBeziersI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawBeziersI pfnGdipDrawBeziersI = (pGdipDrawBeziersI) GetProcAddress(m_hGdiplusLib, "GdipDrawBeziersI");
	if(!pfnGdipDrawBeziersI)
	{
		return GenericError;
	}
	return pfnGdipDrawBeziersI(graphics,pen,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawRectangle(GpGraphics* graphics, GpPen* pen, REAL x, REAL y, REAL width, REAL height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawRectangle pfnGdipDrawRectangle = (pGdipDrawRectangle) GetProcAddress(m_hGdiplusLib, "GdipDrawRectangle");
	if(!pfnGdipDrawRectangle)
	{
		return GenericError;
	}
	return pfnGdipDrawRectangle(graphics,pen,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawRectangleI(GpGraphics* graphics, GpPen* pen, INT x, INT y, INT width, INT height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawRectangleI pfnGdipDrawRectangleI = (pGdipDrawRectangleI) GetProcAddress(m_hGdiplusLib, "GdipDrawRectangleI");
	if(!pfnGdipDrawRectangleI)
	{
		return GenericError;
	}
	return pfnGdipDrawRectangleI(graphics,pen,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawRectangles(GpGraphics* graphics, GpPen* pen, GDIPCONST GpRectF* rects, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawRectangles pfnGdipDrawRectangles = (pGdipDrawRectangles) GetProcAddress(m_hGdiplusLib, "GdipDrawRectangles");
	if(!pfnGdipDrawRectangles)
	{
		return GenericError;
	}
	return pfnGdipDrawRectangles(graphics,pen,rects,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawRectanglesI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpRect* rects, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawRectanglesI pfnGdipDrawRectanglesI = (pGdipDrawRectanglesI) GetProcAddress(m_hGdiplusLib, "GdipDrawRectanglesI");
	if(!pfnGdipDrawRectanglesI)
	{
		return GenericError;
	}
	return pfnGdipDrawRectanglesI(graphics,pen,rects,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawEllipse(GpGraphics* graphics, GpPen* pen, REAL x, REAL y, REAL width, REAL height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawEllipse pfnGdipDrawEllipse = (pGdipDrawEllipse) GetProcAddress(m_hGdiplusLib, "GdipDrawEllipse");
	if(!pfnGdipDrawEllipse)
	{
		return GenericError;
	}
	return pfnGdipDrawEllipse(graphics,pen,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawEllipseI(GpGraphics* graphics, GpPen* pen, INT x, INT y, INT width, INT height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawEllipseI pfnGdipDrawEllipseI = (pGdipDrawEllipseI) GetProcAddress(m_hGdiplusLib, "GdipDrawEllipseI");
	if(!pfnGdipDrawEllipseI)
	{
		return GenericError;
	}
	return pfnGdipDrawEllipseI(graphics,pen,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawPie(GpGraphics* graphics, GpPen* pen, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawPie pfnGdipDrawPie = (pGdipDrawPie) GetProcAddress(m_hGdiplusLib, "GdipDrawPie");
	if(!pfnGdipDrawPie)
	{
		return GenericError;
	}
	return pfnGdipDrawPie(graphics,pen,x,y,width,height,startAngle,sweepAngle);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawPieI(GpGraphics* graphics, GpPen* pen, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawPieI pfnGdipDrawPieI = (pGdipDrawPieI) GetProcAddress(m_hGdiplusLib, "GdipDrawPieI");
	if(!pfnGdipDrawPieI)
	{
		return GenericError;
	}
	return pfnGdipDrawPieI(graphics,pen,x,y,width,height,startAngle,sweepAngle);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawPolygon(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawPolygon pfnGdipDrawPolygon = (pGdipDrawPolygon) GetProcAddress(m_hGdiplusLib, "GdipDrawPolygon");
	if(!pfnGdipDrawPolygon)
	{
		return GenericError;
	}
	return pfnGdipDrawPolygon(graphics,pen,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawPolygonI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawPolygonI pfnGdipDrawPolygonI = (pGdipDrawPolygonI) GetProcAddress(m_hGdiplusLib, "GdipDrawPolygonI");
	if(!pfnGdipDrawPolygonI)
	{
		return GenericError;
	}
	return pfnGdipDrawPolygonI(graphics,pen,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawPath(GpGraphics* graphics, GpPen* pen, GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawPath pfnGdipDrawPath = (pGdipDrawPath) GetProcAddress(m_hGdiplusLib, "GdipDrawPath");
	if(!pfnGdipDrawPath)
	{
		return GenericError;
	}
	return pfnGdipDrawPath(graphics,pen,path);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawCurve(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawCurve pfnGdipDrawCurve = (pGdipDrawCurve) GetProcAddress(m_hGdiplusLib, "GdipDrawCurve");
	if(!pfnGdipDrawCurve)
	{
		return GenericError;
	}
	return pfnGdipDrawCurve(graphics,pen,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawCurveI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawCurveI pfnGdipDrawCurveI = (pGdipDrawCurveI) GetProcAddress(m_hGdiplusLib, "GdipDrawCurveI");
	if(!pfnGdipDrawCurveI)
	{
		return GenericError;
	}
	return pfnGdipDrawCurveI(graphics,pen,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawCurve2(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawCurve2 pfnGdipDrawCurve2 = (pGdipDrawCurve2) GetProcAddress(m_hGdiplusLib, "GdipDrawCurve2");
	if(!pfnGdipDrawCurve2)
	{
		return GenericError;
	}
	return pfnGdipDrawCurve2(graphics,pen,points,count,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawCurve2I(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawCurve2I pfnGdipDrawCurve2I = (pGdipDrawCurve2I) GetProcAddress(m_hGdiplusLib, "GdipDrawCurve2I");
	if(!pfnGdipDrawCurve2I)
	{
		return GenericError;
	}
	return pfnGdipDrawCurve2I(graphics,pen,points,count,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawCurve3(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count, INT offset, INT numberOfSegments, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawCurve3 pfnGdipDrawCurve3 = (pGdipDrawCurve3) GetProcAddress(m_hGdiplusLib, "GdipDrawCurve3");
	if(!pfnGdipDrawCurve3)
	{
		return GenericError;
	}
	return pfnGdipDrawCurve3(graphics,pen,points,count,offset,numberOfSegments,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawCurve3I(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count, INT offset, INT numberOfSegments, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawCurve3I pfnGdipDrawCurve3I = (pGdipDrawCurve3I) GetProcAddress(m_hGdiplusLib, "GdipDrawCurve3I");
	if(!pfnGdipDrawCurve3I)
	{
		return GenericError;
	}
	return pfnGdipDrawCurve3I(graphics,pen,points,count,offset,numberOfSegments,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawClosedCurve(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawClosedCurve pfnGdipDrawClosedCurve = (pGdipDrawClosedCurve) GetProcAddress(m_hGdiplusLib, "GdipDrawClosedCurve");
	if(!pfnGdipDrawClosedCurve)
	{
		return GenericError;
	}
	return pfnGdipDrawClosedCurve(graphics,pen,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawClosedCurveI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawClosedCurveI pfnGdipDrawClosedCurveI = (pGdipDrawClosedCurveI) GetProcAddress(m_hGdiplusLib, "GdipDrawClosedCurveI");
	if(!pfnGdipDrawClosedCurveI)
	{
		return GenericError;
	}
	return pfnGdipDrawClosedCurveI(graphics,pen,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawClosedCurve2(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawClosedCurve2 pfnGdipDrawClosedCurve2 = (pGdipDrawClosedCurve2) GetProcAddress(m_hGdiplusLib, "GdipDrawClosedCurve2");
	if(!pfnGdipDrawClosedCurve2)
	{
		return GenericError;
	}
	return pfnGdipDrawClosedCurve2(graphics,pen,points,count,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawClosedCurve2I(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count, REAL tension)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawClosedCurve2I pfnGdipDrawClosedCurve2I = (pGdipDrawClosedCurve2I) GetProcAddress(m_hGdiplusLib, "GdipDrawClosedCurve2I");
	if(!pfnGdipDrawClosedCurve2I)
	{
		return GenericError;
	}
	return pfnGdipDrawClosedCurve2I(graphics,pen,points,count,tension);
}

GpStatus WINAPI GdiplusWrapper::GdipGraphicsClear(GpGraphics* graphics, ARGB color)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGraphicsClear pfnGdipGraphicsClear = (pGdipGraphicsClear) GetProcAddress(m_hGdiplusLib, "GdipGraphicsClear");
	if(!pfnGdipGraphicsClear)
	{
		return GenericError;
	}
	return pfnGdipGraphicsClear(graphics,color);
}

GpStatus WINAPI GdiplusWrapper::GdipFillRectangle(GpGraphics* graphics, GpBrush* brush, REAL x, REAL y, REAL width, REAL height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillRectangle pfnGdipFillRectangle = (pGdipFillRectangle) GetProcAddress(m_hGdiplusLib, "GdipFillRectangle");
	if(!pfnGdipFillRectangle)
	{
		return GenericError;
	}
	return pfnGdipFillRectangle(graphics,brush,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipFillRectangleI(GpGraphics* graphics, GpBrush* brush, INT x, INT y, INT width, INT height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillRectangleI pfnGdipFillRectangleI = (pGdipFillRectangleI) GetProcAddress(m_hGdiplusLib, "GdipFillRectangleI");
	if(!pfnGdipFillRectangleI)
	{
		return GenericError;
	}
	return pfnGdipFillRectangleI(graphics,brush,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipFillRectangles(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpRectF* rects, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillRectangles pfnGdipFillRectangles = (pGdipFillRectangles) GetProcAddress(m_hGdiplusLib, "GdipFillRectangles");
	if(!pfnGdipFillRectangles)
	{
		return GenericError;
	}
	return pfnGdipFillRectangles(graphics,brush,rects,count);
}

GpStatus WINAPI GdiplusWrapper::GdipFillRectanglesI(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpRect* rects, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillRectanglesI pfnGdipFillRectanglesI = (pGdipFillRectanglesI) GetProcAddress(m_hGdiplusLib, "GdipFillRectanglesI");
	if(!pfnGdipFillRectanglesI)
	{
		return GenericError;
	}
	return pfnGdipFillRectanglesI(graphics,brush,rects,count);
}

GpStatus WINAPI GdiplusWrapper::GdipFillPolygon(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPointF* points, INT count, GpFillMode fillMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillPolygon pfnGdipFillPolygon = (pGdipFillPolygon) GetProcAddress(m_hGdiplusLib, "GdipFillPolygon");
	if(!pfnGdipFillPolygon)
	{
		return GenericError;
	}
	return pfnGdipFillPolygon(graphics,brush,points,count,fillMode);
}

GpStatus WINAPI GdiplusWrapper::GdipFillPolygonI(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPoint* points, INT count, GpFillMode fillMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillPolygonI pfnGdipFillPolygonI = (pGdipFillPolygonI) GetProcAddress(m_hGdiplusLib, "GdipFillPolygonI");
	if(!pfnGdipFillPolygonI)
	{
		return GenericError;
	}
	return pfnGdipFillPolygonI(graphics,brush,points,count,fillMode);
}

GpStatus WINAPI GdiplusWrapper::GdipFillPolygon2(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillPolygon2 pfnGdipFillPolygon2 = (pGdipFillPolygon2) GetProcAddress(m_hGdiplusLib, "GdipFillPolygon2");
	if(!pfnGdipFillPolygon2)
	{
		return GenericError;
	}
	return pfnGdipFillPolygon2(graphics,brush,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipFillPolygon2I(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillPolygon2I pfnGdipFillPolygon2I = (pGdipFillPolygon2I) GetProcAddress(m_hGdiplusLib, "GdipFillPolygon2I");
	if(!pfnGdipFillPolygon2I)
	{
		return GenericError;
	}
	return pfnGdipFillPolygon2I(graphics,brush,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipFillEllipse(GpGraphics* graphics, GpBrush* brush, REAL x, REAL y, REAL width, REAL height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillEllipse pfnGdipFillEllipse = (pGdipFillEllipse) GetProcAddress(m_hGdiplusLib, "GdipFillEllipse");
	if(!pfnGdipFillEllipse)
	{
		return GenericError;
	}
	return pfnGdipFillEllipse(graphics,brush,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipFillEllipseI(GpGraphics* graphics, GpBrush* brush, INT x, INT y, INT width, INT height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillEllipseI pfnGdipFillEllipseI = (pGdipFillEllipseI) GetProcAddress(m_hGdiplusLib, "GdipFillEllipseI");
	if(!pfnGdipFillEllipseI)
	{
		return GenericError;
	}
	return pfnGdipFillEllipseI(graphics,brush,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipFillPie(GpGraphics* graphics, GpBrush* brush, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillPie pfnGdipFillPie = (pGdipFillPie) GetProcAddress(m_hGdiplusLib, "GdipFillPie");
	if(!pfnGdipFillPie)
	{
		return GenericError;
	}
	return pfnGdipFillPie(graphics,brush,x,y,width,height,startAngle,sweepAngle);
}

GpStatus WINAPI GdiplusWrapper::GdipFillPieI(GpGraphics* graphics, GpBrush* brush, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillPieI pfnGdipFillPieI = (pGdipFillPieI) GetProcAddress(m_hGdiplusLib, "GdipFillPieI");
	if(!pfnGdipFillPieI)
	{
		return GenericError;
	}
	return pfnGdipFillPieI(graphics,brush,x,y,width,height,startAngle,sweepAngle);
}

GpStatus WINAPI GdiplusWrapper::GdipFillPath(GpGraphics* graphics, GpBrush* brush, GpPath* path)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillPath pfnGdipFillPath = (pGdipFillPath) GetProcAddress(m_hGdiplusLib, "GdipFillPath");
	if(!pfnGdipFillPath)
	{
		return GenericError;
	}
	return pfnGdipFillPath(graphics,brush,path);
}

GpStatus WINAPI GdiplusWrapper::GdipFillClosedCurve(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPointF* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillClosedCurve pfnGdipFillClosedCurve = (pGdipFillClosedCurve) GetProcAddress(m_hGdiplusLib, "GdipFillClosedCurve");
	if(!pfnGdipFillClosedCurve)
	{
		return GenericError;
	}
	return pfnGdipFillClosedCurve(graphics,brush,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipFillClosedCurveI(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPoint* points, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillClosedCurveI pfnGdipFillClosedCurveI = (pGdipFillClosedCurveI) GetProcAddress(m_hGdiplusLib, "GdipFillClosedCurveI");
	if(!pfnGdipFillClosedCurveI)
	{
		return GenericError;
	}
	return pfnGdipFillClosedCurveI(graphics,brush,points,count);
}

GpStatus WINAPI GdiplusWrapper::GdipFillClosedCurve2(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPointF* points, INT count, REAL tension, GpFillMode fillMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillClosedCurve2 pfnGdipFillClosedCurve2 = (pGdipFillClosedCurve2) GetProcAddress(m_hGdiplusLib, "GdipFillClosedCurve2");
	if(!pfnGdipFillClosedCurve2)
	{
		return GenericError;
	}
	return pfnGdipFillClosedCurve2(graphics,brush,points,count,tension,fillMode);
}

GpStatus WINAPI GdiplusWrapper::GdipFillClosedCurve2I(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPoint* points, INT count, REAL tension, GpFillMode fillMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillClosedCurve2I pfnGdipFillClosedCurve2I = (pGdipFillClosedCurve2I) GetProcAddress(m_hGdiplusLib, "GdipFillClosedCurve2I");
	if(!pfnGdipFillClosedCurve2I)
	{
		return GenericError;
	}
	return pfnGdipFillClosedCurve2I(graphics,brush,points,count,tension,fillMode);
}

GpStatus WINAPI GdiplusWrapper::GdipFillRegion(GpGraphics* graphics, GpBrush* brush, GpRegion* region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFillRegion pfnGdipFillRegion = (pGdipFillRegion) GetProcAddress(m_hGdiplusLib, "GdipFillRegion");
	if(!pfnGdipFillRegion)
	{
		return GenericError;
	}
	return pfnGdipFillRegion(graphics,brush,region);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImage(GpGraphics* graphics, GpImage* image, REAL x, REAL y)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImage pfnGdipDrawImage = (pGdipDrawImage) GetProcAddress(m_hGdiplusLib, "GdipDrawImage");
	if(!pfnGdipDrawImage)
	{
		return GenericError;
	}
	return pfnGdipDrawImage(graphics,image,x,y);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImageI(GpGraphics* graphics, GpImage* image, INT x, INT y)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImageI pfnGdipDrawImageI = (pGdipDrawImageI) GetProcAddress(m_hGdiplusLib, "GdipDrawImageI");
	if(!pfnGdipDrawImageI)
	{
		return GenericError;
	}
	return pfnGdipDrawImageI(graphics,image,x,y);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImageRect(GpGraphics* graphics, GpImage* image, REAL x, REAL y, REAL width, REAL height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImageRect pfnGdipDrawImageRect = (pGdipDrawImageRect) GetProcAddress(m_hGdiplusLib, "GdipDrawImageRect");
	if(!pfnGdipDrawImageRect)
	{
		return GenericError;
	}
	return pfnGdipDrawImageRect(graphics,image,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImageRectI(GpGraphics* graphics, GpImage* image, INT x, INT y, INT width, INT height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImageRectI pfnGdipDrawImageRectI = (pGdipDrawImageRectI) GetProcAddress(m_hGdiplusLib, "GdipDrawImageRectI");
	if(!pfnGdipDrawImageRectI)
	{
		return GenericError;
	}
	return pfnGdipDrawImageRectI(graphics,image,x,y,width,height);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImagePoints(GpGraphics* graphics, GpImage* image, GDIPCONST GpPointF* dstpoints, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImagePoints pfnGdipDrawImagePoints = (pGdipDrawImagePoints) GetProcAddress(m_hGdiplusLib, "GdipDrawImagePoints");
	if(!pfnGdipDrawImagePoints)
	{
		return GenericError;
	}
	return pfnGdipDrawImagePoints(graphics,image,dstpoints,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImagePointsI(GpGraphics* graphics, GpImage* image, GDIPCONST GpPoint* dstpoints, INT count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImagePointsI pfnGdipDrawImagePointsI = (pGdipDrawImagePointsI) GetProcAddress(m_hGdiplusLib, "GdipDrawImagePointsI");
	if(!pfnGdipDrawImagePointsI)
	{
		return GenericError;
	}
	return pfnGdipDrawImagePointsI(graphics,image,dstpoints,count);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImagePointRect(GpGraphics* graphics, GpImage* image, REAL x, REAL y, REAL srcx, REAL srcy, REAL srcwidth, REAL srcheight, GpUnit srcUnit)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImagePointRect pfnGdipDrawImagePointRect = (pGdipDrawImagePointRect) GetProcAddress(m_hGdiplusLib, "GdipDrawImagePointRect");
	if(!pfnGdipDrawImagePointRect)
	{
		return GenericError;
	}
	return pfnGdipDrawImagePointRect(graphics,image,x,y,srcx,srcy,srcwidth,srcheight,srcUnit);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImagePointRectI(GpGraphics* graphics, GpImage* image, INT x, INT y, INT srcx, INT srcy, INT srcwidth, INT srcheight, GpUnit srcUnit)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImagePointRectI pfnGdipDrawImagePointRectI = (pGdipDrawImagePointRectI) GetProcAddress(m_hGdiplusLib, "GdipDrawImagePointRectI");
	if(!pfnGdipDrawImagePointRectI)
	{
		return GenericError;
	}
	return pfnGdipDrawImagePointRectI(graphics,image,x,y,srcx,srcy,srcwidth,srcheight,srcUnit);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImageRectRect(GpGraphics* graphics, GpImage* image, REAL dstx, REAL dsty, REAL dstwidth, REAL dstheight, REAL srcx, REAL srcy, REAL srcwidth, REAL srcheight, GpUnit srcUnit, GDIPCONST GpImageAttributes* imageAttributes, DrawImageAbort callback, VOID* callbackData)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImageRectRect pfnGdipDrawImageRectRect = (pGdipDrawImageRectRect) GetProcAddress(m_hGdiplusLib, "GdipDrawImageRectRect");
	if(!pfnGdipDrawImageRectRect)
	{
		return GenericError;
	}
	return pfnGdipDrawImageRectRect(graphics,image,dstx,dsty,dstwidth,dstheight,srcx,srcy,srcwidth,srcheight,srcUnit,imageAttributes,callback,callbackData);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImageRectRectI(GpGraphics* graphics, GpImage* image, INT dstx, INT dsty, INT dstwidth, INT dstheight, INT srcx, INT srcy, INT srcwidth, INT srcheight, GpUnit srcUnit, GDIPCONST GpImageAttributes* imageAttributes, DrawImageAbort callback, VOID* callbackData)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImageRectRectI pfnGdipDrawImageRectRectI = (pGdipDrawImageRectRectI) GetProcAddress(m_hGdiplusLib, "GdipDrawImageRectRectI");
	if(!pfnGdipDrawImageRectRectI)
	{
		return GenericError;
	}
	return pfnGdipDrawImageRectRectI(graphics,image,dstx,dsty,dstwidth,dstheight,srcx,srcy,srcwidth,srcheight,srcUnit,imageAttributes,callback,callbackData);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImagePointsRect(GpGraphics* graphics, GpImage* image, GDIPCONST GpPointF* points, INT count, REAL srcx, REAL srcy, REAL srcwidth, REAL srcheight, GpUnit srcUnit, GDIPCONST GpImageAttributes* imageAttributes, DrawImageAbort callback, VOID* callbackData)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImagePointsRect pfnGdipDrawImagePointsRect = (pGdipDrawImagePointsRect) GetProcAddress(m_hGdiplusLib, "GdipDrawImagePointsRect");
	if(!pfnGdipDrawImagePointsRect)
	{
		return GenericError;
	}
	return pfnGdipDrawImagePointsRect(graphics,image,points,count,srcx,srcy,srcwidth,srcheight,srcUnit,imageAttributes,callback,callbackData);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawImagePointsRectI(GpGraphics* graphics, GpImage* image, GDIPCONST GpPoint* points, INT count, INT srcx, INT srcy, INT srcwidth, INT srcheight, GpUnit srcUnit, GDIPCONST GpImageAttributes* imageAttributes, DrawImageAbort callback, VOID* callbackData)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawImagePointsRectI pfnGdipDrawImagePointsRectI = (pGdipDrawImagePointsRectI) GetProcAddress(m_hGdiplusLib, "GdipDrawImagePointsRectI");
	if(!pfnGdipDrawImagePointsRectI)
	{
		return GenericError;
	}
	return pfnGdipDrawImagePointsRectI(graphics,image,points,count,srcx,srcy,srcwidth,srcheight,srcUnit,imageAttributes,callback,callbackData);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileDestPoint(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST PointF& destPoint, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileDestPoint pfnGdipEnumerateMetafileDestPoint = (pGdipEnumerateMetafileDestPoint) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileDestPoint");
	if(!pfnGdipEnumerateMetafileDestPoint)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileDestPoint(graphics,metafile,destPoint,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileDestPointI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Point& destPoint, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileDestPointI pfnGdipEnumerateMetafileDestPointI = (pGdipEnumerateMetafileDestPointI) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileDestPointI");
	if(!pfnGdipEnumerateMetafileDestPointI)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileDestPointI(graphics,metafile,destPoint,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileDestRect(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST RectF& destRect, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileDestRect pfnGdipEnumerateMetafileDestRect = (pGdipEnumerateMetafileDestRect) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileDestRect");
	if(!pfnGdipEnumerateMetafileDestRect)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileDestRect(graphics,metafile,destRect,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileDestRectI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Rect& destRect, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileDestRectI pfnGdipEnumerateMetafileDestRectI = (pGdipEnumerateMetafileDestRectI) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileDestRectI");
	if(!pfnGdipEnumerateMetafileDestRectI)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileDestRectI(graphics,metafile,destRect,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileDestPoints(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST PointF* destPoints, INT count, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileDestPoints pfnGdipEnumerateMetafileDestPoints = (pGdipEnumerateMetafileDestPoints) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileDestPoints");
	if(!pfnGdipEnumerateMetafileDestPoints)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileDestPoints(graphics,metafile,destPoints,count,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileDestPointsI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Point* destPoints, INT count, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileDestPointsI pfnGdipEnumerateMetafileDestPointsI = (pGdipEnumerateMetafileDestPointsI) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileDestPointsI");
	if(!pfnGdipEnumerateMetafileDestPointsI)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileDestPointsI(graphics,metafile,destPoints,count,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileSrcRectDestPoint(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST PointF& destPoint, GDIPCONST RectF& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileSrcRectDestPoint pfnGdipEnumerateMetafileSrcRectDestPoint = (pGdipEnumerateMetafileSrcRectDestPoint) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileSrcRectDestPoint");
	if(!pfnGdipEnumerateMetafileSrcRectDestPoint)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileSrcRectDestPoint(graphics,metafile,destPoint,srcRect,srcUnit,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileSrcRectDestPointI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Point& destPoint, GDIPCONST Rect& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileSrcRectDestPointI pfnGdipEnumerateMetafileSrcRectDestPointI = (pGdipEnumerateMetafileSrcRectDestPointI) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileSrcRectDestPointI");
	if(!pfnGdipEnumerateMetafileSrcRectDestPointI)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileSrcRectDestPointI(graphics,metafile,destPoint,srcRect,srcUnit,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileSrcRectDestRect(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST RectF& destRect, GDIPCONST RectF& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileSrcRectDestRect pfnGdipEnumerateMetafileSrcRectDestRect = (pGdipEnumerateMetafileSrcRectDestRect) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileSrcRectDestRect");
	if(!pfnGdipEnumerateMetafileSrcRectDestRect)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileSrcRectDestRect(graphics,metafile,destRect,srcRect,srcUnit,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileSrcRectDestRectI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Rect& destRect, GDIPCONST Rect& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileSrcRectDestRectI pfnGdipEnumerateMetafileSrcRectDestRectI = (pGdipEnumerateMetafileSrcRectDestRectI) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileSrcRectDestRectI");
	if(!pfnGdipEnumerateMetafileSrcRectDestRectI)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileSrcRectDestRectI(graphics,metafile,destRect,srcRect,srcUnit,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileSrcRectDestPoints(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST PointF* destPoints, INT count, GDIPCONST RectF& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileSrcRectDestPoints pfnGdipEnumerateMetafileSrcRectDestPoints = (pGdipEnumerateMetafileSrcRectDestPoints) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileSrcRectDestPoints");
	if(!pfnGdipEnumerateMetafileSrcRectDestPoints)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileSrcRectDestPoints(graphics,metafile,destPoints,count,srcRect,srcUnit,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipEnumerateMetafileSrcRectDestPointsI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Point* destPoints, INT count, GDIPCONST Rect& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEnumerateMetafileSrcRectDestPointsI pfnGdipEnumerateMetafileSrcRectDestPointsI = (pGdipEnumerateMetafileSrcRectDestPointsI) GetProcAddress(m_hGdiplusLib, "GdipEnumerateMetafileSrcRectDestPointsI");
	if(!pfnGdipEnumerateMetafileSrcRectDestPointsI)
	{
		return GenericError;
	}
	return pfnGdipEnumerateMetafileSrcRectDestPointsI(graphics,metafile,destPoints,count,srcRect,srcUnit,callback,callbackData,imageAttributes);
}

GpStatus WINAPI GdiplusWrapper::GdipPlayMetafileRecord(GDIPCONST GpMetafile* metafile, EmfPlusRecordType recordType, UINT flags, UINT dataSize, GDIPCONST BYTE* data)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPlayMetafileRecord pfnGdipPlayMetafileRecord = (pGdipPlayMetafileRecord) GetProcAddress(m_hGdiplusLib, "GdipPlayMetafileRecord");
	if(!pfnGdipPlayMetafileRecord)
	{
		return GenericError;
	}
	return pfnGdipPlayMetafileRecord(metafile,recordType,flags,dataSize,data);
}

GpStatus WINAPI GdiplusWrapper::GdipSetClipGraphics(GpGraphics* graphics, GpGraphics* srcgraphics, CombineMode combineMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetClipGraphics pfnGdipSetClipGraphics = (pGdipSetClipGraphics) GetProcAddress(m_hGdiplusLib, "GdipSetClipGraphics");
	if(!pfnGdipSetClipGraphics)
	{
		return GenericError;
	}
	return pfnGdipSetClipGraphics(graphics,srcgraphics,combineMode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetClipRect(GpGraphics* graphics, REAL x, REAL y, REAL width, REAL height, CombineMode combineMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetClipRect pfnGdipSetClipRect = (pGdipSetClipRect) GetProcAddress(m_hGdiplusLib, "GdipSetClipRect");
	if(!pfnGdipSetClipRect)
	{
		return GenericError;
	}
	return pfnGdipSetClipRect(graphics,x,y,width,height,combineMode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetClipRectI(GpGraphics* graphics, INT x, INT y, INT width, INT height, CombineMode combineMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetClipRectI pfnGdipSetClipRectI = (pGdipSetClipRectI) GetProcAddress(m_hGdiplusLib, "GdipSetClipRectI");
	if(!pfnGdipSetClipRectI)
	{
		return GenericError;
	}
	return pfnGdipSetClipRectI(graphics,x,y,width,height,combineMode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetClipPath(GpGraphics* graphics, GpPath* path, CombineMode combineMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetClipPath pfnGdipSetClipPath = (pGdipSetClipPath) GetProcAddress(m_hGdiplusLib, "GdipSetClipPath");
	if(!pfnGdipSetClipPath)
	{
		return GenericError;
	}
	return pfnGdipSetClipPath(graphics,path,combineMode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetClipRegion(GpGraphics* graphics, GpRegion* region, CombineMode combineMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetClipRegion pfnGdipSetClipRegion = (pGdipSetClipRegion) GetProcAddress(m_hGdiplusLib, "GdipSetClipRegion");
	if(!pfnGdipSetClipRegion)
	{
		return GenericError;
	}
	return pfnGdipSetClipRegion(graphics,region,combineMode);
}

GpStatus WINAPI GdiplusWrapper::GdipSetClipHrgn(GpGraphics* graphics, HRGN hRgn, CombineMode combineMode)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetClipHrgn pfnGdipSetClipHrgn = (pGdipSetClipHrgn) GetProcAddress(m_hGdiplusLib, "GdipSetClipHrgn");
	if(!pfnGdipSetClipHrgn)
	{
		return GenericError;
	}
	return pfnGdipSetClipHrgn(graphics,hRgn,combineMode);
}

GpStatus WINAPI GdiplusWrapper::GdipResetClip(GpGraphics* graphics)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipResetClip pfnGdipResetClip = (pGdipResetClip) GetProcAddress(m_hGdiplusLib, "GdipResetClip");
	if(!pfnGdipResetClip)
	{
		return GenericError;
	}
	return pfnGdipResetClip(graphics);
}

GpStatus WINAPI GdiplusWrapper::GdipTranslateClip(GpGraphics* graphics, REAL dx, REAL dy)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTranslateClip pfnGdipTranslateClip = (pGdipTranslateClip) GetProcAddress(m_hGdiplusLib, "GdipTranslateClip");
	if(!pfnGdipTranslateClip)
	{
		return GenericError;
	}
	return pfnGdipTranslateClip(graphics,dx,dy);
}

GpStatus WINAPI GdiplusWrapper::GdipTranslateClipI(GpGraphics* graphics, INT dx, INT dy)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipTranslateClipI pfnGdipTranslateClipI = (pGdipTranslateClipI) GetProcAddress(m_hGdiplusLib, "GdipTranslateClipI");
	if(!pfnGdipTranslateClipI)
	{
		return GenericError;
	}
	return pfnGdipTranslateClipI(graphics,dx,dy);
}

GpStatus WINAPI GdiplusWrapper::GdipGetClip(GpGraphics* graphics, GpRegion* region)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetClip pfnGdipGetClip = (pGdipGetClip) GetProcAddress(m_hGdiplusLib, "GdipGetClip");
	if(!pfnGdipGetClip)
	{
		return GenericError;
	}
	return pfnGdipGetClip(graphics,region);
}

GpStatus WINAPI GdiplusWrapper::GdipGetClipBounds(GpGraphics* graphics, GpRectF* rect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetClipBounds pfnGdipGetClipBounds = (pGdipGetClipBounds) GetProcAddress(m_hGdiplusLib, "GdipGetClipBounds");
	if(!pfnGdipGetClipBounds)
	{
		return GenericError;
	}
	return pfnGdipGetClipBounds(graphics,rect);
}

GpStatus WINAPI GdiplusWrapper::GdipGetClipBoundsI(GpGraphics* graphics, GpRect* rect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetClipBoundsI pfnGdipGetClipBoundsI = (pGdipGetClipBoundsI) GetProcAddress(m_hGdiplusLib, "GdipGetClipBoundsI");
	if(!pfnGdipGetClipBoundsI)
	{
		return GenericError;
	}
	return pfnGdipGetClipBoundsI(graphics,rect);
}

GpStatus WINAPI GdiplusWrapper::GdipIsClipEmpty(GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsClipEmpty pfnGdipIsClipEmpty = (pGdipIsClipEmpty) GetProcAddress(m_hGdiplusLib, "GdipIsClipEmpty");
	if(!pfnGdipIsClipEmpty)
	{
		return GenericError;
	}
	return pfnGdipIsClipEmpty(graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipGetVisibleClipBounds(GpGraphics* graphics, GpRectF* rect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetVisibleClipBounds pfnGdipGetVisibleClipBounds = (pGdipGetVisibleClipBounds) GetProcAddress(m_hGdiplusLib, "GdipGetVisibleClipBounds");
	if(!pfnGdipGetVisibleClipBounds)
	{
		return GenericError;
	}
	return pfnGdipGetVisibleClipBounds(graphics,rect);
}

GpStatus WINAPI GdiplusWrapper::GdipGetVisibleClipBoundsI(GpGraphics* graphics, GpRect* rect)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetVisibleClipBoundsI pfnGdipGetVisibleClipBoundsI = (pGdipGetVisibleClipBoundsI) GetProcAddress(m_hGdiplusLib, "GdipGetVisibleClipBoundsI");
	if(!pfnGdipGetVisibleClipBoundsI)
	{
		return GenericError;
	}
	return pfnGdipGetVisibleClipBoundsI(graphics,rect);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisibleClipEmpty(GpGraphics* graphics, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisibleClipEmpty pfnGdipIsVisibleClipEmpty = (pGdipIsVisibleClipEmpty) GetProcAddress(m_hGdiplusLib, "GdipIsVisibleClipEmpty");
	if(!pfnGdipIsVisibleClipEmpty)
	{
		return GenericError;
	}
	return pfnGdipIsVisibleClipEmpty(graphics,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisiblePoint(GpGraphics* graphics, REAL x, REAL y, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisiblePoint pfnGdipIsVisiblePoint = (pGdipIsVisiblePoint) GetProcAddress(m_hGdiplusLib, "GdipIsVisiblePoint");
	if(!pfnGdipIsVisiblePoint)
	{
		return GenericError;
	}
	return pfnGdipIsVisiblePoint(graphics,x,y,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisiblePointI(GpGraphics* graphics, INT x, INT y, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisiblePointI pfnGdipIsVisiblePointI = (pGdipIsVisiblePointI) GetProcAddress(m_hGdiplusLib, "GdipIsVisiblePointI");
	if(!pfnGdipIsVisiblePointI)
	{
		return GenericError;
	}
	return pfnGdipIsVisiblePointI(graphics,x,y,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisibleRect(GpGraphics* graphics, REAL x, REAL y, REAL width, REAL height, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisibleRect pfnGdipIsVisibleRect = (pGdipIsVisibleRect) GetProcAddress(m_hGdiplusLib, "GdipIsVisibleRect");
	if(!pfnGdipIsVisibleRect)
	{
		return GenericError;
	}
	return pfnGdipIsVisibleRect(graphics,x,y,width,height,result);
}

GpStatus WINAPI GdiplusWrapper::GdipIsVisibleRectI(GpGraphics* graphics, INT x, INT y, INT width, INT height, BOOL* result)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsVisibleRectI pfnGdipIsVisibleRectI = (pGdipIsVisibleRectI) GetProcAddress(m_hGdiplusLib, "GdipIsVisibleRectI");
	if(!pfnGdipIsVisibleRectI)
	{
		return GenericError;
	}
	return pfnGdipIsVisibleRectI(graphics,x,y,width,height,result);
}

GpStatus WINAPI GdiplusWrapper::GdipSaveGraphics(GpGraphics* graphics, GraphicsState* state)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSaveGraphics pfnGdipSaveGraphics = (pGdipSaveGraphics) GetProcAddress(m_hGdiplusLib, "GdipSaveGraphics");
	if(!pfnGdipSaveGraphics)
	{
		return GenericError;
	}
	return pfnGdipSaveGraphics(graphics,state);
}

GpStatus WINAPI GdiplusWrapper::GdipRestoreGraphics(GpGraphics* graphics, GraphicsState state)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRestoreGraphics pfnGdipRestoreGraphics = (pGdipRestoreGraphics) GetProcAddress(m_hGdiplusLib, "GdipRestoreGraphics");
	if(!pfnGdipRestoreGraphics)
	{
		return GenericError;
	}
	return pfnGdipRestoreGraphics(graphics,state);
}

GpStatus WINAPI GdiplusWrapper::GdipBeginContainer(GpGraphics* graphics, GDIPCONST GpRectF* dstrect, GDIPCONST GpRectF* srcrect, GpUnit unit, GraphicsContainer* state)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipBeginContainer pfnGdipBeginContainer = (pGdipBeginContainer) GetProcAddress(m_hGdiplusLib, "GdipBeginContainer");
	if(!pfnGdipBeginContainer)
	{
		return GenericError;
	}
	return pfnGdipBeginContainer(graphics,dstrect,srcrect,unit,state);
}

GpStatus WINAPI GdiplusWrapper::GdipBeginContainerI(GpGraphics* graphics, GDIPCONST GpRect* dstrect, GDIPCONST GpRect* srcrect, GpUnit unit, GraphicsContainer* state)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipBeginContainerI pfnGdipBeginContainerI = (pGdipBeginContainerI) GetProcAddress(m_hGdiplusLib, "GdipBeginContainerI");
	if(!pfnGdipBeginContainerI)
	{
		return GenericError;
	}
	return pfnGdipBeginContainerI(graphics,dstrect,srcrect,unit,state);
}

GpStatus WINAPI GdiplusWrapper::GdipBeginContainer2(GpGraphics* graphics, GraphicsContainer* state)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipBeginContainer2 pfnGdipBeginContainer2 = (pGdipBeginContainer2) GetProcAddress(m_hGdiplusLib, "GdipBeginContainer2");
	if(!pfnGdipBeginContainer2)
	{
		return GenericError;
	}
	return pfnGdipBeginContainer2(graphics,state);
}

GpStatus WINAPI GdiplusWrapper::GdipEndContainer(GpGraphics* graphics, GraphicsContainer state)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEndContainer pfnGdipEndContainer = (pGdipEndContainer) GetProcAddress(m_hGdiplusLib, "GdipEndContainer");
	if(!pfnGdipEndContainer)
	{
		return GenericError;
	}
	return pfnGdipEndContainer(graphics,state);
}

GpStatus GdiplusWrapper::GdipGetMetafileHeaderFromWmf(HMETAFILE hWmf, GDIPCONST WmfPlaceableFileHeader* wmfPlaceableFileHeader, MetafileHeader* header)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetMetafileHeaderFromWmf pfnGdipGetMetafileHeaderFromWmf = (pGdipGetMetafileHeaderFromWmf) GetProcAddress(m_hGdiplusLib, "GdipGetMetafileHeaderFromWmf");
	if(!pfnGdipGetMetafileHeaderFromWmf)
	{
		return GenericError;
	}
	return pfnGdipGetMetafileHeaderFromWmf(hWmf,wmfPlaceableFileHeader,header);
}

GpStatus WINAPI GdiplusWrapper::GdipGetMetafileHeaderFromEmf(HENHMETAFILE hEmf, MetafileHeader* header)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetMetafileHeaderFromEmf pfnGdipGetMetafileHeaderFromEmf = (pGdipGetMetafileHeaderFromEmf) GetProcAddress(m_hGdiplusLib, "GdipGetMetafileHeaderFromEmf");
	if(!pfnGdipGetMetafileHeaderFromEmf)
	{
		return GenericError;
	}
	return pfnGdipGetMetafileHeaderFromEmf(hEmf,header);
}

GpStatus WINAPI GdiplusWrapper::GdipGetMetafileHeaderFromFile(GDIPCONST WCHAR* filename, MetafileHeader* header)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetMetafileHeaderFromFile pfnGdipGetMetafileHeaderFromFile = (pGdipGetMetafileHeaderFromFile) GetProcAddress(m_hGdiplusLib, "GdipGetMetafileHeaderFromFile");
	if(!pfnGdipGetMetafileHeaderFromFile)
	{
		return GenericError;
	}
	return pfnGdipGetMetafileHeaderFromFile(filename,header);
}

GpStatus WINAPI GdiplusWrapper::GdipGetMetafileHeaderFromStream(IStream* stream, MetafileHeader* header)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetMetafileHeaderFromStream pfnGdipGetMetafileHeaderFromStream = (pGdipGetMetafileHeaderFromStream) GetProcAddress(m_hGdiplusLib, "GdipGetMetafileHeaderFromStream");
	if(!pfnGdipGetMetafileHeaderFromStream)
	{
		return GenericError;
	}
	return pfnGdipGetMetafileHeaderFromStream(stream,header);
}

GpStatus WINAPI GdiplusWrapper::GdipGetMetafileHeaderFromMetafile(GpMetafile* metafile, MetafileHeader* header)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetMetafileHeaderFromMetafile pfnGdipGetMetafileHeaderFromMetafile = (pGdipGetMetafileHeaderFromMetafile) GetProcAddress(m_hGdiplusLib, "GdipGetMetafileHeaderFromMetafile");
	if(!pfnGdipGetMetafileHeaderFromMetafile)
	{
		return GenericError;
	}
	return pfnGdipGetMetafileHeaderFromMetafile(metafile,header);
}

GpStatus WINAPI GdiplusWrapper::GdipGetHemfFromMetafile(GpMetafile* metafile, HENHMETAFILE* hEmf)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetHemfFromMetafile pfnGdipGetHemfFromMetafile = (pGdipGetHemfFromMetafile) GetProcAddress(m_hGdiplusLib, "GdipGetHemfFromMetafile");
	if(!pfnGdipGetHemfFromMetafile)
	{
		return GenericError;
	}
	return pfnGdipGetHemfFromMetafile(metafile,hEmf);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateStreamOnFile(GDIPCONST WCHAR* filename, UINT access, IStream** stream)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateStreamOnFile pfnGdipCreateStreamOnFile = (pGdipCreateStreamOnFile) GetProcAddress(m_hGdiplusLib, "GdipCreateStreamOnFile");
	if(!pfnGdipCreateStreamOnFile)
	{
		return GenericError;
	}
	return pfnGdipCreateStreamOnFile(filename,access,stream);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateMetafileFromWmf(HMETAFILE hWmf, BOOL deleteWmf, GDIPCONST WmfPlaceableFileHeader* wmfPlaceableFileHeader, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateMetafileFromWmf pfnGdipCreateMetafileFromWmf = (pGdipCreateMetafileFromWmf) GetProcAddress(m_hGdiplusLib, "GdipCreateMetafileFromWmf");
	if(!pfnGdipCreateMetafileFromWmf)
	{
		return GenericError;
	}
	return pfnGdipCreateMetafileFromWmf(hWmf,deleteWmf,wmfPlaceableFileHeader,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateMetafileFromEmf(HENHMETAFILE hEmf, BOOL deleteEmf, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateMetafileFromEmf pfnGdipCreateMetafileFromEmf = (pGdipCreateMetafileFromEmf) GetProcAddress(m_hGdiplusLib, "GdipCreateMetafileFromEmf");
	if(!pfnGdipCreateMetafileFromEmf)
	{
		return GenericError;
	}
	return pfnGdipCreateMetafileFromEmf(hEmf,deleteEmf,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateMetafileFromFile(GDIPCONST WCHAR* file, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateMetafileFromFile pfnGdipCreateMetafileFromFile = (pGdipCreateMetafileFromFile) GetProcAddress(m_hGdiplusLib, "GdipCreateMetafileFromFile");
	if(!pfnGdipCreateMetafileFromFile)
	{
		return GenericError;
	}
	return pfnGdipCreateMetafileFromFile(file,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateMetafileFromWmfFile(GDIPCONST WCHAR* file, GDIPCONST WmfPlaceableFileHeader* wmfPlaceableFileHeader, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateMetafileFromWmfFile pfnGdipCreateMetafileFromWmfFile = (pGdipCreateMetafileFromWmfFile) GetProcAddress(m_hGdiplusLib, "GdipCreateMetafileFromWmfFile");
	if(!pfnGdipCreateMetafileFromWmfFile)
	{
		return GenericError;
	}
	return pfnGdipCreateMetafileFromWmfFile(file,wmfPlaceableFileHeader,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateMetafileFromStream(IStream* stream, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateMetafileFromStream pfnGdipCreateMetafileFromStream = (pGdipCreateMetafileFromStream) GetProcAddress(m_hGdiplusLib, "GdipCreateMetafileFromStream");
	if(!pfnGdipCreateMetafileFromStream)
	{
		return GenericError;
	}
	return pfnGdipCreateMetafileFromStream(stream,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipRecordMetafile(HDC referenceHdc, EmfType type, GDIPCONST GpRectF* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRecordMetafile pfnGdipRecordMetafile = (pGdipRecordMetafile) GetProcAddress(m_hGdiplusLib, "GdipRecordMetafile");
	if(!pfnGdipRecordMetafile)
	{
		return GenericError;
	}
	return pfnGdipRecordMetafile(referenceHdc,type,frameRect,frameUnit,description,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipRecordMetafileI(HDC referenceHdc, EmfType type, GDIPCONST GpRect* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRecordMetafileI pfnGdipRecordMetafileI = (pGdipRecordMetafileI) GetProcAddress(m_hGdiplusLib, "GdipRecordMetafileI");
	if(!pfnGdipRecordMetafileI)
	{
		return GenericError;
	}
	return pfnGdipRecordMetafileI(referenceHdc,type,frameRect,frameUnit,description,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipRecordMetafileFileName(GDIPCONST WCHAR* fileName, HDC referenceHdc, EmfType type, GDIPCONST GpRectF* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRecordMetafileFileName pfnGdipRecordMetafileFileName = (pGdipRecordMetafileFileName) GetProcAddress(m_hGdiplusLib, "GdipRecordMetafileFileName");
	if(!pfnGdipRecordMetafileFileName)
	{
		return GenericError;
	}
	return pfnGdipRecordMetafileFileName(fileName,referenceHdc,type,frameRect,frameUnit,description,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipRecordMetafileFileNameI(GDIPCONST WCHAR* fileName, HDC referenceHdc, EmfType type, GDIPCONST GpRect* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRecordMetafileFileNameI pfnGdipRecordMetafileFileNameI = (pGdipRecordMetafileFileNameI) GetProcAddress(m_hGdiplusLib, "GdipRecordMetafileFileNameI");
	if(!pfnGdipRecordMetafileFileNameI)
	{
		return GenericError;
	}
	return pfnGdipRecordMetafileFileNameI(fileName,referenceHdc,type,frameRect,frameUnit,description,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipRecordMetafileStream(IStream* stream, HDC referenceHdc, EmfType type, GDIPCONST GpRectF* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRecordMetafileStream pfnGdipRecordMetafileStream = (pGdipRecordMetafileStream) GetProcAddress(m_hGdiplusLib, "GdipRecordMetafileStream");
	if(!pfnGdipRecordMetafileStream)
	{
		return GenericError;
	}
	return pfnGdipRecordMetafileStream(stream,referenceHdc,type,frameRect,frameUnit,description,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipRecordMetafileStreamI(IStream* stream, HDC referenceHdc, EmfType type, GDIPCONST GpRect* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipRecordMetafileStreamI pfnGdipRecordMetafileStreamI = (pGdipRecordMetafileStreamI) GetProcAddress(m_hGdiplusLib, "GdipRecordMetafileStreamI");
	if(!pfnGdipRecordMetafileStreamI)
	{
		return GenericError;
	}
	return pfnGdipRecordMetafileStreamI(stream,referenceHdc,type,frameRect,frameUnit,description,metafile);
}

GpStatus WINAPI GdiplusWrapper::GdipSetMetafileDownLevelRasterizationLimit(GpMetafile* metafile, UINT metafileRasterizationLimitDpi)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetMetafileDownLevelRasterizationLimit pfnGdipSetMetafileDownLevelRasterizationLimit = (pGdipSetMetafileDownLevelRasterizationLimit) GetProcAddress(m_hGdiplusLib, "GdipSetMetafileDownLevelRasterizationLimit");
	if(!pfnGdipSetMetafileDownLevelRasterizationLimit)
	{
		return GenericError;
	}
	return pfnGdipSetMetafileDownLevelRasterizationLimit(metafile,metafileRasterizationLimitDpi);
}

GpStatus WINAPI GdiplusWrapper::GdipGetMetafileDownLevelRasterizationLimit(GDIPCONST GpMetafile* metafile, UINT* metafileRasterizationLimitDpi)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetMetafileDownLevelRasterizationLimit pfnGdipGetMetafileDownLevelRasterizationLimit = (pGdipGetMetafileDownLevelRasterizationLimit) GetProcAddress(m_hGdiplusLib, "GdipGetMetafileDownLevelRasterizationLimit");
	if(!pfnGdipGetMetafileDownLevelRasterizationLimit)
	{
		return GenericError;
	}
	return pfnGdipGetMetafileDownLevelRasterizationLimit(metafile,metafileRasterizationLimitDpi);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageDecodersSize(UINT* numDecoders, UINT* size)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageDecodersSize pfnGdipGetImageDecodersSize = (pGdipGetImageDecodersSize) GetProcAddress(m_hGdiplusLib, "GdipGetImageDecodersSize");
	if(!pfnGdipGetImageDecodersSize)
	{
		return GenericError;
	}
	return pfnGdipGetImageDecodersSize(numDecoders,size);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageDecoders(UINT numDecoders, UINT size, ImageCodecInfo* decoders)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageDecoders pfnGdipGetImageDecoders = (pGdipGetImageDecoders) GetProcAddress(m_hGdiplusLib, "GdipGetImageDecoders");
	if(!pfnGdipGetImageDecoders)
	{
		return GenericError;
	}
	return pfnGdipGetImageDecoders(numDecoders,size,decoders);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageEncodersSize(UINT* numEncoders, UINT* size)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageEncodersSize pfnGdipGetImageEncodersSize = (pGdipGetImageEncodersSize) GetProcAddress(m_hGdiplusLib, "GdipGetImageEncodersSize");
	if(!pfnGdipGetImageEncodersSize)
	{
		return GenericError;
	}
	return pfnGdipGetImageEncodersSize(numEncoders,size);
}

GpStatus WINAPI GdiplusWrapper::GdipGetImageEncoders(UINT numEncoders, UINT size, ImageCodecInfo* encoders)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetImageEncoders pfnGdipGetImageEncoders = (pGdipGetImageEncoders) GetProcAddress(m_hGdiplusLib, "GdipGetImageEncoders");
	if(!pfnGdipGetImageEncoders)
	{
		return GenericError;
	}
	return pfnGdipGetImageEncoders(numEncoders,size,encoders);
}

GpStatus WINAPI GdiplusWrapper::GdipComment(GpGraphics* graphics, UINT sizeData, GDIPCONST BYTE* data)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipComment pfnGdipComment = (pGdipComment) GetProcAddress(m_hGdiplusLib, "GdipComment");
	if(!pfnGdipComment)
	{
		return GenericError;
	}
	return pfnGdipComment(graphics,sizeData,data);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateFontFamilyFromName(GDIPCONST WCHAR* name, GpFontCollection* fontCollection, GpFontFamily** FontFamily)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateFontFamilyFromName pfnGdipCreateFontFamilyFromName = (pGdipCreateFontFamilyFromName) GetProcAddress(m_hGdiplusLib, "GdipCreateFontFamilyFromName");
	if(!pfnGdipCreateFontFamilyFromName)
	{
		return GenericError;
	}
	return pfnGdipCreateFontFamilyFromName(name,fontCollection,FontFamily);
}

GpStatus WINAPI GdiplusWrapper::GdipDeleteFontFamily(GpFontFamily* FontFamily)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeleteFontFamily pfnGdipDeleteFontFamily = (pGdipDeleteFontFamily) GetProcAddress(m_hGdiplusLib, "GdipDeleteFontFamily");
	if(!pfnGdipDeleteFontFamily)
	{
		return GenericError;
	}
	return pfnGdipDeleteFontFamily(FontFamily);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneFontFamily(GpFontFamily* FontFamily, GpFontFamily** clonedFontFamily)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneFontFamily pfnGdipCloneFontFamily = (pGdipCloneFontFamily) GetProcAddress(m_hGdiplusLib, "GdipCloneFontFamily");
	if(!pfnGdipCloneFontFamily)
	{
		return GenericError;
	}
	return pfnGdipCloneFontFamily(FontFamily,clonedFontFamily);
}

GpStatus WINAPI GdiplusWrapper::GdipGetGenericFontFamilySansSerif(GpFontFamily** nativeFamily)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetGenericFontFamilySansSerif pfnGdipGetGenericFontFamilySansSerif = (pGdipGetGenericFontFamilySansSerif) GetProcAddress(m_hGdiplusLib, "GdipGetGenericFontFamilySansSerif");
	if(!pfnGdipGetGenericFontFamilySansSerif)
	{
		return GenericError;
	}
	return pfnGdipGetGenericFontFamilySansSerif(nativeFamily);
}

GpStatus WINAPI GdiplusWrapper::GdipGetGenericFontFamilySerif(GpFontFamily** nativeFamily)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetGenericFontFamilySerif pfnGdipGetGenericFontFamilySerif = (pGdipGetGenericFontFamilySerif) GetProcAddress(m_hGdiplusLib, "GdipGetGenericFontFamilySerif");
	if(!pfnGdipGetGenericFontFamilySerif)
	{
		return GenericError;
	}
	return pfnGdipGetGenericFontFamilySerif(nativeFamily);
}

GpStatus WINAPI GdiplusWrapper::GdipGetGenericFontFamilyMonospace(GpFontFamily** nativeFamily)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetGenericFontFamilyMonospace pfnGdipGetGenericFontFamilyMonospace = (pGdipGetGenericFontFamilyMonospace) GetProcAddress(m_hGdiplusLib, "GdipGetGenericFontFamilyMonospace");
	if(!pfnGdipGetGenericFontFamilyMonospace)
	{
		return GenericError;
	}
	return pfnGdipGetGenericFontFamilyMonospace(nativeFamily);
}

GpStatus WINAPI GdiplusWrapper::GdipGetFamilyName(GDIPCONST GpFontFamily* family, WCHAR name[LF_FACESIZE], LANGID language)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetFamilyName pfnGdipGetFamilyName = (pGdipGetFamilyName) GetProcAddress(m_hGdiplusLib, "GdipGetFamilyName");
	if(!pfnGdipGetFamilyName)
	{
		return GenericError;
	}
	return pfnGdipGetFamilyName(family,name,language);
}

GpStatus WINAPI GdiplusWrapper::GdipIsStyleAvailable(GDIPCONST GpFontFamily* family, INT style, BOOL* IsStyleAvailable)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipIsStyleAvailable pfnGdipIsStyleAvailable = (pGdipIsStyleAvailable) GetProcAddress(m_hGdiplusLib, "GdipIsStyleAvailable");
	if(!pfnGdipIsStyleAvailable)
	{
		return GenericError;
	}
	return pfnGdipIsStyleAvailable(family,style,IsStyleAvailable);
}

GpStatus WINAPI GdiplusWrapper::GdipFontCollectionEnumerable(GpFontCollection* fontCollection, GpGraphics* graphics, INT* numFound)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFontCollectionEnumerable pfnGdipFontCollectionEnumerable = (pGdipFontCollectionEnumerable) GetProcAddress(m_hGdiplusLib, "GdipFontCollectionEnumerable");
	if(!pfnGdipFontCollectionEnumerable)
	{
		return GenericError;
	}
	return pfnGdipFontCollectionEnumerable(fontCollection,graphics,numFound);
}

GpStatus WINAPI GdiplusWrapper::GdipFontCollectionEnumerate(GpFontCollection* fontCollection, INT numSought, GpFontFamily* gpfamilies[], INT* numFound, GpGraphics* graphics)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipFontCollectionEnumerate pfnGdipFontCollectionEnumerate = (pGdipFontCollectionEnumerate) GetProcAddress(m_hGdiplusLib, "GdipFontCollectionEnumerate");
	if(!pfnGdipFontCollectionEnumerate)
	{
		return GenericError;
	}
	return pfnGdipFontCollectionEnumerate(fontCollection,numSought,gpfamilies,numFound,graphics);
}

GpStatus WINAPI GdiplusWrapper::GdipGetEmHeight(GDIPCONST GpFontFamily* family, INT style, UINT16* EmHeight)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetEmHeight pfnGdipGetEmHeight = (pGdipGetEmHeight) GetProcAddress(m_hGdiplusLib, "GdipGetEmHeight");
	if(!pfnGdipGetEmHeight)
	{
		return GenericError;
	}
	return pfnGdipGetEmHeight(family,style,EmHeight);
}

GpStatus WINAPI GdiplusWrapper::GdipGetCellAscent(GDIPCONST GpFontFamily* family, INT style, UINT16* CellAscent)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetCellAscent pfnGdipGetCellAscent = (pGdipGetCellAscent) GetProcAddress(m_hGdiplusLib, "GdipGetCellAscent");
	if(!pfnGdipGetCellAscent)
	{
		return GenericError;
	}
	return pfnGdipGetCellAscent(family,style,CellAscent);
}

GpStatus WINAPI GdiplusWrapper::GdipGetCellDescent(GDIPCONST GpFontFamily* family, INT style, UINT16* CellDescent)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetCellDescent pfnGdipGetCellDescent = (pGdipGetCellDescent) GetProcAddress(m_hGdiplusLib, "GdipGetCellDescent");
	if(!pfnGdipGetCellDescent)
	{
		return GenericError;
	}
	return pfnGdipGetCellDescent(family,style,CellDescent);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLineSpacing(GDIPCONST GpFontFamily* family, INT style, UINT16* LineSpacing)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLineSpacing pfnGdipGetLineSpacing = (pGdipGetLineSpacing) GetProcAddress(m_hGdiplusLib, "GdipGetLineSpacing");
	if(!pfnGdipGetLineSpacing)
	{
		return GenericError;
	}
	return pfnGdipGetLineSpacing(family,style,LineSpacing);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateFontFromDC(HDC hdc, GpFont** font)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateFontFromDC pfnGdipCreateFontFromDC = (pGdipCreateFontFromDC) GetProcAddress(m_hGdiplusLib, "GdipCreateFontFromDC");
	if(!pfnGdipCreateFontFromDC)
	{
		return GenericError;
	}
	return pfnGdipCreateFontFromDC(hdc,font);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateFontFromLogfontA(HDC hdc, GDIPCONST LOGFONTA* logfont, GpFont** font)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateFontFromLogfontA pfnGdipCreateFontFromLogfontA = (pGdipCreateFontFromLogfontA) GetProcAddress(m_hGdiplusLib, "GdipCreateFontFromLogfontA");
	if(!pfnGdipCreateFontFromLogfontA)
	{
		return GenericError;
	}
	return pfnGdipCreateFontFromLogfontA(hdc,logfont,font);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateFontFromLogfontW(HDC hdc, GDIPCONST LOGFONTW* logfont, GpFont** font)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateFontFromLogfontW pfnGdipCreateFontFromLogfontW = (pGdipCreateFontFromLogfontW) GetProcAddress(m_hGdiplusLib, "GdipCreateFontFromLogfontW");
	if(!pfnGdipCreateFontFromLogfontW)
	{
		return GenericError;
	}
	return pfnGdipCreateFontFromLogfontW(hdc,logfont,font);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateFont(GDIPCONST GpFontFamily* fontFamily, REAL emSize, INT style, Unit unit, GpFont** font)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateFont pfnGdipCreateFont = (pGdipCreateFont) GetProcAddress(m_hGdiplusLib, "GdipCreateFont");
	if(!pfnGdipCreateFont)
	{
		return GenericError;
	}
	return pfnGdipCreateFont(fontFamily,emSize,style,unit,font);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneFont(GpFont* font, GpFont** cloneFont)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneFont pfnGdipCloneFont = (pGdipCloneFont) GetProcAddress(m_hGdiplusLib, "GdipCloneFont");
	if(!pfnGdipCloneFont)
	{
		return GenericError;
	}
	return pfnGdipCloneFont(font,cloneFont);
}

GpStatus WINAPI GdiplusWrapper::GdipDeleteFont(GpFont* font)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeleteFont pfnGdipDeleteFont = (pGdipDeleteFont) GetProcAddress(m_hGdiplusLib, "GdipDeleteFont");
	if(!pfnGdipDeleteFont)
	{
		return GenericError;
	}
	return pfnGdipDeleteFont(font);
}

GpStatus WINAPI GdiplusWrapper::GdipGetFamily(GpFont* font, GpFontFamily** family)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetFamily pfnGdipGetFamily = (pGdipGetFamily) GetProcAddress(m_hGdiplusLib, "GdipGetFamily");
	if(!pfnGdipGetFamily)
	{
		return GenericError;
	}
	return pfnGdipGetFamily(font,family);
}

GpStatus WINAPI GdiplusWrapper::GdipGetFontStyle(GpFont* font, INT* style)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetFontStyle pfnGdipGetFontStyle = (pGdipGetFontStyle) GetProcAddress(m_hGdiplusLib, "GdipGetFontStyle");
	if(!pfnGdipGetFontStyle)
	{
		return GenericError;
	}
	return pfnGdipGetFontStyle(font,style);
}

GpStatus WINAPI GdiplusWrapper::GdipGetFontSize(GpFont* font, REAL* size)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetFontSize pfnGdipGetFontSize = (pGdipGetFontSize) GetProcAddress(m_hGdiplusLib, "GdipGetFontSize");
	if(!pfnGdipGetFontSize)
	{
		return GenericError;
	}
	return pfnGdipGetFontSize(font,size);
}

GpStatus WINAPI GdiplusWrapper::GdipGetFontUnit(GpFont* font, Unit* unit)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetFontUnit pfnGdipGetFontUnit = (pGdipGetFontUnit) GetProcAddress(m_hGdiplusLib, "GdipGetFontUnit");
	if(!pfnGdipGetFontUnit)
	{
		return GenericError;
	}
	return pfnGdipGetFontUnit(font,unit);
}

GpStatus WINAPI GdiplusWrapper::GdipGetFontHeight(GDIPCONST GpFont* font, GDIPCONST GpGraphics* graphics, REAL* height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetFontHeight pfnGdipGetFontHeight = (pGdipGetFontHeight) GetProcAddress(m_hGdiplusLib, "GdipGetFontHeight");
	if(!pfnGdipGetFontHeight)
	{
		return GenericError;
	}
	return pfnGdipGetFontHeight(font,graphics,height);
}

GpStatus WINAPI GdiplusWrapper::GdipGetFontHeightGivenDPI(GDIPCONST GpFont* font, REAL dpi, REAL* height)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetFontHeightGivenDPI pfnGdipGetFontHeightGivenDPI = (pGdipGetFontHeightGivenDPI) GetProcAddress(m_hGdiplusLib, "GdipGetFontHeightGivenDPI");
	if(!pfnGdipGetFontHeightGivenDPI)
	{
		return GenericError;
	}
	return pfnGdipGetFontHeightGivenDPI(font,dpi,height);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLogFontA(GpFont* font, GpGraphics* graphics, LOGFONTA* logfontA)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLogFontA pfnGdipGetLogFontA = (pGdipGetLogFontA) GetProcAddress(m_hGdiplusLib, "GdipGetLogFontA");
	if(!pfnGdipGetLogFontA)
	{
		return GenericError;
	}
	return pfnGdipGetLogFontA(font,graphics,logfontA);
}

GpStatus WINAPI GdiplusWrapper::GdipGetLogFontW(GpFont* font, GpGraphics* graphics, LOGFONTW* logfontW)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetLogFontW pfnGdipGetLogFontW = (pGdipGetLogFontW) GetProcAddress(m_hGdiplusLib, "GdipGetLogFontW");
	if(!pfnGdipGetLogFontW)
	{
		return GenericError;
	}
	return pfnGdipGetLogFontW(font,graphics,logfontW);
}

GpStatus WINAPI GdiplusWrapper::GdipNewInstalledFontCollection(GpFontCollection** fontCollection)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipNewInstalledFontCollection pfnGdipNewInstalledFontCollection = (pGdipNewInstalledFontCollection) GetProcAddress(m_hGdiplusLib, "GdipNewInstalledFontCollection");
	if(!pfnGdipNewInstalledFontCollection)
	{
		return GenericError;
	}
	return pfnGdipNewInstalledFontCollection(fontCollection);
}

GpStatus WINAPI GdiplusWrapper::GdipNewPrivateFontCollection(GpFontCollection** fontCollection)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipNewPrivateFontCollection pfnGdipNewPrivateFontCollection = (pGdipNewPrivateFontCollection) GetProcAddress(m_hGdiplusLib, "GdipNewPrivateFontCollection");
	if(!pfnGdipNewPrivateFontCollection)
	{
		return GenericError;
	}
	return pfnGdipNewPrivateFontCollection(fontCollection);
}

GpStatus WINAPI GdiplusWrapper::GdipDeletePrivateFontCollection(GpFontCollection** fontCollection)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeletePrivateFontCollection pfnGdipDeletePrivateFontCollection = (pGdipDeletePrivateFontCollection) GetProcAddress(m_hGdiplusLib, "GdipDeletePrivateFontCollection");
	if(!pfnGdipDeletePrivateFontCollection)
	{
		return GenericError;
	}
	return pfnGdipDeletePrivateFontCollection(fontCollection);
}

GpStatus WINAPI GdiplusWrapper::GdipGetFontCollectionFamilyCount(GpFontCollection* fontCollection, INT* numFound)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetFontCollectionFamilyCount pfnGdipGetFontCollectionFamilyCount = (pGdipGetFontCollectionFamilyCount) GetProcAddress(m_hGdiplusLib, "GdipGetFontCollectionFamilyCount");
	if(!pfnGdipGetFontCollectionFamilyCount)
	{
		return GenericError;
	}
	return pfnGdipGetFontCollectionFamilyCount(fontCollection,numFound);
}

GpStatus WINAPI GdiplusWrapper::GdipGetFontCollectionFamilyList(GpFontCollection* fontCollection, INT numSought, GpFontFamily* gpfamilies[], INT* numFound)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetFontCollectionFamilyList pfnGdipGetFontCollectionFamilyList = (pGdipGetFontCollectionFamilyList) GetProcAddress(m_hGdiplusLib, "GdipGetFontCollectionFamilyList");
	if(!pfnGdipGetFontCollectionFamilyList)
	{
		return GenericError;
	}
	return pfnGdipGetFontCollectionFamilyList(fontCollection,numSought,gpfamilies,numFound);
}

GpStatus WINAPI GdiplusWrapper::GdipPrivateAddFontFile(GpFontCollection* fontCollection, GDIPCONST WCHAR* filename)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPrivateAddFontFile pfnGdipPrivateAddFontFile = (pGdipPrivateAddFontFile) GetProcAddress(m_hGdiplusLib, "GdipPrivateAddFontFile");
	if(!pfnGdipPrivateAddFontFile)
	{
		return GenericError;
	}
	return pfnGdipPrivateAddFontFile(fontCollection,filename);
}

GpStatus WINAPI GdiplusWrapper::GdipPrivateAddMemoryFont(GpFontCollection* fontCollection, GDIPCONST void* memory, INT length)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipPrivateAddMemoryFont pfnGdipPrivateAddMemoryFont = (pGdipPrivateAddMemoryFont) GetProcAddress(m_hGdiplusLib, "GdipPrivateAddMemoryFont");
	if(!pfnGdipPrivateAddMemoryFont)
	{
		return GenericError;
	}
	return pfnGdipPrivateAddMemoryFont(fontCollection,memory,length);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawString(GpGraphics* graphics, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFont* font, GDIPCONST RectF* layoutRect, GDIPCONST GpStringFormat* stringFormat, GDIPCONST GpBrush* brush)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawString pfnGdipDrawString = (pGdipDrawString) GetProcAddress(m_hGdiplusLib, "GdipDrawString");
	if(!pfnGdipDrawString)
	{
		return GenericError;
	}
	return pfnGdipDrawString(graphics,string,length,font,layoutRect,stringFormat,brush);
}

GpStatus WINAPI GdiplusWrapper::GdipMeasureString(GpGraphics* graphics, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFont* font, GDIPCONST RectF* layoutRect, GDIPCONST GpStringFormat* stringFormat, RectF* boundingBox, INT* codepointsFitted, INT* linesFilled)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipMeasureString pfnGdipMeasureString = (pGdipMeasureString) GetProcAddress(m_hGdiplusLib, "GdipMeasureString");
	if(!pfnGdipMeasureString)
	{
		return GenericError;
	}
	return pfnGdipMeasureString(graphics,string,length,font,layoutRect,stringFormat,boundingBox,codepointsFitted,linesFilled);
}

GpStatus WINAPI GdiplusWrapper::GdipMeasureCharacterRanges(GpGraphics* graphics, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFont* font, GDIPCONST RectF& layoutRect, GDIPCONST GpStringFormat* stringFormat, INT regionCount, GpRegion** regions)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipMeasureCharacterRanges pfnGdipMeasureCharacterRanges = (pGdipMeasureCharacterRanges) GetProcAddress(m_hGdiplusLib, "GdipMeasureCharacterRanges");
	if(!pfnGdipMeasureCharacterRanges)
	{
		return GenericError;
	}
	return pfnGdipMeasureCharacterRanges(graphics,string,length,font,layoutRect,stringFormat,regionCount,regions);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawDriverString(GpGraphics* graphics, GDIPCONST UINT16* text, INT length, GDIPCONST GpFont* font, GDIPCONST GpBrush* brush, GDIPCONST PointF* positions, INT flags, GDIPCONST GpMatrix* matrix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawDriverString pfnGdipDrawDriverString = (pGdipDrawDriverString) GetProcAddress(m_hGdiplusLib, "GdipDrawDriverString");
	if(!pfnGdipDrawDriverString)
	{
		return GenericError;
	}
	return pfnGdipDrawDriverString(graphics,text,length,font,brush,positions,flags,matrix);
}

GpStatus WINAPI GdiplusWrapper::GdipMeasureDriverString(GpGraphics* graphics, GDIPCONST UINT16* text, INT length, GDIPCONST GpFont* font, GDIPCONST PointF* positions, INT flags, GDIPCONST GpMatrix* matrix, RectF* boundingBox)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipMeasureDriverString pfnGdipMeasureDriverString = (pGdipMeasureDriverString) GetProcAddress(m_hGdiplusLib, "GdipMeasureDriverString");
	if(!pfnGdipMeasureDriverString)
	{
		return GenericError;
	}
	return pfnGdipMeasureDriverString(graphics,text,length,font,positions,flags,matrix,boundingBox);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateStringFormat(INT formatAttributes, LANGID language, GpStringFormat** format)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateStringFormat pfnGdipCreateStringFormat = (pGdipCreateStringFormat) GetProcAddress(m_hGdiplusLib, "GdipCreateStringFormat");
	if(!pfnGdipCreateStringFormat)
	{
		return GenericError;
	}
	return pfnGdipCreateStringFormat(formatAttributes,language,format);
}

GpStatus WINAPI GdiplusWrapper::GdipStringFormatGetGenericDefault(GpStringFormat** format)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipStringFormatGetGenericDefault pfnGdipStringFormatGetGenericDefault = (pGdipStringFormatGetGenericDefault) GetProcAddress(m_hGdiplusLib, "GdipStringFormatGetGenericDefault");
	if(!pfnGdipStringFormatGetGenericDefault)
	{
		return GenericError;
	}
	return pfnGdipStringFormatGetGenericDefault(format);
}

GpStatus WINAPI GdiplusWrapper::GdipStringFormatGetGenericTypographic(GpStringFormat** format)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipStringFormatGetGenericTypographic pfnGdipStringFormatGetGenericTypographic = (pGdipStringFormatGetGenericTypographic) GetProcAddress(m_hGdiplusLib, "GdipStringFormatGetGenericTypographic");
	if(!pfnGdipStringFormatGetGenericTypographic)
	{
		return GenericError;
	}
	return pfnGdipStringFormatGetGenericTypographic(format);
}

GpStatus WINAPI GdiplusWrapper::GdipDeleteStringFormat(GpStringFormat* format)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeleteStringFormat pfnGdipDeleteStringFormat = (pGdipDeleteStringFormat) GetProcAddress(m_hGdiplusLib, "GdipDeleteStringFormat");
	if(!pfnGdipDeleteStringFormat)
	{
		return GenericError;
	}
	return pfnGdipDeleteStringFormat(format);
}

GpStatus WINAPI GdiplusWrapper::GdipCloneStringFormat(GDIPCONST GpStringFormat* format, GpStringFormat** newFormat)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCloneStringFormat pfnGdipCloneStringFormat = (pGdipCloneStringFormat) GetProcAddress(m_hGdiplusLib, "GdipCloneStringFormat");
	if(!pfnGdipCloneStringFormat)
	{
		return GenericError;
	}
	return pfnGdipCloneStringFormat(format,newFormat);
}

GpStatus WINAPI GdiplusWrapper::GdipSetStringFormatFlags(GpStringFormat* format, INT flags)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetStringFormatFlags pfnGdipSetStringFormatFlags = (pGdipSetStringFormatFlags) GetProcAddress(m_hGdiplusLib, "GdipSetStringFormatFlags");
	if(!pfnGdipSetStringFormatFlags)
	{
		return GenericError;
	}
	return pfnGdipSetStringFormatFlags(format,flags);
}

GpStatus WINAPI GdiplusWrapper::GdipGetStringFormatFlags(GDIPCONST GpStringFormat* format, INT* flags)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetStringFormatFlags pfnGdipGetStringFormatFlags = (pGdipGetStringFormatFlags) GetProcAddress(m_hGdiplusLib, "GdipGetStringFormatFlags");
	if(!pfnGdipGetStringFormatFlags)
	{
		return GenericError;
	}
	return pfnGdipGetStringFormatFlags(format,flags);
}

GpStatus WINAPI GdiplusWrapper::GdipSetStringFormatAlign(GpStringFormat* format, StringAlignment align)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetStringFormatAlign pfnGdipSetStringFormatAlign = (pGdipSetStringFormatAlign) GetProcAddress(m_hGdiplusLib, "GdipSetStringFormatAlign");
	if(!pfnGdipSetStringFormatAlign)
	{
		return GenericError;
	}
	return pfnGdipSetStringFormatAlign(format,align);
}

GpStatus WINAPI GdiplusWrapper::GdipGetStringFormatAlign(GDIPCONST GpStringFormat* format, StringAlignment* align)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetStringFormatAlign pfnGdipGetStringFormatAlign = (pGdipGetStringFormatAlign) GetProcAddress(m_hGdiplusLib, "GdipGetStringFormatAlign");
	if(!pfnGdipGetStringFormatAlign)
	{
		return GenericError;
	}
	return pfnGdipGetStringFormatAlign(format,align);
}

GpStatus WINAPI GdiplusWrapper::GdipSetStringFormatLineAlign(GpStringFormat* format, StringAlignment align)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetStringFormatLineAlign pfnGdipSetStringFormatLineAlign = (pGdipSetStringFormatLineAlign) GetProcAddress(m_hGdiplusLib, "GdipSetStringFormatLineAlign");
	if(!pfnGdipSetStringFormatLineAlign)
	{
		return GenericError;
	}
	return pfnGdipSetStringFormatLineAlign(format,align);
}

GpStatus WINAPI GdiplusWrapper::GdipGetStringFormatLineAlign(GDIPCONST GpStringFormat* format, StringAlignment* align)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetStringFormatLineAlign pfnGdipGetStringFormatLineAlign = (pGdipGetStringFormatLineAlign) GetProcAddress(m_hGdiplusLib, "GdipGetStringFormatLineAlign");
	if(!pfnGdipGetStringFormatLineAlign)
	{
		return GenericError;
	}
	return pfnGdipGetStringFormatLineAlign(format,align);
}

GpStatus WINAPI GdiplusWrapper::GdipSetStringFormatTrimming(GpStringFormat* format, StringTrimming trimming)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetStringFormatTrimming pfnGdipSetStringFormatTrimming = (pGdipSetStringFormatTrimming) GetProcAddress(m_hGdiplusLib, "GdipSetStringFormatTrimming");
	if(!pfnGdipSetStringFormatTrimming)
	{
		return GenericError;
	}
	return pfnGdipSetStringFormatTrimming(format,trimming);
}

GpStatus WINAPI GdiplusWrapper::GdipGetStringFormatTrimming(GDIPCONST GpStringFormat* format, StringTrimming* trimming)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetStringFormatTrimming pfnGdipGetStringFormatTrimming = (pGdipGetStringFormatTrimming) GetProcAddress(m_hGdiplusLib, "GdipGetStringFormatTrimming");
	if(!pfnGdipGetStringFormatTrimming)
	{
		return GenericError;
	}
	return pfnGdipGetStringFormatTrimming(format,trimming);
}

GpStatus WINAPI GdiplusWrapper::GdipSetStringFormatHotkeyPrefix(GpStringFormat* format, INT hotkeyPrefix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetStringFormatHotkeyPrefix pfnGdipSetStringFormatHotkeyPrefix = (pGdipSetStringFormatHotkeyPrefix) GetProcAddress(m_hGdiplusLib, "GdipSetStringFormatHotkeyPrefix");
	if(!pfnGdipSetStringFormatHotkeyPrefix)
	{
		return GenericError;
	}
	return pfnGdipSetStringFormatHotkeyPrefix(format,hotkeyPrefix);
}

GpStatus WINAPI GdiplusWrapper::GdipGetStringFormatHotkeyPrefix(GDIPCONST GpStringFormat* format, INT* hotkeyPrefix)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetStringFormatHotkeyPrefix pfnGdipGetStringFormatHotkeyPrefix = (pGdipGetStringFormatHotkeyPrefix) GetProcAddress(m_hGdiplusLib, "GdipGetStringFormatHotkeyPrefix");
	if(!pfnGdipGetStringFormatHotkeyPrefix)
	{
		return GenericError;
	}
	return pfnGdipGetStringFormatHotkeyPrefix(format,hotkeyPrefix);
}

GpStatus WINAPI GdiplusWrapper::GdipSetStringFormatTabStops(GpStringFormat* format, REAL firstTabOffset, INT count, GDIPCONST REAL* tabStops)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetStringFormatTabStops pfnGdipSetStringFormatTabStops = (pGdipSetStringFormatTabStops) GetProcAddress(m_hGdiplusLib, "GdipSetStringFormatTabStops");
	if(!pfnGdipSetStringFormatTabStops)
	{
		return GenericError;
	}
	return pfnGdipSetStringFormatTabStops(format,firstTabOffset,count,tabStops);
}

GpStatus WINAPI GdiplusWrapper::GdipGetStringFormatTabStops(GDIPCONST GpStringFormat* format, INT count, REAL* firstTabOffset, REAL* tabStops)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetStringFormatTabStops pfnGdipGetStringFormatTabStops = (pGdipGetStringFormatTabStops) GetProcAddress(m_hGdiplusLib, "GdipGetStringFormatTabStops");
	if(!pfnGdipGetStringFormatTabStops)
	{
		return GenericError;
	}
	return pfnGdipGetStringFormatTabStops(format,count,firstTabOffset,tabStops);
}

GpStatus WINAPI GdiplusWrapper::GdipGetStringFormatTabStopCount(GDIPCONST GpStringFormat* format, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetStringFormatTabStopCount pfnGdipGetStringFormatTabStopCount = (pGdipGetStringFormatTabStopCount) GetProcAddress(m_hGdiplusLib, "GdipGetStringFormatTabStopCount");
	if(!pfnGdipGetStringFormatTabStopCount)
	{
		return GenericError;
	}
	return pfnGdipGetStringFormatTabStopCount(format,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetStringFormatDigitSubstitution(GpStringFormat* format, LANGID language, StringDigitSubstitute substitute)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetStringFormatDigitSubstitution pfnGdipSetStringFormatDigitSubstitution = (pGdipSetStringFormatDigitSubstitution) GetProcAddress(m_hGdiplusLib, "GdipSetStringFormatDigitSubstitution");
	if(!pfnGdipSetStringFormatDigitSubstitution)
	{
		return GenericError;
	}
	return pfnGdipSetStringFormatDigitSubstitution(format,language,substitute);
}

GpStatus WINAPI GdiplusWrapper::GdipGetStringFormatDigitSubstitution(GDIPCONST GpStringFormat* format, LANGID* language, StringDigitSubstitute* substitute)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetStringFormatDigitSubstitution pfnGdipGetStringFormatDigitSubstitution = (pGdipGetStringFormatDigitSubstitution) GetProcAddress(m_hGdiplusLib, "GdipGetStringFormatDigitSubstitution");
	if(!pfnGdipGetStringFormatDigitSubstitution)
	{
		return GenericError;
	}
	return pfnGdipGetStringFormatDigitSubstitution(format,language,substitute);
}

GpStatus WINAPI GdiplusWrapper::GdipGetStringFormatMeasurableCharacterRangeCount(GDIPCONST GpStringFormat* format, INT* count)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipGetStringFormatMeasurableCharacterRangeCount pfnGdipGetStringFormatMeasurableCharacterRangeCount = (pGdipGetStringFormatMeasurableCharacterRangeCount) GetProcAddress(m_hGdiplusLib, "GdipGetStringFormatMeasurableCharacterRangeCount");
	if(!pfnGdipGetStringFormatMeasurableCharacterRangeCount)
	{
		return GenericError;
	}
	return pfnGdipGetStringFormatMeasurableCharacterRangeCount(format,count);
}

GpStatus WINAPI GdiplusWrapper::GdipSetStringFormatMeasurableCharacterRanges(GpStringFormat* format, INT rangeCount, GDIPCONST CharacterRange* ranges)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipSetStringFormatMeasurableCharacterRanges pfnGdipSetStringFormatMeasurableCharacterRanges = (pGdipSetStringFormatMeasurableCharacterRanges) GetProcAddress(m_hGdiplusLib, "GdipSetStringFormatMeasurableCharacterRanges");
	if(!pfnGdipSetStringFormatMeasurableCharacterRanges)
	{
		return GenericError;
	}
	return pfnGdipSetStringFormatMeasurableCharacterRanges(format,rangeCount,ranges);
}

GpStatus WINAPI GdiplusWrapper::GdipCreateCachedBitmap(GpBitmap* bitmap, GpGraphics* graphics, GpCachedBitmap** cachedBitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipCreateCachedBitmap pfnGdipCreateCachedBitmap = (pGdipCreateCachedBitmap) GetProcAddress(m_hGdiplusLib, "GdipCreateCachedBitmap");
	if(!pfnGdipCreateCachedBitmap)
	{
		return GenericError;
	}
	return pfnGdipCreateCachedBitmap(bitmap,graphics,cachedBitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipDeleteCachedBitmap(GpCachedBitmap* cachedBitmap)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDeleteCachedBitmap pfnGdipDeleteCachedBitmap = (pGdipDeleteCachedBitmap) GetProcAddress(m_hGdiplusLib, "GdipDeleteCachedBitmap");
	if(!pfnGdipDeleteCachedBitmap)
	{
		return GenericError;
	}
	return pfnGdipDeleteCachedBitmap(cachedBitmap);
}

GpStatus WINAPI GdiplusWrapper::GdipDrawCachedBitmap(GpGraphics* graphics, GpCachedBitmap* cachedBitmap, INT x, INT y)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipDrawCachedBitmap pfnGdipDrawCachedBitmap = (pGdipDrawCachedBitmap) GetProcAddress(m_hGdiplusLib, "GdipDrawCachedBitmap");
	if(!pfnGdipDrawCachedBitmap)
	{
		return GenericError;
	}
	return pfnGdipDrawCachedBitmap(graphics,cachedBitmap,x,y);
}

UINT WINAPI GdiplusWrapper::GdipEmfToWmfBits(HENHMETAFILE hemf, UINT cbData16, LPBYTE pData16, INT iMapMode, INT eFlags)
{
	if(!IsInitialized())
	{
		return GenericError;
	}
	pGdipEmfToWmfBits pfnGdipEmfToWmfBits = (pGdipEmfToWmfBits) GetProcAddress(m_hGdiplusLib, "GdipEmfToWmfBits");
	if(!pfnGdipEmfToWmfBits)
	{
		return GenericError;
	}
	return pfnGdipEmfToWmfBits(hemf,cbData16,pData16,iMapMode,eFlags);
}

HPALETTE WINAPI GdiplusWrapper::GdipCreateHalftonePalette()
{
	if(!IsInitialized())
	{
		return NULL;
	}
	pGdipCreateHalftonePalette pfnGdipCreateHalftonePalette = (pGdipCreateHalftonePalette) GetProcAddress(m_hGdiplusLib, "GdipCreateHalftonePalette");
	if(!pfnGdipCreateHalftonePalette)
	{
		return NULL;
	}
	return pfnGdipCreateHalftonePalette();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

extern "C"
{
	Status
	WINAPI GdiplusStartup(OUT ULONG_PTR* token, const GdiplusStartupInput* input, OUT GdiplusStartupOutput* output)
	{
		return gdiplusWrapper.GdiplusStartup(token, input, output);
	}

	VOID
	WINAPI GdiplusShutdown(ULONG_PTR token)
	{
		gdiplusWrapper.GdiplusShutdown(token);
	}
}

void* WINAPI DllExports::GdipAlloc(size_t size)
{
	return gdiplusWrapper.GdipAlloc(size);
}

void WINAPI DllExports::GdipFree(void* ptr)
{
	return gdiplusWrapper.GdipFree(ptr);
}

GpStatus WINAPI DllExports::GdipCreatePath(GpFillMode brushMode, GpPath** path)
{
	return gdiplusWrapper.GdipCreatePath(brushMode, path);
}

GpStatus WINAPI DllExports::GdipCreatePath2(GDIPCONST GpPointF* points, GDIPCONST BYTE* types, INT count, GpFillMode fillMode, GpPath** path)
{
	return gdiplusWrapper.GdipCreatePath2(points, types, count, fillMode, path);
}

GpStatus WINAPI DllExports::GdipCreatePath2I(GDIPCONST GpPoint* points, GDIPCONST BYTE* types, INT count, GpFillMode fillMode, GpPath** path)
{
	return gdiplusWrapper.GdipCreatePath2I(points, types, count, fillMode, path);
}

GpStatus WINAPI DllExports::GdipClonePath(GpPath* path, GpPath** clonePath)
{
	return gdiplusWrapper.GdipClonePath(path, clonePath);
}

GpStatus WINAPI DllExports::GdipDeletePath(GpPath* path)
{
	return gdiplusWrapper.GdipDeletePath(path);
}

GpStatus WINAPI DllExports::GdipResetPath(GpPath* path)
{
	return gdiplusWrapper.GdipResetPath(path);
}

GpStatus WINAPI DllExports::GdipGetPointCount(GpPath* path, INT* count)
{
	return gdiplusWrapper.GdipGetPointCount(path, count);
}

GpStatus WINAPI DllExports::GdipGetPathTypes(GpPath* path, BYTE* types, INT count)
{
	return gdiplusWrapper.GdipGetPathTypes(path, types, count);
}

GpStatus WINAPI DllExports::GdipGetPathPoints(GpPath* path, GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipGetPathPoints(path, points, count);
}

GpStatus WINAPI DllExports::GdipGetPathPointsI(GpPath* path, GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipGetPathPointsI(path, points, count);
}

GpStatus WINAPI DllExports::GdipGetPathFillMode(GpPath* path, GpFillMode* fillmode)
{
	return gdiplusWrapper.GdipGetPathFillMode(path, fillmode);
}

GpStatus WINAPI DllExports::GdipSetPathFillMode(GpPath* path, GpFillMode fillmode)
{
	return gdiplusWrapper.GdipSetPathFillMode(path, fillmode);
}

GpStatus WINAPI DllExports::GdipGetPathData(GpPath* path, GpPathData* pathData)
{
	return gdiplusWrapper.GdipGetPathData(path, pathData);
}

GpStatus WINAPI DllExports::GdipStartPathFigure(GpPath* path)
{
	return gdiplusWrapper.GdipStartPathFigure(path);
}

GpStatus WINAPI DllExports::GdipClosePathFigure(GpPath* path)
{
	return gdiplusWrapper.GdipClosePathFigure(path);
}

GpStatus WINAPI DllExports::GdipClosePathFigures(GpPath* path)
{
	return gdiplusWrapper.GdipClosePathFigures(path);
}

GpStatus WINAPI DllExports::GdipSetPathMarker(GpPath* path)
{
	return gdiplusWrapper.GdipSetPathMarker(path);
}

GpStatus WINAPI DllExports::GdipClearPathMarkers(GpPath* path)
{
	return gdiplusWrapper.GdipClearPathMarkers(path);
}

GpStatus WINAPI DllExports::GdipReversePath(GpPath* path)
{
	return gdiplusWrapper.GdipReversePath(path);
}

GpStatus WINAPI DllExports::GdipGetPathLastPoint(GpPath* path, GpPointF* lastPoint)
{
	return gdiplusWrapper.GdipGetPathLastPoint(path, lastPoint);
}

GpStatus WINAPI DllExports::GdipAddPathLine(GpPath* path, REAL x1, REAL y1, REAL x2, REAL y2)
{
	return gdiplusWrapper.GdipAddPathLine(path, x1, y1, x2, y2);
}

GpStatus WINAPI DllExports::GdipAddPathLine2(GpPath* path, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipAddPathLine2(path, points, count);
}

GpStatus WINAPI DllExports::GdipAddPathArc(GpPath* path, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
{
	return gdiplusWrapper.GdipAddPathArc(path, x, y, width, height, startAngle, sweepAngle);
}

GpStatus WINAPI DllExports::GdipAddPathBezier(GpPath* path, REAL x1, REAL y1, REAL x2, REAL y2, REAL x3, REAL y3, REAL x4, REAL y4)
{
	return gdiplusWrapper.GdipAddPathBezier(path, x1, y1, x2, y2, x3, y3, x4, y4);
}

GpStatus WINAPI DllExports::GdipAddPathBeziers(GpPath* path, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipAddPathBeziers(path, points, count);
}

GpStatus WINAPI DllExports::GdipAddPathCurve(GpPath* path, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipAddPathCurve(path, points, count);
}

GpStatus WINAPI DllExports::GdipAddPathCurve2(GpPath* path, GDIPCONST GpPointF* points, INT count, REAL tension)
{
	return gdiplusWrapper.GdipAddPathCurve2(path, points, count, tension);
}

GpStatus WINAPI DllExports::GdipAddPathCurve3(GpPath* path, GDIPCONST GpPointF* points, INT count, INT offset, INT numberOfSegments, REAL tension)
{
	return gdiplusWrapper.GdipAddPathCurve3(path, points, count, offset, numberOfSegments, tension);
}

GpStatus WINAPI DllExports::GdipAddPathClosedCurve(GpPath* path, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipAddPathClosedCurve(path, points, count);
}

GpStatus WINAPI DllExports::GdipAddPathClosedCurve2(GpPath* path, GDIPCONST GpPointF* points, INT count, REAL tension)
{
	return gdiplusWrapper.GdipAddPathClosedCurve2(path, points, count, tension);
}

GpStatus WINAPI DllExports::GdipAddPathRectangle(GpPath* path, REAL x, REAL y, REAL width, REAL height)
{
	return gdiplusWrapper.GdipAddPathRectangle(path, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipAddPathRectangles(GpPath* path, GDIPCONST GpRectF* rects, INT count)
{
	return gdiplusWrapper.GdipAddPathRectangles(path, rects, count);
}

GpStatus WINAPI DllExports::GdipAddPathEllipse(GpPath* path, REAL x, REAL y, REAL width, REAL height)
{
	return gdiplusWrapper.GdipAddPathEllipse(path, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipAddPathPie(GpPath* path, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
{
	return gdiplusWrapper.GdipAddPathPie(path, x, y, width, height, startAngle, sweepAngle);
}

GpStatus WINAPI DllExports::GdipAddPathPolygon(GpPath* path, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipAddPathPolygon(path, points, count);
}

GpStatus WINAPI DllExports::GdipAddPathPath(GpPath* path, GDIPCONST GpPath* addingPath, BOOL connect)
{
	return gdiplusWrapper.GdipAddPathPath(path, addingPath, connect);
}

GpStatus WINAPI DllExports::GdipAddPathString(GpPath* path, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFontFamily* family, INT style, REAL emSize, GDIPCONST RectF* layoutRect, GDIPCONST GpStringFormat* format)
{
	return gdiplusWrapper.GdipAddPathString(path, string, length, family, style, emSize, layoutRect, format);
}

GpStatus WINAPI DllExports::GdipAddPathStringI(GpPath* path, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFontFamily* family, INT style, REAL emSize, GDIPCONST Rect* layoutRect, GDIPCONST GpStringFormat* format)
{
	return gdiplusWrapper.GdipAddPathStringI(path, string, length, family, style, emSize, layoutRect, format);
}

GpStatus WINAPI DllExports::GdipAddPathLineI(GpPath* path, INT x1, INT y1, INT x2, INT y2)
{
	return gdiplusWrapper.GdipAddPathLineI(path, x1, y1, x2, y2);
}

GpStatus WINAPI DllExports::GdipAddPathLine2I(GpPath* path, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipAddPathLine2I(path, points, count);
}

GpStatus WINAPI DllExports::GdipAddPathArcI(GpPath* path, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
{
	return gdiplusWrapper.GdipAddPathArcI(path, x, y, width, height, startAngle, sweepAngle);
}

GpStatus WINAPI DllExports::GdipAddPathBezierI(GpPath* path, INT x1, INT y1, INT x2, INT y2, INT x3, INT y3, INT x4, INT y4)
{
	return gdiplusWrapper.GdipAddPathBezierI(path, x1, y1, x2, y2, x3, y3, x4, y4);
}

GpStatus WINAPI DllExports::GdipAddPathBeziersI(GpPath* path, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipAddPathBeziersI(path, points, count);
}

GpStatus WINAPI DllExports::GdipAddPathCurveI(GpPath* path, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipAddPathCurveI(path, points, count);
}

GpStatus WINAPI DllExports::GdipAddPathCurve2I(GpPath* path, GDIPCONST GpPoint* points, INT count, REAL tension)
{
	return gdiplusWrapper.GdipAddPathCurve2I(path, points, count, tension);
}

GpStatus WINAPI DllExports::GdipAddPathCurve3I(GpPath* path, GDIPCONST GpPoint* points, INT count, INT offset, INT numberOfSegments, REAL tension)
{
	return gdiplusWrapper.GdipAddPathCurve3I(path, points, count, offset, numberOfSegments, tension);
}

GpStatus WINAPI DllExports::GdipAddPathClosedCurveI(GpPath* path, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipAddPathClosedCurveI(path, points, count);
}

GpStatus WINAPI DllExports::GdipAddPathClosedCurve2I(GpPath* path, GDIPCONST GpPoint* points, INT count, REAL tension)
{
	return gdiplusWrapper.GdipAddPathClosedCurve2I(path, points, count, tension);
}

GpStatus WINAPI DllExports::GdipAddPathRectangleI(GpPath* path, INT x, INT y, INT width, INT height)
{
	return gdiplusWrapper.GdipAddPathRectangleI(path, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipAddPathRectanglesI(GpPath* path, GDIPCONST GpRect* rects, INT count)
{
	return gdiplusWrapper.GdipAddPathRectanglesI(path, rects, count);
}

GpStatus WINAPI DllExports::GdipAddPathEllipseI(GpPath* path, INT x, INT y, INT width, INT height)
{
	return gdiplusWrapper.GdipAddPathEllipseI(path, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipAddPathPieI(GpPath* path, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
{
	return gdiplusWrapper.GdipAddPathPieI(path, x, y, width, height, startAngle, sweepAngle);
}

GpStatus WINAPI DllExports::GdipAddPathPolygonI(GpPath* path, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipAddPathPolygonI(path, points, count);
}

GpStatus WINAPI DllExports::GdipFlattenPath(GpPath* path, GpMatrix* matrix, REAL flatness)
{
	return gdiplusWrapper.GdipFlattenPath(path, matrix, flatness);
}

GpStatus WINAPI DllExports::GdipWindingModeOutline(GpPath* path, GpMatrix* matrix, REAL flatness)
{
	return gdiplusWrapper.GdipWindingModeOutline(path, matrix, flatness);
}

GpStatus WINAPI DllExports::GdipWidenPath(GpPath* nativePath, GpPen* pen, GpMatrix* matrix, REAL flatness)
{
	return gdiplusWrapper.GdipWidenPath(nativePath, pen, matrix, flatness);
}

GpStatus WINAPI DllExports::GdipWarpPath(GpPath* path, GpMatrix* matrix, GDIPCONST GpPointF* points, INT count, REAL srcx, REAL srcy, REAL srcwidth, REAL srcheight, WarpMode warpMode, REAL flatness)
{
	return gdiplusWrapper.GdipWarpPath(path, matrix, points, count, srcx, srcy, srcwidth, srcheight, warpMode, flatness);
}

GpStatus WINAPI DllExports::GdipTransformPath(GpPath* path, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipTransformPath(path, matrix);
}

GpStatus WINAPI DllExports::GdipGetPathWorldBounds(GpPath* path, GpRectF* bounds, GDIPCONST GpMatrix* matrix, GDIPCONST GpPen* pen)
{
	return gdiplusWrapper.GdipGetPathWorldBounds(path, bounds, matrix, pen);
}

GpStatus WINAPI DllExports::GdipGetPathWorldBoundsI(GpPath* path, GpRect* bounds, GDIPCONST GpMatrix* matrix, GDIPCONST GpPen* pen)
{
	return gdiplusWrapper.GdipGetPathWorldBoundsI(path, bounds, matrix, pen);
}

GpStatus WINAPI DllExports::GdipIsVisiblePathPoint(GpPath* path, REAL x, REAL y, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisiblePathPoint(path, x, y, graphics, result);
}

GpStatus WINAPI DllExports::GdipIsVisiblePathPointI(GpPath* path, INT x, INT y, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisiblePathPointI(path, x, y, graphics, result);
}

GpStatus WINAPI DllExports::GdipIsOutlineVisiblePathPoint(GpPath* path, REAL x, REAL y, GpPen* pen, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsOutlineVisiblePathPoint(path, x, y, pen, graphics, result);
}

GpStatus WINAPI DllExports::GdipIsOutlineVisiblePathPointI(GpPath* path, INT x, INT y, GpPen* pen, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsOutlineVisiblePathPointI(path, x, y, pen, graphics, result);
}

GpStatus WINAPI DllExports::GdipCreatePathIter(GpPathIterator** iterator, GpPath* path)
{
	return gdiplusWrapper.GdipCreatePathIter(iterator, path);
}

GpStatus WINAPI DllExports::GdipDeletePathIter(GpPathIterator* iterator)
{
	return gdiplusWrapper.GdipDeletePathIter(iterator);
}

GpStatus WINAPI DllExports::GdipPathIterNextSubpath(GpPathIterator* iterator, INT* resultCount, INT* startIndex, INT* endIndex, BOOL* isClosed)
{
	return gdiplusWrapper.GdipPathIterNextSubpath(iterator, resultCount, startIndex, endIndex, isClosed);
}

GpStatus WINAPI DllExports::GdipPathIterNextSubpathPath(GpPathIterator* iterator, INT* resultCount, GpPath* path, BOOL* isClosed)
{
	return gdiplusWrapper.GdipPathIterNextSubpathPath(iterator, resultCount, path, isClosed);
}

GpStatus WINAPI DllExports::GdipPathIterNextPathType(GpPathIterator* iterator, INT* resultCount, BYTE* pathType, INT* startIndex, INT* endIndex)
{
	return gdiplusWrapper.GdipPathIterNextPathType(iterator, resultCount, pathType, startIndex, endIndex);
}

GpStatus WINAPI DllExports::GdipPathIterNextMarker(GpPathIterator* iterator, INT* resultCount, INT* startIndex, INT* endIndex)
{
	return gdiplusWrapper.GdipPathIterNextMarker(iterator, resultCount, startIndex, endIndex);
}

GpStatus WINAPI DllExports::GdipPathIterNextMarkerPath(GpPathIterator* iterator, INT* resultCount, GpPath* path)
{
	return gdiplusWrapper.GdipPathIterNextMarkerPath(iterator, resultCount, path);
}

GpStatus WINAPI DllExports::GdipPathIterGetCount(GpPathIterator* iterator, INT* count)
{
	return gdiplusWrapper.GdipPathIterGetCount(iterator, count);
}

GpStatus WINAPI DllExports::GdipPathIterGetSubpathCount(GpPathIterator* iterator, INT* count)
{
	return gdiplusWrapper.GdipPathIterGetSubpathCount(iterator, count);
}

GpStatus WINAPI DllExports::GdipPathIterIsValid(GpPathIterator* iterator, BOOL* valid)
{
	return gdiplusWrapper.GdipPathIterIsValid(iterator, valid);
}

GpStatus WINAPI DllExports::GdipPathIterHasCurve(GpPathIterator* iterator, BOOL* hasCurve)
{
	return gdiplusWrapper.GdipPathIterHasCurve(iterator, hasCurve);
}

GpStatus WINAPI DllExports::GdipPathIterRewind(GpPathIterator* iterator)
{
	return gdiplusWrapper.GdipPathIterRewind(iterator);
}

GpStatus WINAPI DllExports::GdipPathIterEnumerate(GpPathIterator* iterator, INT* resultCount, GpPointF* points, BYTE* types, INT count)
{
	return gdiplusWrapper.GdipPathIterEnumerate(iterator, resultCount, points, types, count);
}

GpStatus WINAPI DllExports::GdipPathIterCopyData(GpPathIterator* iterator, INT* resultCount, GpPointF* points, BYTE* types, INT startIndex, INT endIndex)
{
	return gdiplusWrapper.GdipPathIterCopyData(iterator, resultCount, points, types, startIndex, endIndex);
}

GpStatus WINAPI DllExports::GdipCreateMatrix(GpMatrix** matrix)
{
	return gdiplusWrapper.GdipCreateMatrix(matrix);
}

GpStatus WINAPI DllExports::GdipCreateMatrix2(REAL m11, REAL m12, REAL m21, REAL m22, REAL dx, REAL dy, GpMatrix** matrix)
{
	return gdiplusWrapper.GdipCreateMatrix2(m11, m12, m21, m22, dx, dy, matrix);
}

GpStatus WINAPI DllExports::GdipCreateMatrix3(GDIPCONST GpRectF* rect, GDIPCONST GpPointF* dstplg, GpMatrix** matrix)
{
	return gdiplusWrapper.GdipCreateMatrix3(rect, dstplg, matrix);
}

GpStatus WINAPI DllExports::GdipCreateMatrix3I(GDIPCONST GpRect* rect, GDIPCONST GpPoint* dstplg, GpMatrix** matrix)
{
	return gdiplusWrapper.GdipCreateMatrix3I(rect, dstplg, matrix);
}

GpStatus WINAPI DllExports::GdipCloneMatrix(GpMatrix* matrix, GpMatrix** cloneMatrix)
{
	return gdiplusWrapper.GdipCloneMatrix(matrix, cloneMatrix);
}

GpStatus WINAPI DllExports::GdipDeleteMatrix(GpMatrix* matrix)
{
	return gdiplusWrapper.GdipDeleteMatrix(matrix);
}

GpStatus WINAPI DllExports::GdipSetMatrixElements(GpMatrix* matrix, REAL m11, REAL m12, REAL m21, REAL m22, REAL dx, REAL dy)
{
	return gdiplusWrapper.GdipSetMatrixElements(matrix, m11, m12, m21, m22, dx, dy);
}

GpStatus WINAPI DllExports::GdipMultiplyMatrix(GpMatrix* matrix, GpMatrix* matrix2, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipMultiplyMatrix(matrix, matrix2, order);
}

GpStatus WINAPI DllExports::GdipTranslateMatrix(GpMatrix* matrix, REAL offsetX, REAL offsetY, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipTranslateMatrix(matrix, offsetX, offsetY, order);
}

GpStatus WINAPI DllExports::GdipScaleMatrix(GpMatrix* matrix, REAL scaleX, REAL scaleY, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipScaleMatrix(matrix, scaleX, scaleY, order);
}

GpStatus WINAPI DllExports::GdipRotateMatrix(GpMatrix* matrix, REAL angle, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipRotateMatrix(matrix, angle, order);
}

GpStatus WINAPI DllExports::GdipShearMatrix(GpMatrix* matrix, REAL shearX, REAL shearY, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipShearMatrix(matrix, shearX, shearY, order);
}

GpStatus WINAPI DllExports::GdipInvertMatrix(GpMatrix* matrix)
{
	return gdiplusWrapper.GdipInvertMatrix(matrix);
}

GpStatus WINAPI DllExports::GdipTransformMatrixPoints(GpMatrix* matrix, GpPointF* pts, INT count)
{
	return gdiplusWrapper.GdipTransformMatrixPoints(matrix, pts, count);
}

GpStatus WINAPI DllExports::GdipTransformMatrixPointsI(GpMatrix* matrix, GpPoint* pts, INT count)
{
	return gdiplusWrapper.GdipTransformMatrixPointsI(matrix, pts, count);
}

GpStatus WINAPI DllExports::GdipVectorTransformMatrixPoints(GpMatrix* matrix, GpPointF* pts, INT count)
{
	return gdiplusWrapper.GdipVectorTransformMatrixPoints(matrix, pts, count);
}

GpStatus WINAPI DllExports::GdipVectorTransformMatrixPointsI(GpMatrix* matrix, GpPoint* pts, INT count)
{
	return gdiplusWrapper.GdipVectorTransformMatrixPointsI(matrix, pts, count);
}

GpStatus WINAPI DllExports::GdipGetMatrixElements(GDIPCONST GpMatrix* matrix, REAL* matrixOut)
{
	return gdiplusWrapper.GdipGetMatrixElements(matrix, matrixOut);
}

GpStatus WINAPI DllExports::GdipIsMatrixInvertible(GDIPCONST GpMatrix* matrix, BOOL* result)
{
	return gdiplusWrapper.GdipIsMatrixInvertible(matrix, result);
}

GpStatus WINAPI DllExports::GdipIsMatrixIdentity(GDIPCONST GpMatrix* matrix, BOOL* result)
{
	return gdiplusWrapper.GdipIsMatrixIdentity(matrix, result);
}

GpStatus WINAPI DllExports::GdipIsMatrixEqual(GDIPCONST GpMatrix* matrix, GDIPCONST GpMatrix* matrix2, BOOL* result)
{
	return gdiplusWrapper.GdipIsMatrixEqual(matrix, matrix2, result);
}

GpStatus WINAPI DllExports::GdipCreateRegion(GpRegion** region)
{
	return gdiplusWrapper.GdipCreateRegion(region);
}

GpStatus WINAPI DllExports::GdipCreateRegionRect(GDIPCONST GpRectF* rect, GpRegion** region)
{
	return gdiplusWrapper.GdipCreateRegionRect(rect, region);
}

GpStatus WINAPI DllExports::GdipCreateRegionRectI(GDIPCONST GpRect* rect, GpRegion** region)
{
	return gdiplusWrapper.GdipCreateRegionRectI(rect, region);
}

GpStatus WINAPI DllExports::GdipCreateRegionPath(GpPath* path, GpRegion** region)
{
	return gdiplusWrapper.GdipCreateRegionPath(path, region);
}

GpStatus WINAPI DllExports::GdipCreateRegionRgnData(GDIPCONST BYTE* regionData, INT size, GpRegion** region)
{
	return gdiplusWrapper.GdipCreateRegionRgnData(regionData, size, region);
}

GpStatus WINAPI DllExports::GdipCreateRegionHrgn(HRGN hRgn, GpRegion** region)
{
	return gdiplusWrapper.GdipCreateRegionHrgn(hRgn, region);
}

GpStatus WINAPI DllExports::GdipCloneRegion(GpRegion* region, GpRegion** cloneRegion)
{
	return gdiplusWrapper.GdipCloneRegion(region, cloneRegion);
}

GpStatus WINAPI DllExports::GdipDeleteRegion(GpRegion* region)
{
	return gdiplusWrapper.GdipDeleteRegion(region);
}

GpStatus WINAPI DllExports::GdipSetInfinite(GpRegion* region)
{
	return gdiplusWrapper.GdipSetInfinite(region);
}

GpStatus WINAPI DllExports::GdipSetEmpty(GpRegion* region)
{
	return gdiplusWrapper.GdipSetEmpty(region);
}

GpStatus WINAPI DllExports::GdipCombineRegionRect(GpRegion* region, GDIPCONST GpRectF* rect, CombineMode combineMode)
{
	return gdiplusWrapper.GdipCombineRegionRect(region, rect, combineMode);
}

GpStatus WINAPI DllExports::GdipCombineRegionRectI(GpRegion* region, GDIPCONST GpRect* rect, CombineMode combineMode)
{
	return gdiplusWrapper.GdipCombineRegionRectI(region, rect, combineMode);
}

GpStatus WINAPI DllExports::GdipCombineRegionPath(GpRegion* region, GpPath* path, CombineMode combineMode)
{
	return gdiplusWrapper.GdipCombineRegionPath(region, path, combineMode);
}

GpStatus WINAPI DllExports::GdipCombineRegionRegion(GpRegion* region, GpRegion* region2, CombineMode combineMode)
{
	return gdiplusWrapper.GdipCombineRegionRegion(region, region2, combineMode);
}

GpStatus WINAPI DllExports::GdipTranslateRegion(GpRegion* region, REAL dx, REAL dy)
{
	return gdiplusWrapper.GdipTranslateRegion(region, dx, dy);
}

GpStatus WINAPI DllExports::GdipTranslateRegionI(GpRegion* region, INT dx, INT dy)
{
	return gdiplusWrapper.GdipTranslateRegionI(region, dx, dy);
}

GpStatus WINAPI DllExports::GdipTransformRegion(GpRegion* region, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipTransformRegion(region, matrix);
}

GpStatus WINAPI DllExports::GdipGetRegionBounds(GpRegion* region, GpGraphics* graphics, GpRectF* rect)
{
	return gdiplusWrapper.GdipGetRegionBounds(region, graphics, rect);
}

GpStatus WINAPI DllExports::GdipGetRegionBoundsI(GpRegion* region, GpGraphics* graphics, GpRect* rect)
{
	return gdiplusWrapper.GdipGetRegionBoundsI(region, graphics, rect);
}

GpStatus WINAPI DllExports::GdipGetRegionHRgn(GpRegion* region, GpGraphics* graphics, HRGN* hRgn)
{
	return gdiplusWrapper.GdipGetRegionHRgn(region, graphics, hRgn);
}

GpStatus WINAPI DllExports::GdipIsEmptyRegion(GpRegion* region, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsEmptyRegion(region, graphics, result);
}

GpStatus WINAPI DllExports::GdipIsInfiniteRegion(GpRegion* region, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsInfiniteRegion(region, graphics, result);
}

GpStatus WINAPI DllExports::GdipIsEqualRegion(GpRegion* region, GpRegion* region2, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsEqualRegion(region, region2, graphics, result);
}

GpStatus WINAPI DllExports::GdipGetRegionDataSize(GpRegion* region, UINT* bufferSize)
{
	return gdiplusWrapper.GdipGetRegionDataSize(region, bufferSize);
}

GpStatus WINAPI DllExports::GdipGetRegionData(GpRegion* region, BYTE* buffer, UINT bufferSize, UINT* sizeFilled)
{
	return gdiplusWrapper.GdipGetRegionData(region, buffer, bufferSize, sizeFilled);
}

GpStatus WINAPI DllExports::GdipIsVisibleRegionPoint(GpRegion* region, REAL x, REAL y, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisibleRegionPoint(region, x, y, graphics, result);
}

GpStatus WINAPI DllExports::GdipIsVisibleRegionPointI(GpRegion* region, INT x, INT y, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisibleRegionPointI(region, x, y, graphics, result);
}

GpStatus WINAPI DllExports::GdipIsVisibleRegionRect(GpRegion* region, REAL x, REAL y, REAL width, REAL height, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisibleRegionRect(region, x, y, width, height, graphics, result);
}

GpStatus WINAPI DllExports::GdipIsVisibleRegionRectI(GpRegion* region, INT x, INT y, INT width, INT height, GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisibleRegionRectI(region, x, y, width, height, graphics, result);
}

GpStatus WINAPI DllExports::GdipGetRegionScansCount(GpRegion* region, UINT* count, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipGetRegionScansCount(region, count, matrix);
}

GpStatus WINAPI DllExports::GdipGetRegionScans(GpRegion* region, GpRectF* rects, INT* count, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipGetRegionScans(region, rects, count, matrix);
}

GpStatus WINAPI DllExports::GdipGetRegionScansI(GpRegion* region, GpRect* rects, INT* count, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipGetRegionScansI(region, rects, count, matrix);
}

GpStatus WINAPI DllExports::GdipCloneBrush(GpBrush* brush, GpBrush** cloneBrush)
{
	return gdiplusWrapper.GdipCloneBrush(brush, cloneBrush);
}

GpStatus WINAPI DllExports::GdipDeleteBrush(GpBrush* brush)
{
	return gdiplusWrapper.GdipDeleteBrush(brush);
}

GpStatus WINAPI DllExports::GdipGetBrushType(GpBrush* brush, GpBrushType* type)
{
	return gdiplusWrapper.GdipGetBrushType(brush, type);
}

GpStatus WINAPI DllExports::GdipCreateHatchBrush(GpHatchStyle hatchstyle, ARGB forecol, ARGB backcol, GpHatch** brush)
{
	return gdiplusWrapper.GdipCreateHatchBrush(hatchstyle, forecol, backcol, brush);
}

GpStatus WINAPI DllExports::GdipGetHatchStyle(GpHatch* brush, GpHatchStyle* hatchstyle)
{
	return gdiplusWrapper.GdipGetHatchStyle(brush, hatchstyle);
}

GpStatus WINAPI DllExports::GdipGetHatchForegroundColor(GpHatch* brush, ARGB* forecol)
{
	return gdiplusWrapper.GdipGetHatchForegroundColor(brush, forecol);
}

GpStatus WINAPI DllExports::GdipGetHatchBackgroundColor(GpHatch* brush, ARGB* backcol)
{
	return gdiplusWrapper.GdipGetHatchBackgroundColor(brush, backcol);
}

GpStatus WINAPI DllExports::GdipCreateTexture(GpImage* image, GpWrapMode wrapmode, GpTexture** texture)
{
	return gdiplusWrapper.GdipCreateTexture(image, wrapmode, texture);
}

GpStatus WINAPI DllExports::GdipCreateTexture2(GpImage* image, GpWrapMode wrapmode, REAL x, REAL y, REAL width, REAL height, GpTexture** texture)
{
	return gdiplusWrapper.GdipCreateTexture2(image, wrapmode, x, y, width, height, texture);
}

GpStatus WINAPI DllExports::GdipCreateTextureIA(GpImage* image, GDIPCONST GpImageAttributes* imageAttributes, REAL x, REAL y, REAL width, REAL height, GpTexture** texture)
{
	return gdiplusWrapper.GdipCreateTextureIA(image, imageAttributes, x, y, width, height, texture);
}

GpStatus WINAPI DllExports::GdipCreateTexture2I(GpImage* image, GpWrapMode wrapmode, INT x, INT y, INT width, INT height, GpTexture** texture)
{
	return gdiplusWrapper.GdipCreateTexture2I(image, wrapmode, x, y, width, height, texture);
}

GpStatus WINAPI DllExports::GdipCreateTextureIAI(GpImage* image, GDIPCONST GpImageAttributes* imageAttributes, INT x, INT y, INT width, INT height, GpTexture** texture)
{
	return gdiplusWrapper.GdipCreateTextureIAI(image, imageAttributes, x, y, width, height, texture);
}

GpStatus WINAPI DllExports::GdipGetTextureTransform(GpTexture* brush, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipGetTextureTransform(brush, matrix);
}

GpStatus WINAPI DllExports::GdipSetTextureTransform(GpTexture* brush, GDIPCONST GpMatrix* matrix)
{
	return gdiplusWrapper.GdipSetTextureTransform(brush, matrix);
}

GpStatus WINAPI DllExports::GdipResetTextureTransform(GpTexture* brush)
{
	return gdiplusWrapper.GdipResetTextureTransform(brush);
}

GpStatus WINAPI DllExports::GdipMultiplyTextureTransform(GpTexture* brush, GDIPCONST GpMatrix* matrix, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipMultiplyTextureTransform(brush, matrix, order);
}

GpStatus WINAPI DllExports::GdipTranslateTextureTransform(GpTexture* brush, REAL dx, REAL dy, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipTranslateTextureTransform(brush, dx, dy, order);
}

GpStatus WINAPI DllExports::GdipScaleTextureTransform(GpTexture* brush, REAL sx, REAL sy, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipScaleTextureTransform(brush, sx, sy, order);
}

GpStatus WINAPI DllExports::GdipRotateTextureTransform(GpTexture* brush, REAL angle, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipRotateTextureTransform(brush, angle, order);
}

GpStatus WINAPI DllExports::GdipSetTextureWrapMode(GpTexture* brush, GpWrapMode wrapmode)
{
	return gdiplusWrapper.GdipSetTextureWrapMode(brush, wrapmode);
}

GpStatus WINAPI DllExports::GdipGetTextureWrapMode(GpTexture* brush, GpWrapMode* wrapmode)
{
	return gdiplusWrapper.GdipGetTextureWrapMode(brush, wrapmode);
}

GpStatus WINAPI DllExports::GdipGetTextureImage(GpTexture* brush, GpImage** image)
{
	return gdiplusWrapper.GdipGetTextureImage(brush, image);
}

GpStatus WINAPI DllExports::GdipCreateSolidFill(ARGB color, GpSolidFill** brush)
{
	return gdiplusWrapper.GdipCreateSolidFill(color, brush);
}

GpStatus WINAPI DllExports::GdipSetSolidFillColor(GpSolidFill* brush, ARGB color)
{
	return gdiplusWrapper.GdipSetSolidFillColor(brush, color);
}

GpStatus WINAPI DllExports::GdipGetSolidFillColor(GpSolidFill* brush, ARGB* color)
{
	return gdiplusWrapper.GdipGetSolidFillColor(brush, color);
}

GpStatus WINAPI DllExports::GdipCreateLineBrush(GDIPCONST GpPointF* point1, GDIPCONST GpPointF* point2, ARGB color1, ARGB color2, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	return gdiplusWrapper.GdipCreateLineBrush(point1, point2, color1, color2, wrapMode, lineGradient);
}

GpStatus WINAPI DllExports::GdipCreateLineBrushI(GDIPCONST GpPoint* point1, GDIPCONST GpPoint* point2, ARGB color1, ARGB color2, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	return gdiplusWrapper.GdipCreateLineBrushI(point1, point2, color1, color2, wrapMode, lineGradient);
}

GpStatus WINAPI DllExports::GdipCreateLineBrushFromRect(GDIPCONST GpRectF* rect, ARGB color1, ARGB color2, LinearGradientMode mode, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	return gdiplusWrapper.GdipCreateLineBrushFromRect(rect, color1, color2, mode, wrapMode, lineGradient);
}

GpStatus WINAPI DllExports::GdipCreateLineBrushFromRectI(GDIPCONST GpRect* rect, ARGB color1, ARGB color2, LinearGradientMode mode, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	return gdiplusWrapper.GdipCreateLineBrushFromRectI(rect, color1, color2, mode, wrapMode, lineGradient);
}

GpStatus WINAPI DllExports::GdipCreateLineBrushFromRectWithAngle(GDIPCONST GpRectF* rect, ARGB color1, ARGB color2, REAL angle, BOOL isAngleScalable, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	return gdiplusWrapper.GdipCreateLineBrushFromRectWithAngle(rect, color1, color2, angle, isAngleScalable, wrapMode, lineGradient);
}

GpStatus WINAPI DllExports::GdipCreateLineBrushFromRectWithAngleI(GDIPCONST GpRect* rect, ARGB color1, ARGB color2, REAL angle, BOOL isAngleScalable, GpWrapMode wrapMode, GpLineGradient** lineGradient)
{
	return gdiplusWrapper.GdipCreateLineBrushFromRectWithAngleI(rect, color1, color2, angle, isAngleScalable, wrapMode, lineGradient);
}

GpStatus WINAPI DllExports::GdipSetLineColors(GpLineGradient* brush, ARGB color1, ARGB color2)
{
	return gdiplusWrapper.GdipSetLineColors(brush, color1, color2);
}

GpStatus WINAPI DllExports::GdipGetLineColors(GpLineGradient* brush, ARGB* colors)
{
	return gdiplusWrapper.GdipGetLineColors(brush, colors);
}

GpStatus WINAPI DllExports::GdipGetLineRect(GpLineGradient* brush, GpRectF* rect)
{
	return gdiplusWrapper.GdipGetLineRect(brush, rect);
}

GpStatus WINAPI DllExports::GdipGetLineRectI(GpLineGradient* brush, GpRect* rect)
{
	return gdiplusWrapper.GdipGetLineRectI(brush, rect);
}

GpStatus WINAPI DllExports::GdipSetLineGammaCorrection(GpLineGradient* brush, BOOL useGammaCorrection)
{
	return gdiplusWrapper.GdipSetLineGammaCorrection(brush, useGammaCorrection);
}

GpStatus WINAPI DllExports::GdipGetLineGammaCorrection(GpLineGradient* brush, BOOL* useGammaCorrection)
{
	return gdiplusWrapper.GdipGetLineGammaCorrection(brush, useGammaCorrection);
}

GpStatus WINAPI DllExports::GdipGetLineBlendCount(GpLineGradient* brush, INT* count)
{
	return gdiplusWrapper.GdipGetLineBlendCount(brush, count);
}

GpStatus WINAPI DllExports::GdipGetLineBlend(GpLineGradient* brush, REAL* blend, REAL* positions, INT count)
{
	return gdiplusWrapper.GdipGetLineBlend(brush, blend, positions, count);
}

GpStatus WINAPI DllExports::GdipSetLineBlend(GpLineGradient* brush, GDIPCONST REAL* blend, GDIPCONST REAL* positions, INT count)
{
	return gdiplusWrapper.GdipSetLineBlend(brush, blend, positions, count);
}

GpStatus WINAPI DllExports::GdipGetLinePresetBlendCount(GpLineGradient* brush, INT* count)
{
	return gdiplusWrapper.GdipGetLinePresetBlendCount(brush, count);
}

GpStatus WINAPI DllExports::GdipGetLinePresetBlend(GpLineGradient* brush, ARGB* blend, REAL* positions, INT count)
{
	return gdiplusWrapper.GdipGetLinePresetBlend(brush, blend, positions, count);
}

GpStatus WINAPI DllExports::GdipSetLinePresetBlend(GpLineGradient* brush, GDIPCONST ARGB* blend, GDIPCONST REAL* positions, INT count)
{
	return gdiplusWrapper.GdipSetLinePresetBlend(brush, blend, positions, count);
}

GpStatus WINAPI DllExports::GdipSetLineSigmaBlend(GpLineGradient* brush, REAL focus, REAL scale)
{
	return gdiplusWrapper.GdipSetLineSigmaBlend(brush, focus, scale);
}

GpStatus WINAPI DllExports::GdipSetLineLinearBlend(GpLineGradient* brush, REAL focus, REAL scale)
{
	return gdiplusWrapper.GdipSetLineLinearBlend(brush, focus, scale);
}

GpStatus WINAPI DllExports::GdipSetLineWrapMode(GpLineGradient* brush, GpWrapMode wrapmode)
{
	return gdiplusWrapper.GdipSetLineWrapMode(brush, wrapmode);
}

GpStatus WINAPI DllExports::GdipGetLineWrapMode(GpLineGradient* brush, GpWrapMode* wrapmode)
{
	return gdiplusWrapper.GdipGetLineWrapMode(brush, wrapmode);
}

GpStatus WINAPI DllExports::GdipGetLineTransform(GpLineGradient* brush, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipGetLineTransform(brush, matrix);
}

GpStatus WINAPI DllExports::GdipSetLineTransform(GpLineGradient* brush, GDIPCONST GpMatrix* matrix)
{
	return gdiplusWrapper.GdipSetLineTransform(brush, matrix);
}

GpStatus WINAPI DllExports::GdipResetLineTransform(GpLineGradient* brush)
{
	return gdiplusWrapper.GdipResetLineTransform(brush);
}

GpStatus WINAPI DllExports::GdipMultiplyLineTransform(GpLineGradient* brush, GDIPCONST GpMatrix* matrix, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipMultiplyLineTransform(brush, matrix, order);
}

GpStatus WINAPI DllExports::GdipTranslateLineTransform(GpLineGradient* brush, REAL dx, REAL dy, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipTranslateLineTransform(brush, dx, dy, order);
}

GpStatus WINAPI DllExports::GdipScaleLineTransform(GpLineGradient* brush, REAL sx, REAL sy, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipScaleLineTransform(brush, sx, sy, order);
}

GpStatus WINAPI DllExports::GdipRotateLineTransform(GpLineGradient* brush, REAL angle, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipRotateLineTransform(brush, angle, order);
}

GpStatus WINAPI DllExports::GdipCreatePathGradient(GDIPCONST GpPointF* points, INT count, GpWrapMode wrapMode, GpPathGradient** polyGradient)
{
	return gdiplusWrapper.GdipCreatePathGradient(points, count, wrapMode, polyGradient);
}

GpStatus WINAPI DllExports::GdipCreatePathGradientI(GDIPCONST GpPoint* points, INT count, GpWrapMode wrapMode, GpPathGradient** polyGradient)
{
	return gdiplusWrapper.GdipCreatePathGradientI(points, count, wrapMode, polyGradient);
}

GpStatus WINAPI DllExports::GdipCreatePathGradientFromPath(GDIPCONST GpPath* path, GpPathGradient** polyGradient)
{
	return gdiplusWrapper.GdipCreatePathGradientFromPath(path, polyGradient);
}

GpStatus WINAPI DllExports::GdipGetPathGradientCenterColor(GpPathGradient* brush, ARGB* colors)
{
	return gdiplusWrapper.GdipGetPathGradientCenterColor(brush, colors);
}

GpStatus WINAPI DllExports::GdipSetPathGradientCenterColor(GpPathGradient* brush, ARGB colors)
{
	return gdiplusWrapper.GdipSetPathGradientCenterColor(brush, colors);
}

GpStatus WINAPI DllExports::GdipGetPathGradientSurroundColorsWithCount(GpPathGradient* brush, ARGB* color, INT* count)
{
	return gdiplusWrapper.GdipGetPathGradientSurroundColorsWithCount(brush, color, count);
}

GpStatus WINAPI DllExports::GdipSetPathGradientSurroundColorsWithCount(GpPathGradient* brush, GDIPCONST ARGB* color, INT* count)
{
	return gdiplusWrapper.GdipSetPathGradientSurroundColorsWithCount(brush, color, count);
}

GpStatus WINAPI DllExports::GdipGetPathGradientPath(GpPathGradient* brush, GpPath* path)
{
	return gdiplusWrapper.GdipGetPathGradientPath(brush, path);
}

GpStatus WINAPI DllExports::GdipSetPathGradientPath(GpPathGradient* brush, GDIPCONST GpPath* path)
{
	return gdiplusWrapper.GdipSetPathGradientPath(brush, path);
}

GpStatus WINAPI DllExports::GdipGetPathGradientCenterPoint(GpPathGradient* brush, GpPointF* points)
{
	return gdiplusWrapper.GdipGetPathGradientCenterPoint(brush, points);
}

GpStatus WINAPI DllExports::GdipGetPathGradientCenterPointI(GpPathGradient* brush, GpPoint* points)
{
	return gdiplusWrapper.GdipGetPathGradientCenterPointI(brush, points);
}

GpStatus WINAPI DllExports::GdipSetPathGradientCenterPoint(GpPathGradient* brush, GDIPCONST GpPointF* points)
{
	return gdiplusWrapper.GdipSetPathGradientCenterPoint(brush, points);
}

GpStatus WINAPI DllExports::GdipSetPathGradientCenterPointI(GpPathGradient* brush, GDIPCONST GpPoint* points)
{
	return gdiplusWrapper.GdipSetPathGradientCenterPointI(brush, points);
}

GpStatus WINAPI DllExports::GdipGetPathGradientRect(GpPathGradient* brush, GpRectF* rect)
{
	return gdiplusWrapper.GdipGetPathGradientRect(brush, rect);
}

GpStatus WINAPI DllExports::GdipGetPathGradientRectI(GpPathGradient* brush, GpRect* rect)
{
	return gdiplusWrapper.GdipGetPathGradientRectI(brush, rect);
}

GpStatus WINAPI DllExports::GdipGetPathGradientPointCount(GpPathGradient* brush, INT* count)
{
	return gdiplusWrapper.GdipGetPathGradientPointCount(brush, count);
}

GpStatus WINAPI DllExports::GdipGetPathGradientSurroundColorCount(GpPathGradient* brush, INT* count)
{
	return gdiplusWrapper.GdipGetPathGradientSurroundColorCount(brush, count);
}

GpStatus WINAPI DllExports::GdipSetPathGradientGammaCorrection(GpPathGradient* brush, BOOL useGammaCorrection)
{
	return gdiplusWrapper.GdipSetPathGradientGammaCorrection(brush, useGammaCorrection);
}

GpStatus WINAPI DllExports::GdipGetPathGradientGammaCorrection(GpPathGradient* brush, BOOL* useGammaCorrection)
{
	return gdiplusWrapper.GdipGetPathGradientGammaCorrection(brush, useGammaCorrection);
}

GpStatus WINAPI DllExports::GdipGetPathGradientBlendCount(GpPathGradient* brush, INT* count)
{
	return gdiplusWrapper.GdipGetPathGradientBlendCount(brush, count);
}

GpStatus WINAPI DllExports::GdipGetPathGradientBlend(GpPathGradient* brush, REAL* blend, REAL* positions, INT count)
{
	return gdiplusWrapper.GdipGetPathGradientBlend(brush, blend, positions, count);
}

GpStatus WINAPI DllExports::GdipSetPathGradientBlend(GpPathGradient* brush, GDIPCONST REAL* blend, GDIPCONST REAL* positions, INT count)
{
	return gdiplusWrapper.GdipSetPathGradientBlend(brush, blend, positions, count);
}

GpStatus WINAPI DllExports::GdipGetPathGradientPresetBlendCount(GpPathGradient* brush, INT* count)
{
	return gdiplusWrapper.GdipGetPathGradientPresetBlendCount(brush, count);
}

GpStatus WINAPI DllExports::GdipGetPathGradientPresetBlend(GpPathGradient* brush, ARGB* blend, REAL* positions, INT count)
{
	return gdiplusWrapper.GdipGetPathGradientPresetBlend(brush, blend, positions, count);
}

GpStatus WINAPI DllExports::GdipSetPathGradientPresetBlend(GpPathGradient* brush, GDIPCONST ARGB* blend, GDIPCONST REAL* positions, INT count)
{
	return gdiplusWrapper.GdipSetPathGradientPresetBlend(brush, blend, positions, count);
}

GpStatus WINAPI DllExports::GdipSetPathGradientSigmaBlend(GpPathGradient* brush, REAL focus, REAL scale)
{
	return gdiplusWrapper.GdipSetPathGradientSigmaBlend(brush, focus, scale);
}

GpStatus WINAPI DllExports::GdipSetPathGradientLinearBlend(GpPathGradient* brush, REAL focus, REAL scale)
{
	return gdiplusWrapper.GdipSetPathGradientLinearBlend(brush, focus, scale);
}

GpStatus WINAPI DllExports::GdipGetPathGradientWrapMode(GpPathGradient* brush, GpWrapMode* wrapmode)
{
	return gdiplusWrapper.GdipGetPathGradientWrapMode(brush, wrapmode);
}

GpStatus WINAPI DllExports::GdipSetPathGradientWrapMode(GpPathGradient* brush, GpWrapMode wrapmode)
{
	return gdiplusWrapper.GdipSetPathGradientWrapMode(brush, wrapmode);
}

GpStatus WINAPI DllExports::GdipGetPathGradientTransform(GpPathGradient* brush, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipGetPathGradientTransform(brush, matrix);
}

GpStatus WINAPI DllExports::GdipSetPathGradientTransform(GpPathGradient* brush, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipSetPathGradientTransform(brush, matrix);
}

GpStatus WINAPI DllExports::GdipResetPathGradientTransform(GpPathGradient* brush)
{
	return gdiplusWrapper.GdipResetPathGradientTransform(brush);
}

GpStatus WINAPI DllExports::GdipMultiplyPathGradientTransform(GpPathGradient* brush, GDIPCONST GpMatrix* matrix, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipMultiplyPathGradientTransform(brush, matrix, order);
}

GpStatus WINAPI DllExports::GdipTranslatePathGradientTransform(GpPathGradient* brush, REAL dx, REAL dy, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipTranslatePathGradientTransform(brush, dx, dy, order);
}

GpStatus WINAPI DllExports::GdipScalePathGradientTransform(GpPathGradient* brush, REAL sx, REAL sy, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipScalePathGradientTransform(brush, sx, sy, order);
}

GpStatus WINAPI DllExports::GdipRotatePathGradientTransform(GpPathGradient* brush, REAL angle, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipRotatePathGradientTransform(brush, angle, order);
}

GpStatus WINAPI DllExports::GdipGetPathGradientFocusScales(GpPathGradient* brush, REAL* xScale, REAL* yScale)
{
	return gdiplusWrapper.GdipGetPathGradientFocusScales(brush, xScale, yScale);
}

GpStatus WINAPI DllExports::GdipSetPathGradientFocusScales(GpPathGradient* brush, REAL xScale, REAL yScale)
{
	return gdiplusWrapper.GdipSetPathGradientFocusScales(brush, xScale, yScale);
}

GpStatus WINAPI DllExports::GdipCreatePen1(ARGB color, REAL width, GpUnit unit, GpPen** pen)
{
	return gdiplusWrapper.GdipCreatePen1(color, width, unit, pen);
}

GpStatus WINAPI DllExports::GdipCreatePen2(GpBrush* brush, REAL width, GpUnit unit, GpPen** pen)
{
	return gdiplusWrapper.GdipCreatePen2(brush, width, unit, pen);
}

GpStatus WINAPI DllExports::GdipClonePen(GpPen* pen, GpPen** clonepen)
{
	return gdiplusWrapper.GdipClonePen(pen, clonepen);
}

GpStatus WINAPI DllExports::GdipDeletePen(GpPen* pen)
{
	return gdiplusWrapper.GdipDeletePen(pen);
}

GpStatus WINAPI DllExports::GdipSetPenWidth(GpPen* pen, REAL width)
{
	return gdiplusWrapper.GdipSetPenWidth(pen, width);
}

GpStatus WINAPI DllExports::GdipGetPenWidth(GpPen* pen, REAL* width)
{
	return gdiplusWrapper.GdipGetPenWidth(pen, width);
}

GpStatus WINAPI DllExports::GdipSetPenUnit(GpPen* pen, GpUnit unit)
{
	return gdiplusWrapper.GdipSetPenUnit(pen, unit);
}

GpStatus WINAPI DllExports::GdipGetPenUnit(GpPen* pen, GpUnit* unit)
{
	return gdiplusWrapper.GdipGetPenUnit(pen, unit);
}

GpStatus WINAPI DllExports::GdipSetPenLineCap197819(GpPen* pen, GpLineCap startCap, GpLineCap endCap, GpDashCap dashCap)
{
	return gdiplusWrapper.GdipSetPenLineCap197819(pen, startCap, endCap, dashCap);
}

GpStatus WINAPI DllExports::GdipSetPenStartCap(GpPen* pen, GpLineCap startCap)
{
	return gdiplusWrapper.GdipSetPenStartCap(pen, startCap);
}

GpStatus WINAPI DllExports::GdipSetPenEndCap(GpPen* pen, GpLineCap endCap)
{
	return gdiplusWrapper.GdipSetPenEndCap(pen, endCap);
}

GpStatus WINAPI DllExports::GdipSetPenDashCap197819(GpPen* pen, GpDashCap dashCap)
{
	return gdiplusWrapper.GdipSetPenDashCap197819(pen, dashCap);
}

GpStatus WINAPI DllExports::GdipGetPenStartCap(GpPen* pen, GpLineCap* startCap)
{
	return gdiplusWrapper.GdipGetPenStartCap(pen, startCap);
}

GpStatus WINAPI DllExports::GdipGetPenEndCap(GpPen* pen, GpLineCap* endCap)
{
	return gdiplusWrapper.GdipGetPenEndCap(pen, endCap);
}

GpStatus WINAPI DllExports::GdipGetPenDashCap197819(GpPen* pen, GpDashCap* dashCap)
{
	return gdiplusWrapper.GdipGetPenDashCap197819(pen, dashCap);
}

GpStatus WINAPI DllExports::GdipSetPenLineJoin(GpPen* pen, GpLineJoin lineJoin)
{
	return gdiplusWrapper.GdipSetPenLineJoin(pen, lineJoin);
}

GpStatus WINAPI DllExports::GdipGetPenLineJoin(GpPen* pen, GpLineJoin* lineJoin)
{
	return gdiplusWrapper.GdipGetPenLineJoin(pen, lineJoin);
}

GpStatus WINAPI DllExports::GdipSetPenCustomStartCap(GpPen* pen, GpCustomLineCap* customCap)
{
	return gdiplusWrapper.GdipSetPenCustomStartCap(pen, customCap);
}

GpStatus WINAPI DllExports::GdipGetPenCustomStartCap(GpPen* pen, GpCustomLineCap** customCap)
{
	return gdiplusWrapper.GdipGetPenCustomStartCap(pen, customCap);
}

GpStatus WINAPI DllExports::GdipSetPenCustomEndCap(GpPen* pen, GpCustomLineCap* customCap)
{
	return gdiplusWrapper.GdipSetPenCustomEndCap(pen, customCap);
}

GpStatus WINAPI DllExports::GdipGetPenCustomEndCap(GpPen* pen, GpCustomLineCap** customCap)
{
	return gdiplusWrapper.GdipGetPenCustomEndCap(pen, customCap);
}

GpStatus WINAPI DllExports::GdipSetPenMiterLimit(GpPen* pen, REAL miterLimit)
{
	return gdiplusWrapper.GdipSetPenMiterLimit(pen, miterLimit);
}

GpStatus WINAPI DllExports::GdipGetPenMiterLimit(GpPen* pen, REAL* miterLimit)
{
	return gdiplusWrapper.GdipGetPenMiterLimit(pen, miterLimit);
}

GpStatus WINAPI DllExports::GdipSetPenMode(GpPen* pen, GpPenAlignment penMode)
{
	return gdiplusWrapper.GdipSetPenMode(pen, penMode);
}

GpStatus WINAPI DllExports::GdipGetPenMode(GpPen* pen, GpPenAlignment* penMode)
{
	return gdiplusWrapper.GdipGetPenMode(pen, penMode);
}

GpStatus WINAPI DllExports::GdipSetPenTransform(GpPen* pen, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipSetPenTransform(pen, matrix);
}

GpStatus WINAPI DllExports::GdipGetPenTransform(GpPen* pen, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipGetPenTransform(pen, matrix);
}

GpStatus WINAPI DllExports::GdipResetPenTransform(GpPen* pen)
{
	return gdiplusWrapper.GdipResetPenTransform(pen);
}

GpStatus WINAPI DllExports::GdipMultiplyPenTransform(GpPen* pen, GDIPCONST GpMatrix* matrix, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipMultiplyPenTransform(pen, matrix, order);
}

GpStatus WINAPI DllExports::GdipTranslatePenTransform(GpPen* pen, REAL dx, REAL dy, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipTranslatePenTransform(pen, dx, dy, order);
}

GpStatus WINAPI DllExports::GdipScalePenTransform(GpPen* pen, REAL sx, REAL sy, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipScalePenTransform(pen, sx, sy, order);
}

GpStatus WINAPI DllExports::GdipRotatePenTransform(GpPen* pen, REAL angle, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipRotatePenTransform(pen, angle, order);
}

GpStatus WINAPI DllExports::GdipSetPenColor(GpPen* pen, ARGB argb)
{
	return gdiplusWrapper.GdipSetPenColor(pen, argb);
}

GpStatus WINAPI DllExports::GdipGetPenColor(GpPen* pen, ARGB* argb)
{
	return gdiplusWrapper.GdipGetPenColor(pen, argb);
}

GpStatus WINAPI DllExports::GdipSetPenBrushFill(GpPen* pen, GpBrush* brush)
{
	return gdiplusWrapper.GdipSetPenBrushFill(pen, brush);
}

GpStatus WINAPI DllExports::GdipGetPenBrushFill(GpPen* pen, GpBrush** brush)
{
	return gdiplusWrapper.GdipGetPenBrushFill(pen, brush);
}

GpStatus WINAPI DllExports::GdipGetPenFillType(GpPen* pen, GpPenType* type)
{
	return gdiplusWrapper.GdipGetPenFillType(pen, type);
}

GpStatus WINAPI DllExports::GdipGetPenDashStyle(GpPen* pen, GpDashStyle* dashstyle)
{
	return gdiplusWrapper.GdipGetPenDashStyle(pen, dashstyle);
}

GpStatus WINAPI DllExports::GdipSetPenDashStyle(GpPen* pen, GpDashStyle dashstyle)
{
	return gdiplusWrapper.GdipSetPenDashStyle(pen, dashstyle);
}

GpStatus WINAPI DllExports::GdipGetPenDashOffset(GpPen* pen, REAL* offset)
{
	return gdiplusWrapper.GdipGetPenDashOffset(pen, offset);
}

GpStatus WINAPI DllExports::GdipSetPenDashOffset(GpPen* pen, REAL offset)
{
	return gdiplusWrapper.GdipSetPenDashOffset(pen, offset);
}

GpStatus WINAPI DllExports::GdipGetPenDashCount(GpPen* pen, INT* count)
{
	return gdiplusWrapper.GdipGetPenDashCount(pen, count);
}

GpStatus WINAPI DllExports::GdipSetPenDashArray(GpPen* pen, GDIPCONST REAL* dash, INT count)
{
	return gdiplusWrapper.GdipSetPenDashArray(pen, dash, count);
}

GpStatus WINAPI DllExports::GdipGetPenDashArray(GpPen* pen, REAL* dash, INT count)
{
	return gdiplusWrapper.GdipGetPenDashArray(pen, dash, count);
}

GpStatus WINAPI DllExports::GdipGetPenCompoundCount(GpPen* pen, INT* count)
{
	return gdiplusWrapper.GdipGetPenCompoundCount(pen, count);
}

GpStatus WINAPI DllExports::GdipSetPenCompoundArray(GpPen* pen, GDIPCONST REAL* dash, INT count)
{
	return gdiplusWrapper.GdipSetPenCompoundArray(pen, dash, count);
}

GpStatus WINAPI DllExports::GdipGetPenCompoundArray(GpPen* pen, REAL* dash, INT count)
{
	return gdiplusWrapper.GdipGetPenCompoundArray(pen, dash, count);
}

GpStatus WINAPI DllExports::GdipCreateCustomLineCap(GpPath* fillPath, GpPath* strokePath, GpLineCap baseCap, REAL baseInset, GpCustomLineCap** customCap)
{
	return gdiplusWrapper.GdipCreateCustomLineCap(fillPath, strokePath, baseCap, baseInset, customCap);
}

GpStatus WINAPI DllExports::GdipDeleteCustomLineCap(GpCustomLineCap* customCap)
{
	return gdiplusWrapper.GdipDeleteCustomLineCap(customCap);
}

GpStatus WINAPI DllExports::GdipCloneCustomLineCap(GpCustomLineCap* customCap, GpCustomLineCap** clonedCap)
{
	return gdiplusWrapper.GdipCloneCustomLineCap(customCap, clonedCap);
}

GpStatus WINAPI DllExports::GdipGetCustomLineCapType(GpCustomLineCap* customCap, CustomLineCapType* capType)
{
	return gdiplusWrapper.GdipGetCustomLineCapType(customCap, capType);
}

GpStatus WINAPI DllExports::GdipSetCustomLineCapStrokeCaps(GpCustomLineCap* customCap, GpLineCap startCap, GpLineCap endCap)
{
	return gdiplusWrapper.GdipSetCustomLineCapStrokeCaps(customCap, startCap, endCap);
}

GpStatus WINAPI DllExports::GdipGetCustomLineCapStrokeCaps(GpCustomLineCap* customCap, GpLineCap* startCap, GpLineCap* endCap)
{
	return gdiplusWrapper.GdipGetCustomLineCapStrokeCaps(customCap, startCap, endCap);
}

GpStatus WINAPI DllExports::GdipSetCustomLineCapStrokeJoin(GpCustomLineCap* customCap, GpLineJoin lineJoin)
{
	return gdiplusWrapper.GdipSetCustomLineCapStrokeJoin(customCap, lineJoin);
}

GpStatus WINAPI DllExports::GdipGetCustomLineCapStrokeJoin(GpCustomLineCap* customCap, GpLineJoin* lineJoin)
{
	return gdiplusWrapper.GdipGetCustomLineCapStrokeJoin(customCap, lineJoin);
}

GpStatus WINAPI DllExports::GdipSetCustomLineCapBaseCap(GpCustomLineCap* customCap, GpLineCap baseCap)
{
	return gdiplusWrapper.GdipSetCustomLineCapBaseCap(customCap, baseCap);
}

GpStatus WINAPI DllExports::GdipGetCustomLineCapBaseCap(GpCustomLineCap* customCap, GpLineCap* baseCap)
{
	return gdiplusWrapper.GdipGetCustomLineCapBaseCap(customCap, baseCap);
}

GpStatus WINAPI DllExports::GdipSetCustomLineCapBaseInset(GpCustomLineCap* customCap, REAL inset)
{
	return gdiplusWrapper.GdipSetCustomLineCapBaseInset(customCap, inset);
}

GpStatus WINAPI DllExports::GdipGetCustomLineCapBaseInset(GpCustomLineCap* customCap, REAL* inset)
{
	return gdiplusWrapper.GdipGetCustomLineCapBaseInset(customCap, inset);
}

GpStatus WINAPI DllExports::GdipSetCustomLineCapWidthScale(GpCustomLineCap* customCap, REAL widthScale)
{
	return gdiplusWrapper.GdipSetCustomLineCapWidthScale(customCap, widthScale);
}

GpStatus WINAPI DllExports::GdipGetCustomLineCapWidthScale(GpCustomLineCap* customCap, REAL* widthScale)
{
	return gdiplusWrapper.GdipGetCustomLineCapWidthScale(customCap, widthScale);
}

GpStatus WINAPI DllExports::GdipCreateAdjustableArrowCap(REAL height, REAL width, BOOL isFilled, GpAdjustableArrowCap** cap)
{
	return gdiplusWrapper.GdipCreateAdjustableArrowCap(height, width, isFilled, cap);
}

GpStatus WINAPI DllExports::GdipSetAdjustableArrowCapHeight(GpAdjustableArrowCap* cap, REAL height)
{
	return gdiplusWrapper.GdipSetAdjustableArrowCapHeight(cap, height);
}

GpStatus WINAPI DllExports::GdipGetAdjustableArrowCapHeight(GpAdjustableArrowCap* cap, REAL* height)
{
	return gdiplusWrapper.GdipGetAdjustableArrowCapHeight(cap, height);
}

GpStatus WINAPI DllExports::GdipSetAdjustableArrowCapWidth(GpAdjustableArrowCap* cap, REAL width)
{
	return gdiplusWrapper.GdipSetAdjustableArrowCapWidth(cap, width);
}

GpStatus WINAPI DllExports::GdipGetAdjustableArrowCapWidth(GpAdjustableArrowCap* cap, REAL* width)
{
	return gdiplusWrapper.GdipGetAdjustableArrowCapWidth(cap, width);
}

GpStatus WINAPI DllExports::GdipSetAdjustableArrowCapMiddleInset(GpAdjustableArrowCap* cap, REAL middleInset)
{
	return gdiplusWrapper.GdipSetAdjustableArrowCapMiddleInset(cap, middleInset);
}

GpStatus WINAPI DllExports::GdipGetAdjustableArrowCapMiddleInset(GpAdjustableArrowCap* cap, REAL* middleInset)
{
	return gdiplusWrapper.GdipGetAdjustableArrowCapMiddleInset(cap, middleInset);
}

GpStatus WINAPI DllExports::GdipSetAdjustableArrowCapFillState(GpAdjustableArrowCap* cap, BOOL fillState)
{
	return gdiplusWrapper.GdipSetAdjustableArrowCapFillState(cap, fillState);
}

GpStatus WINAPI DllExports::GdipGetAdjustableArrowCapFillState(GpAdjustableArrowCap* cap, BOOL* fillState)
{
	return gdiplusWrapper.GdipGetAdjustableArrowCapFillState(cap, fillState);
}

GpStatus WINAPI DllExports::GdipLoadImageFromStream(IStream* stream, GpImage** image)
{
	return gdiplusWrapper.GdipLoadImageFromStream(stream, image);
}

GpStatus WINAPI DllExports::GdipLoadImageFromFile(GDIPCONST WCHAR* filename, GpImage** image)
{
	return gdiplusWrapper.GdipLoadImageFromFile(filename, image);
}

GpStatus WINAPI DllExports::GdipLoadImageFromStreamICM(IStream* stream, GpImage** image)
{
	return gdiplusWrapper.GdipLoadImageFromStreamICM(stream, image);
}

GpStatus WINAPI DllExports::GdipLoadImageFromFileICM(GDIPCONST WCHAR* filename, GpImage** image)
{
	return gdiplusWrapper.GdipLoadImageFromFileICM(filename, image);
}

GpStatus WINAPI DllExports::GdipCloneImage(GpImage* image, GpImage** cloneImage)
{
	return gdiplusWrapper.GdipCloneImage(image, cloneImage);
}

GpStatus WINAPI DllExports::GdipDisposeImage(GpImage* image)
{
	return gdiplusWrapper.GdipDisposeImage(image);
}

GpStatus WINAPI DllExports::GdipSaveImageToFile(GpImage* image, GDIPCONST WCHAR* filename, GDIPCONST CLSID* clsidEncoder, GDIPCONST EncoderParameters* encoderParams)
{
	return gdiplusWrapper.GdipSaveImageToFile(image, filename, clsidEncoder, encoderParams);
}

GpStatus WINAPI DllExports::GdipSaveImageToStream(GpImage* image, IStream* stream, GDIPCONST CLSID* clsidEncoder, GDIPCONST EncoderParameters* encoderParams)
{
	return gdiplusWrapper.GdipSaveImageToStream(image, stream, clsidEncoder, encoderParams);
}

GpStatus WINAPI DllExports::GdipSaveAdd(GpImage* image, GDIPCONST EncoderParameters* encoderParams)
{
	return gdiplusWrapper.GdipSaveAdd(image, encoderParams);
}

GpStatus WINAPI DllExports::GdipSaveAddImage(GpImage* image, GpImage* newImage, GDIPCONST EncoderParameters* encoderParams)
{
	return gdiplusWrapper.GdipSaveAddImage(image, newImage, encoderParams);
}

GpStatus WINAPI DllExports::GdipGetImageGraphicsContext(GpImage* image, GpGraphics** graphics)
{
	return gdiplusWrapper.GdipGetImageGraphicsContext(image, graphics);
}

GpStatus WINAPI DllExports::GdipGetImageBounds(GpImage* image, GpRectF* srcRect, GpUnit* srcUnit)
{
	return gdiplusWrapper.GdipGetImageBounds(image, srcRect, srcUnit);
}

GpStatus WINAPI DllExports::GdipGetImageDimension(GpImage* image, REAL* width, REAL* height)
{
	return gdiplusWrapper.GdipGetImageDimension(image, width, height);
}

GpStatus WINAPI DllExports::GdipGetImageType(GpImage* image, ImageType* type)
{
	return gdiplusWrapper.GdipGetImageType(image, type);
}

GpStatus WINAPI DllExports::GdipGetImageWidth(GpImage* image, UINT* width)
{
	return gdiplusWrapper.GdipGetImageWidth(image, width);
}

GpStatus WINAPI DllExports::GdipGetImageHeight(GpImage* image, UINT* height)
{
	return gdiplusWrapper.GdipGetImageHeight(image, height);
}

GpStatus WINAPI DllExports::GdipGetImageHorizontalResolution(GpImage* image, REAL* resolution)
{
	return gdiplusWrapper.GdipGetImageHorizontalResolution(image, resolution);
}

GpStatus WINAPI DllExports::GdipGetImageVerticalResolution(GpImage* image, REAL* resolution)
{
	return gdiplusWrapper.GdipGetImageVerticalResolution(image, resolution);
}

GpStatus WINAPI DllExports::GdipGetImageFlags(GpImage* image, UINT* flags)
{
	return gdiplusWrapper.GdipGetImageFlags(image, flags);
}

GpStatus WINAPI DllExports::GdipGetImageRawFormat(GpImage* image, GUID* format)
{
	return gdiplusWrapper.GdipGetImageRawFormat(image, format);
}

GpStatus WINAPI DllExports::GdipGetImagePixelFormat(GpImage* image, PixelFormat* format)
{
	return gdiplusWrapper.GdipGetImagePixelFormat(image, format);
}

GpStatus WINAPI DllExports::GdipGetImageThumbnail(GpImage* image, UINT thumbWidth, UINT thumbHeight, GpImage** thumbImage, GetThumbnailImageAbort callback, VOID* callbackData)
{
	return gdiplusWrapper.GdipGetImageThumbnail(image, thumbWidth, thumbHeight, thumbImage, callback, callbackData);
}

GpStatus WINAPI DllExports::GdipGetEncoderParameterListSize(GpImage* image, GDIPCONST CLSID* clsidEncoder, UINT* size)
{
	return gdiplusWrapper.GdipGetEncoderParameterListSize(image, clsidEncoder, size);
}

GpStatus WINAPI DllExports::GdipGetEncoderParameterList(GpImage* image, GDIPCONST CLSID* clsidEncoder, UINT size, EncoderParameters* buffer)
{
	return gdiplusWrapper.GdipGetEncoderParameterList(image, clsidEncoder, size, buffer);
}

GpStatus WINAPI DllExports::GdipImageGetFrameDimensionsCount(GpImage* image, UINT* count)
{
	return gdiplusWrapper.GdipImageGetFrameDimensionsCount(image, count);
}

GpStatus WINAPI DllExports::GdipImageGetFrameDimensionsList(GpImage* image, GUID* dimensionIDs, UINT count)
{
	return gdiplusWrapper.GdipImageGetFrameDimensionsList(image, dimensionIDs, count);
}

GpStatus WINAPI DllExports::GdipImageGetFrameCount(GpImage* image, GDIPCONST GUID* dimensionID, UINT* count)
{
	return gdiplusWrapper.GdipImageGetFrameCount(image, dimensionID, count);
}

GpStatus WINAPI DllExports::GdipImageSelectActiveFrame(GpImage* image, GDIPCONST GUID* dimensionID, UINT frameIndex)
{
	return gdiplusWrapper.GdipImageSelectActiveFrame(image, dimensionID, frameIndex);
}

GpStatus WINAPI DllExports::GdipImageRotateFlip(GpImage* image, RotateFlipType rfType)
{
	return gdiplusWrapper.GdipImageRotateFlip(image, rfType);
}

GpStatus WINAPI DllExports::GdipGetImagePalette(GpImage* image, ColorPalette* palette, INT size)
{
	return gdiplusWrapper.GdipGetImagePalette(image, palette, size);
}

GpStatus WINAPI DllExports::GdipSetImagePalette(GpImage* image, GDIPCONST ColorPalette* palette)
{
	return gdiplusWrapper.GdipSetImagePalette(image, palette);
}

GpStatus WINAPI DllExports::GdipGetImagePaletteSize(GpImage* image, INT* size)
{
	return gdiplusWrapper.GdipGetImagePaletteSize(image, size);
}

GpStatus WINAPI DllExports::GdipGetPropertyCount(GpImage* image, UINT* numOfProperty)
{
	return gdiplusWrapper.GdipGetPropertyCount(image, numOfProperty);
}

GpStatus WINAPI DllExports::GdipGetPropertyIdList(GpImage* image, UINT numOfProperty, PROPID* list)
{
	return gdiplusWrapper.GdipGetPropertyIdList(image, numOfProperty, list);
}

GpStatus WINAPI DllExports::GdipGetPropertyItemSize(GpImage* image, PROPID propId, UINT* size)
{
	return gdiplusWrapper.GdipGetPropertyItemSize(image, propId, size);
}

GpStatus WINAPI DllExports::GdipGetPropertyItem(GpImage* image, PROPID propId, UINT propSize, PropertyItem* buffer)
{
	return gdiplusWrapper.GdipGetPropertyItem(image, propId, propSize, buffer);
}

GpStatus WINAPI DllExports::GdipGetPropertySize(GpImage* image, UINT* totalBufferSize, UINT* numProperties)
{
	return gdiplusWrapper.GdipGetPropertySize(image, totalBufferSize, numProperties);
}

GpStatus WINAPI DllExports::GdipGetAllPropertyItems(GpImage* image, UINT totalBufferSize, UINT numProperties, PropertyItem* allItems)
{
	return gdiplusWrapper.GdipGetAllPropertyItems(image, totalBufferSize, numProperties, allItems);
}

GpStatus WINAPI DllExports::GdipRemovePropertyItem(GpImage* image, PROPID propId)
{
	return gdiplusWrapper.GdipRemovePropertyItem(image, propId);
}

GpStatus WINAPI DllExports::GdipSetPropertyItem(GpImage* image, GDIPCONST PropertyItem* item)
{
	return gdiplusWrapper.GdipSetPropertyItem(image, item);
}

GpStatus WINAPI DllExports::GdipImageForceValidation(GpImage* image)
{
	return gdiplusWrapper.GdipImageForceValidation(image);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromStream(IStream* stream, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromStream(stream, bitmap);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromFile(GDIPCONST WCHAR* filename, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromFile(filename, bitmap);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromStreamICM(IStream* stream, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromStreamICM(stream, bitmap);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromFileICM(GDIPCONST WCHAR* filename, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromFileICM(filename, bitmap);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromScan0(INT width, INT height, INT stride, PixelFormat format, BYTE* scan0, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromScan0(width, height, stride, format, scan0, bitmap);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromGraphics(INT width, INT height, GpGraphics* target, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromGraphics(width, height, target, bitmap);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromDirectDrawSurface(IDirectDrawSurface7* surface, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromDirectDrawSurface(surface, bitmap);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromGdiDib(GDIPCONST BITMAPINFO* gdiBitmapInfo, VOID* gdiBitmapData, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromGdiDib(gdiBitmapInfo, gdiBitmapData, bitmap);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromHBITMAP(HBITMAP hbm, HPALETTE hpal, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromHBITMAP(hbm, hpal, bitmap);
}

GpStatus WINAPI DllExports::GdipCreateHBITMAPFromBitmap(GpBitmap* bitmap, HBITMAP* hbmReturn, ARGB background)
{
	return gdiplusWrapper.GdipCreateHBITMAPFromBitmap(bitmap, hbmReturn, background);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromHICON(HICON hicon, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromHICON(hicon, bitmap);
}

GpStatus WINAPI DllExports::GdipCreateHICONFromBitmap(GpBitmap* bitmap, HICON* hbmReturn)
{
	return gdiplusWrapper.GdipCreateHICONFromBitmap(bitmap, hbmReturn);
}

GpStatus WINAPI DllExports::GdipCreateBitmapFromResource(HINSTANCE hInstance, GDIPCONST WCHAR* lpBitmapName, GpBitmap** bitmap)
{
	return gdiplusWrapper.GdipCreateBitmapFromResource(hInstance, lpBitmapName, bitmap);
}

GpStatus WINAPI DllExports::GdipCloneBitmapArea(REAL x, REAL y, REAL width, REAL height, PixelFormat format, GpBitmap* srcBitmap, GpBitmap** dstBitmap)
{
	return gdiplusWrapper.GdipCloneBitmapArea(x, y, width, height, format, srcBitmap, dstBitmap);
}

GpStatus WINAPI DllExports::GdipCloneBitmapAreaI(INT x, INT y, INT width, INT height, PixelFormat format, GpBitmap* srcBitmap, GpBitmap** dstBitmap)
{
	return gdiplusWrapper.GdipCloneBitmapAreaI(x, y, width, height, format, srcBitmap, dstBitmap);
}

GpStatus WINAPI DllExports::GdipBitmapLockBits(GpBitmap* bitmap, GDIPCONST GpRect* rect, UINT flags, PixelFormat format, BitmapData* lockedBitmapData)
{
	return gdiplusWrapper.GdipBitmapLockBits(bitmap, rect, flags, format, lockedBitmapData);
}

GpStatus WINAPI DllExports::GdipBitmapUnlockBits(GpBitmap* bitmap, BitmapData* lockedBitmapData)
{
	return gdiplusWrapper.GdipBitmapUnlockBits(bitmap, lockedBitmapData);
}

GpStatus WINAPI DllExports::GdipBitmapGetPixel(GpBitmap* bitmap, INT x, INT y, ARGB* color)
{
	return gdiplusWrapper.GdipBitmapGetPixel(bitmap, x, y, color);
}

GpStatus WINAPI DllExports::GdipBitmapSetPixel(GpBitmap* bitmap, INT x, INT y, ARGB color)
{
	return gdiplusWrapper.GdipBitmapSetPixel(bitmap, x, y, color);
}

GpStatus WINAPI DllExports::GdipBitmapSetResolution(GpBitmap* bitmap, REAL xdpi, REAL ydpi)
{
	return gdiplusWrapper.GdipBitmapSetResolution(bitmap, xdpi, ydpi);
}

GpStatus WINAPI DllExports::GdipCreateImageAttributes(GpImageAttributes** imageattr)
{
	return gdiplusWrapper.GdipCreateImageAttributes(imageattr);
}

GpStatus WINAPI DllExports::GdipCloneImageAttributes(GDIPCONST GpImageAttributes* imageattr, GpImageAttributes** cloneImageattr)
{
	return gdiplusWrapper.GdipCloneImageAttributes(imageattr, cloneImageattr);
}

GpStatus WINAPI DllExports::GdipDisposeImageAttributes(GpImageAttributes* imageattr)
{
	return gdiplusWrapper.GdipDisposeImageAttributes(imageattr);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesToIdentity(GpImageAttributes* imageattr, ColorAdjustType type)
{
	return gdiplusWrapper.GdipSetImageAttributesToIdentity(imageattr, type);
}

GpStatus WINAPI DllExports::GdipResetImageAttributes(GpImageAttributes* imageattr, ColorAdjustType type)
{
	return gdiplusWrapper.GdipResetImageAttributes(imageattr, type);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesColorMatrix(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, GDIPCONST ColorMatrix* colorMatrix, GDIPCONST ColorMatrix* grayMatrix, ColorMatrixFlags flags)
{
	return gdiplusWrapper.GdipSetImageAttributesColorMatrix(imageattr, type, enableFlag, colorMatrix, grayMatrix, flags);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesThreshold(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, REAL threshold)
{
	return gdiplusWrapper.GdipSetImageAttributesThreshold(imageattr, type, enableFlag, threshold);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesGamma(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, REAL gamma)
{
	return gdiplusWrapper.GdipSetImageAttributesGamma(imageattr, type, enableFlag, gamma);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesNoOp(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag)
{
	return gdiplusWrapper.GdipSetImageAttributesNoOp(imageattr, type, enableFlag);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesColorKeys(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, ARGB colorLow, ARGB colorHigh)
{
	return gdiplusWrapper.GdipSetImageAttributesColorKeys(imageattr, type, enableFlag, colorLow, colorHigh);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesOutputChannel(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, ColorChannelFlags channelFlags)
{
	return gdiplusWrapper.GdipSetImageAttributesOutputChannel(imageattr, type, enableFlag, channelFlags);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesOutputChannelColorProfile(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, GDIPCONST WCHAR* colorProfileFilename)
{
	return gdiplusWrapper.GdipSetImageAttributesOutputChannelColorProfile(imageattr, type, enableFlag, colorProfileFilename);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesRemapTable(GpImageAttributes* imageattr, ColorAdjustType type, BOOL enableFlag, UINT mapSize, GDIPCONST ColorMap* map)
{
	return gdiplusWrapper.GdipSetImageAttributesRemapTable(imageattr, type, enableFlag, mapSize, map);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesWrapMode(GpImageAttributes* imageAttr, WrapMode wrap, ARGB argb, BOOL clamp)
{
	return gdiplusWrapper.GdipSetImageAttributesWrapMode(imageAttr, wrap, argb, clamp);
}

GpStatus WINAPI DllExports::GdipSetImageAttributesICMMode(GpImageAttributes* imageAttr, BOOL on)
{
	return gdiplusWrapper.GdipSetImageAttributesICMMode(imageAttr, on);
}

GpStatus WINAPI DllExports::GdipGetImageAttributesAdjustedPalette(GpImageAttributes* imageAttr, ColorPalette* colorPalette, ColorAdjustType colorAdjustType)
{
	return gdiplusWrapper.GdipGetImageAttributesAdjustedPalette(imageAttr, colorPalette, colorAdjustType);
}

GpStatus WINAPI DllExports::GdipFlush(GpGraphics* graphics, GpFlushIntention intention)
{
	return gdiplusWrapper.GdipFlush(graphics, intention);
}

GpStatus WINAPI DllExports::GdipCreateFromHDC(HDC hdc, GpGraphics** graphics)
{
	return gdiplusWrapper.GdipCreateFromHDC(hdc, graphics);
}

GpStatus WINAPI DllExports::GdipCreateFromHDC2(HDC hdc, HANDLE hDevice, GpGraphics** graphics)
{
	return gdiplusWrapper.GdipCreateFromHDC2(hdc, hDevice, graphics);
}

GpStatus WINAPI DllExports::GdipCreateFromHWND(HWND hwnd, GpGraphics** graphics)
{
	return gdiplusWrapper.GdipCreateFromHWND(hwnd, graphics);
}

GpStatus WINAPI DllExports::GdipCreateFromHWNDICM(HWND hwnd, GpGraphics** graphics)
{
	return gdiplusWrapper.GdipCreateFromHWNDICM(hwnd, graphics);
}

GpStatus WINAPI DllExports::GdipDeleteGraphics(GpGraphics* graphics)
{
	return gdiplusWrapper.GdipDeleteGraphics(graphics);
}

GpStatus WINAPI DllExports::GdipGetDC(GpGraphics* graphics, HDC* hdc)
{
	return gdiplusWrapper.GdipGetDC(graphics, hdc);
}

GpStatus WINAPI DllExports::GdipReleaseDC(GpGraphics* graphics, HDC hdc)
{
	return gdiplusWrapper.GdipReleaseDC(graphics, hdc);
}

GpStatus WINAPI DllExports::GdipSetCompositingMode(GpGraphics* graphics, CompositingMode compositingMode)
{
	return gdiplusWrapper.GdipSetCompositingMode(graphics, compositingMode);
}

GpStatus WINAPI DllExports::GdipGetCompositingMode(GpGraphics* graphics, CompositingMode* compositingMode)
{
	return gdiplusWrapper.GdipGetCompositingMode(graphics, compositingMode);
}

GpStatus WINAPI DllExports::GdipSetRenderingOrigin(GpGraphics* graphics, INT x, INT y)
{
	return gdiplusWrapper.GdipSetRenderingOrigin(graphics, x, y);
}

GpStatus WINAPI DllExports::GdipGetRenderingOrigin(GpGraphics* graphics, INT* x, INT* y)
{
	return gdiplusWrapper.GdipGetRenderingOrigin(graphics, x, y);
}

GpStatus WINAPI DllExports::GdipSetCompositingQuality(GpGraphics* graphics, CompositingQuality compositingQuality)
{
	return gdiplusWrapper.GdipSetCompositingQuality(graphics, compositingQuality);
}

GpStatus WINAPI DllExports::GdipGetCompositingQuality(GpGraphics* graphics, CompositingQuality* compositingQuality)
{
	return gdiplusWrapper.GdipGetCompositingQuality(graphics, compositingQuality);
}

GpStatus WINAPI DllExports::GdipSetSmoothingMode(GpGraphics* graphics, SmoothingMode smoothingMode)
{
	return gdiplusWrapper.GdipSetSmoothingMode(graphics, smoothingMode);
}

GpStatus WINAPI DllExports::GdipGetSmoothingMode(GpGraphics* graphics, SmoothingMode* smoothingMode)
{
	return gdiplusWrapper.GdipGetSmoothingMode(graphics, smoothingMode);
}

GpStatus WINAPI DllExports::GdipSetPixelOffsetMode(GpGraphics* graphics, PixelOffsetMode pixelOffsetMode)
{
	return gdiplusWrapper.GdipSetPixelOffsetMode(graphics, pixelOffsetMode);
}

GpStatus WINAPI DllExports::GdipGetPixelOffsetMode(GpGraphics* graphics, PixelOffsetMode* pixelOffsetMode)
{
	return gdiplusWrapper.GdipGetPixelOffsetMode(graphics, pixelOffsetMode);
}

GpStatus WINAPI DllExports::GdipSetTextRenderingHint(GpGraphics* graphics, TextRenderingHint mode)
{
	return gdiplusWrapper.GdipSetTextRenderingHint(graphics, mode);
}

GpStatus WINAPI DllExports::GdipGetTextRenderingHint(GpGraphics* graphics, TextRenderingHint* mode)
{
	return gdiplusWrapper.GdipGetTextRenderingHint(graphics, mode);
}

GpStatus WINAPI DllExports::GdipSetTextContrast(GpGraphics* graphics, UINT contrast)
{
	return gdiplusWrapper.GdipSetTextContrast(graphics, contrast);
}

GpStatus WINAPI DllExports::GdipGetTextContrast(GpGraphics* graphics, UINT* contrast)
{
	return gdiplusWrapper.GdipGetTextContrast(graphics, contrast);
}

GpStatus WINAPI DllExports::GdipSetInterpolationMode(GpGraphics* graphics, InterpolationMode interpolationMode)
{
	return gdiplusWrapper.GdipSetInterpolationMode(graphics, interpolationMode);
}

GpStatus WINAPI DllExports::GdipGetInterpolationMode(GpGraphics* graphics, InterpolationMode* interpolationMode)
{
	return gdiplusWrapper.GdipGetInterpolationMode(graphics, interpolationMode);
}

GpStatus WINAPI DllExports::GdipSetWorldTransform(GpGraphics* graphics, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipSetWorldTransform(graphics, matrix);
}

GpStatus WINAPI DllExports::GdipResetWorldTransform(GpGraphics* graphics)
{
	return gdiplusWrapper.GdipResetWorldTransform(graphics);
}

GpStatus WINAPI DllExports::GdipMultiplyWorldTransform(GpGraphics* graphics, GDIPCONST GpMatrix* matrix, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipMultiplyWorldTransform(graphics, matrix, order);
}

GpStatus WINAPI DllExports::GdipTranslateWorldTransform(GpGraphics* graphics, REAL dx, REAL dy, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipTranslateWorldTransform(graphics, dx, dy, order);
}

GpStatus WINAPI DllExports::GdipScaleWorldTransform(GpGraphics* graphics, REAL sx, REAL sy, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipScaleWorldTransform(graphics, sx, sy, order);
}

GpStatus WINAPI DllExports::GdipRotateWorldTransform(GpGraphics* graphics, REAL angle, GpMatrixOrder order)
{
	return gdiplusWrapper.GdipRotateWorldTransform(graphics, angle, order);
}

GpStatus WINAPI DllExports::GdipGetWorldTransform(GpGraphics* graphics, GpMatrix* matrix)
{
	return gdiplusWrapper.GdipGetWorldTransform(graphics, matrix);
}

GpStatus WINAPI DllExports::GdipResetPageTransform(GpGraphics* graphics)
{
	return gdiplusWrapper.GdipResetPageTransform(graphics);
}

GpStatus WINAPI DllExports::GdipGetPageUnit(GpGraphics* graphics, GpUnit* unit)
{
	return gdiplusWrapper.GdipGetPageUnit(graphics, unit);
}

GpStatus WINAPI DllExports::GdipGetPageScale(GpGraphics* graphics, REAL* scale)
{
	return gdiplusWrapper.GdipGetPageScale(graphics, scale);
}

GpStatus WINAPI DllExports::GdipSetPageUnit(GpGraphics* graphics, GpUnit unit)
{
	return gdiplusWrapper.GdipSetPageUnit(graphics, unit);
}

GpStatus WINAPI DllExports::GdipSetPageScale(GpGraphics* graphics, REAL scale)
{
	return gdiplusWrapper.GdipSetPageScale(graphics, scale);
}

GpStatus WINAPI DllExports::GdipGetDpiX(GpGraphics* graphics, REAL* dpi)
{
	return gdiplusWrapper.GdipGetDpiX(graphics, dpi);
}

GpStatus WINAPI DllExports::GdipGetDpiY(GpGraphics* graphics, REAL* dpi)
{
	return gdiplusWrapper.GdipGetDpiY(graphics, dpi);
}

GpStatus WINAPI DllExports::GdipTransformPoints(GpGraphics* graphics, GpCoordinateSpace destSpace, GpCoordinateSpace srcSpace, GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipTransformPoints(graphics, destSpace, srcSpace, points, count);
}

GpStatus WINAPI DllExports::GdipTransformPointsI(GpGraphics* graphics, GpCoordinateSpace destSpace, GpCoordinateSpace srcSpace, GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipTransformPointsI(graphics, destSpace, srcSpace, points, count);
}

GpStatus WINAPI DllExports::GdipGetNearestColor(GpGraphics* graphics, ARGB* argb)
{
	return gdiplusWrapper.GdipGetNearestColor(graphics, argb);
}

GpStatus WINAPI DllExports::GdipDrawLine(GpGraphics* graphics, GpPen* pen, REAL x1, REAL y1, REAL x2, REAL y2)
{
	return gdiplusWrapper.GdipDrawLine(graphics, pen, x1, y1, x2, y2);
}

GpStatus WINAPI DllExports::GdipDrawLineI(GpGraphics* graphics, GpPen* pen, INT x1, INT y1, INT x2, INT y2)
{
	return gdiplusWrapper.GdipDrawLineI(graphics, pen, x1, y1, x2, y2);
}

GpStatus WINAPI DllExports::GdipDrawLines(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipDrawLines(graphics, pen, points, count);
}

GpStatus WINAPI DllExports::GdipDrawLinesI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipDrawLinesI(graphics, pen, points, count);
}

GpStatus WINAPI DllExports::GdipDrawArc(GpGraphics* graphics, GpPen* pen, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
{
	return gdiplusWrapper.GdipDrawArc(graphics, pen, x, y, width, height, startAngle, sweepAngle);
}

GpStatus WINAPI DllExports::GdipDrawArcI(GpGraphics* graphics, GpPen* pen, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
{
	return gdiplusWrapper.GdipDrawArcI(graphics, pen, x, y, width, height, startAngle, sweepAngle);
}

GpStatus WINAPI DllExports::GdipDrawBezier(GpGraphics* graphics, GpPen* pen, REAL x1, REAL y1, REAL x2, REAL y2, REAL x3, REAL y3, REAL x4, REAL y4)
{
	return gdiplusWrapper.GdipDrawBezier(graphics, pen, x1, y1, x2, y2, x3, y3, x4, y4);
}

GpStatus WINAPI DllExports::GdipDrawBezierI(GpGraphics* graphics, GpPen* pen, INT x1, INT y1, INT x2, INT y2, INT x3, INT y3, INT x4, INT y4)
{
	return gdiplusWrapper.GdipDrawBezierI(graphics, pen, x1, y1, x2, y2, x3, y3, x4, y4);
}

GpStatus WINAPI DllExports::GdipDrawBeziers(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipDrawBeziers(graphics, pen, points, count);
}

GpStatus WINAPI DllExports::GdipDrawBeziersI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipDrawBeziersI(graphics, pen, points, count);
}

GpStatus WINAPI DllExports::GdipDrawRectangle(GpGraphics* graphics, GpPen* pen, REAL x, REAL y, REAL width, REAL height)
{
	return gdiplusWrapper.GdipDrawRectangle(graphics, pen, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipDrawRectangleI(GpGraphics* graphics, GpPen* pen, INT x, INT y, INT width, INT height)
{
	return gdiplusWrapper.GdipDrawRectangleI(graphics, pen, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipDrawRectangles(GpGraphics* graphics, GpPen* pen, GDIPCONST GpRectF* rects, INT count)
{
	return gdiplusWrapper.GdipDrawRectangles(graphics, pen, rects, count);
}

GpStatus WINAPI DllExports::GdipDrawRectanglesI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpRect* rects, INT count)
{
	return gdiplusWrapper.GdipDrawRectanglesI(graphics, pen, rects, count);
}

GpStatus WINAPI DllExports::GdipDrawEllipse(GpGraphics* graphics, GpPen* pen, REAL x, REAL y, REAL width, REAL height)
{
	return gdiplusWrapper.GdipDrawEllipse(graphics, pen, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipDrawEllipseI(GpGraphics* graphics, GpPen* pen, INT x, INT y, INT width, INT height)
{
	return gdiplusWrapper.GdipDrawEllipseI(graphics, pen, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipDrawPie(GpGraphics* graphics, GpPen* pen, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
{
	return gdiplusWrapper.GdipDrawPie(graphics, pen, x, y, width, height, startAngle, sweepAngle);
}

GpStatus WINAPI DllExports::GdipDrawPieI(GpGraphics* graphics, GpPen* pen, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
{
	return gdiplusWrapper.GdipDrawPieI(graphics, pen, x, y, width, height, startAngle, sweepAngle);
}

GpStatus WINAPI DllExports::GdipDrawPolygon(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipDrawPolygon(graphics, pen, points, count);
}

GpStatus WINAPI DllExports::GdipDrawPolygonI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipDrawPolygonI(graphics, pen, points, count);
}

GpStatus WINAPI DllExports::GdipDrawPath(GpGraphics* graphics, GpPen* pen, GpPath* path)
{
	return gdiplusWrapper.GdipDrawPath(graphics, pen, path);
}

GpStatus WINAPI DllExports::GdipDrawCurve(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipDrawCurve(graphics, pen, points, count);
}

GpStatus WINAPI DllExports::GdipDrawCurveI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipDrawCurveI(graphics, pen, points, count);
}

GpStatus WINAPI DllExports::GdipDrawCurve2(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count, REAL tension)
{
	return gdiplusWrapper.GdipDrawCurve2(graphics, pen, points, count, tension);
}

GpStatus WINAPI DllExports::GdipDrawCurve2I(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count, REAL tension)
{
	return gdiplusWrapper.GdipDrawCurve2I(graphics, pen, points, count, tension);
}

GpStatus WINAPI DllExports::GdipDrawCurve3(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count, INT offset, INT numberOfSegments, REAL tension)
{
	return gdiplusWrapper.GdipDrawCurve3(graphics, pen, points, count, offset, numberOfSegments, tension);
}

GpStatus WINAPI DllExports::GdipDrawCurve3I(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count, INT offset, INT numberOfSegments, REAL tension)
{
	return gdiplusWrapper.GdipDrawCurve3I(graphics, pen, points, count, offset, numberOfSegments, tension);
}

GpStatus WINAPI DllExports::GdipDrawClosedCurve(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipDrawClosedCurve(graphics, pen, points, count);
}

GpStatus WINAPI DllExports::GdipDrawClosedCurveI(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipDrawClosedCurveI(graphics, pen, points, count);
}

GpStatus WINAPI DllExports::GdipDrawClosedCurve2(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPointF* points, INT count, REAL tension)
{
	return gdiplusWrapper.GdipDrawClosedCurve2(graphics, pen, points, count, tension);
}

GpStatus WINAPI DllExports::GdipDrawClosedCurve2I(GpGraphics* graphics, GpPen* pen, GDIPCONST GpPoint* points, INT count, REAL tension)
{
	return gdiplusWrapper.GdipDrawClosedCurve2I(graphics, pen, points, count, tension);
}

GpStatus WINAPI DllExports::GdipGraphicsClear(GpGraphics* graphics, ARGB color)
{
	return gdiplusWrapper.GdipGraphicsClear(graphics, color);
}

GpStatus WINAPI DllExports::GdipFillRectangle(GpGraphics* graphics, GpBrush* brush, REAL x, REAL y, REAL width, REAL height)
{
	return gdiplusWrapper.GdipFillRectangle(graphics, brush, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipFillRectangleI(GpGraphics* graphics, GpBrush* brush, INT x, INT y, INT width, INT height)
{
	return gdiplusWrapper.GdipFillRectangleI(graphics, brush, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipFillRectangles(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpRectF* rects, INT count)
{
	return gdiplusWrapper.GdipFillRectangles(graphics, brush, rects, count);
}

GpStatus WINAPI DllExports::GdipFillRectanglesI(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpRect* rects, INT count)
{
	return gdiplusWrapper.GdipFillRectanglesI(graphics, brush, rects, count);
}

GpStatus WINAPI DllExports::GdipFillPolygon(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPointF* points, INT count, GpFillMode fillMode)
{
	return gdiplusWrapper.GdipFillPolygon(graphics, brush, points, count, fillMode);
}

GpStatus WINAPI DllExports::GdipFillPolygonI(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPoint* points, INT count, GpFillMode fillMode)
{
	return gdiplusWrapper.GdipFillPolygonI(graphics, brush, points, count, fillMode);
}

GpStatus WINAPI DllExports::GdipFillPolygon2(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipFillPolygon2(graphics, brush, points, count);
}

GpStatus WINAPI DllExports::GdipFillPolygon2I(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipFillPolygon2I(graphics, brush, points, count);
}

GpStatus WINAPI DllExports::GdipFillEllipse(GpGraphics* graphics, GpBrush* brush, REAL x, REAL y, REAL width, REAL height)
{
	return gdiplusWrapper.GdipFillEllipse(graphics, brush, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipFillEllipseI(GpGraphics* graphics, GpBrush* brush, INT x, INT y, INT width, INT height)
{
	return gdiplusWrapper.GdipFillEllipseI(graphics, brush, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipFillPie(GpGraphics* graphics, GpBrush* brush, REAL x, REAL y, REAL width, REAL height, REAL startAngle, REAL sweepAngle)
{
	return gdiplusWrapper.GdipFillPie(graphics, brush, x, y, width, height, startAngle, sweepAngle);
}

GpStatus WINAPI DllExports::GdipFillPieI(GpGraphics* graphics, GpBrush* brush, INT x, INT y, INT width, INT height, REAL startAngle, REAL sweepAngle)
{
	return gdiplusWrapper.GdipFillPieI(graphics, brush, x, y, width, height, startAngle, sweepAngle);
}

GpStatus WINAPI DllExports::GdipFillPath(GpGraphics* graphics, GpBrush* brush, GpPath* path)
{
	return gdiplusWrapper.GdipFillPath(graphics, brush, path);
}

GpStatus WINAPI DllExports::GdipFillClosedCurve(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPointF* points, INT count)
{
	return gdiplusWrapper.GdipFillClosedCurve(graphics, brush, points, count);
}

GpStatus WINAPI DllExports::GdipFillClosedCurveI(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPoint* points, INT count)
{
	return gdiplusWrapper.GdipFillClosedCurveI(graphics, brush, points, count);
}

GpStatus WINAPI DllExports::GdipFillClosedCurve2(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPointF* points, INT count, REAL tension, GpFillMode fillMode)
{
	return gdiplusWrapper.GdipFillClosedCurve2(graphics, brush, points, count, tension, fillMode);
}

GpStatus WINAPI DllExports::GdipFillClosedCurve2I(GpGraphics* graphics, GpBrush* brush, GDIPCONST GpPoint* points, INT count, REAL tension, GpFillMode fillMode)
{
	return gdiplusWrapper.GdipFillClosedCurve2I(graphics, brush, points, count, tension, fillMode);
}

GpStatus WINAPI DllExports::GdipFillRegion(GpGraphics* graphics, GpBrush* brush, GpRegion* region)
{
	return gdiplusWrapper.GdipFillRegion(graphics, brush, region);
}

GpStatus WINAPI DllExports::GdipDrawImage(GpGraphics* graphics, GpImage* image, REAL x, REAL y)
{
	return gdiplusWrapper.GdipDrawImage(graphics, image, x, y);
}

GpStatus WINAPI DllExports::GdipDrawImageI(GpGraphics* graphics, GpImage* image, INT x, INT y)
{
	return gdiplusWrapper.GdipDrawImageI(graphics, image, x, y);
}

GpStatus WINAPI DllExports::GdipDrawImageRect(GpGraphics* graphics, GpImage* image, REAL x, REAL y, REAL width, REAL height)
{
	return gdiplusWrapper.GdipDrawImageRect(graphics, image, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipDrawImageRectI(GpGraphics* graphics, GpImage* image, INT x, INT y, INT width, INT height)
{
	return gdiplusWrapper.GdipDrawImageRectI(graphics, image, x, y, width, height);
}

GpStatus WINAPI DllExports::GdipDrawImagePoints(GpGraphics* graphics, GpImage* image, GDIPCONST GpPointF* dstpoints, INT count)
{
	return gdiplusWrapper.GdipDrawImagePoints(graphics, image, dstpoints, count);
}

GpStatus WINAPI DllExports::GdipDrawImagePointsI(GpGraphics* graphics, GpImage* image, GDIPCONST GpPoint* dstpoints, INT count)
{
	return gdiplusWrapper.GdipDrawImagePointsI(graphics, image, dstpoints, count);
}

GpStatus WINAPI DllExports::GdipDrawImagePointRect(GpGraphics* graphics, GpImage* image, REAL x, REAL y, REAL srcx, REAL srcy, REAL srcwidth, REAL srcheight, GpUnit srcUnit)
{
	return gdiplusWrapper.GdipDrawImagePointRect(graphics, image, x, y, srcx, srcy, srcwidth, srcheight, srcUnit);
}

GpStatus WINAPI DllExports::GdipDrawImagePointRectI(GpGraphics* graphics, GpImage* image, INT x, INT y, INT srcx, INT srcy, INT srcwidth, INT srcheight, GpUnit srcUnit)
{
	return gdiplusWrapper.GdipDrawImagePointRectI(graphics, image, x, y, srcx, srcy, srcwidth, srcheight, srcUnit);
}

GpStatus WINAPI DllExports::GdipDrawImageRectRect(GpGraphics* graphics, GpImage* image, REAL dstx, REAL dsty, REAL dstwidth, REAL dstheight, REAL srcx, REAL srcy, REAL srcwidth, REAL srcheight, GpUnit srcUnit, GDIPCONST GpImageAttributes* imageAttributes, DrawImageAbort callback, VOID* callbackData)
{
	return gdiplusWrapper.GdipDrawImageRectRect(graphics, image, dstx, dsty, dstwidth, dstheight, srcx, srcy, srcwidth, srcheight, srcUnit, imageAttributes, callback, callbackData);
}

GpStatus WINAPI DllExports::GdipDrawImageRectRectI(GpGraphics* graphics, GpImage* image, INT dstx, INT dsty, INT dstwidth, INT dstheight, INT srcx, INT srcy, INT srcwidth, INT srcheight, GpUnit srcUnit, GDIPCONST GpImageAttributes* imageAttributes, DrawImageAbort callback, VOID* callbackData)
{
	return gdiplusWrapper.GdipDrawImageRectRectI(graphics, image, dstx, dsty, dstwidth, dstheight, srcx, srcy, srcwidth, srcheight, srcUnit, imageAttributes, callback, callbackData);
}

GpStatus WINAPI DllExports::GdipDrawImagePointsRect(GpGraphics* graphics, GpImage* image, GDIPCONST GpPointF* points, INT count, REAL srcx, REAL srcy, REAL srcwidth, REAL srcheight, GpUnit srcUnit, GDIPCONST GpImageAttributes* imageAttributes, DrawImageAbort callback, VOID* callbackData)
{
	return gdiplusWrapper.GdipDrawImagePointsRect(graphics, image, points, count, srcx, srcy, srcwidth, srcheight, srcUnit, imageAttributes, callback, callbackData);
}

GpStatus WINAPI DllExports::GdipDrawImagePointsRectI(GpGraphics* graphics, GpImage* image, GDIPCONST GpPoint* points, INT count, INT srcx, INT srcy, INT srcwidth, INT srcheight, GpUnit srcUnit, GDIPCONST GpImageAttributes* imageAttributes, DrawImageAbort callback, VOID* callbackData)
{
	return gdiplusWrapper.GdipDrawImagePointsRectI(graphics, image, points, count, srcx, srcy, srcwidth, srcheight, srcUnit, imageAttributes, callback, callbackData);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileDestPoint(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST PointF& destPoint, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileDestPoint(graphics, metafile, destPoint, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileDestPointI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Point& destPoint, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileDestPointI(graphics, metafile, destPoint, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileDestRect(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST RectF& destRect, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileDestRect(graphics, metafile, destRect, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileDestRectI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Rect& destRect, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileDestRectI(graphics, metafile, destRect, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileDestPoints(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST PointF* destPoints, INT count, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileDestPoints(graphics, metafile, destPoints, count, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileDestPointsI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Point* destPoints, INT count, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileDestPointsI(graphics, metafile, destPoints, count, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileSrcRectDestPoint(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST PointF& destPoint, GDIPCONST RectF& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileSrcRectDestPoint(graphics, metafile, destPoint, srcRect, srcUnit, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileSrcRectDestPointI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Point& destPoint, GDIPCONST Rect& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileSrcRectDestPointI(graphics, metafile, destPoint, srcRect, srcUnit, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileSrcRectDestRect(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST RectF& destRect, GDIPCONST RectF& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileSrcRectDestRect(graphics, metafile, destRect, srcRect, srcUnit, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileSrcRectDestRectI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Rect& destRect, GDIPCONST Rect& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileSrcRectDestRectI(graphics, metafile, destRect, srcRect, srcUnit, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileSrcRectDestPoints(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST PointF* destPoints, INT count, GDIPCONST RectF& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileSrcRectDestPoints(graphics, metafile, destPoints, count, srcRect, srcUnit, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipEnumerateMetafileSrcRectDestPointsI(GpGraphics* graphics, GDIPCONST GpMetafile* metafile, GDIPCONST Point* destPoints, INT count, GDIPCONST Rect& srcRect, Unit srcUnit, EnumerateMetafileProc callback, VOID* callbackData, GDIPCONST GpImageAttributes* imageAttributes)
{
	return gdiplusWrapper.GdipEnumerateMetafileSrcRectDestPointsI(graphics, metafile, destPoints, count, srcRect, srcUnit, callback, callbackData, imageAttributes);
}

GpStatus WINAPI DllExports::GdipPlayMetafileRecord(GDIPCONST GpMetafile* metafile, EmfPlusRecordType recordType, UINT flags, UINT dataSize, GDIPCONST BYTE* data)
{
	return gdiplusWrapper.GdipPlayMetafileRecord(metafile, recordType, flags, dataSize, data);
}

GpStatus WINAPI DllExports::GdipSetClipGraphics(GpGraphics* graphics, GpGraphics* srcgraphics, CombineMode combineMode)
{
	return gdiplusWrapper.GdipSetClipGraphics(graphics, srcgraphics, combineMode);
}

GpStatus WINAPI DllExports::GdipSetClipRect(GpGraphics* graphics, REAL x, REAL y, REAL width, REAL height, CombineMode combineMode)
{
	return gdiplusWrapper.GdipSetClipRect(graphics, x, y, width, height, combineMode);
}

GpStatus WINAPI DllExports::GdipSetClipRectI(GpGraphics* graphics, INT x, INT y, INT width, INT height, CombineMode combineMode)
{
	return gdiplusWrapper.GdipSetClipRectI(graphics, x, y, width, height, combineMode);
}

GpStatus WINAPI DllExports::GdipSetClipPath(GpGraphics* graphics, GpPath* path, CombineMode combineMode)
{
	return gdiplusWrapper.GdipSetClipPath(graphics, path, combineMode);
}

GpStatus WINAPI DllExports::GdipSetClipRegion(GpGraphics* graphics, GpRegion* region, CombineMode combineMode)
{
	return gdiplusWrapper.GdipSetClipRegion(graphics, region, combineMode);
}

GpStatus WINAPI DllExports::GdipSetClipHrgn(GpGraphics* graphics, HRGN hRgn, CombineMode combineMode)
{
	return gdiplusWrapper.GdipSetClipHrgn(graphics, hRgn, combineMode);
}

GpStatus WINAPI DllExports::GdipResetClip(GpGraphics* graphics)
{
	return gdiplusWrapper.GdipResetClip(graphics);
}

GpStatus WINAPI DllExports::GdipTranslateClip(GpGraphics* graphics, REAL dx, REAL dy)
{
	return gdiplusWrapper.GdipTranslateClip(graphics, dx, dy);
}

GpStatus WINAPI DllExports::GdipTranslateClipI(GpGraphics* graphics, INT dx, INT dy)
{
	return gdiplusWrapper.GdipTranslateClipI(graphics, dx, dy);
}

GpStatus WINAPI DllExports::GdipGetClip(GpGraphics* graphics, GpRegion* region)
{
	return gdiplusWrapper.GdipGetClip(graphics, region);
}

GpStatus WINAPI DllExports::GdipGetClipBounds(GpGraphics* graphics, GpRectF* rect)
{
	return gdiplusWrapper.GdipGetClipBounds(graphics, rect);
}

GpStatus WINAPI DllExports::GdipGetClipBoundsI(GpGraphics* graphics, GpRect* rect)
{
	return gdiplusWrapper.GdipGetClipBoundsI(graphics, rect);
}

GpStatus WINAPI DllExports::GdipIsClipEmpty(GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsClipEmpty(graphics, result);
}

GpStatus WINAPI DllExports::GdipGetVisibleClipBounds(GpGraphics* graphics, GpRectF* rect)
{
	return gdiplusWrapper.GdipGetVisibleClipBounds(graphics, rect);
}

GpStatus WINAPI DllExports::GdipGetVisibleClipBoundsI(GpGraphics* graphics, GpRect* rect)
{
	return gdiplusWrapper.GdipGetVisibleClipBoundsI(graphics, rect);
}

GpStatus WINAPI DllExports::GdipIsVisibleClipEmpty(GpGraphics* graphics, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisibleClipEmpty(graphics, result);
}

GpStatus WINAPI DllExports::GdipIsVisiblePoint(GpGraphics* graphics, REAL x, REAL y, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisiblePoint(graphics, x, y, result);
}

GpStatus WINAPI DllExports::GdipIsVisiblePointI(GpGraphics* graphics, INT x, INT y, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisiblePointI(graphics, x, y, result);
}

GpStatus WINAPI DllExports::GdipIsVisibleRect(GpGraphics* graphics, REAL x, REAL y, REAL width, REAL height, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisibleRect(graphics, x, y, width, height, result);
}

GpStatus WINAPI DllExports::GdipIsVisibleRectI(GpGraphics* graphics, INT x, INT y, INT width, INT height, BOOL* result)
{
	return gdiplusWrapper.GdipIsVisibleRectI(graphics, x, y, width, height, result);
}

GpStatus WINAPI DllExports::GdipSaveGraphics(GpGraphics* graphics, GraphicsState* state)
{
	return gdiplusWrapper.GdipSaveGraphics(graphics, state);
}

GpStatus WINAPI DllExports::GdipRestoreGraphics(GpGraphics* graphics, GraphicsState state)
{
	return gdiplusWrapper.GdipRestoreGraphics(graphics, state);
}

GpStatus WINAPI DllExports::GdipBeginContainer(GpGraphics* graphics, GDIPCONST GpRectF* dstrect, GDIPCONST GpRectF* srcrect, GpUnit unit, GraphicsContainer* state)
{
	return gdiplusWrapper.GdipBeginContainer(graphics, dstrect, srcrect, unit, state);
}

GpStatus WINAPI DllExports::GdipBeginContainerI(GpGraphics* graphics, GDIPCONST GpRect* dstrect, GDIPCONST GpRect* srcrect, GpUnit unit, GraphicsContainer* state)
{
	return gdiplusWrapper.GdipBeginContainerI(graphics, dstrect, srcrect, unit, state);
}

GpStatus WINAPI DllExports::GdipBeginContainer2(GpGraphics* graphics, GraphicsContainer* state)
{
	return gdiplusWrapper.GdipBeginContainer2(graphics, state);
}

GpStatus WINAPI DllExports::GdipEndContainer(GpGraphics* graphics, GraphicsContainer state)
{
	return gdiplusWrapper.GdipEndContainer(graphics, state);
}

GpStatus DllExports::GdipGetMetafileHeaderFromWmf(HMETAFILE hWmf, GDIPCONST WmfPlaceableFileHeader* wmfPlaceableFileHeader, MetafileHeader* header)
{
	return gdiplusWrapper.GdipGetMetafileHeaderFromWmf(hWmf, wmfPlaceableFileHeader, header);
}

GpStatus WINAPI DllExports::GdipGetMetafileHeaderFromEmf(HENHMETAFILE hEmf, MetafileHeader* header)
{
	return gdiplusWrapper.GdipGetMetafileHeaderFromEmf(hEmf, header);
}

GpStatus WINAPI DllExports::GdipGetMetafileHeaderFromFile(GDIPCONST WCHAR* filename, MetafileHeader* header)
{
	return gdiplusWrapper.GdipGetMetafileHeaderFromFile(filename, header);
}

GpStatus WINAPI DllExports::GdipGetMetafileHeaderFromStream(IStream* stream, MetafileHeader* header)
{
	return gdiplusWrapper.GdipGetMetafileHeaderFromStream(stream, header);
}

GpStatus WINAPI DllExports::GdipGetMetafileHeaderFromMetafile(GpMetafile* metafile, MetafileHeader* header)
{
	return gdiplusWrapper.GdipGetMetafileHeaderFromMetafile(metafile, header);
}

GpStatus WINAPI DllExports::GdipGetHemfFromMetafile(GpMetafile* metafile, HENHMETAFILE* hEmf)
{
	return gdiplusWrapper.GdipGetHemfFromMetafile(metafile, hEmf);
}

GpStatus WINAPI DllExports::GdipCreateStreamOnFile(GDIPCONST WCHAR* filename, UINT access, IStream** stream)
{
	return gdiplusWrapper.GdipCreateStreamOnFile(filename, access, stream);
}

GpStatus WINAPI DllExports::GdipCreateMetafileFromWmf(HMETAFILE hWmf, BOOL deleteWmf, GDIPCONST WmfPlaceableFileHeader* wmfPlaceableFileHeader, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipCreateMetafileFromWmf(hWmf, deleteWmf, wmfPlaceableFileHeader, metafile);
}

GpStatus WINAPI DllExports::GdipCreateMetafileFromEmf(HENHMETAFILE hEmf, BOOL deleteEmf, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipCreateMetafileFromEmf(hEmf, deleteEmf, metafile);
}

GpStatus WINAPI DllExports::GdipCreateMetafileFromFile(GDIPCONST WCHAR* file, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipCreateMetafileFromFile(file, metafile);
}

GpStatus WINAPI DllExports::GdipCreateMetafileFromWmfFile(GDIPCONST WCHAR* file, GDIPCONST WmfPlaceableFileHeader* wmfPlaceableFileHeader, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipCreateMetafileFromWmfFile(file, wmfPlaceableFileHeader, metafile);
}

GpStatus WINAPI DllExports::GdipCreateMetafileFromStream(IStream* stream, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipCreateMetafileFromStream(stream, metafile);
}

GpStatus WINAPI DllExports::GdipRecordMetafile(HDC referenceHdc, EmfType type, GDIPCONST GpRectF* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipRecordMetafile(referenceHdc, type, frameRect, frameUnit, description, metafile);
}

GpStatus WINAPI DllExports::GdipRecordMetafileI(HDC referenceHdc, EmfType type, GDIPCONST GpRect* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipRecordMetafileI(referenceHdc, type, frameRect, frameUnit, description, metafile);
}

GpStatus WINAPI DllExports::GdipRecordMetafileFileName(GDIPCONST WCHAR* fileName, HDC referenceHdc, EmfType type, GDIPCONST GpRectF* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipRecordMetafileFileName(fileName, referenceHdc, type, frameRect, frameUnit, description, metafile);
}

GpStatus WINAPI DllExports::GdipRecordMetafileFileNameI(GDIPCONST WCHAR* fileName, HDC referenceHdc, EmfType type, GDIPCONST GpRect* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipRecordMetafileFileNameI(fileName, referenceHdc, type, frameRect, frameUnit, description, metafile);
}

GpStatus WINAPI DllExports::GdipRecordMetafileStream(IStream* stream, HDC referenceHdc, EmfType type, GDIPCONST GpRectF* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipRecordMetafileStream(stream, referenceHdc, type, frameRect, frameUnit, description, metafile);
}

GpStatus WINAPI DllExports::GdipRecordMetafileStreamI(IStream* stream, HDC referenceHdc, EmfType type, GDIPCONST GpRect* frameRect, MetafileFrameUnit frameUnit, GDIPCONST WCHAR* description, GpMetafile** metafile)
{
	return gdiplusWrapper.GdipRecordMetafileStreamI(stream, referenceHdc, type, frameRect, frameUnit, description, metafile);
}

GpStatus WINAPI DllExports::GdipSetMetafileDownLevelRasterizationLimit(GpMetafile* metafile, UINT metafileRasterizationLimitDpi)
{
	return gdiplusWrapper.GdipSetMetafileDownLevelRasterizationLimit(metafile, metafileRasterizationLimitDpi);
}

GpStatus WINAPI DllExports::GdipGetMetafileDownLevelRasterizationLimit(GDIPCONST GpMetafile* metafile, UINT* metafileRasterizationLimitDpi)
{
	return gdiplusWrapper.GdipGetMetafileDownLevelRasterizationLimit(metafile, metafileRasterizationLimitDpi);
}

GpStatus WINAPI DllExports::GdipGetImageDecodersSize(UINT* numDecoders, UINT* size)
{
	return gdiplusWrapper.GdipGetImageDecodersSize(numDecoders, size);
}

GpStatus WINAPI DllExports::GdipGetImageDecoders(UINT numDecoders, UINT size, ImageCodecInfo* decoders)
{
	return gdiplusWrapper.GdipGetImageDecoders(numDecoders, size, decoders);
}

GpStatus WINAPI DllExports::GdipGetImageEncodersSize(UINT* numEncoders, UINT* size)
{
	return gdiplusWrapper.GdipGetImageEncodersSize(numEncoders, size);
}

GpStatus WINAPI DllExports::GdipGetImageEncoders(UINT numEncoders, UINT size, ImageCodecInfo* encoders)
{
	return gdiplusWrapper.GdipGetImageEncoders(numEncoders, size, encoders);
}

GpStatus WINAPI DllExports::GdipComment(GpGraphics* graphics, UINT sizeData, GDIPCONST BYTE* data)
{
	return gdiplusWrapper.GdipComment(graphics, sizeData, data);
}

GpStatus WINAPI DllExports::GdipCreateFontFamilyFromName(GDIPCONST WCHAR* name, GpFontCollection* fontCollection, GpFontFamily** FontFamily)
{
	return gdiplusWrapper.GdipCreateFontFamilyFromName(name, fontCollection, FontFamily);
}

GpStatus WINAPI DllExports::GdipDeleteFontFamily(GpFontFamily* FontFamily)
{
	return gdiplusWrapper.GdipDeleteFontFamily(FontFamily);
}

GpStatus WINAPI DllExports::GdipCloneFontFamily(GpFontFamily* FontFamily, GpFontFamily** clonedFontFamily)
{
	return gdiplusWrapper.GdipCloneFontFamily(FontFamily, clonedFontFamily);
}

GpStatus WINAPI DllExports::GdipGetGenericFontFamilySansSerif(GpFontFamily** nativeFamily)
{
	return gdiplusWrapper.GdipGetGenericFontFamilySansSerif(nativeFamily);
}

GpStatus WINAPI DllExports::GdipGetGenericFontFamilySerif(GpFontFamily** nativeFamily)
{
	return gdiplusWrapper.GdipGetGenericFontFamilySerif(nativeFamily);
}

GpStatus WINAPI DllExports::GdipGetGenericFontFamilyMonospace(GpFontFamily** nativeFamily)
{
	return gdiplusWrapper.GdipGetGenericFontFamilyMonospace(nativeFamily);
}

GpStatus WINAPI DllExports::GdipGetFamilyName(GDIPCONST GpFontFamily* family, WCHAR name[LF_FACESIZE], LANGID language)
{
	return gdiplusWrapper.GdipGetFamilyName(family, name, language);
}

GpStatus WINAPI DllExports::GdipIsStyleAvailable(GDIPCONST GpFontFamily* family, INT style, BOOL* IsStyleAvailable)
{
	return gdiplusWrapper.GdipIsStyleAvailable(family, style, IsStyleAvailable);
}

GpStatus WINAPI DllExports::GdipFontCollectionEnumerable(GpFontCollection* fontCollection, GpGraphics* graphics, INT* numFound)
{
	return gdiplusWrapper.GdipFontCollectionEnumerable(fontCollection, graphics, numFound);
}

GpStatus WINAPI DllExports::GdipFontCollectionEnumerate(GpFontCollection* fontCollection, INT numSought, GpFontFamily* gpfamilies[], INT* numFound, GpGraphics* graphics)
{
	return gdiplusWrapper.GdipFontCollectionEnumerate(fontCollection, numSought, gpfamilies, numFound, graphics);
}

GpStatus WINAPI DllExports::GdipGetEmHeight(GDIPCONST GpFontFamily* family, INT style, UINT16* EmHeight)
{
	return gdiplusWrapper.GdipGetEmHeight(family, style, EmHeight);
}

GpStatus WINAPI DllExports::GdipGetCellAscent(GDIPCONST GpFontFamily* family, INT style, UINT16* CellAscent)
{
	return gdiplusWrapper.GdipGetCellAscent(family, style, CellAscent);
}

GpStatus WINAPI DllExports::GdipGetCellDescent(GDIPCONST GpFontFamily* family, INT style, UINT16* CellDescent)
{
	return gdiplusWrapper.GdipGetCellDescent(family, style, CellDescent);
}

GpStatus WINAPI DllExports::GdipGetLineSpacing(GDIPCONST GpFontFamily* family, INT style, UINT16* LineSpacing)
{
	return gdiplusWrapper.GdipGetLineSpacing(family, style, LineSpacing);
}

GpStatus WINAPI DllExports::GdipCreateFontFromDC(HDC hdc, GpFont** font)
{
	return gdiplusWrapper.GdipCreateFontFromDC(hdc, font);
}

GpStatus WINAPI DllExports::GdipCreateFontFromLogfontA(HDC hdc, GDIPCONST LOGFONTA* logfont, GpFont** font)
{
	return gdiplusWrapper.GdipCreateFontFromLogfontA(hdc, logfont, font);
}

GpStatus WINAPI DllExports::GdipCreateFontFromLogfontW(HDC hdc, GDIPCONST LOGFONTW* logfont, GpFont** font)
{
	return gdiplusWrapper.GdipCreateFontFromLogfontW(hdc, logfont, font);
}

GpStatus WINAPI DllExports::GdipCreateFont(GDIPCONST GpFontFamily* fontFamily, REAL emSize, INT style, Unit unit, GpFont** font)
{
	return gdiplusWrapper.GdipCreateFont(fontFamily, emSize, style, unit, font);
}

GpStatus WINAPI DllExports::GdipCloneFont(GpFont* font, GpFont** cloneFont)
{
	return gdiplusWrapper.GdipCloneFont(font, cloneFont);
}

GpStatus WINAPI DllExports::GdipDeleteFont(GpFont* font)
{
	return gdiplusWrapper.GdipDeleteFont(font);
}

GpStatus WINAPI DllExports::GdipGetFamily(GpFont* font, GpFontFamily** family)
{
	return gdiplusWrapper.GdipGetFamily(font, family);
}

GpStatus WINAPI DllExports::GdipGetFontStyle(GpFont* font, INT* style)
{
	return gdiplusWrapper.GdipGetFontStyle(font, style);
}

GpStatus WINAPI DllExports::GdipGetFontSize(GpFont* font, REAL* size)
{
	return gdiplusWrapper.GdipGetFontSize(font, size);
}

GpStatus WINAPI DllExports::GdipGetFontUnit(GpFont* font, Unit* unit)
{
	return gdiplusWrapper.GdipGetFontUnit(font, unit);
}

GpStatus WINAPI DllExports::GdipGetFontHeight(GDIPCONST GpFont* font, GDIPCONST GpGraphics* graphics, REAL* height)
{
	return gdiplusWrapper.GdipGetFontHeight(font, graphics, height);
}

GpStatus WINAPI DllExports::GdipGetFontHeightGivenDPI(GDIPCONST GpFont* font, REAL dpi, REAL* height)
{
	return gdiplusWrapper.GdipGetFontHeightGivenDPI(font, dpi, height);
}

GpStatus WINAPI DllExports::GdipGetLogFontA(GpFont* font, GpGraphics* graphics, LOGFONTA* logfontA)
{
	return gdiplusWrapper.GdipGetLogFontA(font, graphics, logfontA);
}

GpStatus WINAPI DllExports::GdipGetLogFontW(GpFont* font, GpGraphics* graphics, LOGFONTW* logfontW)
{
	return gdiplusWrapper.GdipGetLogFontW(font, graphics, logfontW);
}

GpStatus WINAPI DllExports::GdipNewInstalledFontCollection(GpFontCollection** fontCollection)
{
	return gdiplusWrapper.GdipNewInstalledFontCollection(fontCollection);
}

GpStatus WINAPI DllExports::GdipNewPrivateFontCollection(GpFontCollection** fontCollection)
{
	return gdiplusWrapper.GdipNewPrivateFontCollection(fontCollection);
}

GpStatus WINAPI DllExports::GdipDeletePrivateFontCollection(GpFontCollection** fontCollection)
{
	return gdiplusWrapper.GdipDeletePrivateFontCollection(fontCollection);
}

GpStatus WINAPI DllExports::GdipGetFontCollectionFamilyCount(GpFontCollection* fontCollection, INT* numFound)
{
	return gdiplusWrapper.GdipGetFontCollectionFamilyCount(fontCollection, numFound);
}

GpStatus WINAPI DllExports::GdipGetFontCollectionFamilyList(GpFontCollection* fontCollection, INT numSought, GpFontFamily* gpfamilies[], INT* numFound)
{
	return gdiplusWrapper.GdipGetFontCollectionFamilyList(fontCollection, numSought, gpfamilies, numFound);
}

GpStatus WINAPI DllExports::GdipPrivateAddFontFile(GpFontCollection* fontCollection, GDIPCONST WCHAR* filename)
{
	return gdiplusWrapper.GdipPrivateAddFontFile(fontCollection, filename);
}

GpStatus WINAPI DllExports::GdipPrivateAddMemoryFont(GpFontCollection* fontCollection, GDIPCONST void* memory, INT length)
{
	return gdiplusWrapper.GdipPrivateAddMemoryFont(fontCollection, memory, length);
}

GpStatus WINAPI DllExports::GdipDrawString(GpGraphics* graphics, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFont* font, GDIPCONST RectF* layoutRect, GDIPCONST GpStringFormat* stringFormat, GDIPCONST GpBrush* brush)
{
	return gdiplusWrapper.GdipDrawString(graphics, string, length, font, layoutRect, stringFormat, brush);
}

GpStatus WINAPI DllExports::GdipMeasureString(GpGraphics* graphics, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFont* font, GDIPCONST RectF* layoutRect, GDIPCONST GpStringFormat* stringFormat, RectF* boundingBox, INT* codepointsFitted, INT* linesFilled)
{
	return gdiplusWrapper.GdipMeasureString(graphics, string, length, font, layoutRect, stringFormat, boundingBox, codepointsFitted, linesFilled);
}

GpStatus WINAPI DllExports::GdipMeasureCharacterRanges(GpGraphics* graphics, GDIPCONST WCHAR* string, INT length, GDIPCONST GpFont* font, GDIPCONST RectF& layoutRect, GDIPCONST GpStringFormat* stringFormat, INT regionCount, GpRegion** regions)
{
	return gdiplusWrapper.GdipMeasureCharacterRanges(graphics, string, length, font, layoutRect, stringFormat, regionCount, regions);
}

GpStatus WINAPI DllExports::GdipDrawDriverString(GpGraphics* graphics, GDIPCONST UINT16* text, INT length, GDIPCONST GpFont* font, GDIPCONST GpBrush* brush, GDIPCONST PointF* positions, INT flags, GDIPCONST GpMatrix* matrix)
{
	return gdiplusWrapper.GdipDrawDriverString(graphics, text, length, font, brush, positions, flags, matrix);
}

GpStatus WINAPI DllExports::GdipMeasureDriverString(GpGraphics* graphics, GDIPCONST UINT16* text, INT length, GDIPCONST GpFont* font, GDIPCONST PointF* positions, INT flags, GDIPCONST GpMatrix* matrix, RectF* boundingBox)
{
	return gdiplusWrapper.GdipMeasureDriverString(graphics, text, length, font, positions, flags, matrix, boundingBox);
}

GpStatus WINAPI DllExports::GdipCreateStringFormat(INT formatAttributes, LANGID language, GpStringFormat** format)
{
	return gdiplusWrapper.GdipCreateStringFormat(formatAttributes, language, format);
}

GpStatus WINAPI DllExports::GdipStringFormatGetGenericDefault(GpStringFormat** format)
{
	return gdiplusWrapper.GdipStringFormatGetGenericDefault(format);
}

GpStatus WINAPI DllExports::GdipStringFormatGetGenericTypographic(GpStringFormat** format)
{
	return gdiplusWrapper.GdipStringFormatGetGenericTypographic(format);
}

GpStatus WINAPI DllExports::GdipDeleteStringFormat(GpStringFormat* format)
{
	return gdiplusWrapper.GdipDeleteStringFormat(format);
}

GpStatus WINAPI DllExports::GdipCloneStringFormat(GDIPCONST GpStringFormat* format, GpStringFormat** newFormat)
{
	return gdiplusWrapper.GdipCloneStringFormat(format, newFormat);
}

GpStatus WINAPI DllExports::GdipSetStringFormatFlags(GpStringFormat* format, INT flags)
{
	return gdiplusWrapper.GdipSetStringFormatFlags(format, flags);
}

GpStatus WINAPI DllExports::GdipGetStringFormatFlags(GDIPCONST GpStringFormat* format, INT* flags)
{
	return gdiplusWrapper.GdipGetStringFormatFlags(format, flags);
}

GpStatus WINAPI DllExports::GdipSetStringFormatAlign(GpStringFormat* format, StringAlignment align)
{
	return gdiplusWrapper.GdipSetStringFormatAlign(format, align);
}

GpStatus WINAPI DllExports::GdipGetStringFormatAlign(GDIPCONST GpStringFormat* format, StringAlignment* align)
{
	return gdiplusWrapper.GdipGetStringFormatAlign(format, align);
}

GpStatus WINAPI DllExports::GdipSetStringFormatLineAlign(GpStringFormat* format, StringAlignment align)
{
	return gdiplusWrapper.GdipSetStringFormatLineAlign(format, align);
}

GpStatus WINAPI DllExports::GdipGetStringFormatLineAlign(GDIPCONST GpStringFormat* format, StringAlignment* align)
{
	return gdiplusWrapper.GdipGetStringFormatLineAlign(format, align);
}

GpStatus WINAPI DllExports::GdipSetStringFormatTrimming(GpStringFormat* format, StringTrimming trimming)
{
	return gdiplusWrapper.GdipSetStringFormatTrimming(format, trimming);
}

GpStatus WINAPI DllExports::GdipGetStringFormatTrimming(GDIPCONST GpStringFormat* format, StringTrimming* trimming)
{
	return gdiplusWrapper.GdipGetStringFormatTrimming(format, trimming);
}

GpStatus WINAPI DllExports::GdipSetStringFormatHotkeyPrefix(GpStringFormat* format, INT hotkeyPrefix)
{
	return gdiplusWrapper.GdipSetStringFormatHotkeyPrefix(format, hotkeyPrefix);
}

GpStatus WINAPI DllExports::GdipGetStringFormatHotkeyPrefix(GDIPCONST GpStringFormat* format, INT* hotkeyPrefix)
{
	return gdiplusWrapper.GdipGetStringFormatHotkeyPrefix(format, hotkeyPrefix);
}

GpStatus WINAPI DllExports::GdipSetStringFormatTabStops(GpStringFormat* format, REAL firstTabOffset, INT count, GDIPCONST REAL* tabStops)
{
	return gdiplusWrapper.GdipSetStringFormatTabStops(format, firstTabOffset, count, tabStops);
}

GpStatus WINAPI DllExports::GdipGetStringFormatTabStops(GDIPCONST GpStringFormat* format, INT count, REAL* firstTabOffset, REAL* tabStops)
{
	return gdiplusWrapper.GdipGetStringFormatTabStops(format, count, firstTabOffset, tabStops);
}

GpStatus WINAPI DllExports::GdipGetStringFormatTabStopCount(GDIPCONST GpStringFormat* format, INT* count)
{
	return gdiplusWrapper.GdipGetStringFormatTabStopCount(format, count);
}

GpStatus WINAPI DllExports::GdipSetStringFormatDigitSubstitution(GpStringFormat* format, LANGID language, StringDigitSubstitute substitute)
{
	return gdiplusWrapper.GdipSetStringFormatDigitSubstitution(format, language, substitute);
}

GpStatus WINAPI DllExports::GdipGetStringFormatDigitSubstitution(GDIPCONST GpStringFormat* format, LANGID* language, StringDigitSubstitute* substitute)
{
	return gdiplusWrapper.GdipGetStringFormatDigitSubstitution(format, language, substitute);
}

GpStatus WINAPI DllExports::GdipGetStringFormatMeasurableCharacterRangeCount(GDIPCONST GpStringFormat* format, INT* count)
{
	return gdiplusWrapper.GdipGetStringFormatMeasurableCharacterRangeCount(format, count);
}

GpStatus WINAPI DllExports::GdipSetStringFormatMeasurableCharacterRanges(GpStringFormat* format, INT rangeCount, GDIPCONST CharacterRange* ranges)
{
	return gdiplusWrapper.GdipSetStringFormatMeasurableCharacterRanges(format, rangeCount, ranges);
}

GpStatus WINAPI DllExports::GdipCreateCachedBitmap(GpBitmap* bitmap, GpGraphics* graphics, GpCachedBitmap** cachedBitmap)
{
	return gdiplusWrapper.GdipCreateCachedBitmap(bitmap, graphics, cachedBitmap);
}

GpStatus WINAPI DllExports::GdipDeleteCachedBitmap(GpCachedBitmap* cachedBitmap)
{
	return gdiplusWrapper.GdipDeleteCachedBitmap(cachedBitmap);
}

GpStatus WINAPI DllExports::GdipDrawCachedBitmap(GpGraphics* graphics, GpCachedBitmap* cachedBitmap, INT x, INT y)
{
	return gdiplusWrapper.GdipDrawCachedBitmap(graphics, cachedBitmap, x, y);
}

UINT WINAPI DllExports::GdipEmfToWmfBits(HENHMETAFILE hemf, UINT cbData16, LPBYTE pData16, INT iMapMode, INT eFlags)
{
	return gdiplusWrapper.GdipEmfToWmfBits(hemf, cbData16, pData16, iMapMode, eFlags);
}

HPALETTE WINAPI DllExports::GdipCreateHalftonePalette()
{
	return gdiplusWrapper.GdipCreateHalftonePalette();
}

